
double ttintegralnew(double v0,double g,double R,double psi,int *method)
// double v0,g,R,psi;
// int *method;
{
/* (integral) d-arclength / velocity            */
/* equals                                       */
/* (integral) R d(psi) / [ v0 + g*z ]           */
/* but                                          */
/* v0 is zero, and z=R*cos(phi) do equals       */
/* (integral) R d(psi) / g*R*cos(phi)           */
/* equals                                       */
/* (1/g) * (integral) d(psi) / cos(phi)         */

	double integral294a();
	*method=294;
	return(integral294a(DTOR*psi)/g);
	}

double ttintegralold(double v0,double g,double R,double psi,int *method)
// double v0,g,R,psi;
// int *method;
{
/* indefinite integral of d(psi)/(b+c*cos(psi)) */
/* see G&R p148 2.553.3 for this integral */

	double b, c, bb, cc, s, x, y1, y2, t, z, psia;

	/* actually, psi had better be between -90 and 90 */
	if( psi<=(-90.0) || psi>=(90.0) ) {
		fprintf(stderr,"warning: psi %f out of range\n", psi );
		}

	/* the integrand is symmetric in psi, so always do integral */
	/* between 0 and +psi, reversing sign of result if psi<0    */

	b=v0;
	c=g*R;
	bb=b*b;
	cc=c*c;

	psia=fabs(psi);

	if( fabs(b/c) < 1.0e-6 ) { /* b==0.0 case */
		*method=0;
		z=log(fabs(tan((PI/4.0)+(DTOR*psia/2.0))))/g;
		}
	else if( bb>cc ) {
		*method=1;
		t = tan(DTOR*psia/2.0);
		s=sqrt(bb-cc);
		x=2.0/s;
		y1=s*t/(b+c);
		z = R*x*atan(y1);
		}
	else {
		*method=2;
		t = tan(DTOR*psia/2.0);
		s=sqrt(cc-bb);
		x=1.0/s;
		y1=s*t + (b+c);
		y2=s*t - (b+c);
		z=R*x*log(fabs(y1/y2));
		}
	if( psi < 0.0 ) z*=(-1.0);
	return(z);
	}





/* Frechet derivative of traveltime with respect to node velocity */
double dTdmpnew(int mp,struct arc_rec *arc,int n_arcs,double traveltime )
// int mp;
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
{
	int i, nn, method;
	double dv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v += m->weight[i] * dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(phi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(phi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(phi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v -= m->weight[i] * dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dv;

	if( finite(D) ) return(D); else return(0.0);
	
	}

/* Frechet derivative of S traveltime with respect to node P velocity */
double dTsdmpp(int mp,struct arc_rec *arc,int n_arcs,double traveltime )
// int mp;
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
{
	int i, nn, method;
	double dvP, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	struct node_rec *mynode;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dvP=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = (mynode->v_orig + m->weight[i] * dvP)/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(pgi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(pgi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(pgi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dvP;

	if( finite(D) ) return(D); else return(0.0);
	
	}
 
 
 
 
 
/* Frechet derivative of S traveltime with respect to node P:S ratio */
double dTsdmpr(int mp,struct arc_rec *arc,int n_arcs,double traveltime )
// int mp;
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
{
	int i, nn, method;
	double dr, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	struct node_rec *mynode;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dr=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/(mynode->ratio_orig+m->weight[i]*dr);
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(pgi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(pgi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(pgi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dr;

	if( finite(D) ) return(D); else return(0.0);
	
	}

/* Frechet derivative of reflected traveltime with respect to reflector position */
/* (Tricky because the point of reflection moves as the reflector is perturbed) */
double dTdmpr(int mp,int interface,int rinterface,double xia[3],double psia[3],
	double xib[3],double npsib[3],struct arc_rec *arc,int n_arcs,double traveltime,int ng)
// int mp, interface, rinterface;
// double xia[3], psia[3];
// double xib[3], npsib[3];
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
// int ng;
{
	int i, j, nn, method, m_arcs;
	double ds, dTdm;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];
	struct node_rec *mynode;
	int sa, sb, hfa, hfb, eca, ecb;
	double xa[3], ta[3], TTa, Qa, tha, aza, TTA;
	double xb[3], tb[3], TTb, Qb, thb, azb, TTB;
	double psib[3], n[3];
	double psiath, psiaaz, psibth, psibaz;
	double dth, daz;
	double xa0[3], xa1[3], xa2[3], TTa0, TTa1, TTa2, psia0[3], psia1[3], psia2[3];
	double xb0[3], xb1[3], xb2[3], TTb0, TTb1, TTb2, psib0[3], psib1[3], psib2[3];
	double ta0[3], ta1[3], ta2[3];
	double tb0[3], tb1[3], tb2[3];
	double epsilon, xrp[3], v1[3], v2[3], v3, P[4], Pr[4], v[3], h[3], s[3][3], st[3][3];
	int face, debug;
	struct tetra_rec *ptetra;

	double tar[3], ta0r[3], ta1r[3], ta2r[3];
	double tbr[3], tb0r[3], tb1r[3], tb2r[3];
	double xar[3], xa0r[3], xa1r[3], xa2r[3];
	double xbr[3], xb0r[3], xb1r[3], xb2r[3];

	double dtadpsi[2][2], dtbdpsi[2][2], dxadpsi[2][2], dxbdpsi[2][2];
	double dpsidxa[2][2], dpsidxb[2][2], dtadxa[2][2], dtbdxb[2][2];
	double b[2], A[2][2], xsoln[2], xbp[3], xbpr[3];
	double *xf1, *xf2, *xf3;
	double dTTadpsi[2], dTTbdpsi[2], dTTadxa[2], dTTbdxb[2];
	double xf1r[3], xf2r[3], xf3r[3];
	double tarest[2], tbrest[2];
	double TTa0est, TTb0est;

	debug=0;

	ds=1.0e-2;           /* perturbation in node position */
	dth=1.0e-4;          /* perturbation in angle of incidence of a ray */
	daz=1.0e-4;          /* perturbation in azimuth of a ray */
	m = &model_par[mp];  /* pointer to this model parameter */
	nn = m->n;           /* number of nodes in this model parameter */

	if(debug) fprintf(stderr,"dTdm: (row col) = (%d %d) with %d nodes\n\n", ng, mp, nn );
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\tstart a (%f %f %f) (%f %f %f)\n",
		xia[0], xia[1], xia[2], psia[0], psia[1], psia[2] );
	if(debug) fprintf(stderr,"\tend   b (%f %f %f) (%f %f %f)\n",
		xib[0], xib[1], xib[2], npsib[0], npsib[1], npsib[2] );
	if(debug) fprintf(stderr,"\n");

	/* Theory:  The goal, to calculate the derivative dT/dm, is reached */
	/* by first calculating how the perturbation dm moves the location  */
	/* of the point of reflection, and then how that motion effects the */
	/* traveltime.  Rays are shot from source and receiver down to the  */
	/* reflector, with perturbations made (alternately) to both the     */
	/* model and to the takeoff angles.  This allows derivatives like   */
	/* dT/dtheta and dx/dtheta to be calculated via finite differences. */
	/* The chain rule is then used to calculate derivatives like dT/dX. */

	/* Step 1a. Shoot rays to unperturbed reflector                     */
	/* source at xia with tangent psia                                  */
        /* has reflection point xa, tangent ta, and traveltime TTA          */
	/* source at xib with tangent psib                                  */
        /* has reflection point xb, tangent tb, and traveltime TTb          */

	/* source to reflector */
	inv_ray_tangentr( &psiath, &psiaaz, psia);
	sa=shoot_to_interface( xia, psia, xa, ta, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTA, &Qa, &eca );
	if( !(sa&&hfa) ) return(0.0);

	if(debug) fprintf(stderr,"\txa (%f %f %f) ta (%f %f %f) TTA %f\n",
		xa[0],xa[1],xa[2], ta[0],ta[1],ta[2], TTA);

	/* Snell's law is equivalent to the condition that the in-plane     */
	/* components two tangents be equal.  Thus we need to identify the  */
	/* normal, n, to the plane.                                         */
	epsilon=1.0e-6;
	for( i=0; i<3; i++ ) xrp[i]=xa[i]-epsilon*ta[i];
	if( !find_tetra( xrp, &i ) ) return(0.0);
	ptetra=tetra+i;
	if( ! ray_exits_tetrahedron( ptetra->x1, ptetra->x2, ptetra->x3, ptetra->x4,
		xrp, ta, epsilon, v1, v2, &v3, &face ) ) return(0.0);
	face+=1;
	if( face==1 )      {xf1=ptetra->x2; xf2=ptetra->x3; xf3=ptetra->x4;}
	else if( face==2 ) {xf1=ptetra->x1; xf2=ptetra->x3; xf3=ptetra->x4;}
	else if( face==3 ) {xf1=ptetra->x1; xf2=ptetra->x2; xf3=ptetra->x4;}
	else if( face==4 ) {xf1=ptetra->x1; xf2=ptetra->x2; xf3=ptetra->x3;}
	else { return(0.0); }
	eqn_of_plane( xf1, xf2, xf3, P );
	normal_to_plane( P, n );

	/* receiver to reflector */
	for(i=0;i<3;i++) psib[i]=(-npsib[i]); /* reverse the tangent, since shooting backwards */
	inv_ray_tangentr( &psibth, &psibaz, psib);
	sb=shoot_to_interface( xib, psib, xb, tb, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTB, &Qb, &ecb );
	if( !(sb&&hfb) ) return(0.0);

	if(debug) fprintf(stderr,"\txb (%f %f %f) tb (%f %f %f) TTB %f\n",
		xb[0],xb[1],xb[2], tb[0],tb[1],tb[2], TTB);
	if(debug) fprintf(stderr,"\n");

	if(debug) {
		fprintf(stderr,"\tunperturbed nodes at\n");
		for( i=0; i<nn; i++ ) {
			mynode= &node[m->node[i]];
			fprintf(stderr,"\t\t%d %f %f %f\n",i,mynode->x[0],mynode->x[1],mynode->x[2]);
			}
		}

	/* Step 2a. Perturb the reflector by moving all the model-parameter's    */
	/* nodes by a small distance ds parallel to the direction specified      */
	/* by the model parameter.  The velocity eqn of all effected tetrahedra  */
	/* must be recalculated, since the spatial velocity gradiant will cahnge */

	perturb_refl_nodes(m,ds);

	if(debug) {
		fprintf(stderr,"\tperturbed nodes at\n");
		for( i=0; i<nn; i++ ) {
			mynode= &node[m->node[i]];
			fprintf(stderr,"\t\t%d %f %f %f\n",i,mynode->x[0],mynode->x[1],mynode->x[2]);
			}
		fprintf(stderr,"\n");
		}

	/* Step 2b. Shoot ray pencil from source thru the perturbed medium          */
	/* the pencil has 3 rays ray0=(th,az), ray1=(th+dth,az), ray2=(th,az+daz)   */
	/* at the reflection point, the positions are xa0, xa1, xa2 and the         */
	/* tangents are ta0, ta2, ta3 and                                           */
	/* traveltimes are TTa0, TTa1, TTa2                                         */

	ray_tangentr( psiath, psiaaz, psia0 );
	sa=shoot_to_interface( xia, psia0, xa0, ta0, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa0, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psiath+dth, psiaaz, psia1 );
	sa=shoot_to_interface( xia, psia1, xa1, ta1, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa1, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); };

	ray_tangentr( psiath, psiaaz+daz, psia2 );
	sa=shoot_to_interface( xia, psia2, xa2, ta2, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa2, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	if(debug) fprintf(stderr,"\txa0 (%f %f %f) ta0 (%f %f %f) TTa0 %f\n",
		xa0[0],xa0[1],xa0[2],ta0[0],ta0[1],ta0[2],TTa0);
	if(debug) fprintf(stderr,"\txa1 (%f %f %f) ta1 (%f %f %f) TTa1 %f\n",
		xa1[0],xa1[1],xa1[2],ta1[0],ta1[1],ta1[2],TTa1);
	if(debug) fprintf(stderr,"\txa2 (%f %f %f) ta2 (%f %f %f) TTa2 %f\n",
		xa2[0],xa2[1],xa2[2],ta2[0],ta2[1],ta2[2],TTa2);
	if(debug) fprintf(stderr,"\n");

	/* Step 2b. Shoot ray pencil from receiver thru perturbed medium            */
	/* the pencil has 3 rays ray0=(th,az), ray1=(th+dth,az), ray2=(th,az+daz)   */
	/* at the reflection point, the positions are xb0, xb1, xb2 and the         */
	/* tangents are tb0, tb2, tb3                                               */
	/* traveltimes are TTb0, TTb1, TTb2                                         */

	ray_tangentr( psibth, psibaz, psib0 );
	sb=shoot_to_interface( xib, psib0, xb0, tb0, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb0, &Qb, &ecb );
	if( !(sb&&hfb) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psibth+dth, psibaz, psib1 );
	sb=shoot_to_interface( xib, psib1, xb1, tb1, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb1, &Qb, &ecb );
	if( !(sb&&hfb) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psibth, psibaz+daz, psib2 );
	sb=shoot_to_interface( xib, psib2, xb2, tb2, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb2, &Qb, &ecb );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	if(debug) fprintf(stderr,"\txb0 (%f %f %f) tb0 (%f %f %f) TTb0 %f\n",
		xb0[0],xb0[1],xb0[2],tb0[0],tb0[1],tb0[2],TTb0);
	if(debug) fprintf(stderr,"\txb1 (%f %f %f) tb1 (%f %f %f) TTb1 %f\n",
		xb1[0],xb1[1],xb1[2],tb1[0],tb1[1],tb1[2],TTb1);
	if(debug) fprintf(stderr,"\txb2 (%f %f %f) tb2 (%f %f %f) TTb2 %f\n",
		xb2[0],xb2[1],xb2[2],tb2[0],tb2[1],tb2[2],TTb2);
	if(debug) fprintf(stderr,"\n");

	/* Step 2c. Rotate positions and tangents to coordinate system of interface */

	tvh( n, v, h );
	newbasisrot( v, h, n, s );
	for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) st[i][j]=s[j][i];
	rotate3(xa,s,xar); rotate3(xa0,s,xa0r); rotate3(xa1,s,xa1r); rotate3(xa2,s,xa2r);
	rotate3(xb,s,xbr); rotate3(xb0,s,xb0r); rotate3(xb1,s,xb1r); rotate3(xb2,s,xb2r);
	rotate3(ta,s,tar); rotate3(ta0,s,ta0r); rotate3(ta1,s,ta1r); rotate3(ta2,s,ta2r);
	rotate3(tb,s,tbr); rotate3(tb0,s,tb0r); rotate3(tb1,s,tb1r); rotate3(tb2,s,tb2r);
	rotate3(xf1,s,xf1r); rotate3(xf2,s,xf2r); rotate3(xf3,s,xf3r);
	eqn_of_plane( xf1r, xf2r, xf3r, Pr );

	if(debug) fprintf(stderr,"\txar (%f %f %f) tar (%f %f %f)\n",
		xar[0],xar[1],xar[2],tar[0],tar[1],tar[2]);
	if(debug) fprintf(stderr,"\txbr (%f %f %f) tbr (%f %f %f)\n",
		xbr[0],xbr[1],xbr[2],tbr[0],tbr[1],tbr[2]);
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\txa0r (%f %f %f) ta0r (%f %f %f)\n",
		xa0r[0],xa0r[1],xa0r[2],ta0r[0],ta0r[1],ta0r[2]);
	if(debug) fprintf(stderr,"\txa1r (%f %f %f) ta1r (%f %f %f)\n",
		xa1r[0],xa1r[1],xa1r[2],ta1r[0],ta1r[1],ta1r[2]);
	if(debug) fprintf(stderr,"\txa2r (%f %f %f) ta2r (%f %f %f)\n",
		xa2r[0],xa2r[1],xa2r[2],ta2r[0],ta2r[1],ta2r[2]);
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\txb0r (%f %f %f) tb0r (%f %f %f)\n",
		xb0r[0],xb0r[1],xb0r[2],tb0r[0],tb0r[1],tb0r[2]);
	if(debug) fprintf(stderr,"\txb1r (%f %f %f) tb1r (%f %f %f)\n",
		xb1r[0],xb1r[1],xb1r[2],tb1r[0],tb1r[1],tb1r[2]);
	if(debug) fprintf(stderr,"\txb2r (%f %f %f) tb2r (%f %f %f)\n",
		xb2r[0],xb2r[1],xb2r[2],tb2r[0],tb2r[1],tb2r[2]);
	if(debug) fprintf(stderr,"\n");

	/* Step 2d. Compute finite-difference estimatesd of partial derivatives -   */
	/*      reflector position with respect to change in takefoff angles        */
	/*      tangent with respect to change in takefoff angles                   */
	/*      traveltime with respect to change in takefoff angles                */
	/* for both the source-side "a" ray and the receiver-side "b" ray           */

	dxadpsi[0][0] = (xa1r[0]-xa0r[0])/dth; dxadpsi[0][1] = (xa2r[0]-xa0r[0])/daz;
	dxadpsi[1][0] = (xa1r[1]-xa0r[1])/dth; dxadpsi[1][1] = (xa2r[1]-xa0r[1])/daz;

	dxbdpsi[0][0] = (xb1r[0]-xb0r[0])/dth; dxbdpsi[0][1] = (xb2r[0]-xb0r[0])/daz;
	dxbdpsi[1][0] = (xb1r[1]-xb0r[1])/dth; dxbdpsi[1][1] = (xb2r[1]-xb0r[1])/daz;
	
	dtadpsi[0][0] = (ta1r[0]-ta0r[0])/dth; dtadpsi[0][1] = (ta2r[0]-ta0r[0])/daz;
	dtadpsi[1][0] = (ta1r[1]-ta0r[1])/dth; dtadpsi[1][1] = (ta2r[1]-ta0r[1])/daz;

	dtbdpsi[0][0] = (tb1r[0]-tb0r[0])/dth; dtbdpsi[0][1] = (tb2r[0]-tb0r[0])/daz;
	dtbdpsi[1][0] = (tb1r[1]-tb0r[1])/dth; dtbdpsi[1][1] = (tb2r[1]-tb0r[1])/daz;

	dTTadpsi[0] = (TTa1-TTa0)/dth; dTTadpsi[1] = (TTa2-TTa0)/daz;
	dTTbdpsi[0] = (TTb1-TTb0)/dth; dTTbdpsi[1] = (TTb2-TTb0)/daz;

	if(debug) fprintf(stderr,"\tdxa/dpsi %f %f %f %f\n",
		dxadpsi[0][0], dxadpsi[0][1], dxadpsi[1][0], dxadpsi[1][1] );
	if(debug) fprintf(stderr,"\tdxb/dpsi %f %f %f %f\n",
		dxbdpsi[0][0], dxbdpsi[0][1], dxbdpsi[1][0], dxbdpsi[1][1] );
	if(debug) fprintf(stderr,"\n");

	if(debug) fprintf(stderr,"\tdta/dpsi %f %f %f %f\n",
		dtadpsi[0][0], dtadpsi[0][1], dtadpsi[1][0], dtadpsi[1][1] );
	if(debug) fprintf(stderr,"\tdtb/dpsi %f %f %f %f\n",
		dtbdpsi[0][0], dtbdpsi[0][1], dtbdpsi[1][0], dtbdpsi[1][1] );
	if(debug) fprintf(stderr,"\n");

	if(debug) fprintf(stderr,"\tdTTa/dpsi %f %f\n",
		dTTadpsi[0], dTTadpsi[1] );
	if(debug) fprintf(stderr,"\tdTTb/dpsi %f %f\n",
		dTTbdpsi[0], dTTbdpsi[1] );
	if(debug) fprintf(stderr,"\n");

	/* while we have calculated dx/dpsi and dt/dpsi, we really need dt/dx */
	/* in order to impose Snell's law.  So we must apply the chain rule   */

	/* first rely on the fact that a matrix of partials of form dY/dX is the */
	/* matrix inverse of the partials dX/dY                                  */ 
	if( ! invert2x2( dxadpsi, dpsidxa ) ) { perturb_refl_nodes(m,-ds); return(0.0); }
	if( ! invert2x2( dxbdpsi, dpsidxb ) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	/* then use the chain rule dX/dY = dX/dZ dZ/dX                           */
	mult2x2( dtadpsi, dpsidxa, dtadxa );
	mult2x2( dtbdpsi, dpsidxb, dtbdxb );

	if(debug) fprintf(stderr,"\tdta/dxa %f %f %f %f\n",
		dtadxa[0][0], dtadxa[0][1], dtadxa[1][0], dtadxa[1][1] );
	if(debug) fprintf(stderr,"\tdtb/dxb %f %f %f %f\n",
		dtbdxb[0][0], dtbdxb[0][1], dtbdxb[1][0], dtbdxb[1][1] );
	if(debug) fprintf(stderr,"\n");

	/* Step 3: set up linear system for perturbation in point of reflection that */
	/* matches (to first order) the in-plane components of the two tangents      */
	/* Here we are using Taylor's theorem to write to first order                */
	/* ta(x) = ta(xa) + dta/dxa (x-xa) and                                       */
	/* tb(x) = tb(xa) + dtb/dxb (x-xb)                                           */
	/* if we view these equations in a coorinate system that is normal to the    */
	/* reflector, then Snell's law is the condition that the two in-plane        */
	/* components of the tangents are related by ta(x)=-tb(x) (the minus         */
        /* sign arises because we are propagating from point "b" backwards). This    */
	/* yields a 2x2 matrix equation for x, the position of the                   */
	/* point that satisfies Snell's law                                          */
	/* -[ta(xa)+tb(xb)] + [dta/dxa xa - dtb/dxb xb] =  [dta/dxa + dtb/dxb] x     */

	for( i=0; i<2; i++ ) {
		b[i] = -(ta0r[i]+tb0r[i]);
		for( j=0; j<2; j++ ) {
			b[i] += ( dtbdxb[i][j]*xb0r[j] + dtadxa[i][j]*xa0r[j] );
			}
		}
	for( i=0; i<2; i++ ) for( j=0; j<2; j++ ) {
		A[i][j] = dtbdxb[i][j] + dtadxa[i][j];
		}
	if( !solve2x2(A, b, xsoln ) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	/* The estimated tangent is never used */
	if(debug) {
		for( i=0; i<2; i++ ) {
			tarest[i] = ta0r[i];
			tbrest[i] = tb0r[i];
			for( j=0; j<2; j++ ) {
				tarest[i] += dtadxa[i][j]*(xsoln[j]-xa0r[j]);
				tbrest[i] += dtbdxb[i][j]*(xsoln[j]-xb0r[j]);
				}
			}
		}

	if(debug) fprintf(stderr,"\ttarest %f %f\n", tarest[0], tarest[1] );
	if(debug) fprintf(stderr,"\ttbrest %f %f\n", tbrest[0], tbrest[1] );
	if(debug) fprintf(stderr,"\n");

	/* the matrix equation gives only two components of the position of the     */
	/* point of reflection.  I use the equation of the plane (suitably rotated) */
	/* to find the third                                                        */
	xbpr[0]=xsoln[0];
	xbpr[1]=xsoln[1];
	xbpr[2]= -(Pr[0]*xsoln[0] + Pr[1]*xsoln[1] + Pr[3] )/Pr[2];

	/* The unrotated point of reflection is never used */
	if(debug) rotate3(xbpr,st,xbp);

	if(debug) fprintf(stderr,"\tbounce point at %f %f %f \n", xbp[0], xbp[1], xbp[2] );
	if(debug) fprintf(stderr,"\n");

	/* Step 4: Estimate the derivative of traveltime with respect to reflector */
	/* position                                                                */

	/* Use the chain rule: dTdx = dT/dphi dphi/dx. Note dphi/dx appears as a   */
	/* matrix transpose when this formula is case in a linear algebra          */
	for( i=0;i<2;i++ ) {
		dTTadxa[i]=0.0;
		dTTbdxb[i]=0.0;
		for( j=0; j<2; j++ ) {
			dTTadxa[i] += dTTadpsi[j]*dpsidxa[j][i];
			dTTbdxb[i] += dTTbdpsi[j]*dpsidxb[j][i];
			}
		}

	if(debug) fprintf(stderr,"\tdTTa/dxa %f %f\n", dTTadxa[0], dTTadxa[1] );
	if(debug) fprintf(stderr,"\tdTTb/dxb %f %f\n", dTTbdxb[0], dTTbdxb[1] );
	if(debug) fprintf(stderr,"\n");

	/* Setp 5: Derivative calulation. The derivative is the sum of 2 parts */

	/* estimate the traveltime of the perturbed model where snell's law    */
	/* is satisfied                                                        */

	TTa0est=TTa0;
	TTb0est=TTb0;
	for( j=0; j<2; j++ ) {
		TTa0est += dTTadxa[j]*(xsoln[j]-xa0r[j]);
		TTb0est += dTTbdxb[j]*(xsoln[j]-xb0r[j]);
		}

	if(debug) {
		fprintf(stderr,"\tUnperturbed TT %f + %f = %f\n", TTA,  TTB, TTA+TTB );
		fprintf(stderr,"\tPert uncorr TT %f + %f = %f\n", TTa0, TTb0, TTa0+TTb0 );
		fprintf(stderr,"\tPert & corr TT %f + %f = %f\n", TTa0est, TTb0est, TTa0est+TTb0est );
		fprintf(stderr,"\n");
		}

	/* part associated with change in model, but Snell's law is not satisfied */
	dTdm = ( (TTa0est+TTb0est) - (TTA+TTB) ) / ds;

	if(debug) fprintf(stderr,"\tDerivative %f\n\n", dTdm );

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.             */ 
	perturb_refl_nodes(m,-ds);

	if( finite(dTdm) ) return(dTdm); else return(0.0);
	
	}





/* Frechet derivative of direct traveltime with respect to reflector position */
double dTdmprd(int mp,int interface,int rinterface,double xia[3],double psia[3],
	double xib[3],double npsib[3],struct arc_rec *arc,int n_arcs,double traveltime,int ng)
// int mp, interface, rinterface;
// double xia[3], psia[3];
// double xib[3], npsib[3];
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
// int ng;
{
	int i, status, m_arcs, debug, t;
	double dTdm, ds;
	struct model_par_rec *m;
	double xc[3], tc[3];
	int hf, ec;
	double TT, Q, dT, dx[3], xcp[3], v, epsilon;

	m = &model_par[mp];  /* pointer to this model parameter */
	ds=1.0e-2;           /* perturbation in node position */
	epsilon = 1.0e-8;    /* point moved back along ray by this much */
	debug=0;

	/* Theory: Do this one by brute force: perturb the model and */
	/* shoot a ray. This ray will not in general pass thru the   */
	/* receiver point, so apply a first order moveout to correct */
	/* its traveltime to the receiver point.  Then compute first */
	/* order finite difference with unperturbed traveltime.      */

	perturb_refl_nodes(m,ds);
	status=shoot_to_interface( xia, psia, xc, tc, NULL, &m_arcs,
		MAX_ARCS, 10000.0, interface, &hf, &TT, &Q, &ec );
	if( !(status&&hf) ) {
		if(debug) fprintf(stderr,"warning: shoot_to_interface failed in dTdmprd\n");
		perturb_refl_nodes(m,-ds);
		return(0.0);
		}

	for( i=0; i<3; i++ ) xcp[i]=xc[i]-epsilon*tc[i];
	if( find_tetra(xcp,&t) ) v = eval_velocity(&tetra[t],xcp);
	else {
		if(debug) fprintf(stderr,"warning: find_tetra failed in dTdmprd\n");
		perturb_refl_nodes(m,-ds);
		return(0.0);
		}

	for( i=0; i<3; i++ ) dx[i]=xib[i]-xc[i];
	dT=dot( dx, tc ) / v;
	dTdm = ( (TT + dT) - traveltime )/ds;

	if( debug ) {
		fprintf(stderr,"dTdmprd: %d %d\n", ng, mp );
		fprintf(stderr,"         x2  %f %f %f\n", xib[0], xib[1], xib[2] );
		fprintf(stderr,"         x2p %f %f %f\n", xc[0], xc[1], xc[2] );
		fprintf(stderr,"         TT=%f dt=%f traveltime=%f v=%f dTdm=%f\n",
			TT, dT, traveltime, v, dTdm );
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.             */ 
	perturb_refl_nodes(m,-ds);

	if( finite(dTdm) ) return(dTdm); else return(0.0);

	}




double dTdmpold(int mp,struct arc_rec *arc,int n_arcs,double traveltime)
// int mp;
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
{
	int i, nn, method;
	double dv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0, qint1, qint2;
	double rgrad[3], gs, dpsi;

	dv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v+=m->weight[i]*dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the find the amplitude of the gradient in the (x',z') plane*/
		/* and its angle from the z' direction */
		gs=sqrt(rgrad[0]*rgrad[0]+rgrad[2]*rgrad[2]);
		dpsi = RTOD*atan2(rgrad[0],rgrad[2]);
		
		R=arc[i].R;
		/* view the traveltime intergal in a cordinate system */
		/* rotated about y' so that z'' is now parallel to the */
		/* new velocity gradient.  This changes the integration limits */
		psi1=arc[i].psi1-dpsi;
		psi2=arc[i].psi2-dpsi;
		v0=eval_velocity(t,arc[i].xc);
		T2 = ttintegral(v0,gs,R,psi2,&method);
		T1 = ttintegral(v0,gs,R,psi1,&method);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v-=m->weight[i]*dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dv;

	if( finite(D) ) return(D); else return(0.0);
	
	}





double dqinvdmp(int mp,struct arc_rec *arc,int n_arcs,double traveltime,double qinvint )
// int mp;
// struct arc_rec *arc;
// int n_arcs;
// double traveltime;
// double qinvint;
{
	int i, j, nn, method;
	double dqinv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, qinvint0, qinvint1, qinvint2, qinv0;
	double grad[3], rgrad[3];

	dqinv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the qinv equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].qinv += m->weight[i] * dqinv;
	for( i=0; i<n_arcs; i++ ) qinv_eqn( arc[i].ptetra );

	/* calculate the qinv, summing up the contributions for all arcs */
	qinvint0=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;
		for( j=0; j<3; j++ ) grad[j]=t->qinvc[j];
		rotate3(grad,arc[i].s,rgrad);
		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		qinv0=eval_qinv(t,arc[i].xc);
		qinvint2=qintegral( qinv0, rgrad[0], rgrad[2], R, t->gs, psi2 );
		qinvint1=qintegral( qinv0, rgrad[0], rgrad[2], R, t->gs, psi1 );
		qinvint0+=(qinvint2-qinvint1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the qinv equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].qinv -= m->weight[i] * dqinv;
	for( i=0; i<n_arcs; i++ ) qinv_eqn( arc[i].ptetra );

	D = (qinvint0-qinvint)/(traveltime*dqinv);

	if( finite(D) ) return(D); else return(0.0);
	
	}



double integral292( double a, double x )
{
/* (integral ) tan(ax) dx /                          */
/* see CRC Std Math Tables Integral 292              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */
	double ax, value;
	ax=a*x;
	value = log(fabs(cos(ax)))/(-a);
	return(value);
	}

double integral292a( double x )
{
/* (integral ) tan(x) dx /                           */
/* see CRC Std Math Tables Integral 292              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */
	double value;
	value = (-log(fabs(cos(x))));
	return(value);
	}

double inv_integral294a( double x )
{
/* inverts integral 294a                             */
/* Warning! Only checked on interval -PI<x<PI        */

	double PI=3.141592653589793;
	double value;
	value = 2.0*atan(exp(x))-PI/2.0;
	return(value);
	}


double integral294( double a, double x )
{
/* (integral ) dx / cos(ax)                          */
/* see CRC Std Math Tables Integral 294              */
/* integrand is symmetric in x, so always do |x|     */
/* reversing sign for x<0                            */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan(PI/4.0+fabs(a*x/2.0))))/a;
	if( x>=0.0 ) return(value); else return(-value);
	}

double integral294a( double x )
{
/* (integral ) dx / cos(x)                           */
/* see CRC Std Math Tables Integral 294              */
/* integrand is symmetric in x, so always do |x|     */
/* reversing sign for x<0                            */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan((PI/4.0)+fabs(x/2.0))));
	if( x>=0.0) return(value); else return(-value);
	}

double integral295( double a, double x )
{
/* (integral ) dx / sin(ax)                          */
/* see CRC Std Math Tables Integral 294              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */

	double ax, value;
	double PI=3.141592653589793;
	ax = a*x;
	value = log(fabs(tan(ax/2.0)))/a;
	return(value);
	}

double integral295a( double a, double x )
{
/* (integral ) dx / sin(x)                           */
/* see CRC Std Math Tables Integral 294              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan(fabs(x/2.0))));
	return(value);
	}

double integral340( double a, double b, double x )
/* (integral ) dx / (a + b*sin(x) )                   */
/* see CRC Std Math Tables Integral 340               */
/* integrand is antisymmetric in x, so always do |x|  */
/* using same value for x<0                           */
/* I checked this against numerical integration       */
/* for 4*10**4 points on a grid in a=[0.0,2.03],      */
/* b=[0.1,1.01], x1=[-6.27,6.27], x2=[-6.27,6.27]     */
/* and it seemed to work. William Menke, 07-14-2000.  */
{
	double PI=3.141592653589793;
	double tanbranch();
	double a2, b2, root, tanx2, t1, t2, value, x2;

	a2=a*a; b2=b*b; x2=fabs(x/2.0);

	if( a2>b2 ) {
		root = sqrt(a2-b2);
		tanx2 = tan(x2);
		t1 = a*tanx2+b;
		t2 = atan( t1/root ) + PI*tanbranch(x2);
		value = 2.0*t2/root;
		}
	else {
		root = sqrt(b2-a2);
		tanx2 = tan(x2);
		t1 = a*tanx2 + b - root;
		t2 = a*tanx2 + b + root;
		value = log(fabs(t1/t2))/root;
		}
	return(value);
	}

double integral341( double a, double b, double x )
/* (integral ) dx / (a + b*cos(x) )              */
/* see CRC Std Math Tables Integral 341          */
/* integrand is symmetric in x, so always do |x| */
/* reversing sign of result for x<0              */
{
	double PI=3.141592653589793;
	double tanbranch();
	double a2, b2, root, tanx2, t1, t2, value, x2;

	a2=a*a; b2=b*b; x2=fabs(x/2.0);

	if( a2>b2 ) {
		root = sqrt(a2-b2);
		tanx2 = tan(x2);
		t1 = root*tanx2;
		t2 = atan( t1/(a+b) ) + PI*tanbranch(x2);
		value = 2.0*t2/root;
		}
	else {
		root = sqrt(b2-a2);
		tanx2 = tan(x2);
		t1 = root*tanx2 + a + b;
		t2 = root*tanx2 - a - b;
		value = log(fabs(t1/t2))/root;
		}
	if( x>=0.0) return(value); else return(-value);
	}


double integral342( double a, double b, double c, double x )
/* (integral ) dx / (a + b*sin(x) + c*cos(x) )   */
/* see CRC Std Math Tables Integral 342          */
/* I checked this against numerical integration  */
/* for 4*10**5 points on a grid in a=[0.0,2.03], */
/* b=[0.1,1.01], c=[0.1,1.07], x1=[-6.27,6.27],  */
/* x2=[-6.27,6.27] and it seemed to work.        */
/* William Menke, 07-14-2000.                    */
{
	double PI=3.141592653589793;
	double a2, b2, c2, b2pc2;
	double root, tanx2, x2, t1, t2, t3, cx, sx;
	double value;
	double tanbranch();

	a2=a*a; b2=b*b; c2=c*c; b2pc2=b2+c2; x2=x/2.0;

	if( a2 < b2pc2 ) {
		root = sqrt(b2pc2-a2);
		tanx2 = tan(x2);
		t1 = b - root + (a-c)*tanx2;
		t2 = b + root + (a-c)*tanx2;
		value = log(fabs(t1/t2)) / root;
		}
	else if ( a2 > b2pc2 ) {
		root=sqrt(a2-b2pc2);
		tanx2 = tan(x2);
		t1 = b + (a-c)*tanx2;
		t2 = atan( t1 / root ) + PI*tanbranch(x2);
		value = 2.0 * t2 / root;
		}
	else {
		cx=cos(x); sx=sin(x);
		t1 = a - (b+c)*cx - (b-c)*sx;
		t2 = a - (b-c)*cx + (b+c)*sx;
		value = t1/(t2*a);
		}

	return(value);
	}



double qintegral(double qinv0,double qinvx,double qinvz,double R,double vz,double phi )
//double qinv0, qinvx, qinvz, R, vz, phi; // old syntax
{
/* (integral) qinv dT =                                                                */
/* (integral) qinv (1/v) R  d(phi) =                                                   */
/* (integral) R [qinv0 + qinvx*x + qinvz*z] / R*vz*cos(phi) d(phi) =                   */
/* (integral) R [qinv0 + qinvx*R*sin(phi) + qinvz*R*cos(phi)] / R*vz*cos(phi) d(phi) = */
/* (qinv0/vz) (integral) 1/cos(phi) d(phi) +                                           */
/* (R*qinvx/vz) (integral) tan(phi) d(phi) +                                           */
/* (R*qinvz/vz) (integral) d(phi)                                                      */

	double value;

	value = (qinv0/vz)   * integral294a(DTOR*phi)
	      + (R*qinvx/vz) * integral292a(DTOR*phi)
	      + (R*qinvz/vz) * DTOR*phi;

	return(value);
	}
