
/* v(x,y,z) = vc[0]*x + vc[1]*y + vc[2]*z + vc[3] */
int velocity_eqn(struct tetra_rec *t )
// struct tetra_rec *t;
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=t->x1[i];
		G[1][i]=t->x2[i];
		G[2][i]=t->x3[i];
		G[3][i]=t->x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(t->v[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		t->vc[i]=d[i];
		}

	t->gs=sqrt(dot(d,d));

	if( t->gs != 0.0 ) {
		for( i=0; i<3; i++ ) {
			t->g[i]=d[i];
			t->gn[i]=d[i]/t->gs;
			}
		}
	else {
		for( i=0; i<3; i++ ) {
			t->g[i]=d[i];
			t->gn[i]=0.0;
			}
		t->gn[2]=1.0;
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

/* qinv(x,y,z) = qinvc[0]*x + qinvc[1]*y + qinvc[2]*z + qinvc[3] */
int qinv_eqn( struct tetra_rec *t )
// struct tetra_rec *t;
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=t->x1[i];
		G[1][i]=t->x2[i];
		G[2][i]=t->x3[i];
		G[3][i]=t->x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(t->qinv[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		t->qinvc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

int velocity_eqn2(double x1[3],double x2[3],double x3[3],double x4[3],double *v[4],double vc[4])
// double x1[3], x2[3], x3[3], x4[3], *v[4], vc[4];
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=x1[i];
		G[1][i]=x2[i];
		G[2][i]=x3[i];
		G[3][i]=x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(v[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		vc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}


int qinv_eqn2(double x1[3],double x2[3],double x3[3],double x4[3],double *qinv[4],double qinvc[4])
// double x1[3], x2[3], x3[3], x4[3], *qinv[4], qinvc[4];
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=x1[i];
		G[1][i]=x2[i];
		G[2][i]=x3[i];
		G[3][i]=x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(qinv[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		qinvc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}



/* Calculates P=[A,B,C,D], the coefficients of the equation
of the plane Ax + By + Cz + D = 0, given three points x1, x2,
x3 in plane. Returns 1 on success */


int eqn_of_plane(double x1[3],double x2[3],double x3[3],double P[4] )
// double x1[3], x2[3], x3[3];
// double P[4];
{
	double x2mx1[3], x3mx1[3], v[3], a;
	int i;

	/* eqn is (x-x1) (dot) ( (x2-x1) (cross) (x3-x1) )     */
	/* see Rektorys Survey of Applicable Mathematics p 240 */

	for( i=0; i<3; i++ ) {
		x2mx1[i] = x2[i]-x1[i];
		x3mx1[i] = x3[i]-x1[i];
		}

	cross( x2mx1, x3mx1, v );
	P[0] = v[0];
	P[1] = v[1];
	P[2] = v[2];
	P[3] = (-dot(x1,v));

	a=sqrt(P[0]*P[0]+P[1]*P[1]+P[2]*P[2]);
	if( a==0.0 ) a=1.0;
	for( i=0; i<4; i++ ) P[i]/=a;
	
	return(1);
	}


/* Calculates P=[A,B,C,D], the coefficients of the equation
of the plane Ax + By + Cz + D = 0, given the plane's normal, n,
and one point, x. Returns 1 on success */

int eqn_of_plane2(double n[3],double x1[3],double P[4] )
// double n[3], x1[3];
// double P[4];
{
	/* n = grad(Ax + By + Cz + D) = [A, B, C]  */
	/* then D = -(Ax1 + By1 + Cz1)
	/* see Rektorys Survey of Applicable Mathematics p 240 */

	P[0] = n[0];
	P[1] = n[1];
	P[2] = n[2];
	P[3] = (-(P[0]*x1[0]+P[1]*x1[1]+P[2]*x1[2]));
	return(1);
	}


int eqn_of_line(double x0,double y0,double x1,double y1,double L )
// double x0, y0, x1, y1, L[3];
{
	double d;

	d=x0*y1-x1*y0;
	if( d==0.0 ) return(FALSE);
	L[0] = (y0-y1)/d;
	L[1] = (x1-x0)/d;
	L[2] = 1.0;

	return(TRUE);
	}
