


int refine(double x0[3],double xsta,double ysta,int interface,double *th_est,
	double *az_est,struct arc_rec *arc,int *n_arcs,double *traveltime,struct triangle_rec *tri)
// double x0[3], xsta, ysta, *th_est, *az_est;
// int interface;
// struct arc_rec *arc;
// int *n_arcs;
// double *traveltime;
// struct triangle_rec *tri;
{
	int i, ec;
	double th, az, dth, daz, pth, paz, thold, azold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdth, dydth, dxdaz, dydaz, d;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;

	/* bad things may happen if NaN's appear */
	if( (!finite(*th_est)) || (!finite(*az_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	th = *th_est;
	az = *az_est;
	pth = 1.0e-5;
	paz = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;

	refine_star_status=0;
	refine_star_norm=0.0;

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		ray_tangent(th,az,t0);
		sa=shoot_to_interface( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th+pth,az,t0);
		sb=shoot_to_interface( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th,az+paz,t0);
		sc=shoot_to_interface( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		/* perturbation computed by Newton's method */
		dxdth = (x1b[0]-x1a[0])/pth;
		dydth = (x1b[1]-x1a[1])/pth;
		dxdaz = (x1c[0]-x1a[0])/paz;
		dydaz = (x1c[1]-x1a[1])/paz;

		d = dxdth*dydaz-dydth*dxdaz;

		dth = ( dydaz*(xsta-x1a[0]) - dxdaz*(ysta-x1a[1]) )/d;
		daz = (-dydth*(xsta-x1a[0]) + dxdth*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dth*dth+daz*daz); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dth *= f/e;
		daz *= f/e;

		thold=th;
		azold=az;

		/* bad things happen when Nan's appear */
		if( (!finite(dth)) || (!finite(daz)) ) {
			refine_star_status=1;
			return(FALSE);
			}

		th+=dth;
		az+=daz;

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		refine_star_norm=DX+DY;
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) { refine_star_status=2; return(FALSE); }

	*th_est = th;
	*az_est = az;

	return(TRUE);
	}

int refine_refl(double x0[3],double xsta,double ysta,int interface,double *th_est,
	double *az_est,struct arc_rec *arc,int *n_arcs,double *traveltime,struct triangle_rec *tri)
// double x0[3], xsta, ysta, *th_est, *az_est;
// int interface, rinterface;
// struct arc_rec *arc;
// int *n_arcs;
// double *traveltime;
// struct triangle_rec *tri;
{
	int i, ec;
	double th, az, dth, daz, pth, paz, thold, azold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdth, dydth, dxdaz, dydaz, d;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;

	/* bad things may happen if NaN's appear */
	if( (!finite(*th_est)) || (!finite(*az_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	th = *th_est;
	az = *az_est;
	pth = 1.0e-5;
	paz = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;

	refine_star_status=0;
	refine_star_norm=0.0;

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		ray_tangent(th,az,t0);
		sa=shoot_to_interface_refl( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th+pth,az,t0);
		sb=shoot_to_interface_refl( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th,az+paz,t0);
		sc=shoot_to_interface_refl( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		/* perturbation computed by Newton's method */
		dxdth = (x1b[0]-x1a[0])/pth;
		dydth = (x1b[1]-x1a[1])/pth;
		dxdaz = (x1c[0]-x1a[0])/paz;
		dydaz = (x1c[1]-x1a[1])/paz;

		d = dxdth*dydaz-dydth*dxdaz;

		dth = ( dydaz*(xsta-x1a[0]) - dxdaz*(ysta-x1a[1]) )/d;
		daz = (-dydth*(xsta-x1a[0]) + dxdth*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dth*dth+daz*daz); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dth *= f/e;
		daz *= f/e;

		thold=th;
		azold=az;

		/* bad things happen when Nan's appear */
		if( (!finite(dth)) || (!finite(daz)) ) {
			refine_star_status=1;
			return(FALSE);
			}

		th+=dth;
		az+=daz;

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		refine_star_norm=DX+DY;
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) { refine_star_status=2; return(FALSE); }

	*th_est = th;
	*az_est = az;

	return(TRUE);
	}


int refine_turf(double x0[3],double xsta,double ysta,int interface,double *th_est,
	double *az_est,struct arc_rec *arc,int *n_arcs,double *traveltime,struct triangle_rec *tri,
	double azi,double v0,double z0,double T0)
// double x0[3], xsta, ysta, *x_est, *y_est;
// int interface;
// struct arc_rec *arc;
// int *n_arcs;
// double *traveltime;
// struct triangle_rec *tri;
// double azi, v0, z0, T0;
{
	int i, ec;
	double x, y, dx, dy, px, py, xold, yold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdx, dydx, dxdy, dydy, d, theta, ppx, ppy, vp;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;
	int t, my_last_tetra;

	/* bad things may happen if NaN's appear */
	if( (!finite(*x_est)) || (!finite(*y_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	x = *x_est;
	y = *y_est;
	px = 1.0e-5;
	py = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;
	my_last_tetra=(-1);

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		x0[0]=x;
		x0[1]=y;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		my_last_tetra=last_tetra;
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		sa=shoot_to_interface( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) return(FALSE);
		if(traveltime!=NULL)
			(*traveltime) += T0 + ppx*x0[0] + ppy*x0[1];

		x0[0]=x+px;
		x0[1]=y;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;
		sb=shoot_to_interface( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) return(FALSE);

		x0[0]=x;
		x0[1]=y+py;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;
		sc=shoot_to_interface( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) return(FALSE);

		/* perturbation computed by Newton's method */
		dxdx = (x1b[0]-x1a[0])/px;
		dydx = (x1b[1]-x1a[1])/px;
		dxdy = (x1c[0]-x1a[0])/py;
		dydy = (x1c[1]-x1a[1])/py;

		d = dxdx*dydy-dydx*dxdy;

		dx = ( dydy*(xsta-x1a[0]) - dxdy*(ysta-x1a[1]) )/d;
		dy = (-dydx*(xsta-x1a[0]) + dxdx*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dx*dx+dy*dy); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dx *= f/e;
		dy *= f/e;

		xold=x;
		yold=y;
		x+=dx;
		y+=dy;

		/* bad things happen when Nan's appear */
		if( (!finite(x)) || (!finite(y)) ) {
			return(FALSE);
			}

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) return(FALSE);

	*x_est = x;
	*y_est = y;

	return(TRUE);
	}





/* Newton method refine of an already pretty good (azi, theta, T) estimate */
int rayrefine( double *x0, double *x1, double azi, double theta, double T, 
                                       double *newazi, double *newtheta, double *newT, double *newdTdx) 
{
	int i, j, status, tetra1, maxiterations, iter, ierror;
	double paz, pth, pT;
	double v, G[3][3], d[3], maxG, absG, D, Dtol;
	double x1ref[3], x1paz[3], x1pth[3];
	double t1ref[3], t1paz[3], t1pth[3];
	double test1, test2, test3;
	int n_arcs;
	struct arc_rec arc[MAX_ARCS];
	
	if( (!finite(theta)) || (!finite(azi)) || (!finite(T)) ) return(0);

	maxiterations=5; Dtol=0.0005;
	*newazi=azi; *newtheta=theta; *newT=T;

	for( iter=0; iter<maxiterations; iter++ ) {

		if( !shoot_to_traveltime( x0, *newazi, *newtheta, *newT, x1ref, t1ref, arc, &n_arcs )) return(0);
		if( !find_tetra(x1ref,&tetra1) ) return(0);
		v = eval_velocity(&tetra[tetra1],x1ref);

		pth = 1.0e-5;
		paz = 1.0e-5;
		if( !shoot_to_traveltime( x0, *newazi+paz, *newtheta, *newT, x1paz, t1paz, arc, &n_arcs )) return(0);
		if( !shoot_to_traveltime( x0, *newazi, *newtheta+pth, *newT, x1pth, t1pth, arc, &n_arcs )) return(0);
		
		D=0.0;
		for( i=0; i<3; i++ ) {
			G[i][0] = (x1paz[i]-x1ref[i])/paz;
			G[i][1] = (x1pth[i]-x1ref[i])/pth;
			G[i][2] = t1ref[i]*v;
			d[i] = x1[i] - x1ref[i];
			newdTdx[i] = t1ref[i]/v;
			D+=fabs(d[i]);
			}
			
		maxG=0.0;
		for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) if( (absG=fabs(G[i][j]))>maxG ) maxG=absG;
		if( maxG <= 0.0 ) return(0);
		
		gauss( G, d, 3, 3, 1.0e-6*maxG, &ierror, TRUE );
		if( ierror != 0 ) return(0);
		
		if( (!finite(d[0])) || (!finite(d[1])) || (!finite(d[2])) ) return(0);

		*newazi += d[0]; *newtheta += d[1]; *newT += d[2];
		
		if( D < Dtol ) break;
			
		}

	/* plausibility check */
	test1 = fabs( (*newT) - T );
	test2 = fmod( fabs( (*newtheta) - theta ) , 360.0 );
	test3 = fmod( fabs( (*newazi) - azi ) , 360.0);
	if( ((*newT)<=0.0) || (test1>=0.01) || (test2>=1.0) || (test3>=1.0) ) return(0);

	return(1);
	}
