
#define TRUE (1)
#define FALSE (0)
#define MAX_TABLE_COLS (100)
#define MAX_ARCS (10000)
#define EXPLICIT_GTG (0)
#define B_PREC (1.0e-10)
#define TSCALEVALUE (0.1)
void *bsearch();

/* header declaration for sparse matrix routines */
struct sparse_elem {
	double x; /* the value of the element */
	int i, j; /* the row, col index (starts at zero) */
	struct sparse_elem *nd; /* next down */
	struct sparse_elem *no; /* next over */
	};
struct sparse_matrix {
	int n, m, sym; /* rows and columns of matrix; symmetric flag */
	int count; /* current number of non-zero elements */
	struct sparse_elem **row; /* row pointer array of length n */
	struct sparse_elem **col; /* col pointer array of length m */
	};
struct sparse_matrix *new_sparse_matrix(int, int, int);
struct sparse_matrix *copy_sparse_matrix(struct sparse_matrix *A);
int sparse_dim( struct sparse_matrix *, int *, int *, int * );
int sparse_count( struct sparse_matrix * );
int free_sparse_matrix( struct sparse_matrix *);
double get_sparse_value( struct sparse_matrix *, int, int);
int set_sparse_value( struct sparse_matrix *, int, int, double);
int sm_times_v( struct sparse_matrix *, double *, double * );
int smt_times_v( struct sparse_matrix *, double *, double * );
struct sparse_matrix *sm_times_sm( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *smt_times_sm( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *sm_times_smt( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *smt_times_smt( struct sparse_matrix *, struct sparse_matrix *);
double sparse_absmax(struct sparse_matrix *);
int sparse_addtodiag(struct sparse_matrix *, double);
int sparse_truncaterows( struct sparse_matrix *, int);
int sparse_biconj(struct sparse_matrix*,double*,double*,int,double,int,int*,double*,int*);
int sparse_transpose(struct sparse_matrix*);
int sparse_issym(struct sparse_matrix*);
int sparse_add_to_col( struct sparse_matrix *, int, double * );
int sparse_dls_biconj(struct sparse_matrix*,double*,double*,double,int,double,int,int*,double*,int*);
int sparse_GTGpe2times(struct sparse_matrix*,double*,double*,double,double*);
int sparse_GTGpe2solve(double*, double *, double *, int );
double  *AT_times_A_pe2( struct sparse_matrix *, double, int *);
double ATAabsmax( struct sparse_matrix * );

/* I have implemented two basic model designs, that is,
ways of dividing a "rectangular" arrangement of 8 nodes
into tetrahedra. This selectes between them. */
#define load_prismatic_model load_prismatic_model2

#define MAX_TETRAS_TOUCHING_NODE (30)
struct node_rec {  /* node is vertex of tetrahedron */
	int id; /* arbitrary id */
	int ifc; /* a node can sit on an interface of interest */
		/* this interface is given an integer id       */
		/* right now, a node can have only such        */
	double x[3], v, ratio, qinv; /* position, velocity, ratio, 1/Q       */
	double v_orig, ratio_orig, qinv_orig; /* original values */

	int touched; /* for counting rays that interact with this node */
	int hits; /* for counting rays that interact with this node */
	struct tetra_rec *t[MAX_TETRAS_TOUCHING_NODE]; /* list of all tetras including this node */
	int nt; /* elements in tetra list */
	};
int MAX_NODES=0;
int n_nodes=0;
struct node_rec *node;
int QDEFINED=FALSE;

int PERTURBMODEL = 0;
double node_x_perturbation = 0.0;
double node_y_perturbation = 0.0;
double node_z_perturbation = 0.0;
double node_v_perturbation = 0.0;

struct nodeid_element_rec {
	int id;
	int index;
	};
struct nodeid_element_rec *nodeidlist;
int nodeidlist_exists = FALSE;

struct tetra_rec { /* a collection of four nodes */
	int id;
	int n1, n2, n3, n4; /* nodes */
	double *x1, *x2, *x3, *x4; /* coordinates of vertices */
	double *v[4]; /* velocity at vertices */
	double *qinv[4]; /* Q-inverse at vertices */
	double vc[4]; /* coefficients of linear equation for velocity */
	double qinvc[4]; /* coefficients of linear equation for velocity */
	double g[3]; /* gradient vector */
	double gs; /* length of gradient vector */
	double gn[3]; /* tangent to gradient */
	int t1, t2, t3, t4; /* index of adjoing tetra (if any) */
	int ifc[4];  /* interface of faces (if any) */
	int index; /* index of itself */
	int marked; /* a way of marking this tetra */
	int tested; /* another way of marking it */
	};
int n_tetras=0;
int MAX_TETRAS=0;
struct tetra_rec *tetra;

int last_tetra=(-1);  /* used in find_tetra routine */
int find_tetra_hit=0;
int find_tetra_neighbor=0;
int find_tetra_search=0;
int find_tetra_walk=0;
int find_tetra_nofind=0;

struct int_rec {
	double psi, x[3];     /* intersection angle & corresponding point */
	double xme[3];        /* slighty different inside point */
	double dpsi; /* angle with respect to entry angle */
	int face, inside;
	double epsilon, R, val;
	};

struct arc_rec {
	double x0[3], t0[3], x1[3], t1[3];  /*entry and exit location, tangent */
					    /* in untransformed coordinates    */
	double xc[3];                       /* center of circle in untransformed coords */
	double shift[3], s[3][3], st[3][3]; /* shift and rotation */
	double R, psi1, psi2; /* center, entry and exit angles in */
			      /* transformed coordinates (center at origin) */
	int tetra, exit_face; /* id of tetrahedra, exit face */
	double vc[4]; /* velocity coefficients in transformed coordinates */
	double qinvc[4]; /* qinv coefficients in transformed coordinates */
	double T, S, qinvint; /* traveltime, arclength, (integral) qinv dT */
	struct tetra_rec *ptetra; /* pointer back to tetra */
	};

struct ray_rec {
	double th, az;
	double x, y, z, T;
	int ok;
	};
struct ray_rec *ray;
int m_rays, n_rays;

struct turf_rec {
	double x0, y0;
	double x, y, z, T;
	int ok;
	};
struct turf_rec *turf;
int m_turf, n_turf;

struct tt_rec {
	double th, az;
	double T, gs;
	double dTdx[3];
	};
#define MAX_TTS (10000)
struct tt_rec tt[MAX_TTS];
int n_tts;

struct triangle_rec {
	double x0,y0;
	double x1,y1;
	double x2,y2;
	};

struct ttime_rec {
	int ok;
	double x0[3];
	double x1[3];
	int interface;
	double obsTa, preTa, obsTT, preTT, T0, ratio, qinv, preqinv;
	int obsorder, preorder;
	double th, az;
	double vapp;
	int index, pwg;
	};
int MAX_TTIMES;
struct ttime_rec *ttime;
int n_ttimes, n_pwg;

struct model_par_rec {
	int n;      /* number of nodes */
	int *node;  /* nodelist */
	double *weight; /* weight relative to other nodes in the group */
	double *t; /* tangent to directional perturbation */
	};
int MAX_MODEL_PARS;
struct model_par_rec *model_par;
int n_model_pars;
int model_L, model_M, model_N;

int calls_to_shoot;
double model_z_min, model_z_max;
double PI, DTOR, RTOD;

int compare_int();
int compare_tt();
int compare_nodeid();
double eval_velocity();
double eval_velocity2();
double eval_arc_velocity();
double eval_qinv();
double eval_qinv2();
double eval_arc_qinv();
double dot();
double eval_plane();
double eval_plane2();
double ttintegralnew();
double ttintegralold();
double myrand();
double eval_line();
double tetra_volume();
double det3x3();
double dTdmpnew();
double dTdmpold();
double dTsdmpp();
double dTsdmpr();
double dqinvdmp();
double dTdmpr();
double dTdmprd();
double great_circle_azimuth();
double solid_angle();
double area_of_triagle();
double qintegral();
double integral292();
double integral292a();
double integral294();
double integral295();
double integral294a();
double inv_integral294a();
double integral295a();
double integral340();
double integral340();
double integral341();
double integral342();
int x_tetrahedron( double *, double *, double *, double *,
		   double *, double *, double *, double * );
int n_tetra_stack, max_tetra_stack, *tetra_stack;

double refine_star_norm;
int refine_star_status;

#define ttintegral ttintegralnew
#define dTdmp dTdmpnew

long starttime;


struct ev_rec {
	int nev, index, count;
	double x, y, z, t0, rms, mean;
	};

struct jhd_rec {
	int nev, order;
	double T, dTdx, dTdy, dTdz, azi, theta;
	};


struct PS_node {
	double Pv, Sv;
	};
struct PS_tetra {
	double Pvc[4], Pg[3], Pgs, Pgn[3];
	double Svc[4], Sg[3], Sgs, Sgn[3];
	};




