
/* appends to tetralist given 8 nodes.  The first four circle the
   bottom of the prism, the last 4 the top. Returns 1 on success, 0
   on failure. */

int prism_to_tetra1(int id,int n0,int n1,int n2,int n3,int n4,
	int n5,int n6,int n7,int type )
// int id, n0,n1,n2,n3,n4,n5,n6,n7, type;
{
	int c, b, f, l, r;
	/* out of space */
	if( (n_tetras+5) > MAX_TETRAS ) return(0);

	/* no such node */
	if( (n0<0) || (n0>=n_nodes) || (n1<0) || (n1>=n_nodes) ||
	    (n2<0) || (n2>=n_nodes) || (n3<0) || (n3>=n_nodes) ||
	    (n4<0) || (n4>=n_nodes) || (n5<0) || (n5>=n_nodes) ||
	    (n6<0) || (n6>=n_nodes) || (n7<0) || (n7>=n_nodes) ) {
		return(0);
		}

	/* there are two configurations that must be alternated */
	/* chess-board style in a model containing multiple prisms */
	if( type ) {
		if( (c=new_tetra( id, n1, n3, n4, n6 )) == (-1) ) return(0);
		if( (b=new_tetra( id, n0, n1, n3, n4 )) == (-1) ) return(0);
		if( (f=new_tetra( id, n1, n2, n3, n6 )) == (-1) ) return(0);
		if( (l=new_tetra( id, n3, n4, n6, n7 )) == (-1) ) return(0);
		if( (r=new_tetra( id, n1, n4, n5, n6 )) == (-1) ) return(0);
		tetra[c].t1=l; tetra[c].t2=r; tetra[c].t3=f; tetra[c].t4=b;
		tetra[b].t1=c; tetra[f].t2=c; tetra[l].t4=c; tetra[r].t3=c;
		}
	else {
		if( (c=new_tetra( id, n0, n2, n5, n7 )) == (-1) ) return(0);
		if( (r=new_tetra( id, n0, n1, n2, n5 )) == (-1) ) return(0);
		if( (f=new_tetra( id, n2, n5, n6, n7 )) == (-1) ) return(0);
		if( (b=new_tetra( id, n0, n4, n5, n7 )) == (-1) ) return(0);
		if( (l=new_tetra( id, n0, n2, n3, n7 )) == (-1) ) return(0);
		tetra[c].t1=f; tetra[c].t2=b; tetra[c].t3=l; tetra[c].t4=r;
		tetra[r].t2=c; tetra[f].t3=c; tetra[b].t2=c; tetra[l].t3=c;
		}

	return(1);
	}

int prism_to_tetra2(int id,int n0,int n1,int n2,int n3,int n4,
	int n5,int n6,int n7,int type )
// int id, n0,n1,n2,n3,n4,n5,n6,n7, type;
{
	int a, b, c, d, e, f;
	/* out of space */
	if( (n_tetras+6) > MAX_TETRAS ) return(0);

	/* no such node */
	if( (n0<0) || (n0>=n_nodes) || (n1<0) || (n1>=n_nodes) ||
	    (n2<0) || (n2>=n_nodes) || (n3<0) || (n3>=n_nodes) ||
	    (n4<0) || (n4>=n_nodes) || (n5<0) || (n5>=n_nodes) ||
	    (n6<0) || (n6>=n_nodes) || (n7<0) || (n7>=n_nodes) ) {
		return(0);
		}

	if( (a=new_tetra( id, n2, n4, n6, n7 )) == (-1) ) return(0);
	if( (b=new_tetra( id, n2, n3, n4, n7 )) == (-1) ) return(0);
	if( (c=new_tetra( id, n0, n2, n3, n4 )) == (-1) ) return(0);
	tetra[a].t3=b; tetra[b].t2=a;
	tetra[b].t4=c; tetra[c].t1=b;

	if( (d=new_tetra( id, n2, n4, n5, n6 )) == (-1) ) return(0);
	if( (e=new_tetra( id, n1, n2, n4, n5 )) == (-1) ) return(0);
	if( (f=new_tetra( id, n0, n1, n2, n4 )) == (-1) ) return(0);
 	tetra[d].t4=e; tetra[e].t1=d;
	tetra[e].t4=f; tetra[f].t1=e;

	tetra[a].t4=d; tetra[d].t3=a;
 	tetra[c].t3=f; tetra[f].t2=c;

	return(1);
	}



int model_to_P( struct PS_node *my_PS_node, struct PS_tetra *my_PS_tetra, int *model_is_P )
{
	int i, j;
	if( *model_is_P ) return(1);
	
	for( i=0; i<n_nodes; i++ ) {
		node[i].v = my_PS_node[i].Pv;	
		}
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) tetra[i].vc[j] = my_PS_tetra[i].Pvc[j];
		for(j=0;j<3;j++) tetra[i].g[j] = my_PS_tetra[i].Pg[j];
		tetra[i].gs = my_PS_tetra[i].Pgs;
		for(j=0;j<3;j++) tetra[i].gn[j] = my_PS_tetra[i].Pgn[j];
		}
	*model_is_P = 1;
	return(1);
	}

int model_to_S( struct PS_node *my_PS_node, struct PS_tetra *my_PS_tetra, int *model_is_P )
{
	int i, j;
	if( ! (*model_is_P) ) return(1);
	
	for( i=0; i<n_nodes; i++ ) {
		node[i].v = my_PS_node[i].Sv;	
		}
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) tetra[i].vc[j] = my_PS_tetra[i].Svc[j];
		for(j=0;j<3;j++) tetra[i].g[j] = my_PS_tetra[i].Sg[j];
		tetra[i].gs = my_PS_tetra[i].Sgs;
		for(j=0;j<3;j++) tetra[i].gn[j] = my_PS_tetra[i].Sgn[j];
		}
	*model_is_P = 0;
	return(1);
	}
