

int mark_tri_prism( double b1[3],double b2[3],double b3[3],double t1[3],double t2[3],double t3[3] )
// double b1[3], b2[3], b3[3]; // old syntax
// double t1[3], t2[3], t3[3]; // old syntax
{
	double *v1, *v2, *v3, *v4;
	int i, it;

	for( i=0; i<n_tetras; i++ ) { tetra[i].marked=0;  tetra[i].tested=0; }

	for( it=0; it<3; it++ ) {
		if( it==0 ) {     v1=b1; v2=b2; v3=b3; v4=t1;}
		else if( it==1) { v1=b2; v2=b3; v3=t1; v4=t2; }
		else            { v1=b3; v2=t1; v3=t2; v4=t3; }
		mark_tetra_intersection2( v1, v2, v3, v4 );
		}

	return(0);
	}


/* marks model tetrahedra that overlap an arbitrary tetrahedron with
   vertices (x1, ... x4) by setting their marked field.  Note that this
   routine does not first clear the marked fields. */
int mark_tetra_intersection( double *x1, double *x2, double *x3, double *x4 ) {

#define MAX_GEN (5)
#define MAX_COUNT (500000)

	int i, tmean, count, tcount; //n, //unused
	double xmean[3];
	struct stack_rec {
		struct tetra_rec tetra;
		double x1[3], x2[3], x3[3], x4[3];
		int t1, t2, t3, t4;
		int n, generation;
		struct stack_rec *previous;
		};
	struct stack_rec *top=NULL, *t, *tnew;

	/* the idea:
		Mark the model tetrahedra in which the vertices of
		target tetrahedron lie.  Then, if all 4 of the vertices
		of the target tetrahedron are within one tetrahedron,
		then your're done, since the model tetrahedron is entirely
		within it. Otherwise, subdivide the target tetrahedron into
		four smaller tetrahedra, and repeat the process.  The
		work is conducted on a stack.                       */

	/* set up stack */
	tcount=0;
	t = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
	if( t==NULL ) {
		fprintf(stderr,"error: mark_tetra_intersection: out of memory (1)\n");
		exit(-1);
		}
	for( i=0; i<3; i++ ) { t->x1[i]=x1[i]; t->x2[i]=x2[i]; t->x3[i]=x3[i]; t->x4[i]=x4[i]; }
	t->tetra.x1 = t->x1; t->tetra.x2 = t->x2; t->tetra.x3 = t->x3; t->tetra.x4 = t->x4;
	if( !find_tetra( x1, &t->t1 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x1 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x2, &t->t2 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x2 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x3, &t->t3 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x3 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x4, &t->t4 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x4 not in model (1)\n"); exit(-1); }
	tetra[t->t1].marked=1; tetra[t->t2].marked=1; tetra[t->t3].marked=1; tetra[t->t4].marked=1;
	t->previous = NULL; t->n=tcount++; t->generation=0;
	top=t;
	
	count=0;
	while( top != NULL ) {
		count++; if(count>MAX_COUNT) {
			fprintf(stderr,"error: mark_tetra_intersection: stack overflow\n");
			exit(-1);
			}
		
		t=top; top=t->previous;

		if( ((t->t1==t->t2) && (t->t1==t->t3) && (t->t1==t->t4)) || (t->generation>MAX_GEN) ) {
			free(t);
			continue;
			}
	
		for( i=0; i<3; i++ ) xmean[i]=(t->x1[i]+t->x2[i]+t->x3[i]+t->x4[i])/4.0;
		if( !find_tetra( xmean, &tmean ) ) {
			fprintf(stderr,"error: mark_tetra_intersection: xmean not in model (2)\n");
			fprintf(stderr,"       x = (%f %f %f)\n", xmean[0], xmean[1], xmean[2]);

			exit(-1);
			}
		tetra[tmean].marked=1;

		/* first tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=xmean[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=tmean; tnew->t2=t->t2; tnew->t3=t->t3; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* second tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=xmean[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=t->t1; tnew->t2=tmean; tnew->t3=t->t3; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* third tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=xmean[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=t->t1; tnew->t2=t->t2; tnew->t3=tmean; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* fourth tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=xmean[i]; }
		tnew->t1=t->t1; tnew->t2=t->t2; tnew->t3=t->t3; tnew->t4=tmean;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		free(t);
		
		}
				
	return(1);
	}

/* marks model tetrahedra that overlap an arbitrary tetrahedron with
   vertices (x1, ... x4) by setting their marked field.  Note that this
   routine does not first clear the marked fields. */
int mark_tetra_intersection2( double *x1, double *x2, double *x3, double *x4 ) {
	int i, t, ta[4], intersect;
	if( !zero_tetra_stack() ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 1)\n");
		exit(-1);
		}
	if( find_tetra(x1,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x2,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x3,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x4,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (push_tetra, 2)\n");
		exit(-1);
		}
	while( (t=pop_tetra()) != (-1) ) {
		tetra[t].tested = 1;
		intersect=x_tetrahedron( tetra[t].x1, tetra[t].x2, tetra[t].x3, tetra[t].x4,
			       x1, x2, x3, x4 );
		if( intersect ) {
			tetra[t].marked = 1;
			ta[0]=tetra[t].t1; ta[1]=tetra[t].t2; ta[2]=tetra[t].t3; ta[3]=tetra[t].t4;
			for( i=0; i<4; i++ ) {
				if( !push_tetra(ta[i]) ) {
					fprintf(stderr,"error: out of memory (init_tetra_stack, 3)\n");
					exit(-1);
					}
				}
			}
		}
				
	return(1);
	}

