




int circular_ray_exits_tetra(struct tetra_rec *t,double rx0[3],double rt0[3],double rx1[3],
	double rt1[3],struct arc_rec *arc,double *traveltime,double *S,double *qinvint,int *ef )
// struct tetra_rec *t;
// double rx0[3], rt0[3]; /*ray starting position and tangent */
// double rx1[3], rt1[3]; /*ray ending position and tangent   */
// struct arc_rec *arc;
// double *traveltime;
// double *S; /* arc length */
// double *qinvint;
// int *ef; /*exit face*/
{
	int i, j, nints, found, status, method;
	double a, vs, p, theta, R, T, T1, T2, v0, qinv, qinv1, qinv2;
	double b0[3], b1[3], b2[3], s[3][3], st[3][3];
	double nx1[3], nx2[3], nx3[3], nx4[3], rc[3];
	double nrx0[3], nrt0[3], nrx1[3], nrt1[3];
	double shift[3];
	double psi1, psi2;
	struct int_rec intersecs[20];
	double tvec[3];
	double epsilon;
	double vc[4], qinvc[4];
	double xcir[3], rxcir[3];

//	int k; //unused
//	double I[3][3]; //unused

	epsilon=1.0e-10;

	/* velocity at starting point */
	vs = eval_velocity(t, rx0);

	/* new basis in which gradient is vertical and ray in (x',z') plane */
	for( i=0; i<3; i++ ) b2[i]=t->gn[i];
	cross( b2, rt0, b1 ); j=unit_vector(b1);
	if( !j ) { /*special case of ray parallel to gradient */
		do {
			ranvec(tvec);
			cross( b2, tvec, b1 );
			j=unit_vector(b1);
			} while ( !j );
		}
	cross( b1, b2, b0 ); unit_vector(b0);

	newbasisrot( b0, b1, b2, s );
	for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) st[i][j]=s[j][i];

	if( arc!=NULL ) for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) {
		arc->s[i][j]=s[i][j];
		arc->st[i][j]=st[i][j];
		}

	/*rotate everything to new coordinate system */
	rotate3(t->x1,s,nx1);
	rotate3(t->x2,s,nx2);
	rotate3(t->x3,s,nx3);
	rotate3(t->x4,s,nx4);
	rotate3(rx0,s,nrx0);
	rotate3(rt0,s,nrt0);

	/* local ray parameter, angle of incidence, radius of circle */
	a=dot(rt0,b2); if(a<(-1.0)) a=(-1.0); else if( a>1.0 ) a=1.0;
	theta=RTOD*acos(a);
	p = sin(DTOR*theta)/vs;
	a = fabs(t->gs*p);
	if( a>1.0e-12 ) R=1.0/a; else R=1.0e12;

	/* center of circle */
	rc[0]=nrx0[0]+R*cos(DTOR*theta);
	rc[1]=nrx0[1];
	rc[2]=nrx0[2]-R*sin(DTOR*theta);

	/*coordinate shift so that center of circle is origin */
	for( i=0; i<3; i++ ) shift[i]=rc[i];

	if( arc!=NULL ) for( i=0; i<3; i++ ) arc->shift[i]=shift[i];

	for( i=0; i<3; i++ ) {
		nx1[i] -= shift[i];
		nx2[i] -= shift[i];
		nx3[i] -= shift[i];
		nx4[i] -= shift[i];
		nrx0[i] -= shift[i];
		rc[i] -= shift[i];
		}

	status=velocity_eqn2( nx1, nx2, nx3, nx4, t->v, vc );

	if( arc!=NULL ) {
		for( i=0; i<4; i++ ) arc->vc[i]=vc[i];
		arc->tetra = t->id;
		arc->ptetra = t;
		}

	if( (qinvint!=NULL) || (arc!=NULL) ) {
		status=qinv_eqn2( nx1, nx2, nx3, nx4, t->qinv, qinvc );
		}

	/* velocity at center of circle */
	v0=vc[3];

	/* entry angle */
	psi1=270.0+theta; while( psi1>=360.0 ) psi1-=360.0; while( psi1<0.0 ) psi1+=360.0;
	if( arc!=NULL ) arc->psi1 = psi1;

	nints=0.0;
	circle_intersects_plane( R, nx1, nx2, nx3, intersecs, &nints, 4, nx4 );
	circle_intersects_plane( R, nx1, nx2, nx4, intersecs, &nints, 3, nx3 );
	circle_intersects_plane( R, nx2, nx3, nx4, intersecs, &nints, 1, nx1 );
	circle_intersects_plane( R, nx1, nx3, nx4, intersecs, &nints, 2, nx2 );

	for( i=0; i<nints; i++ ) {
		a = intersecs[i].psi-psi1;
		while(a<0.0) a+=360.0;
		while(a>=360.0) a-=360.0;
		if( isnan(a) ) {
			fprintf(stderr,"warning: NaN in circle calculation\n");
			a=360.0;
			}
		if( a>(360.0-epsilon) ) a=0.0;
		intersecs[i].dpsi = a;
		}
        qsort( intersecs, nints, sizeof(struct int_rec), compare_int );

	found=(-1);
	for( i=0; i<nints; i++ ) {
		if( (fabs(R*intersecs[i].dpsi)>epsilon)
		 && (intersecs[i].dpsi<=(180.0+epsilon))
		 && (intersecs[i].inside=
			in_tetrahedron( nx1, nx2, nx3, nx4, intersecs[i].xme)) ) {
			found=i;
			break;
			}
		}

	if( found!=(-1) ) {
		for( i=0; i<3; i++ ) nrx1[i]=intersecs[found].x[i]+shift[i];
		nrt1[0]=sin(DTOR*(90.0+intersecs[found].psi));
		nrt1[1]=0.0;
		nrt1[2]=cos(DTOR*(90.0+intersecs[found].psi));
		rotate3(nrx1,st,rx1);
		rotate3(nrt1,st,rt1);
		psi2=intersecs[found].psi;
		while( psi2 < psi1 ) psi1-=360.0;
		if( (traveltime!=NULL) || (arc!=NULL) ) {
			T2 = ttintegral(v0,t->gs,R,psi2,&method);
			T1 = ttintegral(v0,t->gs,R,psi1,&method);
			T=(T2-T1);
			}
		if( traveltime!=NULL ) (*traveltime)=T;
		(*S)=DTOR*(psi2-psi1)*R;
		if( (qinvint!=NULL) || (arc!=NULL) ) {
			qinv2 = qintegral( qinvc[3], qinvc[0], qinvc[2], R, t->gs, psi2 );
			qinv1 = qintegral( qinvc[3], qinvc[0], qinvc[2], R, t->gs, psi1 );
			qinv = qinv2-qinv1;
			}
		if( qinvint!=NULL ) *qinvint = qinv;
		(*ef)=intersecs[found].face;
		if( arc!=NULL ) {
			arc->exit_face = intersecs[found].face;
			arc->R=R;
			arc->psi2 = intersecs[found].psi;
			while( arc->psi2 < arc->psi1 ) arc->psi1 -= 360.0;
			arc->S = (*S);
			arc->T = T;
			arc->qinvint = qinv;
			rxcir[0]=0.0; rxcir[1]=0.0; rxcir[2]=0.0;
			for( i=0; i<3; i++ ) rxcir[i]+=shift[i];
			rotate3(rxcir,st,xcir);
			for( i=0; i<3; i++ ) arc->xc[i]=xcir[i];
			}
		return(TRUE);
		}
	else {
		if( 0 ) { /* some diagnostics */
			fprintf(stderr,"tetra: %d\n", t->index);
			print_tetra(stderr,t);
			fprintf(stderr,"theta %f p %f R %f vs %f\n", theta, p, R, vs );
			fprintf(stderr,"nx1: %f %f %f\n", nx1[0], nx1[1], nx1[2] );
			fprintf(stderr,"nx2: %f %f %f\n", nx2[0], nx2[1], nx2[2] );
			fprintf(stderr,"nx3: %f %f %f\n", nx3[0], nx3[1], nx3[2] );
			fprintf(stderr,"nx4: %f %f %f\n", nx4[0], nx4[1], nx4[2] );
			fprintf(stderr,"nx0: %f %f %f\n", nrx0[0], nrx0[1], nrx0[2] );
			fprintf(stderr,"psi1: %f\n", psi1);
			fprintf(stderr,"\n");
			for( i=0; i<nints; i++ ) {
				fprintf(stderr,"intersection %d of %d\n", i, nints );
				print_intersec(stderr,&intersecs[i]);
				}
			}
		return(FALSE);
		}

	}

int circle_intersects_plane(double R,double x1[3],double x2[3],double x3[3],
	struct int_rec *intersecs,int *nints,int face,double x4[3] )
// double R; /* radius of circle */
// double x1[3], x2[3], x3[3]; /* points on plane */
// struct int_rec *intersecs;
// int *nints;
// int face;
// double x4[3];
{
	int i;
	double P[4], a, b, c;
	double fabsc, A, A2, D, spsi, cpsi, psi, phi, tmp, th, v;
	double epsilon;

	epsilon=1.0e-6;
	v=0.0;

	/* if all three vertices of the tetrahedron are on the  */
	/* same side of the y=0 plane, then the intersection of */
	/* that face of the tetrahedron with the y=0 plane will */
	/* not be inside the tetrahedron, and can be ignored    */
	if( (x1[1]<0.0) && (x2[1]<0.0) && (x3[1]<0.0) ) return(TRUE);
	if( (x1[1]>0.0) && (x2[1]>0.0) && (x3[1]>0.0) ) return(TRUE);


	/* equation of line of intersection of plane with the (x,z)=(ax+bz+c)=0 plane */
	eqn_of_plane( x1, x2, x3, P );
	a = P[0];
	b = P[2];
	c = P[3];

	/* Now imaginge a line that passes thru the origin and which           */
	/* is the perpendicular bisector of the line associated with the face. */
	/* It has equation -bx+az=0.  The point of intersection is             */
	/* (x,z)=-c(a,b)/(a**2+b**2).  The distance from that point to the     */
	/* origin is D=|c|/sqrt(a*a+b*b).   If D>R then no solution occurs.    */ 
	/* The direction, t,  from the origin to the intersection is           */
	/* (tx, tz) = (sin(psi),cos(psi)) = (-ca,-cb)/sqrt((ac)**2+(bc)**2)    */
	/* where psi is the angle from the z axis to the interstection point.  */
	/* Now imagine that we rotate the coordinates so that the t-direction  */
	/* becomes the new z'-axis.  The equation of line is z'=D (where D is  */
	/* guaranteed positive.  The equation of the circle is (x',z') =       */
	/* (Rsin(phi), Rcos(phi)).  There are two intersections, symmetric     */
	/* about the z' axis, psi and -psi, where psi=acos(D/R) is guaranteed  */
	/* to be in the range 0 to pi/2.  The angle, th, in the unrotated      */
	/* coordinate system between the z-axis and the two intersection       */
	/* points is th=psi+/-phi.                                             */

	A2 = (a*a+b*b);
	if( A2 <= 0.0 ) return(FALSE); /*plane parallel to (x,z)*/

	fabsc=fabs(c);
	A = sqrt(A2);
	D = fabsc/A;
	if( D>R ) return(FALSE); /* no intersection */

	tmp = fabs(c)*A;
	spsi = (-c*a) / tmp;
	cpsi = (-c*b) / tmp;
	psi = atan2( spsi, cpsi );

	phi = acos( D/R );
	
	th = psi+phi;
	intersecs[*nints].psi = RTOD*th;
	intersecs[*nints].x[0]= R*sin(th);
	intersecs[*nints].x[1]= 0.0;
	intersecs[*nints].x[2]= R*cos(th);
	/* xme is near x, but slightly displaced in direction of vertex */
	/* I have variously tried two ways of displacing,               */
	/* towards the other vertex, or back along the ray              */
	if(0) {
		for( i=0; i<3; i++ ) {
			intersecs[*nints].xme[i] =
				(1.0-epsilon)*intersecs[*nints].x[i] +
				epsilon*x4[i];
			}
		}
	else {
		intersecs[*nints].xme[0]= R*sin(th-epsilon/R);
		intersecs[*nints].xme[1]= 0.0;
		intersecs[*nints].xme[2]= R*cos(th-epsilon/R);
		}

	intersecs[*nints].face=face;
	intersecs[*nints].inside=FALSE;
	intersecs[*nints].R=R;
	intersecs[*nints].epsilon=epsilon;
	intersecs[*nints].val=v;
	(*nints)++;

	th = psi-phi;
	intersecs[*nints].psi = RTOD*th;
	intersecs[*nints].x[0]= R*sin(th);
	intersecs[*nints].x[1]= 0.0;
	intersecs[*nints].x[2]= R*cos(th);
	/* xme is near x, but slightly displaced in direction of vertex */
	/* I have variously tried two ways of displacing,               */
	/* towards the other vertex, or back along the ray              */
	if(0) {
		for( i=0; i<3; i++ ) {
			intersecs[*nints].xme[i] =
				(1.0-epsilon)*intersecs[*nints].x[i] +
				epsilon*x4[i];
			}
		}
	else {
		intersecs[*nints].xme[0]= R*sin(th-epsilon/R);
		intersecs[*nints].xme[1]= 0.0;
		intersecs[*nints].xme[2]= R*cos(th-epsilon/R);
		}

	intersecs[*nints].face=face;
	intersecs[*nints].inside=FALSE;
	intersecs[*nints].R=R;
	intersecs[*nints].epsilon=epsilon;
	intersecs[*nints].val=v;
	(*nints)++;

	return(TRUE);
	}


int in_tetrahedron2(double x1[3],double x2[3],double x3[3],double x4[3],double x[3])
// double x1[3], x2[3], x3[3], x4[3], x[3];
{
	double P[4], a, b;
	int on_surface;

	/* theory: use three vertices to define a plane         */
	/* check if the fourth vertice and the point are        */
	/* on the same side of the plane.  If it is for all     */
	/* planes, then the point is inside the tetrahedron     */
	/* two points are on the same side of the plane if      */
	/* the equation of the plane evaluates to the same sign */
	/* see Rektorys Survey of Applicable Mathematics p 243  */

	on_surface=0;

	/* plane 4 */
	eqn_of_plane( x1, x2, x3, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x4 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 3 */
	eqn_of_plane( x1, x2, x4, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x3 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 2 */
	eqn_of_plane( x1, x3, x4, P );
	a=eval_plane( P, x ); 
	b=eval_plane( P, x2 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 1 */
	eqn_of_plane( x2, x3, x4, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x1 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	if( on_surface==1 ) return(1); else return(1);

	}
	 
	 
	 
int in_tetrahedron3(double x1[3],double x2[3],double x3[3],double x4[3],double x[3])
// double x1[3], x2[3], x3[3], x4[3], x[3];
{
	double a, b;
	int on_surface;

	/* theory: use three vertices to define a plane         */
	/* check if the fourth vertice and the point are        */
	/* on the same side of the plane.  If it is for all     */
	/* planes, then the point is inside the tetrahedron     */
	/* two points are on the same side of the plane if      */
	/* the equation of the plane evaluates to the same sign */
	/* see Rektorys Survey of Applicable Mathematics p 243  */

	on_surface=0;

	/* plane 4 */
	a=eval_plane2( x,  x1, x2, x3 );
	b=eval_plane2( x4, x1, x2, x3 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 3 */
	a=eval_plane2( x,  x1, x2, x4 );
	b=eval_plane2( x3, x1, x2, x4 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 2 */
	a=eval_plane2( x,  x1, x3, x4 );
	b=eval_plane2( x2, x1, x3, x4 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 1 */
	a=eval_plane2( x,  x2, x3, x4);
	b=eval_plane2( x1, x2, x3, x4);

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	if( on_surface==1 ) return(1); else return(1);

	}

int in_tetrahedron(double x1[3],double x2[3],double x3[3],double x4[3],double x[3])
// double x1[3], x2[3], x3[3], x4[3], x[3];
{
	int i;
	double a, b;
	double v1[3], v2[3], n[3], tv[3], tx[3];

	/* theory:
		is point x on the same side of each of the 4 planes as
		each excluded vertex?  If so, its in the tetrahedron.
		Given a plane xa, xb, xc, then (xb-xa) cross (xc-xa) is
		a normal to that plane.  if (xd-xa) dot normal has same
		sign as (x-xa) dot normal, then the excluded vertex
		is on the same side of the plane as x.                */

	/* plane 1 */
	for( i=0; i<3; i++ ) {
		v1[i] = x3[i]-x2[i];
		v2[i] = x4[i]-x2[i];
		tv[i] = x1[i]-x2[i];
		tx[i] = x[i] -x2[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 2 */
	for( i=0; i<3; i++ ) {
		v1[i] = x3[i]-x1[i];
		v2[i] = x4[i]-x1[i];
		tv[i] = x2[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 3 */
	for( i=0; i<3; i++ ) {
		v1[i] = x2[i]-x1[i];
		v2[i] = x4[i]-x1[i];
		tv[i] = x3[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 4 */
	for( i=0; i<3; i++ ) {
		v1[i] = x2[i]-x1[i];
		v2[i] = x3[i]-x1[i];
		tv[i] = x4[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	return(1);
	}


/* finds intersection, xi, of ray with the plane, P. The ray is
   specified by a point, x, and a tangent unit vector t.
   The point must be below the plane, in the sense that the computed
   arclength, s, along the tangent from xi to the plane is positive.
   The normal of the plane is also calculated. Returns 1 on
   success, 0 on error. */

int ray_exits_layer(double P[4],double x[3],double t[3],double normal[3],double xi[3],double *s )
// double P[4], x[3], t[3];
// double normal[3], xi[3], *s;
{
	int i;
	double a, xc[3];

	for( i=0; i<3; i++ ) xc[i]=x[i];

	/* calculate intersection of ray with plane, P.  The
	   intersection is found by simultaneously solving equation of plane,
	   Ax + By + Cz + D = 0 and position on ray as function of arclength, a,
	   x = xc[0]+a*t[0], y = xc[1]+a*t[1], z = xc[2]+a*t[2] */

	normal_to_plane( P, normal );
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( isfinite(a) && a > 0.0 ) { /* arclength has right sign */
		*s = a;
		return(1);
		}
	else {
		return(0);
		}

	}

/* finds intersection, xi, of ray with one of the faces of a tetrahedron
   The ray is specified by a point, x, and a tangent unit vector t.
   The point must be inside, and the face with the positive arclength, s,
   along the tangent is computed and that arclength is calculated.
   The face is returned as (0, 1, 2, 3) depending upon which of (x1, x2,
   x3, x4) is not on the plane. The outward pointing normal of the plane
   is also calculated. Returns 1 on success, 0 on error. If
   the point tests outside the tetrahedron or exactly on a face, it will
   be advanced a distance epsilon>0 along the ray and retried.  So set
   epsilon to a small positive number. */

int ray_exits_tetrahedron(double x1[3],double x2[3],double x3[3],double x4[3],double x[3],
	double t[3],double epsilon,double normal[3],double xi[3],double *s,int *face )
// double x1[3], x2[3], x3[3], x4[3], x[3], t[3], epsilon;
// double normal[3], xi[3], *s;
// int *face;
{
	int i, in_tetra1, in_tetra2;
	double a, advance, xc[3], xcc[3], P[4];

	advance=0.0;
	for( i=0; i<3; i++ ) xc[i]=x[i];

	if( (in_tetra1=in_tetrahedron( x1, x2, x3, x4, xc)) <= 0 ) {
		/* if the point is on or near the boundary, then it may evaluate
		   outside due to roundoff error.  Thus we move it a tiny bit,
		   and check again. */
		for( i=0; i<3; i++ ) xc[i] += epsilon*t[i];
		advance=epsilon;
		if( (in_tetra2=in_tetrahedron( x1, x2, x3, x4, xc)) <= 0 ) {
			return(0);
			}
		}

	/* calculate intersection of ray with each face of tetrahedra
	   intersection is found by simultaneously solving equation of plane,
	   Ax + By + Cz + D = 0 and position on ray as function of arclength, a,
	   x = xc[0]+a*t[0], y = xc[1]+a*t[1], z = xc[2]+a*t[2] */

	/* calculate outward facing normal, too.  n=grad(P) points in the
           direction of increasing eqn-of-plane values, thus if the remaining
	   face has a negative eqn-of-plane value, then the normal is
	   outward facing.  Otherwise reverse it. */

	/* face 1 */
	*face=0;
	eqn_of_plane( x2, x3, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x1 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( isfinite(a) && a > 0.0 ) { /* arglength has right sign. Now is the point on
				        the face of the tetrahedron? back it up a bit
					before checking. The isfinite() checks whether
					the face is parallel to the face */
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 2 */
	*face=1;
	eqn_of_plane( x1, x3, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x2 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( isfinite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 3 */
	*face=2;
	eqn_of_plane( x1, x2, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x3 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( isfinite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 4 */
	*face=3;
	eqn_of_plane( x1, x2, x3, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x4 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( isfinite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	return(0);
	}


/* normal to plane */
int normal_to_plane(double P[4],double n[3])
// double P[4], n[3];
{
//	double a; //unused

	/* normal is in direction grad(Ax+By+Cz+D)=(A,B,C) */
	n[0]=P[0], n[1]=P[1]; n[2]=P[2];
	unit_vector(n);
	return(1);
	}


/* great circle azimuth on a sphere in degrees */
/* azimuth is the angle of north_pole - point 1 - point 2 */
double great_circle_azimuth(double lat1,double lon1,double lat2,double lon2 )  
// double lat1, lon1, lat2, lon2;
{
	double A, C, a, b, c; // B, //unused
	double cosA, cosC, cosa, cosb, cosc; // cosB, //unused
	double sinA, sinC, sina, sinb, sinc; // sinB, //unused

	if( (lat1==lat2) && (lon1==lon2) ) {
		return( (double)0.0 );
		}

	a=DTOR*(90.0-lat2);
	b=DTOR*(90.0-lat1);

	C = DTOR*( lon2 - lon1 );

	cosa = cos(a);
	cosb = cos(b);
	sina = sin(a);
	sinb = sin(b);
	cosC = cos(C);
	sinC = sin(C);

	cosc = cosa*cosb + sina*sinb*cosC;
	if (cosc<-1.0) c=PI; else if (cosc>1.0) c=0.0; else c=acos(cosc);
	sinc = sin(c);

	cosA = (cosa - cosb*cosc) / (sinb*sinc);
	sinA = sinC/(sinc*sina);
	if (cosA<-1.0) A=PI; else if (cosA>1.0) A=0.0; else A=acos(cosA);
	if( sinA < 0.0 ) A=(-A);

	return( A/DTOR );
	}

double solid_angle(double colata,double lona,double colatb,double lonb,double colatc,double lonc )
// double colata, lona, colatb, lonb, colatc, lonc;
{
	double lata, latb, latc;
	double A1, A2, B1, B2, C1, C2, A, B, C, E, area;

	lata = 90.0-colata;
	latb = 90.0-colatb;
	latc = 90.0-colatc;

	A1 = great_circle_azimuth( lata,lona, latc,lonc );
	A2 =  great_circle_azimuth( lata,lona, latb,lonb );
	A = A1-A2;
	while( A<0.0) A+=360.0; while( A>=360.0) A-=360.0;

	B1 = great_circle_azimuth( latb,lonb, lata,lona );
	B2 = great_circle_azimuth( latb,lonb, latc,lonc );
	B = B1-B2;
	while( B<0.0) B+=360.0; while( B>=360.0) B-=360.0;

	C1 = great_circle_azimuth( latc,lonc, latb,lonb );
	C2 = great_circle_azimuth( latc,lonc, lata,lona );
	C = C1-C2;
	while( C<0.0) C+=360.0;  while( C>=360.0) C-=360.0;

	E = A + B + C - 180;

	area = E*DTOR;

	if( area > 2.0*PI ) area = 4.0*PI - area;

	return(area);
	}

double area_of_triagle(double x[3],double y[3],double z[3],double normal[3])
// double x[3], y[3], z[3], normal[3];
{
	int i;
	double a[3], b[3], c[3], area, len;

	a[0]=x[1]-x[0]; a[1]=y[1]-y[0]; a[2]=z[1]-z[0];
	b[0]=x[2]-x[0]; b[1]=y[2]-y[0]; b[2]=z[2]-z[0];
	cross( a, b, c );
	len = sqrt(fabs(dot(c,c)));
	for( i=0; i<3; i++ ) normal[i]=c[i]/len;
	if( normal[2] < 0.0 ) for( i=0; i<3; i++ ) normal[i]*=(-1.0);
	
	area = 0.5*len;
	return(area);
	}



