#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "date.h"
#define TRUE (1)
#define FALSE (0)
#define MAX_TABLE_COLS (100)
#define MAX_ARCS (10000)
#define EXPLICIT_GTG (0)
#define B_PREC (1.0e-10)
#define TSCALEVALUE (0.1)
void *bsearch();

/* header declaration for sparse matrix routines */
struct sparse_elem {
	double x; /* the value of the element */
	int i, j; /* the row, col index (starts at zero) */
	struct sparse_elem *nd; /* next down */
	struct sparse_elem *no; /* next over */
	};
struct sparse_matrix {
	int n, m, sym; /* rows and columns of matrix; symmetric flag */
	int count; /* current number of non-zero elements */
	struct sparse_elem **row; /* row pointer array of length n */
	struct sparse_elem **col; /* col pointer array of length m */
	};
struct sparse_matrix *new_sparse_matrix(int, int, int);
struct sparse_matrix *copy_sparse_matrix(struct sparse_matrix *A);
int sparse_dim( struct sparse_matrix *, int *, int *, int * );
int sparse_count( struct sparse_matrix * );
int free_sparse_matrix( struct sparse_matrix *);
double get_sparse_value( struct sparse_matrix *, int, int);
int set_sparse_value( struct sparse_matrix *, int, int, double);
int sm_times_v( struct sparse_matrix *, double *, double * );
int smt_times_v( struct sparse_matrix *, double *, double * );
struct sparse_matrix *sm_times_sm( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *smt_times_sm( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *sm_times_smt( struct sparse_matrix *, struct sparse_matrix *);
struct sparse_matrix *smt_times_smt( struct sparse_matrix *, struct sparse_matrix *);
double sparse_absmax(struct sparse_matrix *);
int sparse_addtodiag(struct sparse_matrix *, double);
int sparse_truncaterows( struct sparse_matrix *, int);
int sparse_biconj(struct sparse_matrix*,double*,double*,int,double,int,int*,double*,int*);
int sparse_transpose(struct sparse_matrix*);
int sparse_issym(struct sparse_matrix*);
int sparse_add_to_col( struct sparse_matrix *, int, double * );
int sparse_dls_biconj(struct sparse_matrix*,double*,double*,double,int,double,int,int*,double*,int*);
int sparse_GTGpe2times(struct sparse_matrix*,double*,double*,double,double*);
int sparse_GTGpe2solve(double*, double *, double *, int );
double  *AT_times_A_pe2( struct sparse_matrix *, double, int *);
double ATAabsmax( struct sparse_matrix * );

/* I have implemented two basic model designs, that is,
ways of dividing a "rectangular" arrangement of 8 nodes
into tetrahedra. This selectes between them. */
#define load_prismatic_model load_prismatic_model2

#define MAX_TETRAS_TOUCHING_NODE (30)
struct node_rec {  /* node is vertex of tetrahedron */
	int id; /* arbitrary id */
	int ifc; /* a node can sit on an interface of interest */
		/* this interface is given an integer id       */
		/* right now, a node can have only such        */
	double x[3], v, ratio, qinv; /* position, velocity, ratio, 1/Q       */
	double v_orig, ratio_orig, qinv_orig; /* original values */

	int touched; /* for counting rays that interact with this node */
	int hits; /* for counting rays that interact with this node */
	struct tetra_rec *t[MAX_TETRAS_TOUCHING_NODE]; /* list of all tetras including this node */
	int nt; /* elements in tetra list */
	};
int MAX_NODES=0;
int n_nodes=0;
struct node_rec *node;
int QDEFINED=FALSE;

int PERTURBMODEL = 0;
double node_x_perturbation = 0.0;
double node_y_perturbation = 0.0;
double node_z_perturbation = 0.0;
double node_v_perturbation = 0.0;

struct nodeid_element_rec {
	int id;
	int index;
	};
struct nodeid_element_rec *nodeidlist;
int nodeidlist_exists = FALSE;

struct tetra_rec { /* a collection of four nodes */
	int id;
	int n1, n2, n3, n4; /* nodes */
	double *x1, *x2, *x3, *x4; /* coordinates of vertices */
	double *v[4]; /* velocity at vertices */
	double *qinv[4]; /* Q-inverse at vertices */
	double vc[4]; /* coefficients of linear equation for velocity */
	double qinvc[4]; /* coefficients of linear equation for velocity */
	double g[3]; /* gradient vector */
	double gs; /* length of gradient vector */
	double gn[3]; /* tangent to gradient */
	int t1, t2, t3, t4; /* index of adjoing tetra (if any) */
	int ifc[4];  /* interface of faces (if any) */
	int index; /* index of itself */
	int marked; /* a way of marking this tetra */
	int tested; /* another way of marking it */
	};
int n_tetras=0;
int MAX_TETRAS=0;
struct tetra_rec *tetra;

int last_tetra=(-1);  /* used in find_tetra routine */
int find_tetra_hit=0;
int find_tetra_neighbor=0;
int find_tetra_search=0;
int find_tetra_walk=0;
int find_tetra_nofind=0;

struct int_rec {
	double psi, x[3];     /* intersection angle & corresponding point */
	double xme[3];        /* slighty different inside point */
	double dpsi; /* angle with respect to entry angle */
	int face, inside;
	double epsilon, R, val;
	};

struct arc_rec {
	double x0[3], t0[3], x1[3], t1[3];  /*entry and exit location, tangent */
					    /* in untransformed coordinates    */
	double xc[3];                       /* center of circle in untransformed coords */
	double shift[3], s[3][3], st[3][3]; /* shift and rotation */
	double R, psi1, psi2; /* center, entry and exit angles in */
			      /* transformed coordinates (center at origin) */
	int tetra, exit_face; /* id of tetrahedra, exit face */
	double vc[4]; /* velocity coefficients in transformed coordinates */
	double qinvc[4]; /* qinv coefficients in transformed coordinates */
	double T, S, qinvint; /* traveltime, arclength, (integral) qinv dT */
	struct tetra_rec *ptetra; /* pointer back to tetra */
	};

struct ray_rec {
	double th, az;
	double x, y, z, T;
	int ok;
	};
struct ray_rec *ray;
int m_rays, n_rays;

struct turf_rec {
	double x0, y0;
	double x, y, z, T;
	int ok;
	};
struct turf_rec *turf;
int m_turf, n_turf;

struct tt_rec {
	double th, az;
	double T, gs;
	double dTdx[3];
	};
#define MAX_TTS (10000)
struct tt_rec tt[MAX_TTS];
int n_tts;

struct triangle_rec {
	double x0,y0;
	double x1,y1;
	double x2,y2;
	};

struct ttime_rec {
	int ok;
	double x0[3];
	double x1[3];
	int interface;
	double obsTa, preTa, obsTT, preTT, T0, ratio, qinv, preqinv;
	int obsorder, preorder;
	double th, az;
	double vapp;
	int index, pwg;
	};
int MAX_TTIMES;
struct ttime_rec *ttime;
int n_ttimes, n_pwg;

struct model_par_rec {
	int n;      /* number of nodes */
	int *node;  /* nodelist */
	double *weight; /* weight relative to other nodes in the group */
	double *t; /* tangent to directional perturbation */
	};
int MAX_MODEL_PARS;
struct model_par_rec *model_par;
int n_model_pars;
int model_L, model_M, model_N;

int calls_to_shoot;
double model_z_min, model_z_max;
double PI, DTOR, RTOD;

int compare_int();
int compare_tt();
int compare_nodeid();
double eval_velocity();
double eval_velocity2();
double eval_arc_velocity();
double eval_qinv();
double eval_qinv2();
double eval_arc_qinv();
double dot();
double eval_plane();
double eval_plane2();
double ttintegralnew();
double ttintegralold();
double myrand();
double eval_line();
double tetra_volume();
double det3x3();
double dTdmpnew();
double dTdmpold();
double dTsdmpp();
double dTsdmpr();
double dqinvdmp();
double dTdmpr();
double dTdmprd();
double great_circle_azimuth();
double solid_angle();
double area_of_triagle();
double qintegral();
double integral292();
double integral292a();
double integral294();
double integral295();
double integral294a();
double inv_integral294a();
double integral295a();
double integral340();
double integral340();
double integral341();
double integral342();
int x_tetrahedron( double *, double *, double *, double *,
		   double *, double *, double *, double * );
int n_tetra_stack, max_tetra_stack, *tetra_stack;

double refine_star_norm;
int refine_star_status;

#define ttintegral ttintegralnew
#define dTdmp dTdmpnew

long starttime;

main(argc, argv )
int argc;
char *argv[];
{
	calls_to_shoot=0;
	PI = 4.0 * atan((double)1.0);
	DTOR = PI/180.0;
	RTOD = 180.0/PI;
	srandom(time(NULL));
	ray=NULL;
	turf=NULL;
	time(&starttime);

	if( argc<2 ) {
		fprintf(stderr,"usage: %s keyword arg1 arg2 ...\n", argv[0] );
		fprintf(stderr,"valid keywords:\n");
		fprintf(stderr,"\tdump_model\n");
		fprintf(stderr,"\tgrid\n");
		fprintf(stderr,"\tinterface_depth\n");
		fprintf(stderr,"\tIinvert\n");
		fprintf(stderr,"\tindex_traveltimes\n");
		fprintf(stderr,"\tinvert\n");
		fprintf(stderr,"\tQinvert\n");
		fprintf(stderr,"\tjhd_table\n");
		fprintf(stderr,"\tlocate\n");
		fprintf(stderr,"\tlocate_and_invert\n");
		fprintf(stderr,"\tmake_model\n");
		fprintf(stderr,"\tmodel_parameters\n");
		fprintf(stderr,"\tperturb_model\n");
		fprintf(stderr,"\tpoint point_list\n");
		fprintf(stderr,"\tPSinvert\n");
		fprintf(stderr,"\troughen_model\n");
		fprintf(stderr,"\tshoot_arc\n");
		fprintf(stderr,"\tshoot_arc_refl\n");
		fprintf(stderr,"\tshoot_star\n");
		fprintf(stderr,"\tshoot_star_refl shoot_star_p shoot_turf\n");
		fprintf(stderr,"\tsource_to_receivers\n");
		fprintf(stderr,"\tsource_to_receivers_refl\n");
		fprintf(stderr,"\tteleseismic_invert\n");
		fprintf(stderr,"teleseism_to_receivers\n");
		fprintf(stderr,"\ttt_surface\n");
		fprintf(stderr,"\ttt_table\n");
		fprintf(stderr,"\tTV_invert\n");
		fprintf(stderr,"\tvelocity\n");
		fprintf(stderr,"\tVinvert\n");
		fprintf(stderr,"\tVIinvert\n");
                write_info_blerb(stderr);
                write_solution_blerb(stderr);
		exit(-1);
		}
	if( !strcmp(argv[1],"shoot_arc") ) 
		main_shootarc(argc-1, argv+1 );
	if( !strcmp(argv[1],"shoot_arc_refl") ) 
		main_shootarcrefl(argc-1, argv+1 );
	else if( !strcmp(argv[1],"shoot_star") ) 
		main_shootstar(argc-1, argv+1 );
	else if( !strcmp(argv[1],"shoot_star_refl") ) 
		main_shootstarrefl(argc-1, argv+1 );
	else if( !strcmp(argv[1],"shoot_star_p") ) 
		main_shootstarp(argc-1, argv+1 );
	else if( !strcmp(argv[1],"point") ) 
		main_point(argc-1, argv+1 );
	else if( !strcmp(argv[1],"point_list") ) 
		main_point_list(argc-1, argv+1 );
	else if( !strcmp(argv[1],"velocity") ) 
		main_velocity(argc-1, argv+1 );
	else if( !strcmp(argv[1],"grid") ) 
		main_grid(argc-1, argv+1 );
	else if( !strcmp(argv[1],"source_to_receivers") ) 
		main_stor(argc-1, argv+1 );
	else if( !strcmp(argv[1],"source_to_receivers_refl") ) 
		main_storrefl(argc-1, argv+1 );
	else if( !strcmp(argv[1],"interface_depth") )
		main_interface_depth(argc-1, argv+1 );
	else if( !strcmp(argv[1],"interface_grid") )
		main_interface_grid(argc-1, argv+1 );
	else if( !strcmp(argv[1],"locate") )
		main_locate(argc-1, argv+1 );
	else if( (!strcmp(argv[1],"invert")) || (!strcmp(argv[1],"Vinvert")) )
		main_invert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"Iinvert") )
		main_Iinvert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"VIinvert") )
		main_VIinvert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"Qinvert") )
		main_qinvert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"PSinvert") )
		main_psinvert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"shoot_turf") ) 
		main_shootturf(argc-1, argv+1 );
	else if( !strcmp(argv[1],"teleseism_to_receivers") ) 
		main_ttor(argc-1, argv+1 );
	else if( (!strcmp(argv[1],"teleseismic_invert")) || (!strcmp(argv[1],"T_invert")) ) 
		main_invert_tele(argc-1, argv+1 );
	else if( !strcmp(argv[1],"TV_invert") ) 
		main_TVinvert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"dump_model") ) 
		main_dump_model(argc-1, argv+1 );
	else if( !strcmp(argv[1],"roughen_model") ) 
		main_roughen_model(argc-1, argv+1 );
	else if( !strcmp(argv[1],"perturb_model") ) 
		main_perturb_model(argc-1, argv+1 );
	else if( !strcmp(argv[1],"tt_table") ) 
		main_tt_table(argc-1, argv+1 );
	else if( !strcmp(argv[1],"tt_surface") ) 
		main_tt_surface(argc-1, argv+1 );
	else if( !strcmp(argv[1],"jhd_table") ) 
		main_jhd_table(argc-1, argv+1 );
	else if( !strcmp(argv[1],"jhd_locate") ) 
		main_jhd_locate(argc-1, argv+1 );
	else if( !strcmp(argv[1],"fit_model") ) 
		main_fitmodel(argc-1, argv+1 );
	else if( !strcmp(argv[1],"model_parameters") ) 
		main_modelpars(argc-1, argv+1 );
	else if( !strcmp(argv[1],"make_model") ) 
		main_make_model(argc-1, argv+1 );
	else if( !strcmp(argv[1],"index_traveltimes") ) 
		main_index_traveltimes(argc-1, argv+1 );
	else if( !strcmp(argv[1],"locate_and_invert") ) 
		main_locate_and_invert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"locate_and_PS_invert") ) 
		main_locate_and_PS_invert(argc-1, argv+1 );
	else if( !strcmp(argv[1],"test") ) 
		main_test(argc-1, argv+1 );
	else {
		fprintf(stderr,"error: %s: bad keyword\n", argv[0] );
		exit(-1);
		}
	exit(0);
	}

main_test(argc, argv)
int argc;
char *argv[];
{
	double x[3];
	int i, t0, t1, t2, t3, t, tt;

	fprintf(stderr,"test not currently implemented\n");

	exit(0);
	}

main_index_traveltimes(argc, argv)
int argc;
char *argv[];
{
	FILE *f;
	double x0[3], stheta, ftheta, dtheta, sazi, fazi, dazi, dT;
	int n_thetas, n_azis, iwavemin=1, iwavemax;
	int i, j, k, l, m, count, iwave, t, tt;
	struct jhd_table_rec {
		double azi, theta;
		float *X, *Y, *Z;
		int iwave, *tlist, nt, hit;
		};
	struct jhd_table_rec *jhd;
	struct index_rec {
		int j, k, l, i;
		struct index_rec *next;
		};
	struct index_rec **first, **last, *my_index;
	int *icount;
	int buf[4];

	int r1, r2, r3;
	double b1[3], b2[3], b3[3], t1[3], t2[3], t3[3], T, dTdx[3], Tb, Tt, Tmin, Tmax;

	if( argc != 4 ) {
		fprintf(stderr,"purpose: produce a index to speed up traveltime calculations\n");
		fprintf(stderr,"usage: index_traveltimes model_file tt_table_file tt_index\n");
		fprintf(stderr,"       model_file: standard model file\n");
		fprintf(stderr,"       tt_table_file: travetime table produced by tt_table\n");
		fprintf(stderr,"       tt_index_file: index file that is created\n");
		exit(-1);
		}

	if( !init_tetra_stack() ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	/* allocate index */
	first = (struct index_rec **) malloc( n_tetras * sizeof(struct index_rec *) );
	last =  (struct index_rec **) malloc( n_tetras * sizeof(struct index_rec *) );
	icount =  (int *) malloc( n_tetras * sizeof(int) );

	if( (first==NULL) || (last==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
		exit(-1);
		}
	for( i=0; i<n_tetras; i++ ) { first[i]=NULL; last[i]=NULL; icount[i]=0;}


	/* read in tt_table */
	if( (f=fopen(argv[2],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open tt_table file <%s> for read\n",
			argv[0], argv[2] );
		exit(-1);
		}

	count=0;
	count+=fread( x0, sizeof(double), 3, f );

	count+=fread( &stheta, sizeof(double), 1, f );
	count+=fread( &ftheta, sizeof(double), 1, f );
	count+=fread( &dtheta, sizeof(double), 1, f );
	count+=fread( &n_thetas, sizeof(int), 1, f );

	count+=fread( &sazi, sizeof(double), 1, f );
	count+=fread( &fazi, sizeof(double), 1, f );
	count+=fread( &dazi, sizeof(double), 1, f );
	count+=fread( &n_azis, sizeof(int), 1, f );

	count+=fread( &dT, sizeof(double), 1, f );

	if( count!=12 ) {
		fprintf(stderr,"error: %s: cant read tt_table file (1)\n", argv[0]);
		exit(-1);
		}

	jhd = (struct jhd_table_rec *) malloc( n_thetas * n_azis * sizeof(struct jhd_table_rec) );
	if( jhd==NULL ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		jhd[j+k*n_azis].iwave=0; jhd[j+k*n_azis].nt=0; jhd[j+k*n_azis].tlist=NULL;
		jhd[j+k*n_azis].X=NULL; jhd[j+k*n_azis].Y=NULL; jhd[j+k*n_azis].Z=NULL;
		}

	while(1) {
		count=0;
		count+=fread( &j, sizeof(int), 1, f );
		count+=fread( &k, sizeof(int), 1, f );
		count+=fread( &iwave, sizeof(int), 1, f );
		if( count == 0 ) break;
		if( iwave == 0 ) continue;
		if( count != 3 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (2)\n", argv[0] );
			exit(-1);
			}
		jhd[j+k*n_azis].azi=sazi+dazi*(double)j;
		jhd[j+k*n_azis].theta=stheta+dtheta*(double)k;
		jhd[j+k*n_azis].iwave=iwave;

		jhd[j+k*n_azis].X = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Y = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Z = (float *) malloc( sizeof(float)*iwave );

		if((jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)){
			fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( jhd[j+k*n_azis].X, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Y, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Z, sizeof(float), iwave, f );
		if( count != (3*iwave) ) {
			fprintf(stderr,"error: %s: cant read tt_table file (3)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( &(jhd[j+k*n_azis].nt), sizeof(int), 1, f );
		if( count != 1 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (4)\n", argv[0] );
			exit(-1);
			}

		jhd[j+k*n_azis].tlist = (int *) malloc( sizeof(int)*jhd[j+k*n_azis].nt );
		if( jhd[j+k*n_azis].tlist == NULL ) {
			fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
			exit(-1);
			}
		count=0;
		count+=fread( jhd[j+k*n_azis].tlist, sizeof(int), jhd[j+k*n_azis].nt, f );
		if( count != jhd[j+k*n_azis].nt ) {
			fprintf(stderr,"error: %s: cant read tt_table file (5)\n", argv[0] );
			exit(-1);
			}
		}
	fclose(f);

fprintf(stderr,"index read in\n");

	/* write out index */
	if( (f=fopen(argv[3],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open tt_index file <%s> for write\n",
			argv[0], argv[3] );
		exit(-1);
		}

	/* loop over all triangular-cross-section ray tubes */
	for( j=0; j<(n_azis-1); j++ ) {
	for( k=0; k<(n_thetas-1); k++ ) {
printf("%d %d\n", j, k);
	for( l=0; l<2; l++ ) {
		if( l==0 ) {
			r1 = j+k*n_azis;
			r2 = (j+1)+k*n_azis;
			r3 = j+(k+1)*n_azis;
			}
		else {
			r1 = (j+1)+k*n_azis;
			r2 = (j+1)+(k+1)*n_azis;
			r3 = j+(k+1)*n_azis;
			}

		if( (jhd[r1].iwave==0) || (jhd[r2].iwave==0) || (jhd[r3].iwave==0) ) {
			continue;
			}

		iwave = jhd[r1].iwave;
		if( jhd[r2].iwave < iwave ) iwave=jhd[r2].iwave;
		if( jhd[r3].iwave < iwave ) iwave=jhd[r3].iwave;

		/* loop over all triangular prisms in ray tube */
		iwavemin=1;
		for( i=iwavemin; i<(iwave-1); i++ ) {

			b1[0]=jhd[r1].X[i]; b1[1]=jhd[r1].Y[i]; b1[2]=jhd[r1].Z[i];
			b2[0]=jhd[r2].X[i]; b2[1]=jhd[r2].Y[i]; b2[2]=jhd[r2].Z[i];
			b3[0]=jhd[r3].X[i]; b3[1]=jhd[r3].Y[i]; b3[2]=jhd[r3].Z[i];

			t1[0]=jhd[r1].X[i+1]; t1[1]=jhd[r1].Y[i+1]; t1[2]=jhd[r1].Z[i+1];
			t2[0]=jhd[r2].X[i+1]; t2[1]=jhd[r2].Y[i+1]; t2[2]=jhd[r2].Z[i+1];
			t3[0]=jhd[r3].X[i+1]; t3[1]=jhd[r3].Y[i+1]; t3[2]=jhd[r3].Z[i+1];

			mark_tri_prism( b1,b2,b3, t1,t2,t3 );
			count=0;
			for( tt=0; tt<n_tetras; tt++ ) if( tetra[tt].marked ) {
				my_index=(struct index_rec *)malloc(sizeof(struct index_rec));
				if( my_index==NULL ) {
					fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
					exit(-1);
					}
				my_index->j=j; my_index->k=k; my_index->l=l; my_index->i=i; 
				my_index->next=NULL;
				icount[tt]++;
				if( first[tt]==NULL ) {
					first[tt]=my_index;
					last[tt]=my_index;
					}
				else {
					last[tt]->next=my_index;
					last[tt]=my_index;
					}
				count++;
				}
			}
		}}}

	/* write out index */
	for( i=0; i<n_tetras; i++ ) if( icount[i]>0 ) {
		buf[0]=i; buf[1]=icount[i];
		fwrite( buf, sizeof(int), 2, f );
		my_index=first[i];
		while( my_index!=NULL ) {
			buf[0]=my_index->j; buf[1]=my_index->k;
			buf[2]=my_index->l; buf[3]=my_index->i;
			fwrite( buf, sizeof(int), 4, f );
			my_index = my_index->next;
			}
		}

	destroy_tetra_stack();

	exit(0);
	}

int mark_tri_prism( b1,b2,b3, t1,t2,t3 )
double b1[3], b2[3], b3[3];
double t1[3], t2[3], t3[3];
{
	double *v1, *v2, *v3, *v4;
	int i, it;

	for( i=0; i<n_tetras; i++ ) { tetra[i].marked=0;  tetra[i].tested=0; }

	for( it=0; it<3; it++ ) {
		if( it==0 ) {     v1=b1; v2=b2; v3=b3; v4=t1;}
		else if( it==1) { v1=b2; v2=b3; v3=t1; v4=t2; }
		else            { v1=b3; v2=t1; v3=t2; v4=t3; }
		mark_tetra_intersection2( v1, v2, v3, v4 );
		}

	return(0);
	}


/* marks model tetrahedra that overlap an arbitrary tetrahedron with
   vertices (x1, ... x4) by setting their marked field.  Note that this
   routine does not first clear the marked fields. */
int mark_tetra_intersection( double *x1, double *x2, double *x3, double *x4 ) {

#define MAX_GEN (5)
#define MAX_COUNT (500000)

	int i, n, tmean, count, tcount;
	double xmean[3];
	struct stack_rec {
		struct tetra_rec tetra;
		double x1[3], x2[3], x3[3], x4[3];
		int t1, t2, t3, t4;
		int n, generation;
		struct stack_rec *previous;
		};
	struct stack_rec *top=NULL, *t, *tnew;

	/* the idea:
		Mark the model tetrahedra in which the vertices of
		target tetrahedron lie.  Then, if all 4 of the vertices
		of the target tetrahedron are within one tetrahedron,
		then your're done, since the model tetrahedron is entirely
		within it. Otherwise, subdivide the target tetrahedron into
		four smaller tetrahedra, and repeat the process.  The
		work is conducted on a stack.                       */

	/* set up stack */
	tcount=0;
	t = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
	if( t==NULL ) {
		fprintf(stderr,"error: mark_tetra_intersection: out of memory (1)\n");
		exit(-1);
		}
	for( i=0; i<3; i++ ) { t->x1[i]=x1[i]; t->x2[i]=x2[i]; t->x3[i]=x3[i]; t->x4[i]=x4[i]; }
	t->tetra.x1 = t->x1; t->tetra.x2 = t->x2; t->tetra.x3 = t->x3; t->tetra.x4 = t->x4;
	if( !find_tetra( x1, &t->t1 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x1 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x2, &t->t2 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x2 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x3, &t->t3 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x3 not in model (1)\n"); exit(-1); }
	if( !find_tetra( x4, &t->t4 ) ) { fprintf(stderr,"error: mark_tetra_intersection: x4 not in model (1)\n"); exit(-1); }
	tetra[t->t1].marked=1; tetra[t->t2].marked=1; tetra[t->t3].marked=1; tetra[t->t4].marked=1;
	t->previous = NULL; t->n=tcount++; t->generation=0;
	top=t;
	
	count=0;
	while( top != NULL ) {
		count++; if(count>MAX_COUNT) {
			fprintf(stderr,"error: mark_tetra_intersection: stack overflow\n");
			exit(-1);
			}
		
		t=top; top=t->previous;

		if( ((t->t1==t->t2) && (t->t1==t->t3) && (t->t1==t->t4)) || (t->generation>MAX_GEN) ) {
			free(t);
			continue;
			}
	
		for( i=0; i<3; i++ ) xmean[i]=(t->x1[i]+t->x2[i]+t->x3[i]+t->x4[i])/4.0;
		if( !find_tetra( xmean, &tmean ) ) {
			fprintf(stderr,"error: mark_tetra_intersection: xmean not in model (2)\n");
			fprintf(stderr,"       x = (%f %f %f)\n", xmean[0], xmean[1], xmean[2]);

			exit(-1);
			}
		tetra[tmean].marked=1;

		/* first tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=xmean[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=tmean; tnew->t2=t->t2; tnew->t3=t->t3; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* second tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=xmean[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=t->t1; tnew->t2=tmean; tnew->t3=t->t3; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* third tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=xmean[i]; tnew->x4[i]=t->x4[i]; }
		tnew->t1=t->t1; tnew->t2=t->t2; tnew->t3=tmean; tnew->t4=t->t4;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		/* fourth tetra */
		tnew = (struct stack_rec *) malloc( sizeof(struct stack_rec) );
		if( tnew==NULL ) {
			fprintf(stderr,"error: mark_tetra_intersection: out of memory (2)\n");
			exit(-1);
			}
		tnew->tetra.x1=tnew->x1; tnew->tetra.x2=tnew->x2; tnew->tetra.x3=tnew->x3; tnew->tetra.x4=tnew->x4;
		for( i=0; i<3; i++ ) { tnew->x1[i]=t->x1[i]; tnew->x2[i]=t->x2[i]; tnew->x3[i]=t->x3[i]; tnew->x4[i]=xmean[i]; }
		tnew->t1=t->t1; tnew->t2=t->t2; tnew->t3=t->t3; tnew->t4=tmean;
		tnew->n=tcount++; tnew->generation=t->generation+1;
		tnew->previous = top;
		top=tnew;
		
		free(t);
		
		}
				
	return(1);
	}

/* marks model tetrahedra that overlap an arbitrary tetrahedron with
   vertices (x1, ... x4) by setting their marked field.  Note that this
   routine does not first clear the marked fields. */
int mark_tetra_intersection2( double *x1, double *x2, double *x3, double *x4 ) {
	int i, t, ta[4], intersect;
	if( !zero_tetra_stack() ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 1)\n");
		exit(-1);
		}
	if( find_tetra(x1,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x2,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x3,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (init_tetra_stack, 2)\n");
		exit(-1);
		}
	if( find_tetra(x4,&t) ) if( !push_tetra(t) ) {
		fprintf(stderr,"error: out of memory (push_tetra, 2)\n");
		exit(-1);
		}
	while( (t=pop_tetra()) != (-1) ) {
		tetra[t].tested = 1;
		intersect=x_tetrahedron( tetra[t].x1, tetra[t].x2, tetra[t].x3, tetra[t].x4,
			       x1, x2, x3, x4 );
		if( intersect ) {
			tetra[t].marked = 1;
			ta[0]=tetra[t].t1; ta[1]=tetra[t].t2; ta[2]=tetra[t].t3; ta[3]=tetra[t].t4;
			for( i=0; i<4; i++ ) {
				if( !push_tetra(ta[i]) ) {
					fprintf(stderr,"error: out of memory (init_tetra_stack, 3)\n");
					exit(-1);
					}
				}
			}
		}
				
	return(1);
	}


main_dump_model(argc, argv)
int argc;
char *argv[];
{
	int i;

	if( argc!=3 ) {
		fprintf(stderr,"purpose: writes out model tetrahedralization\n");
		fprintf(stderr,"usage: %s model_file check_flag > output.txt\n",
			argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      output.txt:\n");
		fprintf(stderr,"         number of nodes\n");
		fprintf(stderr,"         index, x, y, z in km, v in km/s, interface of each node\n");
		fprintf(stderr,"         number of tetrahedra\n");
		fprintf(stderr,"         index, node 1, 2, 3, 4, tetrahedron adjoining face 1, 2, 3, 4, interface of face 1, 2, 3, 4, volume in km**3\n");
		fprintf(stderr,"           -1 indicates no adjoining tetrahedron on a face\n");
		fprintf(stderr,"           and no interface of a face\n");
		fprintf(stderr,"      check_flag: checks model if Y or y, doesnt if N or n\n",
			argv[0]);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	printf("%d\n", n_nodes);

	for( i=0; i<n_nodes; i++ ) {
		if( QDEFINED ) printf("%d\t%f\t%f\t%f\t%f\t%f\t%d\n", i,
		node[i].x[0], node[i].x[1], node[i].x[2], node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else printf("%d\t%f\t%f\t%f\t%f\t%d\n", i,
		node[i].x[0], node[i].x[1], node[i].x[2], node[i].v, node[i].ifc );
		}

	printf("%d\n", n_tetras);

	for( i=0; i<n_tetras; i++ ) printf("%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%e\n", i,
		tetra[i].n1, tetra[i].n2, tetra[i].n3, tetra[i].n4,
		tetra[i].t1, tetra[i].t2, tetra[i].t3, tetra[i].t4,
		tetra[i].ifc[0], tetra[i].ifc[1], tetra[i].ifc[2], tetra[i].ifc[3],
		tetra_volume(i) );

	if( argv[2][0]=='y' || argv[2][0]=='Y' ) {
		i=check_tetras(stderr);
		if(i) {
			printf("Connectivity of this model is good\n");
			}
		else {
			printf("Connectivity of this model is bad\n");
			}
		}
	else {
		printf("Connectivity of this model not checked\n");
		}

	exit(0);
	}

main_perturb_model(argc, argv)
int argc;
char *argv[];
{
	int i, j, found, nblobs, ninterfaces, k, nx, ny, nz;
	FILE *fblobs, *finterfaces;
#define NBLOBS      (1000)
	double  x[NBLOBS],  y[NBLOBS],  z[NBLOBS];
	double sx[NBLOBS], sy[NBLOBS], sz[NBLOBS];
	double dv[NBLOBS], dq[NBLOBS];
#define NINTERFACES (100)
	int interface[NINTERFACES];
	double r2, e;

	if( argc!=4 ) {
		fprintf(stderr,"purpose: perturbs a model with gaussian blobs\n");
		fprintf(stderr,"usage: %s model_file blobs_file.txt interfaces_file.txt > new_model_file\n",
			argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      blobs_file.txt:\n");
		fprintf(stderr,"         number of blobs\n");
		fprintf(stderr,"         x, y, z in km, sx, sy, sz in km, dv in km/s, dQ\n");
		fprintf(stderr,"      each blob adds a perturbation of\n");
		fprintf(stderr,"      dv*exp(-r^2) and (1.0/dQ)*exp(-r^2)\n");
		fprintf(stderr,"         where r^2 = ((x-xnode)/sx)^2+((y-ynode)/sy)^2+(z-znode)/sz)^2]\n");
		fprintf(stderr,"       to the velocity and 1/Q of the nodes\n");
		fprintf(stderr,"       Q perturbations are ignored in models lacking Q\n");
		fprintf(stderr,"      interfaces_file.txt:\n");
		fprintf(stderr,"         number of interfaces\n");
		fprintf(stderr,"         interface_number\n");
		fprintf(stderr,"         interface_number\n");
		fprintf(stderr,"      perturbations are only applied to nodes with interface\n");
		fprintf(stderr,"      numbers specified in this file\n");
		fprintf(stderr,"      output new_model_file has same format as model file\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( (fblobs=fopen(argv[2],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open blobs file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}

	if( fscanf(fblobs,"%d", &nblobs ) !=1 ) {
		fprintf(stderr,"error: %s cant read nblobs from blobs file <%s>\n",
			argv[0], argv[2] );
		exit(-1);
		}

	if( nblobs>NBLOBS ) {
		fprintf(stderr,"error: %s too many blobs\n", argv[0]);
		exit(-1);
		}

	for( i=0; i<nblobs; i++ ) {
		if( fscanf(fblobs,"%le%le%le%le%le%le%le%le\n", &x[i], &y[i], &z[i],
			&sx[i], &sy[i], &sz[i], &dv[i], &dq[i] ) != 8 ) {
			fprintf(stderr,"error: %s cant read blob %d from blobs file <%s>\n",
				argv[0], i, argv[2] );
			exit(-1);
			}
		}

	fclose(fblobs);

	if( (finterfaces=fopen(argv[3],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open interfaces file <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}

	if( fscanf(finterfaces,"%d", &ninterfaces ) !=1 ) {
		fprintf(stderr,"error: %s cant read ninterfaces from interfaces file <%s>\n",
			argv[0], argv[3] );
		exit(-1);
		}

	if( ninterfaces>NINTERFACES ) {
		fprintf(stderr,"error: %s too many interfaces\n");
		exit(-1);
		}

	for( i=0; i<ninterfaces; i++ ) {
		if( fscanf(finterfaces,"%d\n", &interface[i] ) != 1 ) {
			fprintf(stderr,"error: %s cant read interface %d from interfaces file <%s>\n",
				argv[0], i, argv[3] );
			exit(-1);
			}
		}

	fclose(finterfaces);
	
	for( i=0; i<n_nodes; i++ ) {
		found=0;
		for( j=0; j<ninterfaces; j++ ) if( node[i].ifc==interface[j] ) {
			found=1;
			break;
			}
		if( found ) {
			for( k=0; k<nblobs; k++ ) {
				r2 = ((node[i].x[0]-x[k])/sx[k]) * ((node[i].x[0]-x[k])/sx[k])
			 	  +  ((node[i].x[1]-y[k])/sy[k]) * ((node[i].x[1]-y[k])/sy[k])
			  	  +  ((node[i].x[2]-z[k])/sz[k]) * ((node[i].x[2]-z[k])/sz[k]);
				e = exp(-r2);
				node[i].v += dv[k]*e;
				if( QDEFINED ) node[i].qinv += (1.0/dq[k])*e;
				}
			}
		}

	/* write out perturbed model */
	printf("%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		printf("%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		printf("%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	exit(0);
	}

main_roughen_model(argc, argv)
int argc;
char *argv[];
{

	int i, j, k, nx, ny, nz;

	if( argc!=6 ) {
		fprintf(stderr,"purpose: roughens a model with random perturbations\n");
		fprintf(stderr,"         to node positions and velocities.  Idealized\n");
		fprintf(stderr,"         models should be roughened slightly (say\n");
		fprintf(stderr,"         by a 0.001 km in node position and 0.001 km/s\n");
		fprintf(stderr,"         in velocity) to break symmetries that can\n");
		fprintf(stderr,"         cause the raytracing to fail.\n");
		fprintf(stderr,"usage: %s model_file dx dy dz dv > new_model_file\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"         dx, dy, dz in km, dv in km/s\n");
		fprintf(stderr,"      output new_model_file has same format as model_file\n");
		exit(-1);
		}

	if( sscanf(argv[2],"%le", &node_x_perturbation ) !=1 ) {
		fprintf(stderr,"error: %s cant read node_x_perturbation from cmd line <%s>\n",
			argv[0], argv[2] );
		exit(-1);
		}

	if( sscanf(argv[3],"%le", &node_y_perturbation ) !=1 ) {
		fprintf(stderr,"error: %s cant read node_y_perturbation from cmd line <%s>\n",
			argv[0], argv[3] );
		exit(-1);
		}

	if( sscanf(argv[4],"%le", &node_z_perturbation ) !=1 ) {
		fprintf(stderr,"error: %s cant read node_z_perturbation from cmd line <%s>\n",
			argv[0], argv[4] );
		exit(-1);
		}

	if( sscanf(argv[5],"%le", &node_v_perturbation ) !=1 ) {
		fprintf(stderr,"error: %s cant read node_v_perturbation from cmd line <%s>\n",
			argv[0], argv[5] );
		exit(-1);
		}

	PERTURBMODEL=1;

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	/* write out roughewned model */
	printf("%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		printf("%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		printf("%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	exit(0);
	}


main_interface_depth(argc, argv)
int argc;
char *argv[];
{

	double x_interface, y_interface, tangent[3], ntangent[3], xi[3], normal[3], x[3], xs[3], s;
	double epsilon, v, qinv;
	int interface, i, j, t, face, tt;
	struct tetra_rec *p;

	epsilon=1.0e-8;

	if( argc!=5 ) {
		fprintf(stderr,"purpose: interrogates the model to find the depth of an interface\n");

		fprintf(stderr,"usage: %s model_file interface x y > output.txt\n",
			argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      interface ... integer matching an interface in the model\n");
		fprintf(stderr,"      x, y ... units in km\n");
		fprintf(stderr,"      output.txt: x_interface, y_interface, z_interface in km, velocity in km/s\n");
		fprintf(stderr,"      or, if Q defined in model\n");
		fprintf(stderr,"      output.txt: x_interface, y_interface, z_interface in km, velocity in km/s, Q\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x_interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read x from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &y_interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read y from command line\n",
			argv[0] );
		exit(-1);
		}

	tangent[0]=0.0; tangent[1]=0.0; tangent[2]=1.0;
	for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
	xs[0]=x_interface;
	xs[1]=y_interface;
	i=0;
	do {
		xs[2]=model_z_min+(model_z_max-model_z_min)*myrand();
		if( i>100 ) {
			fprintf(stderr,"error: cant find a valid starting depth\n");
			exit(-1);
			}
		i++;
		} while( !find_tetra(xs,&t) );
	i=0;
	for( j=0; j<3; j++ ) x[j]=xs[j];
	find_tetra(x,&t);
	while( t!=(-1) ) {
		p = &tetra[t];
		if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
			p->x4, x, tangent, epsilon/2.0, normal,
			xi, &s, &face ) ) {
				break;
				}
		if( p->ifc[face]==interface ) {
			v = eval_velocity(p,xi);
			if( QDEFINED ) {
				qinv = eval_qinv(p,xi);
				printf("%f\t%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v, 1.0/qinv);
				}
			else printf("%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v);
			exit(0);
			}
		for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;

		if( face == 0 )      t=p->t1;
		else if( face == 1 ) t=p->t2;
		else if( face == 2 ) t=p->t3;
		else if( face == 3 ) t=p->t4;
		else t=(-1);

		if( i>10000 ) {
			fprintf(stderr,"error: too many tetras (1)!\n");
			exit(-1);
			}
		i++;
		}
	i=0;
	for( j=0; j<3; j++ ) x[j]=xs[j];
	find_tetra(x,&t);
	while( t!=(-1) ) {
		p = &tetra[t];
		if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
			p->x4, x, ntangent, epsilon/2.0, normal,
			xi, &s, &face ) ) {
				break;
				}
		if( p->ifc[face]==interface ) {
			v = eval_velocity(p,xi);
			if( QDEFINED ) {
				qinv = eval_qinv(p,xi);
				printf("%f\t%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v, 1.0/qinv);
				}
			else printf("%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v);
			exit(0);
			}
		for( j=0; j<3; j++ ) x[j]=xi[j]+ntangent[j]*epsilon;

		if( face == 0 )      t=p->t1;
		else if( face == 1 ) t=p->t2;
		else if( face == 2 ) t=p->t3;
		else if( face == 3 ) t=p->t4;
		else t=(-1);

		if( i>10000 ) {
			fprintf(stderr,"error: too many tetras (2)!\n");
			exit(-1);
			}
		i++;
		}
	fprintf(stderr,"error: cant find interface %d at (%f, %f)\n", interface,
		x_interface, y_interface );

	exit(-1);
	}

main_interface_grid(argc, argv)
int argc;
char *argv[];
{

	double x_interface, y_interface, z_interface;
	double tangent[3], ntangent[3], xi[3], normal[3], x[3], xs[3], s;
	double epsilon, v;
	int interface, i, j, t, face, tt;
	struct tetra_rec *p;
	double xmin, xmax, ymin, ymax, qinv;
	int nx, ny, ix, iy, found, t_save;

	epsilon=1.0e-8;

	if( argc!=9 ) {
		fprintf(stderr,"purpose: interrogates the model to find the depth of an interface\n");

		fprintf(stderr,"usage: %s model_file interface xmin xmax nx ymin ymax ny > output.txt\n",
			argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      interface .... integer matching an interface in the model\n");
		fprintf(stderr,"      xmin, xmax, nx minimum and maximum x, in km, and number of points to sample, an integer\n");
		fprintf(stderr,"      ymin, ymax, ny minimum and maximum y, in km, and number of points to sample, an integer\n");
		fprintf(stderr,"      output.txt: x_interface, y_interface, z_interface in km, velocity in km/s\n");
		fprintf(stderr,"      or, if Q defined in model\n");
		fprintf(stderr,"      output.txt: x_interface, y_interface, z_interface in km, velocity in km/s, Q\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &xmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmin from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &xmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmax from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%d", &nx) != 1 ) {
		fprintf(stderr,"error: %s: cant read nx from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &ymin) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymin from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &ymax) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymax from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &ny) != 1 ) {
		fprintf(stderr,"error: %s: cant read ny from command line\n",
			argv[0] );
		exit(-1);
		}

	printf("x\ty\tz\tv\n");

	z_interface=model_z_min+(model_z_max-model_z_min)/2.0;

	for( ix=0; ix<nx; ix++ ) for( iy=0; iy<ny; iy++ ) {

	found=FALSE;

	x_interface = xmin + (xmax-xmin)*(double)(ix)/((double)(nx-1));
	y_interface = ymin + (ymax-ymin)*(double)(iy)/((double)(ny-1));

	tangent[0]=0.0; tangent[1]=0.0; tangent[2]=1.0;
	for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
	xs[0]=x_interface;
	xs[1]=y_interface;
	xs[2]=z_interface;
	i=0;
	if( !find_tetra(xs,&t) ) {
		do {
			xs[2]=model_z_min+(model_z_max-model_z_min)*myrand();
			if( i>100 ) {
				fprintf(stderr,"error: cant find a valid starting depth (x,y)=(%f, %f)\n", x_interface, y_interface);
				exit(-1);
				}
			i++;
			} while( !find_tetra(xs,&t) );
		}
	t_save=t;
	i=0;
	for( j=0; j<3; j++ ) x[j]=xs[j];
	while( t!=(-1) ) {
		p = &tetra[t];
		if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
			p->x4, x, tangent, epsilon/2.0, normal,
			xi, &s, &face ) ) {
				break;
				}
		if( p->ifc[face]==interface ) {
			v = eval_velocity(p,xi);
			if( QDEFINED ) {
				qinv = eval_qinv(p,xi);
				printf("%f\t%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v, 1.0/qinv);
				}
			else printf("%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v);
			z_interface=xi[2];
			found=TRUE;
			break;
			}
		for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;

		if( face == 0 )      t=p->t1;
		else if( face == 1 ) t=p->t2;
		else if( face == 2 ) t=p->t3;
		else if( face == 3 ) t=p->t4;
		else t=(-1);

		if( i>10000 ) {
			fprintf(stderr,"error: too many tetras (1)!\n");
			exit(-1);
			}
		i++;
		}
	if( found ) continue;
	i=0;
	t=t_save;
	for( j=0; j<3; j++ ) x[j]=xs[j];
	while( t!=(-1) ) {
		p = &tetra[t];
		if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
			p->x4, x, ntangent, epsilon/2.0, normal,
			xi, &s, &face ) ) {
				break;
				}
		if( p->ifc[face]==interface ) {
			v = eval_velocity(p,xi);
			if( QDEFINED ) {
				qinv = eval_qinv(p,xi);
				printf("%f\t%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v, 1.0/qinv );
				}
			else printf("%f\t%f\t%f\t%f\n", xi[0], xi[1], xi[2], v);
			z_interface=xi[2];
			found=TRUE;
			break;
			}
		for( j=0; j<3; j++ ) x[j]=xi[j]+ntangent[j]*epsilon;

		if( face == 0 )      t=p->t1;
		else if( face == 1 ) t=p->t2;
		else if( face == 2 ) t=p->t3;
		else if( face == 3 ) t=p->t4;
		else t=(-1);

		if( i>10000 ) {
			fprintf(stderr,"error: too many tetras (2)!\n");
			exit(-1);
			}
		i++;
		}

	}

	exit(0);
	}

main_grid(argc, argv)
int argc;
char *argv[];
{

	double a, b, c, d, e, s, x[3], tangent[3], ntangent[3], xi[3], normal[3];
	double epsilon;
	int i, j, n, t, face;
	struct tetra_rec *p;
	int save_tetra=(-1);
	epsilon=1.0e-6;

	if( argc!=9 ) {
		fprintf(stderr,"purpose: interrogates the tetrahedral grid along a plane and produces dots along their faces\n");

		fprintf(stderr,"usage: %s model_file xz xmin xmax zmin zmax y n > output.tsv\n",
			argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"usage: %s model_file xy xmin xmax ymin ymax z n > output.tsv\n",
			argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"usage: %s model_file yz ymin ymax zmin zmax x n > output.tsv\n",
			argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      xz xy yz are keywords\n");
		fprintf(stderr,"      xmin, xmax ... units in km\n");
		fprintf(stderr,"      n an integer number of rows\n");
		fprintf(stderr,"      output.tsv:\n");
		fprintf(stderr,"      x y z\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( !(     (!strcmp(argv[2],"xz"))
		|| (!strcmp(argv[2],"xy"))
		|| (!strcmp(argv[2],"yz")) ) ) {
		fprintf(stderr,"error: %s: bad keywords. must be xz xy or yz\n", argv[0]);
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &a) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &b) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &c) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &d) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &e) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &n) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	printf("x\ty\tz\n");

	if( !strcmp(argv[2],"xy") ) {
		/* x=(a,b) y=(c,d) z=e */
		/* lines of fixed yz and variable x */
		tangent[0]=1.0; tangent[1]=0.0; tangent[2]=0.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=a+epsilon;
			x[1]=(c+epsilon)+(d-c)*(double)(i)/(double)(n-1);
			x[2]=e;
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[0]>(a-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[0]>(b+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		/* x=(a,b) y=(c,d) z=e */
		/* lines of fixed xz and variable y */
		tangent[0]=0.0; tangent[1]=1.0; tangent[2]=0.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=(a+epsilon)+(b-a)*(double)(i)/(double)(n-1);
			x[1]=c+epsilon;
			x[2]=e;
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[1]>(c-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[1]>(d+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		}
	else if( !strcmp(argv[2],"xz") ) {
		/* x=(a,b) z=(c,d) y=e */
		/* lines of fixed yz and variable x */
		tangent[0]=1.0; tangent[1]=0.0; tangent[2]=0.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=a+epsilon;
			x[1]=e;
			x[2]=(c+epsilon)+(d-c)*(double)(i)/(double)(n-1);
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[0]>(a-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[0]>(b+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		/* x=(a,b) z=(c,d) y=e */
		/* lines of fixed xy and variable z */
		tangent[0]=0.0; tangent[1]=0.0; tangent[2]=1.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=(a+epsilon)+(b-a)*(double)(i)/(double)(n-1);
			x[1]=e;
			x[2]=c+epsilon;
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[2]>(c-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[2]>(d+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		}
	if( !strcmp(argv[2],"yz") ) {
		/* y=(a,b) z=(c,d) x=e */
		/* lines of fixed xz and variable y */
		tangent[0]=0.0; tangent[1]=1.0; tangent[2]=0.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=e;
			x[1]=a+epsilon;
			x[2]=(c+epsilon)+(d-c)*(double)(i)/(double)(n-1);
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[1]>(a-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[1]>(b+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		/* y=(a,b) z=(c,d) x=e */
		/* lines of fixed xy and variable z */
		tangent[0]=0.0; tangent[1]=0.0; tangent[2]=1.0;
		for( i=0; i<3; i++ ) ntangent[i]=(-tangent[i]);
		for( i=0; i<n; i++ ) {
			x[0]=e;
			x[1]=(a+epsilon)+(b-a)*(double)(i)/(double)(n-1);
			x[2]=c+epsilon;
			last_tetra=save_tetra;
			if( find_tetra(x,&t) ) {
				save_tetra=t;
				p = &tetra[t];
				if( ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, ntangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						if( x[2]>(c-epsilon) ) {
							printf("%f\t%f\t%f\n",
							xi[0], xi[1], xi[2] );
							}
						}
					}
			while( t!=(-1) ) {
				if( x[2]>(d+2.0*epsilon) ) break;
				p = &tetra[t];
				if( !ray_exits_tetrahedron( p->x1, p->x2, p->x3,
					p->x4, x, tangent, epsilon/2.0, normal,
					xi, &s, &face ) ) {
						break;
					}
				t=neighbor(t,face);
				for( j=0; j<3; j++ ) x[j]=xi[j]+tangent[j]*epsilon;
				printf("%f\t%f\t%f\n", x[0], x[1], x[2] );
				}
			}
		}

	exit(0);
	}

main_velocity(argc, argv)
int argc;
char *argv[];
{

	double a, b, c, d, e, x[3], hx1, hx2, hy1, hy2;
	double f1, df, g1, dg, f2, df2, v, epsilon, qinv;
	int i, j, n, m, t;
	int start_t;

	epsilon=1.0e-6;

	if( (argc<3) || ((strcmp(argv[2],"hz"))&&argc!=10) || ((!strcmp(argv[2],"hz"))&&argc!=11) ) {
		fprintf(stderr,"purpose: interrogates the velocity model along a plane\n");

		fprintf(stderr,"usage: %s model_file xz xmin xmax nx zmin zmax nz y > output.tsv\n",
			argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"usage: %s model_file xy xmin xmax nx ymin ymax ny z > output.tsv\n",
			argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"usage: %s model_file yz ymin ymax ny zmin zmax nz x > output.tsv\n",
			argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"usage: %s model_file hz x1 y1 x2 y2 nh zmin zmax nz > output.tsv\n",
			argv[0]);

		write_model_blerb(stderr);
		fprintf(stderr,"      xz xy yz are keywords\n");
		fprintf(stderr,"      xmin, xmax ... units in km\n");
		fprintf(stderr,"      output.tsv:\n");
		fprintf(stderr,"      x y z v index_of_tetrahedron\n");
		fprintf(stderr,"      or\n");
		fprintf(stderr,"      x y z v Q index_of_tetrahedron\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( !(     (!strcmp(argv[2],"xz"))
		|| (!strcmp(argv[2],"xy"))
		|| (!strcmp(argv[2],"yz"))
		|| (!strcmp(argv[2],"hz")) ) ) {
		fprintf(stderr,"error: %s: bad keywords. must be xz xy yz or hz\n", argv[0]);
		exit(-1);
		}

	/* start cases xy, xz, yz */
	if( (!strcmp(argv[2],"xz")) || (!strcmp(argv[2],"xy")) || (!strcmp(argv[2],"yz")) ) {

	if( sscanf(argv[3], "%le", &a) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &b) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%d", &m) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &c) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &d) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &n) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &e) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}



	f1 = a+epsilon; df=(b-a-epsilon)/(double)(m-1);
	g1 = c+epsilon; dg=(d-c-epsilon)/(double)(n-1);

	if( QDEFINED ) printf("x\ty\tz\tv\tQ\ttetra\n");
	else printf("x\ty\tz\tv\ttetra\n");

	if( !strcmp(argv[2],"xz") ) {
		start_t=last_tetra;
		for( i=0; i<m; i++ ) for( j=0; j<n; j++ ) {
			if( j==0 ) last_tetra=start_t;
			x[0]=f1+df*(double)i;
			x[1]=e;
			x[2]=g1+dg*(double)j;
			if( find_tetra(x,&t) ) {
				if( j==0 ) start_t=t;
				v = eval_velocity(&tetra[t],x);
				if( QDEFINED ) {
					qinv = eval_qinv(&tetra[t],x);
					printf("%f\t%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 1.0/qinv, t );
					}
				else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t );
				}
			}
		}
	else if( !strcmp(argv[2],"xy") ) {
		start_t=last_tetra;
		for( i=0; i<m; i++ ) for( j=0; j<n; j++ ) {
			if( j==0 ) last_tetra=start_t;
			x[0]=f1+df*(double)i;
			x[1]=g1+dg*(double)j;
			x[2]=e;
			if( find_tetra(x,&t) ) {
				if( j==0 ) start_t=t;
				v = eval_velocity(&tetra[t],x);
				if( QDEFINED ) {
					qinv = eval_qinv(&tetra[t],x);
					printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 1.0/qinv, t );
					}
				else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t );
				}
			}
		}
	else if( !strcmp(argv[2],"yz") ) {
		start_t=last_tetra;
		for( i=0; i<m; i++ ) for( j=0; j<n; j++ ) {
			if( j==0 ) last_tetra=start_t;
			x[0]=e;
			x[1]=f1+df*(double)i;
			x[2]=g1+dg*(double)j;
			if( find_tetra(x,&t) ) {
				if( j==0 ) start_t=t;
				v = eval_velocity(&tetra[t],x);
				if( QDEFINED ) {
					qinv = eval_qinv(&tetra[t],x);
					printf("%f\t%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 2.0/qinv, t );
				}
				else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t );
				}
			}
		}
	}

	/* start case hz */
	else {

	if( sscanf(argv[3], "%le", &hx1) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &hy1) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &hx2) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &hy2) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &m) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &c) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &d) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &n) != 1 ) {
		fprintf(stderr,"error: %s: cant read parameter from command line\n",
			argv[0] );
		exit(-1);
		}

	f1 = hx1+epsilon; df=(hx2-hx1-epsilon)/(double)(m-1);
	f2 = hy1+epsilon; df2=(hy2-hy1-epsilon)/(double)(m-1);
	g1 = c+epsilon; dg=(d-c-epsilon)/(double)(n-1);

	if( QDEFINED ) printf("x\ty\tz\tv\tQ\ttetra\n");
	else printf("x\ty\tz\tv\ttetra\n");

	if( !strcmp(argv[2],"hz") ) {
		start_t=last_tetra;
		for( i=0; i<m; i++ ) for( j=0; j<n; j++ ) {
			if( j==0 ) last_tetra=start_t;
			x[0]=f1+df*(double)i;
			x[1]=f2+df2*(double)i;;
			x[2]=g1+dg*(double)j;
			if( find_tetra(x,&t) ) {
				if( j==0 ) start_t=t;
				v = eval_velocity(&tetra[t],x);
				if( QDEFINED ) {
					qinv = eval_qinv(&tetra[t],x);
					printf("%f\t%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 1.0/qinv, t );
					}
				else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t );
				}
			}
		}



		}

	exit(0);
	}


main_point(argc, argv)
int argc;
char *argv[];
{

	double x[3], v, qinv;
	int t;

	if( argc!=5 ) {
		fprintf(stderr,"purpose: interrogrates the velocity model at a point\n");
		fprintf(stderr,"usage: %s model_file x y z > output\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      (x, y, z) cartesian coordinates, +z is down\n");
		fprintf(stderr,"      x, y, z in km\n");
		fprintf(stderr,"      output:\n");
		fprintf(stderr,"      x y z v index_of_tetrahedron\n");
		fprintf(stderr,"      or, when Q is defined in the model\n");
		fprintf(stderr,"      x y z v Q index_of_tetrahedron\n");

		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( !find_tetra(x,&t) ) {
		printf("\not in model\n");
		}
	else {
		v = eval_velocity(&tetra[t],x);
		if( QDEFINED ) {
			qinv = eval_qinv(&tetra[t],x);
			printf("%f\t%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 1.0/qinv, t);
			}
		else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t);
		}

	exit(0);
	}

main_point_list(argc, argv)
int argc;
char *argv[];
{

	double x[3], v, qinv;
	int t;

	if( argc!=2 ) {
		fprintf(stderr,"purpose: interrogrates the velocity model at a list of points\n");
		fprintf(stderr,"usage: %s model_file < x_y_z.txt > output.txt\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      x_y_z.txt: list of x y z points, one per line\n");
		fprintf(stderr,"          (x, y, z) are cartesian coordinates in km, +z is down\n");
		fprintf(stderr,"      output.txt:\n");
		fprintf(stderr,"          list of x y z v index_of_tetrahedron\n");
		fprintf(stderr,"          or, when Q is defined in the model\n");
		fprintf(stderr,"          x y z v Q index_of_tetrahedron\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	while( scanf("%le%le%le", &x[0], &x[1], &x[2] ) == 3 ) {

		if( !find_tetra(x,&t) ) {
			continue;
			}
		else {
			v = eval_velocity(&tetra[t],x);
			if( QDEFINED ) {
				qinv = eval_qinv(&tetra[t],x);
				printf("%f\t%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, 1.0/qinv, t);
				}
			else printf("%f\t%f\t%f\t%f\t%d\n", x[0], x[1], x[2], v, t);
			}

		}

	exit(0);
	}


main_shootarc(argc, argv)
int argc;
char *argv[];
{
	int i, j, interface, hitflag, n_arcs, n, status, nrays, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000], qinvint;
	int tindex[8000];
	FILE *ttfile, *rayfile;

	if( argc!=12 ) {
		fprintf(stderr,"purpose: shoots rays in a arc pattern from a point\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z azimuth theta_min theta_max n_thetas target_interface traveltime_file.tsv ray_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      (x, y, z) cartesian coordinates, +z is down\n");
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      azimuth, degrees from +x to +y\n");
		fprintf(stderr,"      theta_min, theta_max, 0-180 degrees from +z\n");
		fprintf(stderr,"      n_thetas n integer, is number of rays\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		write_traveltime_blerb(stderr);
		write_ray_blerb(stderr);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &azi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimuth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &stheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &ftheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &nrays) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_thetas from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( (ttfile=fopen(argv[10],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime file <%s> for write\n",
			argv[0], argv[10]);
		exit(-1);
		}

	if( !strcmp(argv[11],"-") ) {
		rayfile=stdout;
		}
	else if( (rayfile=fopen(argv[11],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open ray file <%s> for write\n",
			argv[0], argv[11] );
		exit(-1);
		}

	if( QDEFINED ) {
		fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty2\tz1\tT\tQ\n");
		}
	else {
		fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty2\tz1\tT\n");
		}
	fprintf(rayfile,"id\tx\ty\tz\ttetra\n");

	dtheta = (ftheta-stheta)/(double)(nrays-1);

	for( j=0; j<nrays; j++ ) {

		theta=stheta+dtheta*(double)j;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			if( QDEFINED ) fprintf(ttfile,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n", j,
				x0[0], x0[1], x0[2], x1[0], x1[1], x1[2], traveltime,
				traveltime/qinvint);
			else fprintf(ttfile,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n", j,
				x0[0], x0[1], x0[2], x1[0], x1[1], x1[2], traveltime);
			arcstoxyz( arc, n_arcs, rayx, rayy, rayz, tindex, 1.0, &n, 8000 );
			for( i=0; i<n; i++ ) {
				fprintf(rayfile, "%d\t%f\t%f\t%f\t%d\n",
					j, rayx[i], rayy[i], rayz[i], tindex[i] );
				}
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray %d (error code %d)\n", j, ec );
			}
		}

	exit(0);
	}

main_shootarcrefl(argc, argv)
int argc;
char *argv[];
{
	int i, j, interface, rinterface, hitflag, n_arcs, n, status, nrays, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000], qinvint;
	int tindex[8000];
	FILE *ttfile, *rayfile;

	if( argc!=13 ) {
		fprintf(stderr,"purpose: shoots reflected rays in a arc pattern from a point\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z azimuth theta_min theta_max n_thetas target_interface reflector_interface traveltime_file.tsv ray_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      (x, y, z) cartesian coordinates, +z is down\n");
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      azimuth, degrees from +x to +y\n");
		fprintf(stderr,"      theta_min, theta_max, 0-180 degrees from +z\n");
		fprintf(stderr,"      n_thetas n integer, is number of rays\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      reflector_interface to reflect from (must match model)\n");
		write_traveltime_blerb(stderr);
		write_ray_blerb(stderr);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &azi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimuth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &stheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &ftheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &nrays) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_thetas from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &rinterface) != 1 ) {
		fprintf(stderr,"error: %s: cant read reflector_interface from command line\n", argv[0] );
		exit(-1);
		}


	if( (ttfile=fopen(argv[11],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime file <%s> for write\n",
			argv[0], argv[11]);
		exit(-1);
		}

	if( !strcmp(argv[12],"-") ) {
		rayfile=stdout;
		}
	else if( (rayfile=fopen(argv[12],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open ray file <%s> for write\n",
			argv[0], argv[12] );
		exit(-1);
		}

	if( QDEFINED ) {
		fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty2\tz1\tT\tQ\n");
		}
	else {
		fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty2\tz1\tT\n");
		}
	fprintf(rayfile,"id\tx\ty\tz\ttetra\n");

	dtheta = (ftheta-stheta)/(double)(nrays-1);

	for( j=0; j<nrays; j++ ) {

		theta=stheta+dtheta*(double)j;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface_refl( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &traveltime,
			&qinvint, &ec );
		if( status&&hitflag ) {
			if( QDEFINED ) fprintf(ttfile,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n", j,
				x0[0], x0[1], x0[2], x1[0], x1[1], x1[2], traveltime,
				traveltime/qinvint);
			else fprintf(ttfile,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n", j,
				x0[0], x0[1], x0[2], x1[0], x1[1], x1[2], traveltime);
			arcstoxyz( arc, n_arcs, rayx, rayy, rayz, tindex, 1.0, &n, 8000 );
			for( i=0; i<n; i++ ) {
				fprintf(rayfile, "%d\t%f\t%f\t%f\t%d\n",
					j, rayx[i], rayy[i], rayz[i], tindex[i] );
				}
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray %d (error code %d)\n", j, ec );
			}
		}

	exit(0);
	}

main_shootstar(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, interface, hitflag, n_arcs, n, status, n_azis, n_thetas, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4];
	double stheta, ftheta, dtheta, theta, qinvint;
	double sazi, fazi, dazi, azi;
	double traveltime;

	if( argc!=12 ) {
		fprintf(stderr,"purpose: shoots rays in a star pattern from a point\n");
		fprintf(stderr,"	 (constant azimuth and angle of incident increments)\n");

		fprintf(stderr,"usage: %s model_file source_x source_y source_z theta_min theta_max n_thetas azi_min azi_max n_azis target_interface > star_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      theta_min, theta_max, 0-180 degrees from +z\n");
		fprintf(stderr,"      azi_min, azi_max, degrees from +x to +y\n");
		fprintf(stderr,"          azi_min<azi_max, but values > 360 allowed\n");
		fprintf(stderr,"      n_thetas, n_azis, integers, control the number of rays\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      star_file.tsv contains:\n");
		fprintf(stderr,"          theta_index\n");
		fprintf(stderr,"          azi_index\n");
		fprintf(stderr,"          theta in degrees\n");
		fprintf(stderr,"          azimuth in degrees\n");
		fprintf(stderr,"          final x in km\n");
		fprintf(stderr,"          final y in km\n");
		fprintf(stderr,"          final z in km\n");
		fprintf(stderr,"          traveltime in s\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &stheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &ftheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &n_thetas) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_thetas from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &sazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &fazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &n_azis) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_azis from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	printf("th_%d\taz_%d\ttheta\tazimuth\tx\ty\tz\tT\n", n_thetas, n_azis);

	dtheta = (ftheta-stheta)/(double)(n_thetas-1);
	dazi = (fazi-sazi)/(double)(n_azis-1);

	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		azi=sazi+dazi*(double)j;
		theta=stheta+dtheta*(double)k;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\n", k, j,
				theta, azi, x1[0], x1[1], x1[2], traveltime);
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray (%d %d) error code (%d)\n", j, k, ec );
			}
		}

	exit(0);
	}

main_shootstarrefl(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, interface, rinterface, hitflag, n_arcs, n, status, n_azis, n_thetas, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4];
	double stheta, ftheta, dtheta, theta, qinvint;
	double sazi, fazi, dazi, azi;
	double traveltime;

	if( argc!=13 ) {
		fprintf(stderr,"purpose: shoots relected rays in a star pattern from a point\n");
		fprintf(stderr,"	 (constant azimuth and angle of incident increments)\n");

		fprintf(stderr,"usage: %s model_file source_x source_y source_z theta_min theta_max n_thetas azi_min azi_max n_azis target_interface reflector_interface > star_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      theta_min, theta_max, 0-180 degrees from +z\n");
		fprintf(stderr,"      azi_min, azi_max, degrees from +x to +y\n");
		fprintf(stderr,"          azi_min<azi_max, but values > 360 allowed\n");
		fprintf(stderr,"      n_thetas, n_azis, integers, control the number of rays\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      reflector_interface to shoot for (must match model)\n");
		fprintf(stderr,"      star_file.tsv contains:\n");
		fprintf(stderr,"          theta_index\n");
		fprintf(stderr,"          azi_index\n");
		fprintf(stderr,"          theta in degrees\n");
		fprintf(stderr,"          azimuth in degrees\n");
		fprintf(stderr,"          final x in km\n");
		fprintf(stderr,"          final y in km\n");
		fprintf(stderr,"          final z in km\n");
		fprintf(stderr,"          traveltime in s\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &stheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &ftheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &n_thetas) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_thetas from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &sazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &fazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &n_azis) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_azis from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%d", &rinterface) != 1 ) {
		fprintf(stderr,"error: %s: cant read reflector_interface from command line\n", argv[0] );
		exit(-1);
		}

	printf("th_%d\taz_%d\ttheta\tazimuth\tx\ty\tz\tT\n", n_thetas, n_azis);

	dtheta = (ftheta-stheta)/(double)(n_thetas-1);
	dazi = (fazi-sazi)/(double)(n_azis-1);

	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		azi=sazi+dazi*(double)j;
		theta=stheta+dtheta*(double)k;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface_refl( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface,
			&hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\n", k, j,
				theta, azi, x1[0], x1[1], x1[2], traveltime);
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray (%d %d) error code (%d)\n", j, k, ec );
			}
		}

	exit(0);
	}

int shoot_to_interface_refl( x0, t0, x1, t1, arc, n_arcs, nmax, smax,
	interface, rinterface, hitflag, traveltime, qinvint, ec )
double x0[3], t0[3];
double x1[3], t1[3];
struct arc_rec arc[];
int *n_arcs, nmax;
double smax;
int interface, rinterface, *hitflag;
double *traveltime;
double *qinvint;
int *ec;
{
	int i, j;
	int status1, hitflag1, ec1, n_arcs1;
	int status2, hitflag2, ec2, n_arcs2;
	double traveltime1, qinvint1;
	double traveltime2, qinvint2;
	double xr[3], tr[3], trp[3], xrp[3];
	double *ptt, *pqi;
	struct arc_rec *par;

	struct tetra_rec *ptetra;
	int face;
	double n[3], v[3], h[3], P[4], s[3][3], st[3][3];
	double epsilon, v1[3], v2[3], v3;

	if( traveltime==NULL ) ptt=NULL; else ptt=(&traveltime1);
	if( qinvint==NULL ) pqi=NULL; else pqi=(&qinvint1);
	if( arc==NULL ) par=NULL; else par=arc;

	/* propagate to reflecting interrface */
	n_arcs1=0;
	status1=shoot_to_interface( x0, t0, xr, tr, par, &n_arcs1,
		nmax, 10000.0, rinterface, &hitflag1, ptt, pqi, &ec1 );
	if( !(status1 && hitflag1) ) {
		*n_arcs=0; *ec=ec1; *hitflag=hitflag1;
		return(status1);
		}

	/* reflect ray tangent */

	/* face and tetra from last arc (if available) */
	if( arc!=NULL ) {
		ptetra=arc[n_arcs1-1].ptetra;
		if( ptetra == NULL ) { *n_arcs=0; *ec=20; *hitflag=FALSE; return(FALSE); }
		face=arc[n_arcs1-1].exit_face;
		}
	else { /* if no last arc, then compute the hard way */
		epsilon=1.0e-6;
		for( i=0; i<3; i++ ) xrp[i]=xr[i]-epsilon*tr[i];
		if( !find_tetra( xrp, &i ) ) { *n_arcs=0; *ec=22; *hitflag=FALSE; return(FALSE); }
		ptetra=tetra+i;
		if( ! ray_exits_tetrahedron( ptetra->x1, ptetra->x2, ptetra->x3, ptetra->x4,
			xrp, tr, epsilon, v1, v2, &v3, &face ) )
			{ *n_arcs=0; *ec=23; *hitflag=FALSE; return(FALSE); }
		face+=1;
		}

	/* normal to exit face */
	if( face==1 )      eqn_of_plane( ptetra->x2, ptetra->x3, ptetra->x4, P );
	else if( face==2 ) eqn_of_plane( ptetra->x1, ptetra->x3, ptetra->x4, P );
	else if( face==3 ) eqn_of_plane( ptetra->x1, ptetra->x2, ptetra->x4, P );
	else if( face==4 ) eqn_of_plane( ptetra->x1, ptetra->x2, ptetra->x3, P );
	else { *n_arcs, *ec=21; *hitflag=FALSE; return(FALSE); }
	normal_to_plane( P, n );

	/* rotate ray tangent to coordinate system of plane */
	/* flip sign of normal-to-plane component           */
	/* rotate back to apply Snell's law of reflection   */
	tvh( n, v, h );
	newbasisrot( n, v, h, s );
	for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) st[i][j]=s[j][i];
	rotate3(tr,s,trp);
	trp[0]=(-trp[0]);
	rotate3(trp,st,tr);

	/* propagate to target interface */
	if( traveltime==NULL ) ptt=NULL; else ptt=(&traveltime2);
	if( qinvint==NULL ) pqi=NULL; else pqi=(&qinvint2);
	if( arc==NULL ) par=NULL; else par=arc+n_arcs1;

	n_arcs2=0;
	status2=shoot_to_interface( xr, tr, x1, t1, par, &n_arcs2,
		nmax-n_arcs1, 10000.0, interface, &hitflag2, ptt, pqi, &ec2 );

	*n_arcs=n_arcs1+n_arcs2;
	if( traveltime!=NULL ) *traveltime=traveltime1+traveltime2;
	if( qinvint!=NULL ) *qinvint=qinvint1+qinvint2;
	*ec=ec2+10;
	*hitflag=hitflag2;
	return(status2);
	}

main_shootturf(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, interface, hitflag, n_arcs, n, status, n_x, n_y, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double theta, azi, v0, vp, z0, T0, px, py, xmin, xmax, ymin, ymax, dx, dy;
	double x0[3], t0[3], x1[3], t1[3], v[4], qinvint;
	double traveltime;
	int t, my_last_tetra;

	my_last_tetra=(-1);

	if( argc!=13 ) {
		fprintf(stderr,"purpose: shoots teleseismic rays in a grid pattern from a plane\n");
		fprintf(stderr,"	 (constant x and y increments)\n");

		fprintf(stderr,"usage: %s model_file azimuth apparent_velocity z0 t0 x_min x_max n_x y_min y_max n_y target_interface > turf_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      teleseismic azimuth in deg\n");
		fprintf(stderr,"      teleseismic apparent velocity in km/s\n");
		fprintf(stderr,"      teleseism crosses point (0,0,z0) at time t0\n");
		fprintf(stderr,"      	(z0 in km, t0 in s)");
		fprintf(stderr,"      x_min, x_max in km; integer n_x\n");
		fprintf(stderr,"      y_min, y_max in km; integer n_y\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      star_file.tsv contains:\n");
		fprintf(stderr,"          x_index\n");
		fprintf(stderr,"          y_index\n");
		fprintf(stderr,"          x in km\n");
		fprintf(stderr,"          y in km\n");
		fprintf(stderr,"          final x in km\n");
		fprintf(stderr,"          final y in km\n");
		fprintf(stderr,"          final z in km\n");
		fprintf(stderr,"          traveltime in s\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &azi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimuth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &v0) != 1 ) {
		fprintf(stderr,"error: %s: cant read apparent velocity from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &z0) != 1 ) {
		fprintf(stderr,"error: %s: cant read z0 from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &T0) != 1 ) {
		fprintf(stderr,"error: %s: cant read t0 from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &xmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read x_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &xmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read x_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &n_x) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &ymin) != 1 ) {
		fprintf(stderr,"error: %s: cant read y_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%le", &ymax) != 1 ) {
		fprintf(stderr,"error: %s: cant read y_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &n_y) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	printf("x_%d\ty_%d\tx0\ty0\tx\ty\tz\tT\n", n_x, n_y);

	dx = (xmax-xmin)/(double)(n_x-1);
	dy = (ymax-ymin)/(double)(n_y-1);

	for( j=0; j<n_x; j++ ) for( k=0; k<n_y; k++ ) {

		x0[0]=xmin+dx*(double)j;
		x0[1]=ymin+dy*(double)k; /*Bug; dy was dx; fixed 6/6/3 by Menke on the advice of Zor */
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else continue;
		if( vp>=v0 ) continue;
		my_last_tetra=last_tetra;
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		px = t0[0]/vp; py=t0[1]/vp;
		status=shoot_to_interface( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			traveltime += T0 + px*x0[0] + py*x0[1];
			/* bug fix; Menke 3/7/01; k, j to j, k in following printf */
			printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\n", j, k,
				x0[0], x0[1], x1[0], x1[1], x1[2], traveltime);
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray (%d %d) error code (%d)\n", j, k, ec );
			}
		}

	exit(0);
	}


main_shootstarp(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, interface, hitflag, n_arcs, n, status, n_azis, n_vels, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4];
	double pmin, p, vmin, vmax, dp, theta;
	double sazi, fazi, dazi, azi;
	double traveltime, qinvint;
	double v0;
	int myt;

	if( argc!=12 ) {
		fprintf(stderr,"purpose: shoots rays in a star pattern from a point\n");
		fprintf(stderr,"	 (constant azimuth and hoizontal slowness increments)\n");

		fprintf(stderr,"usage: %s model_file source_x source_y source_z v_min v_max n_thetas azi_min azi_max n_azis target_interface > star.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      v_min, v_max, apparent velocity in km/s\n");
		fprintf(stderr,"          (downgoing rays only)\n");
		fprintf(stderr,"      azi_min, azi_max, degrees from +x to +y\n");
		fprintf(stderr,"          azi_min<azi_max, but values > 360 allowed\n");
		fprintf(stderr,"      n_velocities, n_azis, integers, control the number of rays\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      star.tsv contains:\n");
		fprintf(stderr,"          theta_index\n");
		fprintf(stderr,"          azi_index\n");
		fprintf(stderr,"          theta in degrees\n");
		fprintf(stderr,"          azimuth in degrees\n");
		fprintf(stderr,"          final x in km\n");
		fprintf(stderr,"          final y in km\n");
		fprintf(stderr,"          final z in km\n");
		fprintf(stderr,"          traveltime in s\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &vmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read v_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &vmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read v_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &n_vels) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_velocities from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &sazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &fazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &n_azis) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_azis from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( find_tetra(x0,&myt) ) {
		v0 = eval_velocity(&tetra[myt],x0);
		}
	else {
		fprintf(stderr,"error: %d: cant find velocity at source\n", argv[0] );
		exit(-1);
		}

	printf("th_%d\taz_%d\ttheta\tazimuth\tx\ty\tz\tT\n", n_vels, n_azis);

	dazi = (fazi-sazi)/(double)(n_azis-1);
	pmin = 1.0/vmax;
	dp = ((1.0/vmin)-(1.0/vmax))/(double)(n_vels-1);


	for( j=0; j<n_azis; j++ ) for( k=0; k<n_vels; k++ ) {
		azi=sazi+dazi*(double)j;
		p=pmin+dp*(double)k;
		if( fabs(v0*p)>1.0 ) break;
		theta=RTOD*asin(v0*p); 
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\n", k, j,
				theta, azi, x1[0], x1[1], x1[2], traveltime);
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray (%d %d), (error code %d)\n", j, k, ec );
			}
		}

	exit(0);
	}

main_stor(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, interface, hitflag, n_arcs, n, status, nrays, order, ec, in_count;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *stafile, *ttfile, *rayfile;
	char ct[128];
	double xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est, qinvint;
	struct triangle_rec tri;
	double geo_spr;

	if( argc!=10 ) {
		fprintf(stderr,"purpose: point-to-point raytracing from a common source to a set of receivers\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z star.tsv target_interface receivers.tsv predicted_tt_file.tsv ray_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      star.tsv: previously computed\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      input receivers.tsv:\n");
		fprintf(stderr,"              x, y, z, location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              T0, origin time in s\n");
		fprintf(stderr,"              T,  arival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"	      order, 0 for 1-st arrival, 1 for second, etc; or\n");
		fprintf(stderr,"	          order=(-1) causes program to pick closest arrival\n");
		write_predictions_blerb(stderr);
		write_ray_blerb(stderr);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( !load_star_file(argv[5]) ) {
		fprintf(stderr,"error: %s: cant load star.tsv <%s>\n", argv[0], argv[5] );
		exit(-1);
		}

	if( sscanf(argv[6], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( (stafile=fopen(argv[7],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open receivers.tsv <%s> for read\n",
			argv[0], argv[7]);
		exit(-1);
		}
	if( fscanf(stafile,"%s%s%s%s%s%s%s", ct,ct,ct,ct,ct,ct,ct ) != 7 ) {
		fprintf(stderr,"error: %s: cant skip header in receivers.tsv <%s>\n",
			argv[0], argv[7]);
		exit(-1);
		}

	if( (ttfile=fopen(argv[8],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open predicted_tt_file.tsv <%s> for write\n",
			argv[0], argv[8]);
		exit(-1);
		}

	if( !strcmp(argv[9],"-") ) {
		rayfile=stdout;
		}
	else if( (rayfile=fopen(argv[9],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open ray file <%s> for write\n",
			argv[0], argv[9] );
		exit(-1);
		}

	if( QDEFINED ) {
fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\tQ\tntts\n");
		}
	else {
fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\tntts\n");
		}
	fprintf(rayfile,"id\tx\ty\tz\ttetra\n");
	nrays=0; in_count=(-1);
 	while( fscanf(stafile,"%le%le%le%le%le%le%d",&xsta,&ysta,&zsta,&T0,&Ta,&ratio,&order)==7){
		n_tts=0; in_count++;
		for( i=0; i<(m_rays-1); i++ )
		for( j=0; j<(n_rays-1); j++ )
		for( l=0; l<2; l++ ) {
			if( instarbox(i, j, l, xsta, ysta, &th_est,
				&az_est, &tri, x0, interface, &geo_spr) )
			if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
				NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				tt[n_tts].gs=geo_spr;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
			}
		qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
		if( order==(-1) ) setorder( tt, n_tts, Ta-T0, ratio, &order );
		for( j=0; j<n_tts; j++ ) {
			ray_tangent(tt[j].th,tt[j].az,t0);
			if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
				MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
				continue;
				}
			if( !hitflag ) { continue; }
			if( QDEFINED ) 
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%f\t%d\n",
			nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
			T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 	Ta-(T0+traveltime*ratio),
			ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), traveltime/qinvint, in_count );
			else
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%d\n",
			nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
			T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 	Ta-(T0+traveltime*ratio),
			ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), in_count );
			n=0;
			arcstoxyz( arc, n_arcs, rayx, rayy, rayz, tindex, 1.0, &n, 8000 );
			for( i=0; i<n; i++ ) {
				fprintf(rayfile, "%d\t%f\t%f\t%f\t%d\n",
					nrays, rayx[i], rayy[i], rayz[i], tindex[i]);
				}
			nrays++;
			}
		}

	exit(0);
	}

main_storrefl(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, interface, rinterface, hitflag, n_arcs, n, status, nrays, order, ec, in_count;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *stafile, *ttfile, *rayfile;
	char ct[128];
	double xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est, qinvint;
	struct triangle_rec tri;
	double geo_spr;

	if( argc!=11 ) {
		fprintf(stderr,"purpose: point-to-point raytracing from a common source to a set of receivers with a single reflection point\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z star.tsv target_interface reflector_interface receivers.tsv predicted_tt_file.tsv ray_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      star.tsv: previously computed\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      reflector_interface to reflect from (must match model)\n");
		fprintf(stderr,"      input receivers.tsv:\n");
		fprintf(stderr,"              x, y, z, location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              T0, origin time in s\n");
		fprintf(stderr,"              T,  arival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"	      order, 0 for 1-st arrival, 1 for second, etc; or\n");
		fprintf(stderr,"	          order=(-1) causes program to pick closest arrival\n");
		write_predictions_blerb(stderr);
		write_ray_blerb(stderr);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( !load_star_file(argv[5]) ) {
		fprintf(stderr,"error: %s: cant load star.tsv <%s>\n", argv[0], argv[5] );
		exit(-1);
		}

	if( sscanf(argv[6], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &rinterface) != 1 ) {
		fprintf(stderr,"error: %s: cant read reflector_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( (stafile=fopen(argv[8],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open receivers.tsv <%s> for read\n",
			argv[0], argv[8]);
		exit(-1);
		}
	if( fscanf(stafile,"%s%s%s%s%s%s%s", ct,ct,ct,ct,ct,ct,ct ) != 7 ) {
		fprintf(stderr,"error: %s: cant skip header in receivers.tsv <%s>\n",
			argv[0], argv[8]);
		exit(-1);
		}

	if( (ttfile=fopen(argv[9],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open predicted_tt_file.tsv <%s> for write\n",
			argv[0], argv[9]);
		exit(-1);
		}

	if( !strcmp(argv[10],"-") ) {
		rayfile=stdout;
		}
	else if( (rayfile=fopen(argv[10],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open ray file <%s> for write\n",
			argv[0], argv[10] );
		exit(-1);
		}

	if( QDEFINED ) {
fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\tQ\ttts\n");
		}
	else {
fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\ttts\n");
		}
	fprintf(rayfile,"id\tx\ty\tz\ttetra\n");
	nrays=0; in_count=(-1);
 	while( fscanf(stafile,"%le%le%le%le%le%le%d",&xsta,&ysta,&zsta,&T0,&Ta,&ratio,&order)==7){
		n_tts=0; in_count++;
		for( i=0; i<(m_rays-1); i++ )
		for( j=0; j<(n_rays-1); j++ )
		for( l=0; l<2; l++ ) {
			if( instarbox_refl(i, j, l, xsta, ysta, &th_est,
				&az_est, &tri, x0, interface, rinterface, &geo_spr) )
			if( refine_refl( x0, xsta, ysta, interface, rinterface, &th_est, &az_est,
				NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				tt[n_tts].gs=geo_spr;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
			}
		qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
		if( order==(-1) ) setorder( tt, n_tts, Ta-T0, ratio, &order );
		for( j=0; j<n_tts; j++ ) {
			ray_tangent(tt[j].th,tt[j].az,t0);
			if( !shoot_to_interface_refl( x0, t0, x1, t1, arc, &n_arcs,
				MAX_ARCS, 10000.0, interface, rinterface, &hitflag,
				&traveltime, &qinvint, &ec ) ) {
				continue;
				}
			if( !hitflag ) continue;
			if( QDEFINED ) 
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%f\t%d\n",
			nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
			T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 	Ta-(T0+traveltime*ratio),
			ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), traveltime/qinvint, in_count );
			else
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%d\n",
			nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
			T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 	Ta-(T0+traveltime*ratio),
			ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), in_count );
			n=0;
			arcstoxyz( arc, n_arcs, rayx, rayy, rayz, tindex, 1.0, &n, 8000 );
			for( i=0; i<n; i++ ) {
				fprintf(rayfile, "%d\t%f\t%f\t%f\t%d\n",
					nrays, rayx[i], rayy[i], rayz[i], tindex[i]);
				}
			nrays++;
			}
		}

	exit(0);
	}


main_ttor(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, interface, hitflag, n_arcs, n, status, nrays, order, ec, in_count;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4], theta, azi, v0, T0, z0, vp;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *stafile, *ttfile, *rayfile;
	char ct[128];
	double xsta, ysta, zsta, ratio, Ta, x_est, y_est, px, py;
	struct triangle_rec tri;
	int t, my_last_tetra;
	double geo_spr, qinvint;

	if( argc!=11 ) {
		fprintf(stderr,"purpose: plane-wave-to-point raytracing from a common telseismic plane-wave to a set of receivers\n");
		fprintf(stderr,"usage: %s model_file azimuth apparent_velocity z0 t0  turf.tsv target_interface receivers.tsv predicted_tt_file.tsv ray_file.tsv\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      teleseismic azimuth in deg\n");
		fprintf(stderr,"      teleseismic apparent velocity in km/s\n");
		fprintf(stderr,"      teleseism crosses point (0,0,z0) at time t0\n");
		fprintf(stderr,"      	(z0 in km, t0 in s)");
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      star.tsv: previously computed\n");
		fprintf(stderr,"      target_interface to shoot for (must match model)\n");
		fprintf(stderr,"      input receivers.tsv:\n");
		fprintf(stderr,"              x, y, z, location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              T,  observed arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"                  Ratio provides a means of computing traveltimes\n");
		fprintf(stderr,"                  for a teleseismic S wave through a P wave model,\n");
		fprintf(stderr,"                  by the simple device of raytracing a P wave\n");
		fprintf(stderr,"                  and then multiplying the traveltime by ratio.\n");
		fprintf(stderr,"                  Note however that the apparent velocity of this\n");
		fprintf(stderr,"                  S wave is exactly the P wave apparent velocity,\n");
		fprintf(stderr,"                  as specified on the command line, divided by the ratio.\n");
		fprintf(stderr,"	      order, 0 for 1-st arrival, 1 for second, etc; or\n");
		fprintf(stderr,"	          order=(-1) for program to pick closest arrival\n");

		write_predictions_blerb(stderr);
		write_ray_blerb(stderr);
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &azi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimuth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &v0) != 1 ) {
		fprintf(stderr,"error: %s: cant read apparent_velocity from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &z0) != 1 ) {
		fprintf(stderr,"error: %s: cant read z0 from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &T0) != 1 ) {
		fprintf(stderr,"error: %s: cant read t0 from command line\n", argv[0] );
		exit(-1);
		}

	if( !load_turf_file(argv[6]) ) {
		fprintf(stderr,"error: %s: cant load turf.tsv <%s>\n", argv[0], argv[6] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read target_interface from command line\n", argv[0] );
		exit(-1);
		}

	if( (stafile=fopen(argv[8],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open receivers.tsv <%s> for read\n",
			argv[0], argv[8]);
		exit(-1);
		}
	if( fscanf(stafile,"%s%s%s%s%s%s", ct,ct,ct,ct,ct,ct ) != 6 ) {
		fprintf(stderr,"error: %s: cant skip header in receivers.tsv <%s>\n",
			argv[0], argv[8]);
		exit(-1);
		}

	if( (ttfile=fopen(argv[9],"w")) == NULL ) {
		fprintf(stderr,"error: %s: cant open predicted_tt_file.tsv <%s> for write\n",
			argv[0], argv[9]);
		exit(-1);
		}

	if( !strcmp(argv[10],"-") ) {
		rayfile=stdout;
		}
	else if( (rayfile=fopen(argv[10],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open ray file <%s> for write\n",
			argv[0], argv[10] );
		exit(-1);
		}

	if( QDEFINED ) {
	fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\\tQ\tttsn");
		}
else {
fprintf(ttfile,"id\tx0\ty0\tz0\tx1\ty1\tz1\tT0\tTa\tTT\tT\tdT\tratio\tobs_order\tpre_order\tamplitude\ttts\n");
		}
	fprintf(rayfile,"id\tx\ty\tz\ttetra\n");
	nrays=0; in_count=(-1);
 	while( fscanf(stafile,"%le%le%le%le%le%d",&xsta,&ysta,&zsta,&Ta,&ratio,&order)==6){
		n_tts=0; in_count++;
		for( i=0; i<(m_turf-1); i++ )
		for( j=0; j<(n_turf-1); j++ )
		for( l=0; l<2; l++ ) {
			if( inturfbox(i, j, l, xsta, ysta, &x_est,
				&y_est, &tri, x0, interface, azi,v0,z0,T0, &geo_spr) )
			if( refine_turf( x0, xsta, ysta, interface, &x_est, &y_est,
				NULL, &n_arcs, &traveltime, &tri, azi,v0,z0,T0) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=x_est;
				tt[n_tts].th=y_est;
				tt[n_tts].T=traveltime;
				tt[n_tts].gs=geo_spr;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
			}
		qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
		if( order==(-1) ) setorder( tt, n_tts, Ta-T0, ratio, &order );

		for( j=0; j<n_tts; j++ ) {

			x0[0]=tt[j].az;
			x0[1]=tt[j].th;
			x0[2]=z0;
			last_tetra=my_last_tetra;
			if( find_tetra(x0,&t) ) {
				vp = eval_velocity(&tetra[t],x0);
				}
			else continue;
			if( vp>=v0 ) continue;
			my_last_tetra=last_tetra;
			theta = 180.0-RTOD*asin(vp/v0);
			ray_tangent(theta,azi,t0);
			px = t0[0]/vp; py=t0[1]/vp;
			if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
				MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
				continue;
				}
			if( !hitflag ) continue;
			traveltime += T0 + px*x0[0] + py*x0[1];
			if( QDEFINED )
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%f\t%d\n",
				nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
				T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 		Ta-(T0+traveltime*ratio),
				ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), traveltime/qinvint, in_count );
			else
			fprintf(ttfile,
			"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%e\t%d\n",
				nrays, x0[0], x0[1], x0[2], x1[0], x1[1], x1[2],
				T0, Ta, traveltime*ratio, T0+traveltime*ratio,
		 		Ta-(T0+traveltime*ratio),
				ratio, order, j, 1.0/sqrt(fabs(tt[j].gs)), in_count );
			n=0;
			arcstoxyz( arc, n_arcs, rayx, rayy, rayz, tindex, 1.0, &n, 8000 );
			for( i=0; i<n; i++ ) {
				fprintf(rayfile, "%d\t%f\t%f\t%f\t%d\n",
					nrays, rayx[i], rayy[i], rayz[i], tindex[i]);
				}
			nrays++;
			}
		}

	exit(0);
	}


main_locate(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, interface, hitflag, n_arcs, n, status, nrays, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *stafile, *locfile;
	char ct[128];
	double xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est;
	struct triangle_rec tri;

#define MAX_LOCTTS (1000)

	double thmin, thmax, azimin, azimax, vs;
	int nth, nazi;
	double G[MAX_LOCTTS][4], GTG[4][4], GTGc[4][4], GTd[4], d[MAX_LOCTTS], maxG;
	int ng, iter, ierror, min_it, min_c_it, max_it;
	double ts, tolerance, rms, GTGi[4][4], sigmat, sigmax, sigmay, sigmaz, dT, dX;
	double damping;

	if( argc!=18 ) {
		fprintf(stderr,"purpose: locate a source given traveltimes to a set of receivers\n");
		fprintf(stderr,"usage: %s model_file xs ys zs thmin thmax nth azimin azimax nazi traveltimes.tsv tolerance location_file.tsv damping min_iterations minimum_constrained_iterations maximum_iterations\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      (x, y, z) cartesian coordinates, +z is down\n");
		fprintf(stderr,"      xs, ys, zs, trial source location in km\n");
		fprintf(stderr,"      arrivals are searched for in a star with\n");
		fprintf(stderr,"         thmin, thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         nth, integer number of thetas in star\n");
		fprintf(stderr,"         azimin, azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              x, y, z, location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data by interface!)\n");
		fprintf(stderr,"              T,  arival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"	      order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"	         Note: order=(-1) not allowed for locate\n");
		fprintf(stderr,"      tolerance, s, iterations stop when rms traveltime error less than tolerance\n");
		fprintf(stderr,"      damping, fraction of maximum GTG value (e.g. 1.0E-6)\n");
		fprintf(stderr,"      min_iterations, minimum number of iterations (must be >0)\n");
		fprintf(stderr,"      min_constrained_iterations, minimum number of iterations with depth held constant (can be 0)\n");
		fprintf(stderr,"      max_iterations, minimum number of iterations (must be >0)\n");

		write_location_blerb(stderr);
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d locate. Model file: %s\n\n", argv[1] );

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	make_ttime(argv[11]);
	if( (stafile=fopen(argv[11],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open traveltimes.tsv <%s> for read\n",
			argv[0], argv[11]);
		exit(-1);
		}
	if( fscanf(stafile,"%s%s%s%s%s%s%s", ct,ct,ct,ct,ct,ct,ct ) != 7 ) {
		fprintf(stderr,"error: %s: cant skip header in receivers.tsv <%s>\n",
			argv[0], argv[10]);
		exit(-1);
		}
	n_ttimes=0;
	fprintf(stderr,"Input Data\n\n");
	fprintf(stderr,"#\txr\tyr\tzr\tinterface\t\tTa\tratio\torder\n");
 	while( fscanf(stafile,"%le%le%le%d%le%le%d",&xsta,&ysta,&zsta,&interface,&Ta,&ratio,&order)==7){
		if( (n_ttimes>MAX_TTIMES) || (n_ttimes>MAX_LOCTTS) ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%d\t%f\t%f\t%d\n",
			n_ttimes, ttime[n_ttimes].x1[0], ttime[n_ttimes].x1[1],
			ttime[n_ttimes].x1[2], ttime[n_ttimes].interface,
			ttime[n_ttimes].obsTa, ttime[n_ttimes].ratio,
			ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(stafile);

	fprintf(stderr,"Total of %d data\n\n", n_ttimes);

	if( sscanf(argv[12], "%le", &tolerance) != 1 ) {
		fprintf(stderr,"error: %s: cant read tolerance from command line\n", argv[0] );
		exit(-1);
		}
	fprintf(stderr,"tolerance: %e\n", tolerance);


	if( (locfile=fopen(argv[13],"w")) == NULL ) {
		fprintf(stderr,"error: %s cant open location_file.tsv <%s> for write\n",
			argv[0], argv[13] );
		exit(-1);
		}

	if( sscanf(argv[14], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}
	fprintf(stderr,"damping: %e\n", damping);

	if( sscanf(argv[15], "%d", &min_it) != 1 ) {
		fprintf(stderr,"error: %s: cant read minimum_iterations from command line\n", argv[0] );
		exit(-1);
		}
	fprintf(stderr,"minimum_iterations: %d\n", min_it);


	if( sscanf(argv[16], "%d", &min_c_it) != 1 ) {
		fprintf(stderr,"error: %s: cant read minimum_constrained_iterations from command line\n", argv[0] );
		exit(-1);
		}
	fprintf(stderr,"minimum_constrained_iterations: %d\n", min_c_it);


	if( sscanf(argv[17], "%d", &max_it) != 1 ) {
		fprintf(stderr,"error: %s: cant read maximum_iterations from command line\n", argv[0] );
		exit(-1);
		}
	fprintf(stderr,"maximum_iterations: %d\n", max_it);


	for( iter=0; iter<max_it; iter++ ) {

	if( !find_tetra(x0,&i) ) {
		fprintf(stderr,"error: %s: source location (%f, %f, %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
		exit(-1);
		}
	vs=eval_velocity(&tetra[i],x0);

	fprintf(stderr,"\nIteration %d: source at (%f %f %f) with velocity %f\n\n",
		iter, x0[0], x0[1], x0[2], vs );
	timestamp("start of this iteration\n");

	interface=ttime[0].interface;
	fprintf(stderr,"Shooting star pattern to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		interface, thmin, thmax, nth, azimin, azimax, nazi );
	shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
	fprintf(stderr,"done with star\n\n", n_ttimes);

	nrays=0;
	ng=0;
	fprintf(stderr,"Linearized traveltime equations:\n");
	fprintf(stderr,"i\tG[i][0]\tG[i][1]\tG[i][2]\tG[i][3]\td[i]\tobsTa\tpreTT\n");
	for( k=0; k<n_ttimes; k++ ) {
		xsta=ttime[k].x1[0];
		ysta=ttime[k].x1[1];
		if( interface!=ttime[k].interface ) {
			fprintf(stderr,"second interace (%d) encountered. ",
				ttime[k].interface);
			fprintf(stderr,"Shooting another star pattern\n");
			interface=ttime[k].interface;
			shoot_star(thmin, thmax, nth, azimin, azimax, nazi,
				x0, interface);
			fprintf(stderr,"Done with star\n\n");
			}
		n_tts=0;
		for( i=0; i<(m_rays-1); i++ )
		for( j=0; j<(n_rays-1); j++ )
		for( l=0; l<2; l++ ) {
			if( instarbox(i, j, l, xsta, ysta, &th_est,
				&az_est, &tri, x0, interface, NULL) )
			if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
				NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
			}
		qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );

		if( (j=ttime[k].obsorder) < n_tts ) {
			ttime[k].ok=TRUE;
			ttime[k].preorder=j;
			ttime[k].preTT=tt[j].T*ttime[k].ratio;
			ttime[k].th=tt[j].th;
			ttime[k].az=tt[j].az;
			ray_tangent(tt[j].th,tt[j].az,t0);
			G[ng][0]=1.0/vs;
			G[ng][1]=(-t0[0]/vs);
			G[ng][2]=(-t0[1]/vs);
			G[ng][3]=(-t0[2]/vs);
			d[ng]=ttime[k].obsTa - ttime[k].preTT;
			fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",
				ng, G[ng][0], G[ng][1], G[ng][2], G[ng][3],
				d[ng], ttime[k].obsTa, ttime[k].preTT);

			ng++;
			}
		else {
			ttime[k].ok=FALSE;
			fprintf(stderr,"warning: lost observation %d\n", k);
			}
		}

	if( ng==0 ) {
		fprintf(stderr,"Error: No rays traced in this iteration; iterations terminationg\n");
		exit(-1);
		}

	for( i=0; i<4; i++ ) for( j=0; j<4; j++ ) {
		GTG[i][j]=0.0;
		for( k=0; k<ng; k++ ) {
			GTG[i][j] += G[k][i]*G[k][j];
			}
		}

	maxG=fabs(GTG[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ ) if( maxG<fabs(GTG[i][j]) )
		maxG=fabs(GTG[i][j]);

	for( i=0; i<4; i++ ) GTG[i][i]+=maxG*damping;

	if( iter<min_c_it ) {
		fprintf(stderr,"\nThis iteration has constrained depth\n\n");
		for( i=0; i<3; i++ ) {
			GTG[i][3]=0.0;
			GTG[3][i]=0.0;
			}
		}

	for( i=0; i<4; i++ ) for( j=0; j<4; j++ ) GTGc[i][j]=GTG[i][j];

	for( i=0; i<4; i++ ) {
		GTd[i]=0.0;
		for( k=0; k<ng; k++ ) {
			GTd[i] += G[k][i]*d[k];
			}
		}

	if( iter<2 ) GTd[3]=0.0;

	fprintf(stderr,"\nNormal equations:\n");
	fprintf(stderr,"i\tGTG[i][0]\tGTG[i][1]\tGTG[i][2]\tGTG[i][3]\tGTd[i]\n");
	for( i=0; i<4; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\n", i, GTG[i][0],
			GTG[i][1], GTG[i][2], GTG[i][3], GTd[i] );
		}
	fprintf(stderr,"Absolute maximum of GTG is %f\n\n", maxG );

	gauss(GTG,GTd,4,4,1.0e-10*maxG,&ierror,TRUE);
	if( ierror != 0 ) {
		fprintf(stderr,"warning: singular matrix detected\n");
		}

	dX = sqrt(GTd[1]*GTd[1]+GTd[2]*GTd[2]+GTd[3]*GTd[3]);

	fprintf(stderr,"Estimated origin time %f, location perturbations (%f %f %f) with length %f\n\n",
		GTd[0]/vs, GTd[1], GTd[2], GTd[3], dX );

	ts = GTd[0]/vs;
	x0[0] += GTd[1];
	x0[1] += GTd[2];
	x0[2] += GTd[3];

	rms=0.0;
	for( i=0; i<ng; i++ ) {
		dT=0.0;
		for( j=0; j<4; j++ ) dT += G[i][j]*GTd[j];
		dT-=d[i];
		rms+=dT*dT;
		}
	rms = sqrt(rms/(double)ng);
	fprintf(stderr,"Linearized r.m.s. error estimate: %f\n\n", rms );

	gjinv(GTGc, GTGi, 4 , 4, 1.0e-14*maxG, &ierror );
	sigmat=rms*sqrt(GTGi[0][0])/vs;
	sigmax=rms*sqrt(GTGi[1][1]);
	sigmay=rms*sqrt(GTGi[2][2]);
	sigmaz=rms*sqrt(GTGi[3][3]);
	fprintf(stderr,
	"xs\tys\tzs\tt0\tsigma-x\tsigma-y\tsigma-z\tsigma-t\tn-data\titer\trms\n");
	fprintf(stderr, "%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%f\n",
		x0[0], x0[1], x0[2], ts, 
		sigmax, sigmay, sigmaz, sigmat, ng, iter, rms );

	if( ((rms<=tolerance)&&(iter>=min_it)) || (iter==(max_it-1)) ) {
	 fprintf(locfile,
	 "xs\tys\tzs\tt0\tsigma-x\tsigma-y\tsigma-z\tsigma-t\tn-data\titer\trms\n");
	 fprintf(locfile,
	 "%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%d\t%f\n",
		x0[0], x0[1], x0[2], ts, 
		sigmax, sigmay, sigmaz, sigmat, ng, iter, rms );
	 break;
	 }
	}

	exit(0);
	}

main_invert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est;
	struct triangle_rec tri;
	struct tetra_rec *pt;
	int biec;

	double thmin, thmax, azimin, azimax, vs;
	int nth, nazi;
	int ng, itt, ierror, match;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping;
	int tnode[4];
	int bracket_status;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, max_dT, qinvint;
	int iter, nx, ny, nz;

	double weight;

	char hitsfile[256];

	double new_damping, scx;


	if( argc!=13 ) {
		fprintf(stderr,"purpose: invert local traveltime data for improvements to node velocities\n");
		fprintf(stderr,"usage: %s starting_model_file model_pars_file.txt traveltimes.tsv max_dT damping thmin thmax nth azimin azimax nazi new_model_file\n",argv[0]);
		write_model_blerb(stderr);
		write_model_pars_blerb(stderr);
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              T0,  origin time in s\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: (-1) indicates closest arrival\n");
		fprintf(stderr,"      max_dT: if the absolute value of the traveltime residual, |dT|=|Tobs-Tpred(initial_model)|\n");
		fprintf(stderr,"              is greater than max_dt, that data is ignored\n");

		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      arrivals are searched for in a star with\n");
		fprintf(stderr,"         thmin, thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         nth, integer number of thetas in star\n");
		fprintf(stderr,"         azimin, azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d invert. Starting model file: %s\n\n", argv[1] );

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars(argv[2]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);

	if( n_model_pars<=1000 ) {
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
			for( j=0; j<model_par[i].n; j++ ) {
				fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
				}
			}
		}

	make_ttime(argv[3]);
	if( (ttfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);

	fprintf(stderr,"Total of %d data\n\n", n_ttimes);

	if( sscanf(argv[4], "%le", &max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read max_dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[12],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[12] );

	G=new_sparse_matrix(n_ttimes,n_model_pars,0);
	d=(double*)malloc((n_ttimes+2)*sizeof(double));
	ts=(double*)malloc((n_model_pars+2)*sizeof(double));
	ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
	GTd=(double*)malloc((n_model_pars+2)*sizeof(double));
	dm=(double*)malloc((n_model_pars+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (GTd==NULL) ||
		(ts==NULL) || (ts2==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting Point to Point Ray tracing of data\n\n");
	timestamp("at start of raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime*ttime[itt].ratio;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of raytracing");
	
	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	timestamp("after forming G and d");
	fprintf(stderr,"Damping: %e\n\n", damping );
	
	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n\n",
	n_model_pars, maxd );
	
	if( EXPLICIT_GTG ) {
	
	timestamp("starting to form GTG");
	invert_GTG=smt_times_sm(G,G);
	timestamp("after forming GTG");

	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_model_pars, n_model_pars,
	100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );

	timestamp("starting to solve linear system");

	for( i=0; i<n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after solving linear system");
	
	if( n_model_pars<1000 ) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	sm_times_v( invert_GTG, dm, ts );
	for( i=0; i<n_model_pars; i++ ) {
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
                if( n_model_pars<1000 ) {
                	fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
                	}
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);
	
	} /* end of explicit GTG */
	else {
	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );

	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	} /* end of implicit solution */

	if( n_model_pars < 1000 ) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}

	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	sm_times_v( G, dm, ts2 );

	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* update velocity of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += weight*dm[i];
			}
		}
	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);
	timestamp("final");

		
	exit(0);
	}

main_Iinvert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, rinterface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est;
	struct triangle_rec tri;
	struct tetra_rec *pt;

	double thmin, thmax, azimin, azimax, vs;
	int nth, nazi;
	int ng, itt, ierror, match;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping;
	int tnode[4];
	int bracket_status;
	int biec;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, max_dT, qinvint;
	int iter, nx, ny, nz;
	double weight, ntan[3];

	double new_damping, scx;

	char hitsfile[256];

	if( argc!=14 ) {
		fprintf(stderr,"purpose: invert local reflected traveltime data for improved position of flecting interface\n");
		fprintf(stderr,"usage: %s starting_model_file model_pars_file.txt traveltimes.tsv max_dT damping thmin thmax nth azimin azimax nazi new_model_file reflector_interface\n",argv[0]);
		write_model_blerb(stderr);
		write_refl_pars_blerb(stderr);
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              T0,  origin time in s\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: (-1) indicates closest arrival\n");
		fprintf(stderr,"                 Note: The traveltimes are for a reflected wave,\n");
		fprintf(stderr,"                 i.e., a wave reflected off a single interface\n");
		fprintf(stderr,"                 specified by the reflector_interface parameter\n");
		fprintf(stderr,"      max_dT: if the absolute value of the traveltime residual, |dT|=|Tobs-Tpred(initial_model)|\n");
		fprintf(stderr,"              is greater than max_dt, that datum is ignored\n");

		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      arrivals are searched for in a star with\n");
		fprintf(stderr,"         thmin, thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         nth, integer number of thetas in star\n");
		fprintf(stderr,"         azimin, azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		fprintf(stderr,"      reflector_interface: integer number of the reflector interface\n");
		fprintf(stderr,"         Must match an interface in the model file\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d Iinvert.\n\n");

	fprintf(stderr,"Reflecting interface: %s\n\n", argv[13] );
	if( sscanf(argv[13], "%d", &rinterface) != 1 ) {
		fprintf(stderr,"error: %s: cant read reflector_interface from command line\n", argv[0] );
		exit(-1);
		}

	fprintf(stderr,"Starting model file: %s\n\n", argv[1] );
	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars(argv[2]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	while( fscanf(mpfile,"%d",&(model_par[n_model_pars].n)) == 1 ) {
		if( n_model_pars > (MAX_MODEL_PARS-1) ) {
			fprintf(stderr,"error: %s: maximum model parameters (%d) exceeds limit of %d\n", argv[0], n_model_pars, MAX_MODEL_PARS-1 );
			exit(-1);
			}
		model_par[n_model_pars].node = (int *)
			malloc((model_par[n_model_pars].n+2)*sizeof(int));
		if(model_par[n_model_pars].node==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par node)!\n");
			exit(-1);
			}
		model_par[n_model_pars].weight = (double *)
			malloc((model_par[n_model_pars].n+2)*sizeof(double));
		if(model_par[n_model_pars].weight==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par weight)!\n");
			exit(-1);
			}
		model_par[n_model_pars].t = (double *)
			malloc(3*(model_par[n_model_pars].n+2)*sizeof(double));
		if(model_par[n_model_pars].t==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par t)!\n");
			exit(-1);
			}
		for( l=0; l<model_par[n_model_pars].n; l++ ) {
			if(fscanf(mpfile,"%d%d%d%le%le%le%le", &i,&j,&k,ntan,ntan+1,ntan+2,&weight)!=7) {
				fprintf(stderr,"error: %s: cant parse model_par_file <%s>\n", argv[0], argv[2] );
				exit(-1);
				}
			if( i<0 || i>=model_L || j<0 || j>=model_M || k<0 || k>=model_N ) {
				fprintf(stderr,"error: %s: bad node (%d %d %d) in model_par_file <%s>\n", argv[0], i, j, k, argv[2] );
				exit(-1);
				}
			nodeid=k*model_L*model_M+j*model_L+i;
			if( (model_par[n_model_pars].node[l]=findnode(nodeid))==(-1) ) {
				fprintf(stderr,"error: %s: couldnt find node (%d %d %d)\n", argv[0], i, j, k );
				exit(-1);
				}
			if( node[model_par[n_model_pars].node[l]].ifc != rinterface ) {
				fprintf(stderr,"error: %s: node (%d %d %d) not on reflector interface %d\n",
				argv[0], i, j, k, rinterface );
				exit(-1);
				}
			if( weight==0.0 ) {
				fprintf(stderr,"error: %s: model parameter %d has 0 weight\n", argv[0], n_model_pars );
				exit(-1);
				}
			model_par[n_model_pars].weight[l]=weight;
			unit_vector(ntan);
			for(i=0;i<3;i++) model_par[n_model_pars].t[3*l+i]=ntan[i];
			}
		n_model_pars++;
		}
	fclose(mpfile);


	if( n_model_pars < 1000 ) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d (%f %f %f) %f\n", model_par[i].node[j],
				model_par[i].t[3*j], model_par[i].t[3*j+1], model_par[i].t[3*j+2],
				model_par[i].weight[j]);
			}
		}

	make_ttime(argv[3]);
	if( (ttfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);

	fprintf(stderr,"Total of %d data\n\n", n_ttimes);

	if( sscanf(argv[4], "%le", &max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read max_dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[12],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}


	sprintf( hitsfile, "%s.hits", argv[12] );

	G=new_sparse_matrix(n_ttimes,n_model_pars,0);
	d=(double*)malloc((n_ttimes+2)*sizeof(double));
	ts=(double*)malloc((n_model_pars+2)*sizeof(double));
	ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
	GTd=(double*)malloc((n_model_pars+2)*sizeof(double));
	dm=(double*)malloc((n_model_pars+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (GTd==NULL) ||
		(ts==NULL) || (ts2==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting Point to Point Ray tracing of data\n\n");
	timestamp("at start of raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star_refl(thmin, thmax, nth, azimin, azimax, nazi, x0, interface, rinterface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox_refl(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, rinterface, NULL) ) {
		bracket_status=TRUE;
		if( refine_refl( x0, xsta, ysta, interface, rinterface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface_refl( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime*ttime[itt].ratio;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmpr(j,interface,rinterface,x0,t0,x1,t1,arc,n_arcs,traveltime,ng)
				* ttime[itt].ratio );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of raytracing");
	
	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	timestamp("after forming G and d");
	
	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars, maxd );
	
	fprintf(stderr,"Damping: %e\n\n", damping );

	if( EXPLICIT_GTG ) {
	
	timestamp("before forming GTG");

	invert_GTG=smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_model_pars, n_model_pars,
	100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );

	timestamp("after forming GTG");

	for( i=0; i<n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after solving linear system");


	if( n_model_pars<1000) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	sm_times_v( invert_GTG, dm, ts );
	for( i=0; i<n_model_pars; i++ ) {
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
                if( n_model_pars<1000) {
                	fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
                	}
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);
	
	} /* end explicit GTG */
	else {

	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );
	
	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	
	} /* end implicit solution */
	
	if( n_model_pars<1000) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}

	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	sm_times_v( G, dm, ts2 );

	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* update position of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			for( l=0; l<3; l++ )
				node[k].x[l] += dm[i]*model_par[i].weight[j]*model_par[i].t[3*j+l];
			}
		}
	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);
	timestamp("final");
		
	exit(0);
	}

main_qinvert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est, Q;
	struct triangle_rec tri;
	struct tetra_rec *pt;

	double thmin, thmax, azimin, azimax, vs;
	int nth, nazi;
	int ng, itt, ierror, match;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping;
	int tnode[4];
	int bracket_status;
	int biec;


	struct sparse_matrix *G, *invert_GTG;
	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, qinvint;
	int iter, nx, ny, nz;

	double weight;

	char hitsfile[256];

	double new_damping, scx;

	if( argc!=12 ) {
		fprintf(stderr,"purpose: invert local path-averaged quality factor data to improve model\n");
		fprintf(stderr,"usage: %s starting_model_file model_pars_file.txt Qs.tsv damping thmin thmax nth azimin azimax nazi new_model_file\n",argv[0]);
		write_model_blerb(stderr);
		write_model_pars_blerb(stderr);
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              Q,  path-averaged quality factor\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc; or\n");
		fprintf(stderr,"                  order=(-1) to have program use closest arrival\n");
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      arrivals are searched for in a star with\n");
		fprintf(stderr,"         thmin, thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         nth, integer number of thetas in star\n");
		fprintf(stderr,"         azimin, azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d Qinvert. Starting model file: %s\n\n", argv[1] );
	timestamp("at start");


	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( !QDEFINED ) {
		fprintf(stderr,"error: %s: Q must be defined in model file <%s>\n",
		argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars(argv[2]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);

	if( n_model_pars<1000) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
			}
		}

	make_ttime(argv[3]);
	if( (ttfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 9 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tQ\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&Q,&order)==9){
		if( n_ttimes > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many Qs\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=0.0;
		ttime[n_ttimes].obsTT=0.0;
		ttime[n_ttimes].obsTa=0.0;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=1.0;
		ttime[n_ttimes].qinv=1.0/Q;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0], ttime[n_ttimes].x0[1], ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0], ttime[n_ttimes].x1[1], ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface, 1.0/ttime[n_ttimes].qinv, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);

	fprintf(stderr,"Total of %d data\n\n", n_ttimes);

	if( sscanf(argv[4], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[11],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[11] );

	G=new_sparse_matrix(n_ttimes,n_model_pars,0);
	ts=(double*)malloc((n_model_pars+2)*sizeof(double));
	ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
	d=(double*)malloc((n_ttimes+2)*sizeof(double));
	GTd=(double*)malloc((n_model_pars+2)*sizeof(double));
	dm=(double*)malloc((n_model_pars+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (ts==NULL) || (GTd==NULL) ||
		         (ts2==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting Point to Point Ray tracing of data\n\n");
	fprintf(stderr,"#\tobsQinv\tpreQinv\tobsQ\tpreQ\tQinv_residual\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );
	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].preqinv=qinvint/traveltime;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\n", itt, 
			ttime[itt].qinv, ttime[itt].preqinv,
			1.0/ttime[itt].qinv, 1.0/ttime[itt].preqinv,
			ttime[itt].qinv-ttime[itt].preqinv);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].qinv-ttime[itt].preqinv;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the qinv ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dqinvdmp(j,arc,n_arcs,traveltime,qinvint) );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	ng++;

	} /* next traveltime */
	
	timestamp("after raytracing");

	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	k=sparse_count(G);
	maxG=sparse_absmax(G);
	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}
	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	
	fprintf(stderr,"Damping: %e\n\n", damping );

	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr, "GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars, maxd );
	
	if( EXPLICIT_GTG ) {

	invert_GTG=smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}

	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_model_pars, n_model_pars,
	100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );
	timestamp("after forming GTG");

	
	for( i=0; i<n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion (explicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after explicit solution");
	
	sm_times_v( invert_GTG, dm, ts );
	if( n_model_pars < 1000 ) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		if( n_model_pars < 1000 ) fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);
	

	} /* end explicit GTG */
	else {
	
	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );

	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	} /* end implicit GTG */
		
	if( n_model_pars < 1000 ) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}

	sm_times_v( G, dm, ts2 );
	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dQinv\tpred_dQinv\tdifference\n");
	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms Qinv error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));

	/* update velocity of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].qinv += weight*dm[i];
			}
		}
	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}

	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);

	timestamp("finish");

	exit(0);
	}

main_invert_tele(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec, dot0;
	double x0[3], t0[3], x1[3], t1[3], v[4], theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta;
	struct triangle_rec tri;
	struct tetra_rec *pt;
	double *blocksum, *blockones;

	double xmin, xmax, ymin, ymax, vapp, z0, vs, x_est, y_est, vp, px, py;
	int nx, ny, t, my_last_tetra;
	int ng, itt, ierror, match, index;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping, qinvint;
	int tnode[4];
	int biec;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, mean;
	int iter, nz, *dindex, *dptr;

	double weight, t0weight;

	char hitsfile[256];

	double new_damping, scx;
	
	timestamp("at start");


	my_last_tetra=(-1);

	if( (argc!=13) && (argc!=14) ) {
		fprintf(stderr,"purpose: invert telseismic traveltime data to improve model\n");
		fprintf(stderr,"usage: %s starting_model_file model_pars_file.txt traveltimes.tsv damping xmin xmax nx ymin ymax ny z0 new_model_file\n",argv[0]);
		fprintf(stderr,"or\n");
		fprintf(stderr,"       %s starting_model_file model_pars_file.txt traveltimes.tsv damping xmin xmax nx ymin ymax ny z0 new_model_file flag\n",argv[0]);
		write_tele_note(stderr);
		write_model_blerb(stderr);
		write_model_pars_blerb(stderr);
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              index: an integer that is the same for\n");
		fprintf(stderr,"                  all observations of a single teleseismic plane wave\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common index!\n");
		fprintf(stderr,"              azimuth, in deg, of plane wave\n");
		fprintf(stderr,"              apparent-velocity, in km/s, of plane wave\n");
		fprintf(stderr,"                  (but be warned;  azimuth and apparent-velocity ought to be the same for common index!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common index by interface!)\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"                  Note: The ratio allows S wave traveltimes\n");
		fprintf(stderr,"                  to be used in conjunction with a P wave\n");
		fprintf(stderr,"                  model inversion.  S wave apparent velocities\n");
		fprintf(stderr,"                  are specified in the traveltime file, along\n");
		fprintf(stderr,"                  with an appropriate P:S velocity ratio.  The\n");
		fprintf(stderr,"                  S wave apparent velocity is scaled to a P wave\n");
		fprintf(stderr,"                  apparent velocity prior to raytracing, and then\n");
		fprintf(stderr,"                  the final traveltime is scaled back to an S wave\n");
		fprintf(stderr,"                  traveltime after raytracing. All traveltimes\n");
		fprintf(stderr,"                  in a planewave group must have the same ratio.\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: Closest arrival option (-1) cannot\n");
		fprintf(stderr,"                 be used since the arrival time at the bottom\n");
		fprintf(stderr,"                 of the model is presumed unknown.  It will\n");
		fprintf(stderr,"                 be reset to 0.\n");
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      arrivals are searched for in a turf with\n");
		fprintf(stderr,"         xmin, xmax, min, max x values, km\n");
		fprintf(stderr,"         nx, integer number of x values in turf\n");
		fprintf(stderr,"         ymin, ymax, min, max y values, km\n");
		fprintf(stderr,"         ny, integer number of y values in turf\n");
		fprintf(stderr,"      z0, in km, reference depth for plane wave\n");
		fprintf(stderr,"         (rays shot from this depth, so it should\n");
		fprintf(stderr,"         be within the model but near its bottom)\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		fprintf(stderr,"      flag: either y or n to either, whether or not to estimate\n");
		fprintf(stderr,"         plane wave arrival times. Default is n\n");
		exit(-1);
		}
	if( argc==13 ) {
		dot0=FALSE;
		}
	else if ( (argv[13][0]=='y') || (argv[13][0]=='Y') ) {
		dot0=TRUE;
		}
	else if ( (argv[13][0]=='n') || (argv[13][0]=='N') ) {
		dot0=FALSE;
		}
	else {
		fprintf(stderr,"error: %s: bad flag <%s>. Must be y or n\n", argv[0], argv[13] );
		exit(-1);
		}

	if( dot0 ) fprintf(stderr,"Raytrace3d teleseismic_invert with t0. Starting model file: %s\n\n", argv[1] );
	else fprintf(stderr,"Raytrace3d teleseismic_invert without t0. Starting model file: %s\n\n", argv[1] );

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars(argv[2]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);

	if( n_model_pars<100 ) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
			}
		}

	make_ttime(argv[3]);
	if( (ttfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 10 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;

 	while( fscanf(ttfile,"%d%le%le%le%le%le%d%le%le%d", &index,&azi,&vapp,
		&xsta,&ysta,&zsta,&interface,&Ta,&ratio,&order)==10){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		if( order==(-1) ) order=0;
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].index=index;
		ttime[n_ttimes].az=azi;
		ttime[n_ttimes].vapp=vapp;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		/* assign planewave group index */
		if( n_ttimes==0 ) {
			n_pwg=0;
			}
		else if( ttime[n_ttimes].index!=ttime[n_ttimes-1].index ) {
			n_pwg++;
			}
		ttime[n_ttimes].pwg=n_pwg;
		n_ttimes++;
		}
	n_pwg++;
	fclose(ttfile);
	
	if( n_ttimes < 1000 ) {
		fprintf(stderr,"\nInput Data\n\n");
		fprintf(stderr,"#\tindex\tazi\tappvel\txr\tyr\tzr\tinterface\tTa\tratio\torder\tpwg\n");
		for( i=0; i<n_ttimes; i++ ) {
			fprintf(stderr,"%d\t%d\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%d\t%d\n",
				i,
				ttime[i].index,ttime[i].az,ttime[i].vapp,
				ttime[i].x1[0],ttime[i].x1[1],ttime[i].x1[2],
				ttime[i].interface,
				ttime[i].obsTa, 
				ttime[i].ratio, ttime[i].obsorder, ttime[i].pwg );
			}
		}

	fprintf(stderr,"Total of %d data in %d planewave groups\n\n", n_ttimes, n_pwg);

	if( sscanf(argv[4], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &xmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &xmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &nx) != 1 ) {
		fprintf(stderr,"error: %s: cant read nx from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &ymin) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &ymax) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &ny) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%le", &z0) != 1 ) {
		fprintf(stderr,"error: %s: cant read z0 from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[12],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[12] );

	if( dot0 ) {
		G=new_sparse_matrix(n_ttimes,n_model_pars+n_pwg,0);
		ts=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));
		ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
		d=(double*)malloc((n_ttimes+2)*sizeof(double));
		dindex=(int*)malloc((n_ttimes+2)*sizeof(int));
		dptr=(int*)malloc((n_ttimes+2)*sizeof(int));
		GTd=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));
		dm=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));
		}
	else {
		G=new_sparse_matrix(n_ttimes,n_model_pars,0);
		ts=(double*)malloc((n_model_pars+2)*sizeof(double));
		ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
		d=(double*)malloc((n_ttimes+2)*sizeof(double));
		dindex=(int*)malloc((n_ttimes+2)*sizeof(int));
		dptr=(int*)malloc((n_ttimes+2)*sizeof(int));
		GTd=(double*)malloc((n_model_pars+2)*sizeof(double));
		dm=(double*)malloc((n_model_pars+2)*sizeof(double));
		}

	if( (G==NULL) || (d==NULL) || (dindex==NULL) || (dptr==NULL) ||(ts==NULL) ||
		(ts2==NULL) || (GTd==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}

	blocksum=(double*)malloc((n_model_pars+2)*sizeof(double));;
	blockones=(double*)malloc((n_ttimes+2)*sizeof(double));
	if( (blocksum==NULL) || (blockones==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (blocksum ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting Plane-Wave to Point Ray tracing of data\n\n");
	timestamp("starting raytrace");

	fprintf(stderr,"#\tobsTa\tT-predicted\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].index==index && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		index=ttime[itt].index;
		azi=ttime[itt].az;
		vapp=ttime[itt].vapp;
		interface=ttime[itt].interface;
		ratio=ttime[itt].ratio;
		fprintf(stderr,"Shooting turf pattern from reference depth %f at azimuth %f and app-velocity %f and ratio %f to interface %d\n",
		z0, azi, vapp, ratio, interface);
		fprintf(stderr,"Turf ranges: x (%f %f %d) y (%f %f %d)\n",
		xmin, xmax, nx, ymin, ymax, ny );
		shoot_turf(xmin, xmax, nx, ymin, ymax, ny, azi, vapp*ratio, z0, 0.0, interface);
		fprintf(stderr,"done with turf\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	for( i=0; i<(m_turf-1); i++ )
	for( j=0; j<(n_turf-1); j++ )
	for( l=0; l<2; l++ ) {
			if( inturfbox(i, j, l, xsta, ysta, &x_est,
				&y_est, &tri, x0, interface, azi,vapp*ratio,z0,0.0,NULL) )
			if( refine_turf( x0, xsta, ysta, interface, &x_est, &y_est,
				NULL, &n_arcs, &traveltime, &tri, azi,vapp*ratio,z0,0.0) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=x_est;
				tt[n_tts].th=y_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
			status=TRUE;
			x0[0]=tt[j].az;
			x0[1]=tt[j].th;
			x0[2]=z0;
			last_tetra=my_last_tetra;
			if( find_tetra(x0,&t) ) {
				vp = eval_velocity(&tetra[t],x0);
				}
			else status=FALSE;
			if( vp>=(vapp*ratio) ) status=FALSE;
			my_last_tetra=last_tetra;
			theta = 180.0-RTOD*asin(vp/(vapp*ratio));
			ray_tangent(theta,azi,t0);
			px = t0[0]/vp; py=t0[1]/vp;
		if( status) if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( status) if( !hitflag ) status=FALSE;
		}
	else status=FALSE;
	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].x0[0]=tt[j].az;
		ttime[itt].x0[1]=tt[j].th;
		ttime[itt].x0[2]=z0;
		ttime[itt].preTa=ratio*(
			traveltime + px*ttime[itt].x0[0] + py*ttime[itt].x0[1] );
		fprintf(stderr,"%d\t%f\t%f\n", itt, ttime[itt].obsTa, ttime[itt].preTa );
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		continue;
		}

	d[ng]=ttime[itt].obsTa-ttime[itt].preTa;
	dindex[ng]=ttime[itt].index;
	dptr[ng]=itt;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio );
			}
		else {
			/* G[ng][j] = 0.0; */
			;
			}
		}

	ng++;

	} /* next traveltime */
	
	timestamp("after raytracing");


	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d before demean:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	t0weight=maxG/10.0;
	k=sparse_count(G);
	fprintf(stderr,"\nBefore adjustment for t0\n");
	fprintf(stderr,"G is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}
	fprintf(stderr,"d is %d by 1 and has abs max of %f\n\n", ng, maxd );

	if (dot0 ) {
		/* add elements of G associate with dt0 model parameters */
		for( i=0; i<ng; i++ ) {
			j = ttime[dptr[i]].pwg;
			set_sparse_value( G, i, n_model_pars+j, t0weight );
			}
		/* subtract <ta> - <tt> from d */	
		i=0;
		while( i<ng ) {
			j=i;
			while( (j<ng) && (dindex[j]==dindex[i]) ) j++;
			mean=0.0;
			for( k=i; k<j; k++ ) mean+=d[k];
			mean /= (double)(j-i);
			for( k=i; k<j; k++ ) d[k]-=mean;
			i=j;
			}		
		}
	else {
		/* planewave-wide demean of equations */
		fprintf(stderr,"rows of G that are in same planewave group\n");
		i=0;
		while( i<ng ) {
			j=i;
			while( (j<ng) && (dindex[j]==dindex[i]) ) j++;
			fprintf(stderr,"%d to %d\n", i, j-1); fflush(stderr);
			for( k=0; k<ng; k++ ) blockones[k]=0.0;
			mean=1.0/(double)(j-i);
			for( k=i; k<j; k++ ) blockones[k]=mean;
			smt_times_v( G, blockones, blocksum );
			for( l=0; l<n_model_pars; l++ ) {
				for( k=i; k<j; k++ ) blockones[k]=(-blocksum[l]);
				sparse_add_to_col( G, l, blockones );
				}
			mean=0.0;
			for( k=i; k<j; k++ ) mean+=d[k];
			mean /= (double)(j-i);
			for( k=i; k<j; k++ ) d[k]-=mean;
			i=j;
			}
		fprintf(stderr,"\n");
		}

	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d after demean:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nAfter adjustment for t0\n");
	timestamp("after adjustment");

	if( dot0 )
		fprintf(stderr,"G is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars+n_pwg, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	else
		fprintf(stderr,"G is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );

		
	fprintf(stderr,"d is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	
	smt_times_v( G, d, GTd );

	if( dot0 ) {
		maxd=0.0;
		for( i=0; i<n_model_pars+n_pwg; i++ ) {
			dv=fabs(GTd[i]);
			if( dv>maxd ) maxd=dv;
			}
		fprintf(stderr, "GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars+n_pwg, maxd );
		}
	else {
		maxd=0.0;
		for( i=0; i<n_model_pars; i++ ) {
			dv=fabs(GTd[i]);
			if( dv>maxd ) maxd=dv;
			}
		fprintf(stderr, "GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars, maxd );
		}
	
	if( EXPLICIT_GTG ) { /* start of explicit GTG part */

	invert_GTG = smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);
	fprintf(stderr,"Damping: %e\n\n", damping );
	timestamp("after explicitly forming GTG");

	if( dot0 ) {
		fprintf(stderr,
		"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
		n_model_pars+n_pwg, n_model_pars+n_pwg,
		100.0*(double)k/((double)(n_model_pars+n_pwg)*(double)(n_model_pars+n_pwg)), maxG );
		}
	else {
		fprintf(stderr,
		"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
		n_model_pars, n_model_pars,
		100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );
		}

	if( dot0 ) {
		for( i=0; i<n_model_pars+n_pwg; i++ ) dm[i]=0.0;
		if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+n_pwg+10,&iter,&err,&biec) ) {
			fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
			if(biec>0)exit(-1);
			}
		}
	else {
		for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
		if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
			fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
			if(biec>0)exit(-1);
			}
		}
		
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after explicit solution");
	
	sm_times_v( invert_GTG, dm, ts );

	if( n_model_pars<1000) fprintf(stderr,"\nGTG*dm=GTd\n");
	rms=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		if( n_model_pars<1000) fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);
	
	} /* end of explicit GTG part */
	
	else { /* start of implicit GTG part */
	
	fprintf(stderr,"Damping: %e\n\n", damping );
			
	if( dot0 ) {
		scx = ATAabsmax( G );
		new_damping = damping*scx;
		fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
			damping, scx, new_damping );
		for( i=0; i<n_model_pars+n_pwg; i++ ) dm[i]=0.0;
		if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+n_pwg+10, &iter,&err,&biec ) ) {
			fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
			if(biec>0)exit(-1);
			}
		}
	else {
		scx = ATAabsmax( G );
		new_damping = damping*scx;
		fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
			damping, scx, new_damping );
		for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
		if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+10, &iter,&err,&biec ) ) {
			fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
			if(biec>0)exit(-1);
			}
		}
		
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");


	} /* end of implicit GTG part */

	if( n_model_pars<1000 ) {
		fprintf(stderr, "Velocity Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}
		
	if( dot0 ) {
		mean=fabs(dm[n_model_pars]);
		for( i=n_model_pars; i<n_model_pars+n_pwg; i++ ) {
			maxd=fabs(dm[i]);
			if( maxd>mean ) mean=maxd;
			}
		fprintf(stderr,"The largest dt0 is %e\n", mean*t0weight );
		}

	

	sm_times_v( G, dm, ts2 );
	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* update velocity of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += weight*dm[i];
			}
		}
	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}

	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);
	timestamp("at end");

		
	exit(0);
	}


int shoot_to_interface( x0, t0, x1, t1, arc, n_arcs, nmax, smax,
	interface, hitflag, traveltime, qinvint, ec )
double x0[3], t0[3];
double x1[3], t1[3];
struct arc_rec arc[];
int *n_arcs, nmax;
double smax;
int interface, *hitflag;
double *traveltime;
double *qinvint;
int *ec;
{
	int i, myt, ef, hit, exit_face;
	double xx0[3], xx1[3], tt0[3], tt1[3], xe[3], s, S, epsilon=1.0e-6;
	struct tetra_rec *pt;
	double ttt, tt, qinv, qinvt;
	double xm[3];

	calls_to_shoot++;

	(*n_arcs)=0;
	s=0.0;
	ttt=0.0;
	qinvt=0.0;

	for(i=0;i<3;i++) {
		xx0[i]=x0[i];
		tt0[i]=t0[i];
		}

	myt=(-1);
	for(i=0;i<3;i++) xe[i]=xx0[i]+epsilon*tt0[i];
	if( !find_tetra(xe, &myt) ) {
		fprintf(stderr,"error: starting point not in tetra\n");
		*ec=1;
		return(FALSE);
		}

	hit=FALSE;

	while( myt != (-1) ) {

		if( (*n_arcs) >= nmax ) {*ec=2; return(FALSE);}

		if( arc != NULL ) {
			if(!circular_ray_exits_tetra(&(tetra[myt]),xx0,tt0,xx1,tt1,
				&arc[*n_arcs], &tt, &S, &qinv, &exit_face)) {
				*ec=3;
				return(FALSE);
				}
			}
		else {
			if(!circular_ray_exits_tetra(&(tetra[myt]),xx0,tt0,xx1,tt1,
				NULL, &tt, &S, &qinv, &exit_face)) {
				*ec=4;
				return(FALSE);
				}
			}

		s += S;
		ttt += tt;
		qinvt += qinv;
		if( s>smax ) {*ec=5; return(FALSE);}

		for(i=0;i<3;i++) {
			xx0[i]=xx1[i];
			tt0[i]=tt1[i];
			}

		pt=&(tetra[myt]);
		ef=exit_face;
		if( ef == 1 ) myt=pt->t1;
		else if( ef == 2 ) myt=pt->t2;
		else if( ef == 3 ) myt=pt->t3;
		else if( ef == 4 ) myt=pt->t4;
		else {
			fprintf(stderr,"warning: problem with ray\n");
			myt=(-1);
			}

		if( pt->ifc[ef-1] == interface ) {
			hit=TRUE;
			(*n_arcs)++;
			break;
			}

		(*n_arcs)++;

		}

	for(i=0;i<3;i++) {
		x1[i]=xx1[i];
		t1[i]=tt1[i];
		}

	*hitflag = hit;
	if( traveltime != NULL ) *traveltime=ttt;
	if( qinvint != NULL ) *qinvint=qinvt;


	*ec=0;
	return(TRUE);
	}

int shoot_with_max_traveltime( x0, t0, x1, t1, arc, n_arcs, nmax, smax,
	max_traveltime, traveltime, qinvint, ec )
double x0[3], t0[3];
double x1[3], t1[3];
struct arc_rec arc[];
int *n_arcs, nmax;
double smax;
double max_traveltime;
double *traveltime;
double *qinvint;
int *ec;
{
	int i, myt, ef, hit, exit_face;
	double xx0[3], xx1[3], tt0[3], tt1[3], xe[3], s, S, epsilon=1.0e-6;
	struct tetra_rec *pt;
	double ttt, tt, qinv, qinvt;
	double xm[3];

	calls_to_shoot++;

	(*n_arcs)=0;
	s=0.0;
	ttt=0.0;
	qinvt=0.0;

	for(i=0;i<3;i++) {
		xx0[i]=x0[i];
		tt0[i]=t0[i];
		}

	myt=(-1);
	for(i=0;i<3;i++) xe[i]=xx0[i]+epsilon*tt0[i];
	if( !find_tetra(xe, &myt) ) {
		fprintf(stderr,"error: starting point not in tetra\n");
		*ec=1;
		return(FALSE);
		}

	hit=FALSE;

	while( myt != (-1) ) {

		if( (*n_arcs) >= nmax ) {*ec=2; return(FALSE);}

		if( arc != NULL ) {
			if(!circular_ray_exits_tetra(&(tetra[myt]),xx0,tt0,xx1,tt1,
				&arc[*n_arcs], &tt, &S, &qinv, &exit_face)) {
				*ec=3;
				return(FALSE);
				}
			}
		else {
			if(!circular_ray_exits_tetra(&(tetra[myt]),xx0,tt0,xx1,tt1,
				NULL, &tt, &S, &qinv, &exit_face)) {
				*ec=4;
				return(FALSE);
				}
			}

		s += S;
		ttt += tt;
		qinvt += qinv;

		for(i=0;i<3;i++) {
			xx0[i]=xx1[i];
			tt0[i]=tt1[i];
			}

		pt=&(tetra[myt]);
		ef=exit_face;
		if( ef == 1 ) myt=pt->t1;
		else if( ef == 2 ) myt=pt->t2;
		else if( ef == 3 ) myt=pt->t3;
		else if( ef == 4 ) myt=pt->t4;
		else {
			fprintf(stderr,"warning: problem with ray\n");
			myt=(-1);
			}


		(*n_arcs)++;

		if( (ttt>max_traveltime) || (s>smax) ) break;

		}

	for(i=0;i<3;i++) {
		x1[i]=xx1[i];
		t1[i]=tt1[i];
		}

	if( traveltime != NULL ) *traveltime=ttt;
	if( qinvint != NULL ) *qinvint=qinvt;

	*ec=0;
	return(TRUE);
	}

/* v(x,y,z) = vc[0]*x + vc[1]*y + vc[2]*z + vc[3] */
int velocity_eqn( t )
struct tetra_rec *t;
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=t->x1[i];
		G[1][i]=t->x2[i];
		G[2][i]=t->x3[i];
		G[3][i]=t->x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(t->v[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		t->vc[i]=d[i];
		}

	t->gs=sqrt(dot(d,d));

	if( t->gs != 0.0 ) {
		for( i=0; i<3; i++ ) {
			t->g[i]=d[i];
			t->gn[i]=d[i]/t->gs;
			}
		}
	else {
		for( i=0; i<3; i++ ) {
			t->g[i]=d[i];
			t->gn[i]=0.0;
			}
		t->gn[2]=1.0;
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

/* qinv(x,y,z) = qinvc[0]*x + qinvc[1]*y + qinvc[2]*z + qinvc[3] */
int qinv_eqn( t )
struct tetra_rec *t;
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=t->x1[i];
		G[1][i]=t->x2[i];
		G[2][i]=t->x3[i];
		G[3][i]=t->x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(t->qinv[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		t->qinvc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

int velocity_eqn2( x1, x2, x3, x4, v, vc )
double x1[3], x2[3], x3[3], x4[3], *v[4], vc[4];
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=x1[i];
		G[1][i]=x2[i];
		G[2][i]=x3[i];
		G[3][i]=x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(v[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		vc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

int qinv_eqn2( x1, x2, x3, x4, qinv, qinvc )
double x1[3], x2[3], x3[3], x4[3], *qinv[4], qinvc[4];
{
	int i, j, ierror;
	double G[4][4], d[4], test;
	
	for( i=0; i<3; i++ ) {
		G[0][i]=x1[i];
		G[1][i]=x2[i];
		G[2][i]=x3[i];
		G[3][i]=x4[i];
		}
	for( i=0; i<4; i++ ) {
		G[i][3]=1.0;
		d[i]=*(qinv[i]);
		}
	test=fabs(G[0][0]);
	for( i=0; i<4; i++ ) for( j=0; j<4; j++ )
		if( test<fabs(G[i][j]) ) test=fabs(G[i][j]);
	gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );

	for( i=0; i<4; i++ ) {
		qinvc[i]=d[i];
		}

	if( ierror==0 ) return(TRUE); else return(FALSE);
	}

double eval_velocity(t, x)
struct tetra_rec *t;
double x[3];
{
	double v;
	v = x[0]*(t->vc[0]) +  x[1]*(t->vc[1]) +  x[2]*(t->vc[2]) + t->vc[3];
	return(v);
	}

double eval_velocity2(x, vc)
double x[3];
double vc[4];
{
	double v;
	v = x[0]*vc[0] +  x[1]*vc[1] +  x[2]*vc[2] + vc[3];
	return(v);
	}

double eval_qinv(t, x)
struct tetra_rec *t;
double x[3];
{
	double v;
	v = x[0]*(t->qinvc[0]) +  x[1]*(t->qinvc[1]) +  x[2]*(t->qinvc[2]) + t->qinvc[3];
	return(v);
	}

double eval_qinv2(x, qinvc)
double x[3];
double qinvc[4];
{
	double v;
	v = x[0]*qinvc[0] +  x[1]*qinvc[1] +  x[2]*qinvc[2] + qinvc[3];
	return(v);
	}

double eval_arc_velocity(a, x)
struct arc_rec *a;
double x[3];
{
	double v;
	v = x[0]*(a->vc[0]) +  x[1]*(a->vc[1]) +  x[2]*(a->vc[2]) + a->vc[3];
	return(v);
	}

double eval_arc_qinv(a, x)
struct arc_rec *a;
double x[3];
{
	double v;
	v = x[0]*(a->qinvc[0]) +  x[1]*(a->qinvc[1]) +  x[2]*(a->qinvc[2]) + a->qinvc[3];
	return(v);
	}

int circular_ray_exits_tetra( t, rx0, rt0, rx1, rt1, arc, traveltime, S, qinvint, ef )
struct tetra_rec *t;
double rx0[3], rt0[3]; /*ray starting position and tangent */
double rx1[3], rt1[3]; /*ray ending position and tangent   */
struct arc_rec *arc;
double *traveltime;
double *S; /* arc length */
double *qinvint;

int *ef; /*exit face*/
{
	int i, j, nints, found, status, method;
	double a, vs, p, theta, R, T, T1, T2, v0, qinv, qinv1, qinv2;
	double b0[3], b1[3], b2[3], s[3][3], st[3][3];
	double nx1[3], nx2[3], nx3[3], nx4[3], rc[3];
	double nrx0[3], nrt0[3], nrx1[3], nrt1[3];
	double shift[3];
	double psi1, psi2;
	struct int_rec intersecs[20];
	double tvec[3];
	double epsilon;
	double vc[4], qinvc[4];
	double xcir[3], rxcir[3];

	int k;
	double I[3][3];

	epsilon=1.0e-10;

	/* velocity at starting point */
	vs = eval_velocity(t, rx0);

	/* new basis in which gradient is vertical and ray in (x',z') plane */
	for( i=0; i<3; i++ ) b2[i]=t->gn[i];
	cross( b2, rt0, b1 ); j=unit_vector(b1);
	if( !j ) { /*special case of ray parallel to gradient */
		do {
			ranvec(tvec);
			cross( b2, tvec, b1 );
			j=unit_vector(b1);
			} while ( !j );
		}
	cross( b1, b2, b0 ); unit_vector(b0);

	newbasisrot( b0, b1, b2, s );
	for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) st[i][j]=s[j][i];

	if( arc!=NULL ) for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) {
		arc->s[i][j]=s[i][j];
		arc->st[i][j]=st[i][j];
		}

	/*rotate everything to new coordinate system */
	rotate3(t->x1,s,nx1);
	rotate3(t->x2,s,nx2);
	rotate3(t->x3,s,nx3);
	rotate3(t->x4,s,nx4);
	rotate3(rx0,s,nrx0);
	rotate3(rt0,s,nrt0);

	/* local ray parameter, angle of incidence, radius of circle */
	a=dot(rt0,b2); if(a<(-1.0)) a=(-1.0); else if( a>1.0 ) a=1.0;
	theta=RTOD*acos(a);
	p = sin(DTOR*theta)/vs;
	a = fabs(t->gs*p);
	if( a>1.0e-12 ) R=1.0/a; else R=1.0e12;

	/* center of circle */
	rc[0]=nrx0[0]+R*cos(DTOR*theta);
	rc[1]=nrx0[1];
	rc[2]=nrx0[2]-R*sin(DTOR*theta);

	/*coordinate shift so that center of circle is origin */
	for( i=0; i<3; i++ ) shift[i]=rc[i];

	if( arc!=NULL ) for( i=0; i<3; i++ ) arc->shift[i]=shift[i];

	for( i=0; i<3; i++ ) {
		nx1[i] -= shift[i];
		nx2[i] -= shift[i];
		nx3[i] -= shift[i];
		nx4[i] -= shift[i];
		nrx0[i] -= shift[i];
		rc[i] -= shift[i];
		}

	status=velocity_eqn2( nx1, nx2, nx3, nx4, t->v, vc );

	if( arc!=NULL ) {
		for( i=0; i<4; i++ ) arc->vc[i]=vc[i];
		arc->tetra = t->id;
		arc->ptetra = t;
		}

	if( (qinvint!=NULL) || (arc!=NULL) ) {
		status=qinv_eqn2( nx1, nx2, nx3, nx4, t->qinv, qinvc );
		}

	/* velocity at center of circle */
	v0=vc[3];

	/* entry angle */
	psi1=270.0+theta; while( psi1>=360.0 ) psi1-=360.0; while( psi1<0.0 ) psi1+=360.0;
	if( arc!=NULL ) arc->psi1 = psi1;

	nints=0.0;
	circle_intersects_plane( R, nx1, nx2, nx3, intersecs, &nints, 4, nx4 );
	circle_intersects_plane( R, nx1, nx2, nx4, intersecs, &nints, 3, nx3 );
	circle_intersects_plane( R, nx2, nx3, nx4, intersecs, &nints, 1, nx1 );
	circle_intersects_plane( R, nx1, nx3, nx4, intersecs, &nints, 2, nx2 );

	for( i=0; i<nints; i++ ) {
		a = intersecs[i].psi-psi1;
		while(a<0.0) a+=360.0;
		while(a>=360.0) a-=360.0;
		if( isnan(a) ) {
			fprintf(stderr,"warning: NaN in circle calculation\n");
			a=360.0;
			}
		if( a>(360.0-epsilon) ) a=0.0;
		intersecs[i].dpsi = a;
		}
        qsort( intersecs, nints, sizeof(struct int_rec), compare_int );

	found=(-1);
	for( i=0; i<nints; i++ ) {
		if( (fabs(R*intersecs[i].dpsi)>epsilon)
		 && (intersecs[i].dpsi<=(180.0+epsilon))
		 && (intersecs[i].inside=
			in_tetrahedron( nx1, nx2, nx3, nx4, intersecs[i].xme)) ) {
			found=i;
			break;
			}
		}

	if( found!=(-1) ) {
		for( i=0; i<3; i++ ) nrx1[i]=intersecs[found].x[i]+shift[i];
		nrt1[0]=sin(DTOR*(90.0+intersecs[found].psi));
		nrt1[1]=0.0;
		nrt1[2]=cos(DTOR*(90.0+intersecs[found].psi));
		rotate3(nrx1,st,rx1);
		rotate3(nrt1,st,rt1);
		psi2=intersecs[found].psi;
		while( psi2 < psi1 ) psi1-=360.0;
		if( (traveltime!=NULL) || (arc!=NULL) ) {
			T2 = ttintegral(v0,t->gs,R,psi2,&method);
			T1 = ttintegral(v0,t->gs,R,psi1,&method);
			T=(T2-T1);
			}
		if( traveltime!=NULL ) (*traveltime)=T;
		(*S)=DTOR*(psi2-psi1)*R;
		if( (qinvint!=NULL) || (arc!=NULL) ) {
			qinv2 = qintegral( qinvc[3], qinvc[0], qinvc[2], R, t->gs, psi2 );
			qinv1 = qintegral( qinvc[3], qinvc[0], qinvc[2], R, t->gs, psi1 );
			qinv = qinv2-qinv1;
			}
		if( qinvint!=NULL ) *qinvint = qinv;
		(*ef)=intersecs[found].face;
		if( arc!=NULL ) {
			arc->exit_face = intersecs[found].face;
			arc->R=R;
			arc->psi2 = intersecs[found].psi;
			while( arc->psi2 < arc->psi1 ) arc->psi1 -= 360.0;
			arc->S = (*S);
			arc->T = T;
			arc->qinvint = qinv;
			rxcir[0]=0.0; rxcir[1]=0.0; rxcir[2]=0.0;
			for( i=0; i<3; i++ ) rxcir[i]+=shift[i];
			rotate3(rxcir,st,xcir);
			for( i=0; i<3; i++ ) arc->xc[i]=xcir[i];
			}
		return(TRUE);
		}
	else {
		if( 0 ) { /* some diagnostics */
			fprintf(stderr,"tetra: %d\n", t->index);
			print_tetra(stderr,t);
			fprintf(stderr,"theta %f p %f R %f vs %f\n", theta, p, R, vs );
			fprintf(stderr,"nx1: %f %f %f\n", nx1[0], nx1[1], nx1[2] );
			fprintf(stderr,"nx2: %f %f %f\n", nx2[0], nx2[1], nx2[2] );
			fprintf(stderr,"nx3: %f %f %f\n", nx3[0], nx3[1], nx3[2] );
			fprintf(stderr,"nx4: %f %f %f\n", nx4[0], nx4[1], nx4[2] );
			fprintf(stderr,"nx0: %f %f %f\n", nrx0[0], nrx0[1], nrx0[2] );
			fprintf(stderr,"psi1: %f\n", psi1);
			fprintf(stderr,"\n");
			for( i=0; i<nints; i++ ) {
				fprintf(stderr,"intersection %d of %d\n", i, nints );
				print_intersec(stderr,&intersecs[i]);
				}
			}
		return(FALSE);
		}

	}

int compare_int(a, b)
struct int_rec *a, *b;
{
  if( a->dpsi <  b->dpsi ) return(-1);
  else if( a->dpsi == b->dpsi ) return(0);
  else return(1);
  }

int compare_tt(a, b)
struct tt_rec *a, *b;
{
  if( a->T <  b->T ) return(-1);
  else if( a->T == b->T ) return(0);
  else return(1);
  }

int compare_nodeid(a, b)
struct nodeid_element_rec *a, *b;
{
  if( a->id <  b->id ) return(-1);
  else if( a->id == b->id ) return(0);
  else return(1);
  }


int circle_intersects_plane( R, x1, x2, x3, intersecs, nints, face, x4 )
double R; /* radius of circle */
double x1[3], x2[3], x3[3]; /* points on plane */
struct int_rec *intersecs;
int *nints;
int face;
double x4[3];
{
	int i;
	double P[4], a, b, c;
	double fabsc, A, A2, D, spsi, cpsi, psi, phi, tmp, th, v;
	double epsilon;

	epsilon=1.0e-6;
	v=0.0;

	/* if all three vertices of the tetrahedron are on the  */
	/* same side of the y=0 plane, then the intersection of */
	/* that face of the tetrahedron with the y=0 plane will */
	/* not be inside the tetrahedron, and can be ignored    */
	if( (x1[1]<0.0) && (x2[1]<0.0) && (x3[1]<0.0) ) return(TRUE);
	if( (x1[1]>0.0) && (x2[1]>0.0) && (x3[1]>0.0) ) return(TRUE);


	/* equation of line of intersection of plane with the (x,z)=(ax+bz+c)=0 plane */
	eqn_of_plane( x1, x2, x3, P );
	a = P[0];
	b = P[2];
	c = P[3];

	/* Now imaginge a line that passes thru the origin and which           */
	/* is the perpendicular bisector of the line associated with the face. */
	/* It has equation -bx+az=0.  The point of intersection is             */
	/* (x,z)=-c(a,b)/(a**2+b**2).  The distance from that point to the     */
	/* origin is D=|c|/sqrt(a*a+b*b).   If D>R then no solution occurs.    */ 
	/* The direction, t,  from the origin to the intersection is           */
	/* (tx, tz) = (sin(psi),cos(psi)) = (-ca,-cb)/sqrt((ac)**2+(bc)**2)    */
	/* where psi is the angle from the z axis to the interstection point.  */
	/* Now imagine that we rotate the coordinates so that the t-direction  */
	/* becomes the new z'-axis.  The equation of line is z'=D (where D is  */
	/* guaranteed positive.  The equation of the circle is (x',z') =       */
	/* (Rsin(phi), Rcos(phi)).  There are two intersections, symmetric     */
	/* about the z' axis, psi and -psi, where psi=acos(D/R) is guaranteed  */
	/* to be in the range 0 to pi/2.  The angle, th, in the unrotated      */
	/* coordinate system between the z-axis and the two intersection       */
	/* points is th=psi+/-phi.                                             */

	A2 = (a*a+b*b);
	if( A2 <= 0.0 ) return(FALSE); /*plane parallel to (x,z)*/

	fabsc=fabs(c);
	A = sqrt(A2);
	D = fabsc/A;
	if( D>R ) return(FALSE); /* no intersection */

	tmp = fabs(c)*A;
	spsi = (-c*a) / tmp;
	cpsi = (-c*b) / tmp;
	psi = atan2( spsi, cpsi );

	phi = acos( D/R );
	
	th = psi+phi;
	intersecs[*nints].psi = RTOD*th;
	intersecs[*nints].x[0]= R*sin(th);
	intersecs[*nints].x[1]= 0.0;
	intersecs[*nints].x[2]= R*cos(th);
	/* xme is near x, but slightly displaced in direction of vertex */
	/* I have variously tried two ways of displacing,               */
	/* towards the other vertex, or back along the ray              */
	if(0) {
		for( i=0; i<3; i++ ) {
			intersecs[*nints].xme[i] =
				(1.0-epsilon)*intersecs[*nints].x[i] +
				epsilon*x4[i];
			}
		}
	else {
		intersecs[*nints].xme[0]= R*sin(th-epsilon/R);
		intersecs[*nints].xme[1]= 0.0;
		intersecs[*nints].xme[2]= R*cos(th-epsilon/R);
		}

	intersecs[*nints].face=face;
	intersecs[*nints].inside=FALSE;
	intersecs[*nints].R=R;
	intersecs[*nints].epsilon=epsilon;
	intersecs[*nints].val=v;
	(*nints)++;

	th = psi-phi;
	intersecs[*nints].psi = RTOD*th;
	intersecs[*nints].x[0]= R*sin(th);
	intersecs[*nints].x[1]= 0.0;
	intersecs[*nints].x[2]= R*cos(th);
	/* xme is near x, but slightly displaced in direction of vertex */
	/* I have variously tried two ways of displacing,               */
	/* towards the other vertex, or back along the ray              */
	if(0) {
		for( i=0; i<3; i++ ) {
			intersecs[*nints].xme[i] =
				(1.0-epsilon)*intersecs[*nints].x[i] +
				epsilon*x4[i];
			}
		}
	else {
		intersecs[*nints].xme[0]= R*sin(th-epsilon/R);
		intersecs[*nints].xme[1]= 0.0;
		intersecs[*nints].xme[2]= R*cos(th-epsilon/R);
		}

	intersecs[*nints].face=face;
	intersecs[*nints].inside=FALSE;
	intersecs[*nints].R=R;
	intersecs[*nints].epsilon=epsilon;
	intersecs[*nints].val=v;
	(*nints)++;

	return(TRUE);
	}



int newbasisrot( b0, b1, b2, s )
double b0[3], b1[3], b2[3], s[3][3];
{
        int i;

        /* derivation, given a vector [a1, a2, a3]

           let a = c0 b0 + c1 b1 + c2 b2
           then because of orthonoarmality of (b0, b1, b2)
           c0 = b0 (dot) a, c1 = b1 (dot) a, c2 = b2 (dot) a
           which in matric form is

           (c0)   (b0[0] b0[1] b0[2]) (a0)
           (c1) = (b1[0] b1[1] b1[2]) (a1)
           (c2)   (b2[0] b2[1] b2[2]) (a2)                       */

        for(i=0; i<3; i++ ) {
                s[0][i]=b0[i];
                s[1][i]=b1[i];
                s[2][i]=b2[i];
                }

        return(1);
        }

int rotate3(c,s,c2)
double c[3];
double s[3][3];
double c2[3];
{
        int i;
        int p;

        for(i=0;i<3;i++) {
                c2[i]=0.0;
                for(p=0;p<3;p++) {
                        c2[i] += s[i][p]*c[p];
                        }
                }
        return(1);
        }


/* Calculates P=[A,B,C,D], the coefficients of the equation
of the plane Ax + By + Cz + D = 0, given three points x1, x2,
x3 in plane. Returns 1 on success */

int eqn_of_plane( x1, x2, x3, P )
double x1[3], x2[3], x3[3];
double P[4];
{
	double x2mx1[3], x3mx1[3], v[3], a;
	int i;

	/* eqn is (x-x1) (dot) ( (x2-x1) (cross) (x3-x1) )     */
	/* see Rektorys Survey of Applicable Mathematics p 240 */

	for( i=0; i<3; i++ ) {
		x2mx1[i] = x2[i]-x1[i];
		x3mx1[i] = x3[i]-x1[i];
		}

	cross( x2mx1, x3mx1, v );
	P[0] = v[0];
	P[1] = v[1];
	P[2] = v[2];
	P[3] = (-dot(x1,v));

	a=sqrt(P[0]*P[0]+P[1]*P[1]+P[2]*P[2]);
	if( a==0.0 ) a=1.0;
	for( i=0; i<4; i++ ) P[i]/=a;
	
	return(1);
	}

/* Calculates P=[A,B,C,D], the coefficients of the equation
of the plane Ax + By + Cz + D = 0, given the plane's normal, n,
and one point, x. Returns 1 on success */

int eqn_of_plane2( n, x1, P )
double n[3], x1[3];
double P[4];
{
	/* n = grad(Ax + By + Cz + D) = [A, B, C]  */
	/* then D = -(Ax1 + By1 + Cz1)
	/* see Rektorys Survey of Applicable Mathematics p 240 */

	P[0] = n[0];
	P[1] = n[1];
	P[2] = n[2];
	P[3] = (-(P[0]*x1[0]+P[1]*x1[1]+P[2]*x1[2]));
	return(1);
	}

/* evaluates equation of plane, Ax + By + Cz + D, for a given x */
/* The coefficients are sent in P=[A,B,C,D]                     */
double eval_plane( P, x )
double P[4], x[3];
{
	return( P[0]*x[0] + P[1]*x[1] + P[2]*x[2] + P[3] );
	}

/* evaluates equation of plane, just using 3 points on plane */
/* note: doesnt return same value as eval_plane() because of different normalization */
double eval_plane2( x, x1, x2, x3 )
double x[3], x1[3], x2[3], x3[3];
{
	int i;
	double xmx1[3], x2mx1[3], x3mx1[3], v[3];
	/* eqn is (x-x1) (dot) ( (x2-x1) (cross) (x3-x1) )     */
	/* see Rektorys Survey of Applicable Mathematics p 240 */
	for( i=0; i<3; i++ ) {
		xmx1[i]  = x[i] -x1[i];
		x2mx1[i] = x2[i]-x1[i];
		x3mx1[i] = x3[i]-x1[i];
		}
	cross( x2mx1, x3mx1, v );
	return( dot( xmx1, v ) );
	}

/* returns 1 if x is inside the tetrahedra with vertices x1, x2,
   x3, x4, 0 if its on the surface and -1 if its outside */

int in_tetrahedron2( x1, x2, x3, x4, x)
double x1[3], x2[3], x3[3], x4[3], x[3];
{
	double P[4], a, b;
	int on_surface;

	/* theory: use three vertices to define a plane         */
	/* check if the fourth vertice and the point are        */
	/* on the same side of the plane.  If it is for all     */
	/* planes, then the point is inside the tetrahedron     */
	/* two points are on the same side of the plane if      */
	/* the equation of the plane evaluates to the same sign */
	/* see Rektorys Survey of Applicable Mathematics p 243  */

	on_surface=0;

	/* plane 4 */
	eqn_of_plane( x1, x2, x3, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x4 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 3 */
	eqn_of_plane( x1, x2, x4, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x3 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 2 */
	eqn_of_plane( x1, x3, x4, P );
	a=eval_plane( P, x ); 
	b=eval_plane( P, x2 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	/* plane 1 */
	eqn_of_plane( x2, x3, x4, P );
	a=eval_plane( P, x );
	b=eval_plane( P, x1 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( a*b < 0.0 ) return(0);
		}

	if( on_surface==1 ) return(1); else return(1);

	}

int in_tetrahedron3( x1, x2, x3, x4, x)
double x1[3], x2[3], x3[3], x4[3], x[3];
{
	double a, b;
	int on_surface;

	/* theory: use three vertices to define a plane         */
	/* check if the fourth vertice and the point are        */
	/* on the same side of the plane.  If it is for all     */
	/* planes, then the point is inside the tetrahedron     */
	/* two points are on the same side of the plane if      */
	/* the equation of the plane evaluates to the same sign */
	/* see Rektorys Survey of Applicable Mathematics p 243  */

	on_surface=0;

	/* plane 4 */
	a=eval_plane2( x,  x1, x2, x3 );
	b=eval_plane2( x4, x1, x2, x3 );
	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 3 */
	a=eval_plane2( x,  x1, x2, x4 );
	b=eval_plane2( x3, x1, x2, x4 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 2 */
	a=eval_plane2( x,  x1, x3, x4 );
	b=eval_plane2( x2, x1, x3, x4 );

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	/* plane 1 */
	a=eval_plane2( x,  x2, x3, x4);
	b=eval_plane2( x1, x2, x3, x4);

	if( a == 0.0 ) {
		on_surface=1;
		}
	else {
		if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);
		}

	if( on_surface==1 ) return(1); else return(1);

	}

int in_tetrahedron( x1, x2, x3, x4, x)
double x1[3], x2[3], x3[3], x4[3], x[3];
{
	int i;
	double a, b;
	double v1[3], v2[3], n[3], tv[3], tx[3];

	/* theory:
		is point x on the same side of each of the 4 planes as
		each excluded vertex?  If so, its in the tetrahedron.
		Given a plane xa, xb, xc, then (xb-xa) cross (xc-xa) is
		a normal to that plane.  if (xd-xa) dot normal has same
		sign as (x-xa) dot normal, then the excluded vertex
		is on the same side of the plane as x.                */

	/* plane 1 */
	for( i=0; i<3; i++ ) {
		v1[i] = x3[i]-x2[i];
		v2[i] = x4[i]-x2[i];
		tv[i] = x1[i]-x2[i];
		tx[i] = x[i] -x2[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 2 */
	for( i=0; i<3; i++ ) {
		v1[i] = x3[i]-x1[i];
		v2[i] = x4[i]-x1[i];
		tv[i] = x2[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 3 */
	for( i=0; i<3; i++ ) {
		v1[i] = x2[i]-x1[i];
		v2[i] = x4[i]-x1[i];
		tv[i] = x3[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	/* plane 4 */
	for( i=0; i<3; i++ ) {
		v1[i] = x2[i]-x1[i];
		v2[i] = x3[i]-x1[i];
		tv[i] = x4[i]-x1[i];
		tx[i] = x[i] -x1[i];
		}
	cross( v1, v2, n );
	a=dot(n,tv);
	b=dot(n,tx);
	if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) return(0);

	return(1);
	}

/* finds intersection, xi, of ray with the plane, P. The ray is
   specified by a point, x, and a tangent unit vector t.
   The point must be below the plane, in the sense that the computed
   arclength, s, along the tangent from xi to the plane is positive.
   The normal of the plane is also calculated. Returns 1 on
   success, 0 on error. */

int ray_exits_layer( P, x, t, normal, xi, s )
double P[4], x[3], t[3];
double normal[3], xi[3], *s;
{
	int i;
	double a, xc[3];

	for( i=0; i<3; i++ ) xc[i]=x[i];

	/* calculate intersection of ray with plane, P.  The
	   intersection is found by simultaneously solving equation of plane,
	   Ax + By + Cz + D = 0 and position on ray as function of arclength, a,
	   x = xc[0]+a*t[0], y = xc[1]+a*t[1], z = xc[2]+a*t[2] */

	normal_to_plane( P, normal );
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( finite(a) && a > 0.0 ) { /* arclength has right sign */
		*s = a;
		return(1);
		}
	else {
		return(0);
		}

	}

/* finds intersection, xi, of ray with one of the faces of a tetrahedron
   The ray is specified by a point, x, and a tangent unit vector t.
   The point must be inside, and the face with the positive arclength, s,
   along the tangent is computed and that arclength is calculated.
   The face is returned as (0, 1, 2, 3) depending upon which of (x1, x2,
   x3, x4) is not on the plane. The outward pointing normal of the plane
   is also calculated. Returns 1 on success, 0 on error. If
   the point tests outside the tetrahedron or exactly on a face, it will
   be advanced a distance epsilon>0 along the ray and retried.  So set
   epsilon to a small positive number. */

int ray_exits_tetrahedron( x1, x2, x3, x4, x, t, epsilon, normal, xi, s, face )
double x1[3], x2[3], x3[3], x4[3], x[3], t[3], epsilon;
double normal[3], xi[3], *s;
int *face;
{
	int i, in_tetra1, in_tetra2;
	double a, advance, xc[3], xcc[3], P[4];

	advance=0.0;
	for( i=0; i<3; i++ ) xc[i]=x[i];

	if( (in_tetra1=in_tetrahedron( x1, x2, x3, x4, xc)) <= 0 ) {
		/* if the point is on or near the boundary, then it may evaluate
		   outside due to roundoff error.  Thus we move it a tiny bit,
		   and check again. */
		for( i=0; i<3; i++ ) xc[i] += epsilon*t[i];
		advance=epsilon;
		if( (in_tetra2=in_tetrahedron( x1, x2, x3, x4, xc)) <= 0 ) {
			return(0);
			}
		}

	/* calculate intersection of ray with each face of tetrahedra
	   intersection is found by simultaneously solving equation of plane,
	   Ax + By + Cz + D = 0 and position on ray as function of arclength, a,
	   x = xc[0]+a*t[0], y = xc[1]+a*t[1], z = xc[2]+a*t[2] */

	/* calculate outward facing normal, too.  n=grad(P) points in the
           direction of increasing eqn-of-plane values, thus if the remaining
	   face has a negative eqn-of-plane value, then the normal is
	   outward facing.  Otherwise reverse it. */

	/* face 1 */
	*face=0;
	eqn_of_plane( x2, x3, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x1 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( finite(a) && a > 0.0 ) { /* arglength has right sign. Now is the point on
				        the face of the tetrahedron? back it up a bit
					before checking. The finite() checks whether
					the face is parallel to the face */
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 2 */
	*face=1;
	eqn_of_plane( x1, x3, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x2 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( finite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 3 */
	*face=2;
	eqn_of_plane( x1, x2, x4, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x3 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( finite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	/* face 4 */
	*face=3;
	eqn_of_plane( x1, x2, x3, P );
	normal_to_plane( P, normal );
	if( eval_plane( P, x4 ) > 0.0 ) for( i=0; i<3; i++ ) normal[i] = (-normal[i]);
	a = (-(P[0]*xc[0]+P[1]*xc[1]+P[2]*xc[2]+P[3])/(P[0]*t[0]+P[1]*t[1]+P[2]*t[2]));
	for( i=0; i<3; i++ ) xi[i] = xc[i] + a*t[i];
	if( finite(a) && a > 0.0 ) {
		for( i=0; i<3; i++ ) xcc[i] = xi[i]-epsilon*t[i];
		if( in_tetrahedron( x1, x2, x3, x4, xcc) == 1 ) {
			*s = a+advance;
			return(1);
			}
		}

	return(0);
	}

/* normal to plane */
normal_to_plane( P, n )
double P[4], n[3];
{
	double a;

	/* normal is in direction grad(Ax+By+Cz+D)=(A,B,C) */
	n[0]=P[0], n[1]=P[1]; n[2]=P[2];
	unit_vector(n);
	return(1);
	}
             
gauss(a,vec,n,nstore,test,ierror,itriag)
double *a, vec[], test;
int n, nstore, *ierror, itriag;
{
 
/* subroutine gauss, by william menke */
/* july 1978 (modified feb 1983, nov 85) */
 
/* a subroutine to solve a system of n linear equations in n unknowns*/
/* where n doesn't exceed MAX_TABLE_COLS */
/* gaussian reduction with partial pivoting is used */
/* 	a		(sent, destroyed)	n by n matrix		*/
/*	vec		(sent, overwritten)	n vector, replaced w/ solution*/
/* 	nstore		(sent)			dimension of a	*/
/*	test		(sent)			div by zero check number*/
/*	ierror		(returned)		zero on no error*/
/*	itriag		(sent)			matrix triangularized only*/
/*						 on TRUE useful when solving*/
/*						 multiple systems with same a */
	static int isub[MAX_TABLE_COLS], l1;
	int line[MAX_TABLE_COLS], iet, ieb, i, j, k, l, j2;
	double big, testa, b, sum;
	

	iet=0;	/* initial error flags, one for triagularization*/
	ieb=0;	/* one for backsolving */

/* triangularize the matrix a*/
/* replacing the zero elements of the triangularized matrix */
/* with the coefficients needed to transform the vector vec */

	if (itriag) {	/* triangularize matrix */
 
 		for( j=0; j<n; j++ ) {      /*line is an array of flags*/
			line[j]=0; 
			/* elements of a are not moved during pivoting*/
			/* line=0 flags unused lines */
			}    /*end for j*/
			
		for( j=0; j<n-1; j++ ) {
			/*  triangularize matrix by partial pivoting */
                       big = 0.0; /* find biggest element in j-th column*/
				  /* of unused portion of matrix*/
		       for( l1=0; l1<n; l1++ ) {
                               if( line[l1]==0 ) {
                                       testa=(double) fabs(
						(double) (*(a+l1*nstore+j)) );
                                       if (testa>big) {
						i=l1;
						big=testa;
						} /*end if*/
					} /*end if*/
				} /*end for l1*/
                       if( big<=test) {   /* test for div by 0 */
                               iet=1;
                               } /*end if*/
 
                       line[i]=1;  /* selected unused line becomes used line */
                       isub[j]=i;  /* isub points to j-th row of tri. matrix */
 
                       sum=1.0/(*(a+i*nstore+j)); 
				/*reduce matrix towards triangle */
		       for( k=0; k<n; k++ ) {
				if( line[k]==0 ) {
					b=(*(a+k*nstore+j))*sum;
				       	for( l=j+1; l<n; l++ ) {
                                               *(a+k*nstore+l)=
							(*(a+k*nstore+l))
							-b*(*(a+i*nstore+l));
					       } /*end for l*/
                                       *(a+k*nstore+j)=b;
					} /*end if*/
				} /*end for k*/
			} /*end for j*/
 
               for( j=0; j<n; j++ ) {
			/*find last unused row and set its pointer*/
			/*  this row contians the apex of the triangle*/
			if( line[j]==0) {
				l1=j;   /*apex of triangle*/
				isub[n-1]=j;
				break;
				} /*end if*/
			} /*end for j*/
 
		} /*end if itriag true*/
		
	/*start backsolving*/
	
	for( i=0; i<n; i++ ) {	/* invert pointers. line(i) now gives*/
				/* row no in triang matrix of i-th row*/
				/* of actual matrix */
		line[isub[i]] = i;
		} /*end for i*/
 
	for( j=0; j<n-1; j++) { /*transform the vector to match triang. matrix*/
               b=vec[isub[j]];
               for( k=0; k<n; k++ ) {
                      if (line[k]>j) {	/* skip elements outside of triangle*/
                                vec[k]=vec[k]-(*(a+k*nstore+j))*b;
				} /*end if*/
			} /*end for k*/
		} /*end for j*/
 
      b = *(a+l1*nstore+(n-1));   /*apex of triangle*/
      if( ((double)fabs( (double) b))<=test) {
		/*check for div by zero in backsolving*/
		ieb=2;
		} /*end if*/
      vec[isub[n-1]]=vec[isub[n-1]]/b;
 
      for( j=n-2; j>=0; j-- ) {	/* backsolve rest of triangle*/
		sum=vec[isub[j]];
		for( j2=j+1; j2<n; j2++ ) {
			sum = sum - vec[isub[j2]] * (*(a+isub[j]*nstore+j2));
			} /*end for j2*/
			b = *(a+isub[j]*nstore+j);
               if( ((double)fabs((double)b))<=test) {
			/* test for div by 0 in backsolving */
			ieb=2;
			} /*end if*/
		vec[isub[j]]=sum/b;   /*solution returned in vec*/
		} /*end for j*/

/*put the solution vector into the proper order*/

      for( i=0; i<n; i++ ) {    /* reorder solution */
		for( k=i; k<n; k++ ) {  /* search for i-th solution element */
			if( line[k]==i ) {
				j=k;
				break;
				} /*end if*/
			} /*end for k*/
               b = vec[j];       /* swap solution and pointer elements*/
               vec[j] = vec[i];
               vec[i] = b;
               line[j] = line[i];
		} /*end for i*/
 
      *ierror = iet + ieb;   /* set final error flag*/
}

		
#include <stdio.h>
#include <math.h>

/* given a unit vector, t, compute two other mutually orthoginal
unit vectors v and h that are perendicular to t.  If t is vertical
then v=[1,0,0] and h=[0,1,0].  Otherwise v will have a positive
3-component and h will have a zero 3-component.  h = t (cross) v  */

double dot();

tvh( t, v, h)
double t[3], v[3], h[3];
{
	double a;

	if( t[0] == 0.0 && t[1] == 0.0 ) {
		v[0]=1.0; v[1]=0.0; v[2]=0.0;
		}
	else {
		a = t[0]*t[0] + t[1]*t[1];
		v[0] = (-t[0]*t[2]/a);
		v[1] = (-t[1]*t[2]/a);
		v[2] = 1.0;
		unit_vector(v);
		}
	
	cross( t, v, h );
	return(1);
	}

/* dot product between vectors a and b */
double dot( a, b )
double a[3], b[3];
{
	return( a[0]*b[0] + a[1]*b[1] + a[2]*b[2] );
	}

/* cross product c = a (cross) b */
cross( a, b, c )
double a[3], b[3], c[3];
{
	c[0] = a[1]*b[2] - b[1]*a[2];
	c[1] = a[2]*b[0] - b[2]*a[0];
	c[2] = a[0]*b[1] - b[0]*a[1];
	return(1);
	}

/* appends to tetralist given 8 nodes.  The first four circle the
   bottom of the prism, the last 4 the top. Returns 1 on success, 0
   on failure. */

int prism_to_tetra1( id, n0,n1,n2,n3,n4,n5,n6,n7, type )
int id, n0,n1,n2,n3,n4,n5,n6,n7, type;
{
	int c, b, f, l, r;
	/* out of space */
	if( (n_tetras+5) > MAX_TETRAS ) return(0);

	/* no such node */
	if( (n0<0) || (n0>=n_nodes) || (n1<0) || (n1>=n_nodes) ||
	    (n2<0) || (n2>=n_nodes) || (n3<0) || (n3>=n_nodes) ||
	    (n4<0) || (n4>=n_nodes) || (n5<0) || (n5>=n_nodes) ||
	    (n6<0) || (n6>=n_nodes) || (n7<0) || (n7>=n_nodes) ) {
		return(0);
		}

	/* there are two configurations that must be alternated */
	/* chess-board style in a model containing multiple prisms */
	if( type ) {
		if( (c=new_tetra( id, n1, n3, n4, n6 )) == (-1) ) return(0);
		if( (b=new_tetra( id, n0, n1, n3, n4 )) == (-1) ) return(0);
		if( (f=new_tetra( id, n1, n2, n3, n6 )) == (-1) ) return(0);
		if( (l=new_tetra( id, n3, n4, n6, n7 )) == (-1) ) return(0);
		if( (r=new_tetra( id, n1, n4, n5, n6 )) == (-1) ) return(0);
		tetra[c].t1=l; tetra[c].t2=r; tetra[c].t3=f; tetra[c].t4=b;
		tetra[b].t1=c; tetra[f].t2=c; tetra[l].t4=c; tetra[r].t3=c;
		}
	else {
		if( (c=new_tetra( id, n0, n2, n5, n7 )) == (-1) ) return(0);
		if( (r=new_tetra( id, n0, n1, n2, n5 )) == (-1) ) return(0);
		if( (f=new_tetra( id, n2, n5, n6, n7 )) == (-1) ) return(0);
		if( (b=new_tetra( id, n0, n4, n5, n7 )) == (-1) ) return(0);
		if( (l=new_tetra( id, n0, n2, n3, n7 )) == (-1) ) return(0);
		tetra[c].t1=f; tetra[c].t2=b; tetra[c].t3=l; tetra[c].t4=r;
		tetra[r].t2=c; tetra[f].t3=c; tetra[b].t2=c; tetra[l].t3=c;
		}

	return(1);
	}

int prism_to_tetra2( id, n0,n1,n2,n3,n4,n5,n6,n7, type )
int id, n0,n1,n2,n3,n4,n5,n6,n7, type;
{
	int a, b, c, d, e, f;
	/* out of space */
	if( (n_tetras+6) > MAX_TETRAS ) return(0);

	/* no such node */
	if( (n0<0) || (n0>=n_nodes) || (n1<0) || (n1>=n_nodes) ||
	    (n2<0) || (n2>=n_nodes) || (n3<0) || (n3>=n_nodes) ||
	    (n4<0) || (n4>=n_nodes) || (n5<0) || (n5>=n_nodes) ||
	    (n6<0) || (n6>=n_nodes) || (n7<0) || (n7>=n_nodes) ) {
		return(0);
		}

	if( (a=new_tetra( id, n2, n4, n6, n7 )) == (-1) ) return(0);
	if( (b=new_tetra( id, n2, n3, n4, n7 )) == (-1) ) return(0);
	if( (c=new_tetra( id, n0, n2, n3, n4 )) == (-1) ) return(0);
	tetra[a].t3=b; tetra[b].t2=a;
	tetra[b].t4=c; tetra[c].t1=b;

	if( (d=new_tetra( id, n2, n4, n5, n6 )) == (-1) ) return(0);
	if( (e=new_tetra( id, n1, n2, n4, n5 )) == (-1) ) return(0);
	if( (f=new_tetra( id, n0, n1, n2, n4 )) == (-1) ) return(0);
 	tetra[d].t4=e; tetra[e].t1=d;
	tetra[e].t4=f; tetra[f].t1=e;

	tetra[a].t4=d; tetra[d].t3=a;
 	tetra[c].t3=f; tetra[f].t2=c;

	return(1);
	}


int new_node( id, x, y, z, v, qinv, interface )
int id, interface;
double x, y, z, v, qinv;
{

	if( (n_nodes+1) > MAX_NODES ) return(-1);
	
	node[n_nodes].id=id;
	node[n_nodes].ifc=interface;
	node[n_nodes].x[0]=x;
	node[n_nodes].x[1]=y;
	node[n_nodes].x[2]=z;
	node[n_nodes].v=v;
	node[n_nodes].v_orig=v;
	node[n_nodes].ratio=1.0/qinv;
	node[n_nodes].ratio_orig=1.0/qinv;
	node[n_nodes].qinv=qinv;
	node[n_nodes].qinv_orig=qinv;
	node[n_nodes].hits=0;
	node[n_nodes].touched=0;
	node[n_nodes].nt=0;
	if( n_nodes==0 ) {
		model_z_min=z;
		model_z_max=z;
		}
	else if( z<model_z_min ) {
		model_z_min=z;
		}
	else if( z>model_z_max ) {
		model_z_max=z;
		}

	n_nodes++;
	return(n_nodes-1);
	}

int new_tetra( id, n1, n2, n3, n4 )
int id, n1, n2, n3, n4;
{
	int i, if1, if2, if3, if4;
	double V;

	V=1.0e-9;

	if( (n_tetras+1) > MAX_TETRAS ) return(-1);
	if( (n1>=n_nodes) || (n2>=n_nodes) || (n3>=n_nodes) || (n4>=n_nodes) ) return(-1);
	if( (n1<0) || (n2<0) || (n3<0) || (n4<0) ) return(-1);

	tetra[n_tetras].index=n_tetras;
	tetra[n_tetras].id=id;
	tetra[n_tetras].n1=n1;
	tetra[n_tetras].n2=n2;
	tetra[n_tetras].n3=n3;
	tetra[n_tetras].n4=n4;
	tetra[n_tetras].t1=(-1);
	tetra[n_tetras].t2=(-1);
	tetra[n_tetras].t3=(-1);
	tetra[n_tetras].t4=(-1);
	tetra[n_tetras].ifc[0]=(-1);
	tetra[n_tetras].ifc[1]=(-1);
	tetra[n_tetras].ifc[2]=(-1);
	tetra[n_tetras].ifc[3]=(-1);
	tetra[n_tetras].x1=node[n1].x;
	tetra[n_tetras].x2=node[n2].x;
	tetra[n_tetras].x3=node[n3].x;
	tetra[n_tetras].x4=node[n4].x;
	tetra[n_tetras].v[0]=&(node[n1].v);
	tetra[n_tetras].v[1]=&(node[n2].v);
	tetra[n_tetras].v[2]=&(node[n3].v);
	tetra[n_tetras].v[3]=&(node[n4].v);
	tetra[n_tetras].qinv[0]=&(node[n1].qinv);
	tetra[n_tetras].qinv[1]=&(node[n2].qinv);
	tetra[n_tetras].qinv[2]=&(node[n3].qinv);
	tetra[n_tetras].qinv[3]=&(node[n4].qinv);
	velocity_eqn( &(tetra[n_tetras]) );
	qinv_eqn( &(tetra[n_tetras]) );
	tetra[n_tetras].marked=0;
	tetra[n_tetras].tested=0;

	/* a face is on an interface if all its nodes are on it */
	if1=node[n1].ifc;
	if2=node[n2].ifc;
	if3=node[n3].ifc;
	if4=node[n4].ifc;
	if( (if2==if3) && (if3==if4) ) tetra[n_tetras].ifc[0]=if2;
	if( (if3==if4) && (if4==if1) ) tetra[n_tetras].ifc[1]=if3;
	if( (if1==if2) && (if2==if4) ) tetra[n_tetras].ifc[2]=if1;
	if( (if1==if2) && (if2==if3) ) tetra[n_tetras].ifc[3]=if1;

	if( tetra_volume(n_tetras) < V ) {
		fprintf(stderr,"warning: tetrahedron %d has volume < %e\n", n_tetras, V);
		fprintf(stderr,"         node %d: %f %f %f\n", n1,
			node[n1].x[0], node[n1].x[1], node[n1].x[2] );
		fprintf(stderr,"         node %d: %f %f %f\n", n2,
			node[n2].x[0], node[n2].x[1], node[n2].x[2] );
		fprintf(stderr,"         node %d: %f %f %f\n", n3,
			node[n3].x[0], node[n3].x[1], node[n3].x[2] );
		fprintf(stderr,"         node %d: %f %f %f\n", n4,
			node[n4].x[0], node[n4].x[1], node[n4].x[2] );
		}

	/* add this tetra to the tetra list of all of its nodes */
	node[n1].t[node[n1].nt] = &(tetra[n_tetras]);
	node[n1].nt++; if( node[n1].nt>=MAX_TETRAS_TOUCHING_NODE ) return(-1);

	node[n2].t[node[n2].nt] = &(tetra[n_tetras]);
	node[n2].nt++; if( node[n2].nt>=MAX_TETRAS_TOUCHING_NODE ) return(-1);

	node[n3].t[node[n3].nt] = &(tetra[n_tetras]);
	node[n3].nt++; if( node[n3].nt>=MAX_TETRAS_TOUCHING_NODE ) return(-1);

	node[n4].t[node[n4].nt] = &(tetra[n_tetras]);
	node[n4].nt++; if( node[n4].nt>=MAX_TETRAS_TOUCHING_NODE ) return(-1);

	n_tetras++;
	return(n_tetras-1);
	}

/* find index, n, of tetra containing x */
/* return 1 on success, 0 on failure    */
int find_tetra(x, n)
double x[3];
int *n;
{
	int i, j, k, l;
	double v1[3], v2[3], normal[3], tv[3], xm[3], dx[3], b[4], bmax;

	(*n)=(-1);

	/* Method 1: */
	/* try to get to x by moving from adjoining tetrahedron to */
	/* adjoining tetrahedron, starting from the last tetrahedron used */
	/* and always choosing the next tetrahedron adjoining the face that */
	/* has a normal closest to the direction toward x */

	if( last_tetra!=(-1) ) i=last_tetra; else i=0;

	for( j=0; j<n_tetras; j++ ) {

		if( i==(-1) ) break;

		if( in_tetrahedron( tetra[i].x1,
			tetra[i].x2,
             		tetra[i].x3,
			tetra[i].x4, x ) ) {
			last_tetra = i;
			*n = i;
			if( j==0 ) find_tetra_hit++;
			if( j==1 ) find_tetra_neighbor++;
			else find_tetra_walk++;
			return(1);
			}

		/* plane 1 */
		for( k=0; k<3; k++ ) {
			xm[k] = (tetra[i].x2[k]+tetra[i].x3[k]+tetra[i].x4[k])/3.0;
			dx[k] = x[k]-xm[k];
			v1[k] = tetra[i].x3[k]-tetra[i].x2[k];
			v2[k] = tetra[i].x4[k]-tetra[i].x2[k];
			tv[k] = tetra[i].x1[k]-tetra[i].x2[k];
			}
		/* outward pointing normal */
		cross( v1, v2, normal );
		unit_vector(normal);
		if( dot(normal,tv)>0.0 ) for( k=0; k<3; k++ ) normal[k]=(-normal[k]);
		unit_vector(dx);
		b[0]=dot(normal,dx);

		/* plane 2 */
		for( k=0; k<3; k++ ) {
			xm[k] = (tetra[i].x1[k]+tetra[i].x3[k]+tetra[i].x4[k])/3.0;
			dx[k] = x[k]-xm[k];
			v1[k] = tetra[i].x3[k]-tetra[i].x1[k];
			v2[k] = tetra[i].x4[k]-tetra[i].x1[k];
			tv[k] = tetra[i].x2[k]-tetra[i].x1[k];
			}
		cross( v1, v2, normal );
		unit_vector(normal);
		if( dot(normal,tv)>0.0 ) for( k=0; k<3; k++ ) normal[k]=(-normal[k]);
		unit_vector(dx);
		b[1]=dot(normal,dx);

		/* plane 3 */
		for( k=0; k<3; k++ ) {
			xm[k] = (tetra[i].x1[k]+tetra[i].x2[k]+tetra[i].x4[k])/3.0;
			dx[k] = x[k]-xm[k];
			v1[k] = tetra[i].x2[k]-tetra[i].x1[k];
			v2[k] = tetra[i].x4[k]-tetra[i].x1[k];
			tv[k] = tetra[i].x3[k]-tetra[i].x1[k];
			}
		cross( v1, v2, normal );
		unit_vector(normal);
		if( dot(normal,tv)>0.0 ) for( k=0; k<3; k++ ) normal[k]=(-normal[k]);
		unit_vector(dx);
		b[2]=dot(normal,dx);

		/* plane 4 */
		for( k=0; k<3; k++ ) {
			xm[k] = (tetra[i].x1[k]+tetra[i].x2[k]+tetra[i].x3[k])/3.0;
			dx[k] = x[k]-xm[k];
			v1[k] = tetra[i].x2[k]-tetra[i].x1[k];
			v2[k] = tetra[i].x3[k]-tetra[i].x1[k];
			tv[k] = tetra[i].x4[k]-tetra[i].x1[k];
			}
		cross( v1, v2, normal );
		unit_vector(normal);
		if( dot(normal,tv)>0.0 ) for( k=0; k<3; k++ ) normal[k]=(-normal[k]);
		unit_vector(dx);
		b[3]=dot(normal,dx);

		bmax=b[0]; l=0;
		for( k=1; k<4; k++ ) if( b[k]>bmax ) { bmax=b[k]; l=k; }
		if( b[l]>0.0 ) {
			if( l==0 )      i=tetra[i].t1;
			else if( l==1 ) i=tetra[i].t2;
			else if( l==2 ) i=tetra[i].t3;
			else if( l==3 ) i=tetra[i].t4;
			}
		else i=(-1);

		}

	/* Method 2: */
	/* brute force search */

	for( i=0; i<n_tetras; i++ ) {
		if( in_tetrahedron( tetra[i].x1,
				    tetra[i].x2,
             			    tetra[i].x3,
				    tetra[i].x4, x ) ) {
			last_tetra = i;
			*n = i;
			find_tetra_search++;
			return(1);
			}
		}
	find_tetra_nofind++;
	return(0);
	}

int link_tetras()
{
	int i, j, k;
	double x[3], xm[3], n[3], P[4], epsilon, a, b;

	epsilon=1.0e-6;

	for( i=0; i<n_tetras; i++ ) {

		/* face 1 */
		eqn_of_plane(tetra[i].x2,tetra[i].x3,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x2[j]+tetra[i].x3[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x1 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		tetra[i].t1 = k;

		/* face 2 */
		eqn_of_plane(tetra[i].x1,tetra[i].x3,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x3[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x2 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		tetra[i].t2 = k; 

		/* face 3 */
		eqn_of_plane(tetra[i].x1,tetra[i].x2,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x2[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x3 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		tetra[i].t3 = k;

		/* face 4 */
		eqn_of_plane(tetra[i].x1,tetra[i].x2,tetra[i].x3,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x2[j]+tetra[i].x3[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x4 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		tetra[i].t4 = k;

		}

	return(1);
	}

int check_tetras(f)
FILE *f;
{
	int i, j, k, status;
	double x[3], xm[3], n[3], P[4], epsilon, a, b;

	epsilon=1.0e-6;
	status=1;

	for( i=0; i<n_tetras; i++ ) {

		/* face 1 */
		eqn_of_plane(tetra[i].x2,tetra[i].x3,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x2[j]+tetra[i].x3[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x1 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		if( tetra[i].t1 != k ) {
			fprintf(f,"warning: tetra %d face %d link is %d should be %d; fixed\n", i, 1, tetra[i].t1, k );
			tetra[i].t1 = k;
			status=0;
			}

		/* face 2 */
		eqn_of_plane(tetra[i].x1,tetra[i].x3,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x3[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x2 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		if( tetra[i].t2 != k ) {
			fprintf(f,"error: tetra %d face %d link is %d should be %d; fixed\n", i, 2, tetra[i].t2, k );
			tetra[i].t2 = k;
			status=0;
			}

		/* face 3 */
		eqn_of_plane(tetra[i].x1,tetra[i].x2,tetra[i].x4,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x2[j]+tetra[i].x4[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x3 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		if( tetra[i].t3 != k ) {
			fprintf(f,"error: tetra %d face %d link is %d should be %d; fixed\n", i, 3, tetra[i].t3, k );
			tetra[i].t3 = k;
			status=0;
			}

		/* face 4 */
		eqn_of_plane(tetra[i].x1,tetra[i].x2,tetra[i].x3,P);
		normal_to_plane( P, n );
		for( j=0; j<3; j++ ) {
			xm[j]=(tetra[i].x1[j]+tetra[i].x2[j]+tetra[i].x3[j])/3.0;
			x[j]=xm[j]+epsilon*n[j];
			}
		a=eval_plane( P, x );
		b=eval_plane( P, tetra[i].x4 );
		if( (a*b)>0.0 ) for( j=0; j<3; j++ ) x[j]=xm[j]-epsilon*n[j];
		last_tetra=(-1); find_tetra( x, &k );
		if( tetra[i].t4 != k ) {
			fprintf(f,"error: tetra %d face %d link is %d should be %d; fixed\n", i, 4, tetra[i].t4, k );
			tetra[i].t4 = k;
			status=0;
			}
		}

	return(status);
	}


int print_tetra(f,t)
FILE *f;
struct tetra_rec *t;
{
	fprintf(f,"id %d index %d\n", t->id, t->index);
	fprintf(f,"nodes %d %d %d %d\n", t->n1, t->n2, t->n3, t->n4);
	fprintf(f,"vertice 1: %f %f %f %f\n", t->x1[0], t->x1[1], t->x1[2], *(t->v[0]) );
	fprintf(f,"vertice 2: %f %f %f %f\n", t->x2[0], t->x2[1], t->x2[2], *(t->v[1]) );
	fprintf(f,"vertice 3: %f %f %f %f\n", t->x3[0], t->x3[1], t->x3[2], *(t->v[2]) );
	fprintf(f,"vertice 4: %f %f %f %f\n", t->x4[0], t->x4[1], t->x4[2], *(t->v[3]) );
	fprintf(f,"coeffs %f %f %f %f\n", t->vc[0], t->vc[1], t->vc[2], t->vc[3] );
	fprintf(f,"grad %f %f %f\n", t->g[0], t->g[1], t->g[2] );
	fprintf(f,"gn %f %f %f\n", t->gn[0], t->gn[1], t->gn[2] );
	fprintf(f,"gs %f\n", t->gs );
	fprintf(f,"adjo %d %d %d %d\n", t->t1, t->t2, t->t3, t->t4);
	fprintf(f,"intf %d %d %d %d\n", t->ifc[0], t->ifc[1], t->ifc[2], t->ifc[3]);
	fprintf(f,"\n");
	
	return(1);
	}

int print_arc(f,a)
FILE *f;
struct arc_rec *a;
{
	int i;
	if( a->ptetra != NULL ) i=a->ptetra->index; else i=0;
	fprintf(f,"arc t %d f %d R %f psi1 %f psi2 %f dpsi %f T %f S %f\n",
		i, a->exit_face, a->R, a->psi1, a->psi2, a->psi2-a->psi1,
		 a->T, a->S);
	if( a->psi2<a->psi1 ) fprintf(f," *** dpsi<0.0");
	if( a->psi1<(-90.0) || a->psi1>(90.0) ) fprintf(f," *** bad psi1 range");
	if( a->psi2<(-90.0) || a->psi2>(90.0) ) fprintf(f," *** bad psi2 range");
	if( a->T < 0.0 ) fprintf(stderr," *** negative T\n");
	if( a->S < 0.0 ) fprintf(stderr," *** negative S\n");
	return(1);
	}

arcstoxyz( arc, n_arcs, x, y, z, index, ds, n, maxn )
struct arc_rec arc[];
int n_arcs;
double x[], y[], z[];
int index[];
double ds;
int *n;
int maxn;
{
	int i, j, k, l;
	double psi, psi1, psi2, r[3], r2[3];

	(*n)=0;

	for( i=0; i<n_arcs; i++ ) {

                k=(int)(0.5+arc[i].S/ds);
                if( k<1 ) k=1;
                for( j=0; j<=k; j++ ) {
                        if( (*n)>(maxn-2) ) return(FALSE);

			/* point in transformed coordinate system */
			psi1 = arc[i].psi1; while( psi1<0.0 ) psi1+=360.0; while( psi1>=360.0 ) psi1-=360.0;
			psi2 = arc[i].psi2; while( psi2<0.0 ) psi2+=360.0; while( psi2>=360.0 ) psi2-=360.0;
			while( psi2<psi1 ) psi1-=360.0;
                        psi = psi1 +
				((double)j) *
				(psi2-psi1)/(double)k;

                        r[0] = arc[i].R*sin(DTOR*psi);
                        r[1] = 0.0;
                        r[2] = arc[i].R*cos(DTOR*psi);

			/* untransform */
			for( l=0; l<3; l++ ) r[l] += arc[i].shift[l];
			rotate3(r,arc[i].st,r2);

			x[*n] = r2[0];
			y[*n] = r2[1];
			z[*n] = r2[2];
			index[*n] = arc[i].ptetra->index;
                        (*n)++;
                        }
		}
	return(TRUE);
	}

int unit_vector(v)
double v[];
{
	double a;
	int i;
	a=sqrt(dot(v,v));
	if( a==0.0 ) {
		for(i=0;i<3;i++) v[i]=0.0;
		return(FALSE);
		}
	else {
		for(i=0;i<3;i++) v[i]/=a;
		return(TRUE);
		}
	}

double ttintegralnew(v0,g,R,psi,method)
double v0,g,R,psi;
int *method;
{
/* (integral) d-arclength / velocity            */
/* equals                                       */
/* (integral) R d(psi) / [ v0 + g*z ]           */
/* but                                          */
/* v0 is zero, and z=R*cos(phi) do equals       */
/* (integral) R d(psi) / g*R*cos(phi)           */
/* equals                                       */
/* (1/g) * (integral) d(psi) / cos(phi)         */

	double integral294a();
	*method=294;
	return(integral294a(DTOR*psi)/g);
	}

double ttintegralold(v0,g,R,psi,method)
double v0,g,R,psi;
int *method;
{
/* indefinite integral of d(psi)/(b+c*cos(psi)) */
/* see G&R p148 2.553.3 for this integral */

	double b, c, bb, cc, s, x, y1, y2, t, z, psia;

	/* actually, psi had better be between -90 and 90 */
	if( psi<=(-90.0) || psi>=(90.0) ) {
		fprintf(stderr,"warning: psi %f out of range\n", psi );
		}

	/* the integrand is symmetric in psi, so always do integral */
	/* between 0 and +psi, reversing sign of result if psi<0    */

	b=v0;
	c=g*R;
	bb=b*b;
	cc=c*c;

	psia=fabs(psi);

	if( fabs(b/c) < 1.0e-6 ) { /* b==0.0 case */
		*method=0;
		z=log(fabs(tan((PI/4.0)+(DTOR*psia/2.0))))/g;
		}
	else if( bb>cc ) {
		*method=1;
		t = tan(DTOR*psia/2.0);
		s=sqrt(bb-cc);
		x=2.0/s;
		y1=s*t/(b+c);
		z = R*x*atan(y1);
		}
	else {
		*method=2;
		t = tan(DTOR*psia/2.0);
		s=sqrt(cc-bb);
		x=1.0/s;
		y1=s*t + (b+c);
		y2=s*t - (b+c);
		z=R*x*log(fabs(y1/y2));
		}
	if( psi < 0.0 ) z*=(-1.0);
	return(z);
	}


double myrand()
{
        double x;
        x=( ((double)(random())) / ((double)2147483647) );
        return(x);
        }

int ranvec(t)
double t[3];
{
	t[0]=myrand();
	t[1]=myrand();
	t[2]=myrand();
	return(TRUE);
	}

int print_intersec(f,i)
FILE *f;
struct int_rec *i;
{

	fprintf(f,"face %d inside %d psi %f dpsi %f\n",
		i->face, i->inside, i->psi, i->dpsi );
	fprintf(f,"x %f %f %f\n", i->x[0], i->x[1], i->x[2]  );
	fprintf(f,"xme %f %f %f\n", i->xme[0], i->xme[1], i->xme[2]  );
	fprintf(f,"epsilon %e R %f val %f\n", i->epsilon, i->R, i->val );
	fprintf(f,"\n");

	return(TRUE);
	}

int load_prismatic_model1(filename)
char *filename;
{
	int i, j, k;
	int l, m, n, lm, I, u, mmo, status, count;
	int n1,n2,n3,n4,n5,n6,n7,n8;
	int *nodelist;
	double x, y, z, v, Q, qinv;
	FILE *f;
	int this, above, below, right, left, front, behind;
	char line[1024], str[1024];

	if( (f=fopen(filename,"r")) == NULL ) {
		fprintf(stderr,"error: cant open model file <%s> for read\n", filename);
		return(FALSE);
		}
	fgets(line,1024,f);
	if( sscanf(line,"%d%d%d", &l, &m, &n ) != 3 ) {
		fprintf(stderr,"error: cant read model size in <%s> \n", filename);
		return(FALSE);
		}

	model_L=l; model_M=m; model_N=n;

	lm = l * m;
	u = (l-1)*(m-1);
	mmo = m-1;
	if( (nodelist=(int *)malloc((l*n*m+10)*sizeof(int))) == NULL ) {
		fprintf(stderr,"error: out of memory (1) \n");
		return(FALSE);
		}
	for( i=0; i<l; i++ ) for( j=0; j<m; j++ ) for( k=0; k<n; k++ ) {
		*(nodelist + k*lm + j*l + i) = (-1);
		}
	MAX_NODES=(l*n*m+10);
	if( (node=(struct node_rec *)malloc(MAX_NODES*sizeof(struct node_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (2) \n");
		return(FALSE);
		}
	MAX_TETRAS=(5*(l-1)*(n-1)*(m-1)+10);
	if( (tetra=(struct tetra_rec *)malloc(MAX_TETRAS*sizeof(struct tetra_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (3) \n");
		return(FALSE);
		}

	while( fgets(line,1024,f) != NULL ) {
		
		count=sscanf(line,"%s%s%s%s%s%s%s%s%s",str,str,str,str,str,str,str,str,str);
		if( count==8 ) {
			if( sscanf(line,"%d%d%d%le%le%le%le%d",&i,&j,&k,&x,&y,&z,&v,&I )!=8 ) {
				fprintf(stderr,"error: bad line in model file <%s>\n",filename);
				return(FALSE);
				}
			qinv=0.0;
			}
		else if( count==9 ) {
			if( sscanf(line,"%d%d%d%le%le%le%le%le%d",&i,&j,&k,&x,&y,&z,&v,&Q,&I)!=9) {
				fprintf(stderr,"error: bad line in model file <%s>\n",filename);
				return(FALSE);
				}
			qinv=1.0/Q;
			QDEFINED=TRUE;
			}
		else {
			fprintf(stderr,"error: too few fields in model file <%s>\n",filename);
			return(FALSE);
			}

		if(PERTURBMODEL) {
			if(i>0 && i<(l-1) ) x+=node_x_perturbation*(myrand()-0.5);
			if(j>0 && j<(m-1) ) y+=node_y_perturbation*(myrand()-0.5);
			if(k>0 && k<(n-1) ) z+=node_z_perturbation*(myrand()-0.5);
			v+=node_v_perturbation*(myrand()-0.5);
			}
		*(nodelist + k*lm + j*l + i) = (status=new_node(k*lm+j*l+i,x,y,z,v,qinv,I));
		if( status == (-1) ) {
			fprintf(stderr,"error: failed to make node (%d %d %d) \n",i,j,k);
			return(FALSE);
			}
		}

	fclose(f);

	for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) for( k=0; k<(n-1); k++ ) {
		if( *(nodelist + k*lm + j*l + i) == (-1) ) {
			fprintf(stderr,"error: missing node (%d %d %d)\n", i, j, k );
			return(FALSE);
			}
		}

	for( k=0; k<(n-1); k++ ) for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) {
		n1 = *(nodelist + (k+1)*lm + j*l + i);
		n2 = *(nodelist + (k+1)*lm + j*l + (i+1));
		n3 = *(nodelist + (k+1)*lm + (j+1)*l + (i+1));
		n4 = *(nodelist + (k+1)*lm + (j+1)*l + i);
		n5 = *(nodelist + k*lm + j*l + i);
		n6 = *(nodelist + k*lm + j*l + (i+1));
		n7 = *(nodelist + k*lm + (j+1)*l + (i+1));
		n8 = *(nodelist + k*lm + (j+1)*l + i);
		status=prism_to_tetra1(k*lm+j*l+i, n1,n2,n3,n4,n5,n6,n7,n8 , (i+j+k)%2);
		if( !status ) {
			fprintf(stderr,"error: failed to make tetra from prism (%d %d %d) \n",i,j,k);
			return(FALSE);
			}
		}

	for( k=0; k<(n-1); k++ ) for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) {
		this=5*(k*u+i*mmo+j);
		if( k>0 )     above=5*((k-1)*u+i*mmo+j);  else above=(-1);
		if( k<(n-2) ) below=5*((k+1)*u+i*mmo+j);  else below=(-1);
		if( i>0 )     left =5*(k*u+(i-1)*mmo+j);  else left =(-1);
		if( i<(l-2) ) right=5*(k*u+(i+1)*mmo+j);  else right=(-1);
		if( j>0 )    behind=5*(k*u+i*mmo+(j-1));  else behind=(-1);
		if( j<(m-2) ) front=5*(k*u+i*mmo+(j+1));  else front=(-1);

		if( (i+j+k)%2 ) {
			if( above!=(-1) ) {
				tetra[this+4].t1=above+1;
				tetra[this+3].t1=above+4;
				}
			if( below!=(-1) ) {
				tetra[this+2].t4=below+2;
				tetra[this+1].t4=below+3;
				}
			if( right!=(-1) ) {
				tetra[this+2].t3=right+4;
				tetra[this+4].t2=right+3;
				}
			if( left!=(-1) ) {
				tetra[this+3].t3=left+2;
				tetra[this+1].t2=left+1;
				}
			if( front!=(-1) ) {
				tetra[this+2].t1=front+1;
				tetra[this+3].t2=front+3;
				}
			if( behind!=(-1) ) {
				tetra[this+4].t4=behind+2;
				tetra[this+1].t3=behind+4;
				}
			}
		else {
			if( above!=(-1) ) {
				tetra[this+2].t1=above+2;
				tetra[this+3].t1=above+1;
				}
			if( below!=(-1) ) {
				tetra[this+1].t4=below+4;
				tetra[this+4].t4=below+3;
				}
			if( right!=(-1) ) {
				tetra[this+1].t1=right+1;
				tetra[this+2].t4=right+3;
				}
			if( left!=(-1) ) {
				tetra[this+3].t3=left+4;
				tetra[this+4].t2=left+2;
				}
			if( front!=(-1) ) {
				tetra[this+4].t1=front+1;
				tetra[this+2].t2=front+4;
				}
			if( behind!=(-1) ) {
				tetra[this+1].t3=behind+2;
				tetra[this+3].t4=behind+3;
				}
			}
		}

	if( m*n*l < (100) ) check_tetras(stderr);

	return(TRUE);
	}

int load_prismatic_model2(filename)
char *filename;
{
	int i, j, k;
	int l, m, n, lm, I, u, mmo, status, count;
	int n1,n2,n3,n4,n5,n6,n7,n8;
	int *nodelist;
	double x, y, z, v, Q, qinv;
	FILE *ff;
	int this, above, below, right, left, front, behind;
	int a=0, b=1, c=2, d=3, e=4, f=5;
	char line[1024], str[1024];

	if( (ff=fopen(filename,"r")) == NULL ) {
		fprintf(stderr,"error: cant open model file <%s> for read\n", filename);
		return(FALSE);
		}

	fgets(line,1024,ff);
	if( sscanf(line,"%d%d%d", &l, &m, &n ) != 3 ) {
		fprintf(stderr,"error: cant read model size in <%s> \n", filename);
		return(FALSE);
		}

	model_L=l; model_M=m; model_N=n;

	lm = l * m;
	u = (l-1)*(m-1);
	mmo = m-1;
	if( (nodelist=(int *)malloc((l*n*m+10)*sizeof(int))) == NULL ) {
		fprintf(stderr,"error: out of memory (1) \n");
		return(FALSE);
		}
	for( i=0; i<l; i++ ) for( j=0; j<m; j++ ) for( k=0; k<n; k++ ) {
		*(nodelist + k*lm + j*l + i) = (-1);
		}
	MAX_NODES=(l*n*m+10);
	if( (node=(struct node_rec *)malloc(MAX_NODES*sizeof(struct node_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (2) \n");
		return(FALSE);
		}
	MAX_TETRAS=(6*(l-1)*(n-1)*(m-1)+10);
	if( (tetra=(struct tetra_rec *)malloc(MAX_TETRAS*sizeof(struct tetra_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (3) \n");
		return(FALSE);
		}

	while( fgets(line,1024,ff) != NULL ) {
		count=sscanf(line,"%s%s%s%s%s%s%s%s%s",str,str,str,str,str,str,str,str,str);
		if( count==8 ) {
			if( sscanf(line,"%d%d%d%le%le%le%le%d",&i,&j,&k,&x,&y,&z,&v,&I )!=8 ) {
				fprintf(stderr,"error: bad line in model file <%s>\n",filename);
				return(FALSE);
				}
			qinv=0.0;
			}
		else if( count==9 ) {
			if( sscanf(line,"%d%d%d%le%le%le%le%le%d",&i,&j,&k,&x,&y,&z,&v,&Q,&I)!=9) {
				fprintf(stderr,"error: bad line in model file <%s>\n",filename);
				return(FALSE);
				}
			qinv=1.0/Q;
			QDEFINED=TRUE;
			}
		else {
			fprintf(stderr,"error: too few fields in model file <%s>\n",filename);
			return(FALSE);
			}

		if(PERTURBMODEL) {
			if(i>0 && i<(l-1) ) x+=node_x_perturbation*(myrand()-0.5);
			if(j>0 && j<(m-1) ) y+=node_y_perturbation*(myrand()-0.5);
			if(k>0 && k<(n-1) ) z+=node_z_perturbation*(myrand()-0.5);
			v+=node_v_perturbation*(myrand()-0.5);
			}
		*(nodelist + k*lm + j*l + i) = (status=new_node(k*lm+j*l+i,x,y,z,v,qinv,I));
		if( status == (-1) ) {
			fprintf(stderr,"error: failed to make node (%d %d %d) \n",i,j,k);
			return(FALSE);
			}
		}

	fclose(ff);

	for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) for( k=0; k<(n-1); k++ ) {
		if( *(nodelist + k*lm + j*l + i) == (-1) ) {
			fprintf(stderr,"error: missing node (%d %d %d)\n", i, j, k );
			return(FALSE);
			}
		}

	for( k=0; k<(n-1); k++ ) for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) {
		n1 = *(nodelist + (k+1)*lm + j*l + i);
		n2 = *(nodelist + (k+1)*lm + j*l + (i+1));
		n3 = *(nodelist + (k+1)*lm + (j+1)*l + (i+1));
		n4 = *(nodelist + (k+1)*lm + (j+1)*l + i);
		n5 = *(nodelist + k*lm + j*l + i);
		n6 = *(nodelist + k*lm + j*l + (i+1));
		n7 = *(nodelist + k*lm + (j+1)*l + (i+1));
		n8 = *(nodelist + k*lm + (j+1)*l + i);
		status=prism_to_tetra2(k*lm+j*l+i, n1,n2,n3,n4,n5,n6,n7,n8 , 0);
		if( !status ) {
			fprintf(stderr,"error: failed to make tetra from prism (%d %d %d) \n",i,j,k);
			return(FALSE);
			}
		}

	for( k=0; k<(n-1); k++ ) for( i=0; i<(l-1); i++ ) for( j=0; j<(m-1); j++ ) {
		this=6*(k*u+i*mmo+j);
		if( k>0 )     above=6*((k-1)*u+i*mmo+j);  else above=(-1);
		if( k<(n-2) ) below=6*((k+1)*u+i*mmo+j);  else below=(-1);
		if( i>0 )     left =6*(k*u+(i-1)*mmo+j);  else left =(-1);
		if( i<(l-2) ) right=6*(k*u+(i+1)*mmo+j);  else right=(-1);
		if( j>0 )    behind=6*(k*u+i*mmo+(j-1));  else behind=(-1);
		if( j<(m-2) ) front=6*(k*u+i*mmo+(j+1));  else front=(-1);

		if( above!=(-1) ) {
			tetra[this+a].t1=above+c;
			tetra[this+d].t1=above+f;
			}
		if( below!=(-1) ) {
			tetra[this+c].t4=below+a;
			tetra[this+f].t4=below+d;
			}
		if( right!=(-1) ) {
			tetra[this+d].t2=right+b;
			tetra[this+e].t3=right+c;
			}
		if( left!=(-1) ) {
			tetra[this+b].t1=left+d;
			tetra[this+c].t2=left+e;
			}
		if( front!=(-1) ) {
			tetra[this+a].t2=front+e;
			tetra[this+b].t3=front+f;
			}
		if( behind!=(-1) ) {
			tetra[this+e].t2=behind+a;
			tetra[this+f].t3=behind+b;
			}
		}

	if( m*n*l < (100) ) check_tetras(stderr);

	return(TRUE);
	}


int write_model_blerb(f)
FILE *f;
{
	fprintf(f,"      (x, y, z) cartesian coordinates, +z is down\n");
	fprintf(f,"      input model_file for prismatic model of LxMxN nodes:\n");
	fprintf(f,"           line 1: L M N\n");
	fprintf(f,"           subsequent lines i j k x y z v interface\n");
	fprintf(f,"               i j k, with 0<=i<L & etc. are node indices\n");
	fprintf(f,"           		in a cartesian grid of LxMxN nodes\n");
	fprintf(f,"               x y z are node positions, in km\n");
	fprintf(f,"                     (need not be regularly spaced)\n");
	fprintf(f,"               v is velocity in km/s\n");
	fprintf(f,"               interface, an integer, associates the nodes with an interface\n");
	fprintf(f,"			(interface numbers are arbitrary)\n");
	fprintf(f,"           or\n");
	fprintf(f,"           subsequent lines i j k x y z v Q interface\n");
	fprintf(f,"               Q is quality factor\n");
	fprintf(f,"           or\n");
	fprintf(f,"           subsequent lines i j k x y z v ratio interface\n");
	fprintf(f,"               ratio is compressional to shear wave velocity ratio (e.g 1.76)\n");
	return(TRUE);
	}
	
int write_solution_blerb(f)
FILE *f;
{
	fprintf(f,"\n");
	fprintf(f,"      A note on the method use to solve large, sparse linear systems\n");
	fprintf(f,"           Many raytrace3d functions require the solution of large\n");
	fprintf(f,"           systems of linear equations, i.e. the N by M matrix equation\n");
	fprintf(f,"                          G m = d\n");
	fprintf(f,"           where the matrix G is sparse.  This code uses a pointer-based\n");
	fprintf(f,"           method of storing just the non-zero elements of G.  When the\n");
	fprintf(f,"           damped least-squares solution:\n");
	fprintf(f,"             A m = b, where A=(G-traspose G + e I) and b = G-transpose d\n");
	fprintf(f,"           (where e is the damping parameter) is required, an iterative, \n");
	fprintf(f,"	      biconjugate gradient method is used.  This method starts with\n");
	fprintf(f,"           the solution m=0 and iteratively improves it.  The method only\n");
	fprintf(f,"           needs to compute A v, A-transpose v, and v / diag(A), where v\n");
	fprintf(f,"           is an arbitray vector.  These operations are implemented directly\n");
	fprintf(f,"           from G, so that A is never explicitly computed (e.g. \n");
	fprintf(f,"                     A v = G-transpose ( G v ) + e v).\n");
	return(TRUE);
	}

int write_model_pars_blerb(f)
FILE *f;
{
	fprintf(f,"        model_pars_file.txt:\n");
	fprintf(f,"          a model parameter is a group of one or mode nodes\n");
	fprintf(f,"          whose velocity is to be adjusted as a unit\n");
	fprintf(f,"          the file contains any number of groups of:\n");
	fprintf(f,"             number_of_nodes in group\n");
	fprintf(f,"             i j k weight of first node in group (matching model file)\n");
	fprintf(f,"             i j k weight of second node in group\n");
	fprintf(f,"		etc.\n");
	fprintf(f,"          where weight scales the overall motion of that node\n");
	fprintf(f,"          relative to others in the group. As the problem is solved\n");
	fprintf(f,"          via least-squares, overlapping groups are perfectly\n");
	fprintf(f,"          acceptable and will impose overall smoothness on the\n");
	fprintf(f,"          velocity perturbations. Negative weights are fine\n");
	fprintf(f,"          as well and could be used, for example, to implement\n");
	fprintf(f,"          lateral gradients in velocity, by defining three model\n");
	fprintf(f,"          parameters for every interface, each containing all\n");
	fprintf(f,"          the nodes on the interface, one model parameter with\n");
	fprintf(f,"          constant weights, one that scales linearly with the\n");
	fprintf(f,"          x-position of the nodes, the other that scales linearly\n");
	fprintf(f,"          with the y-position of the nodes.\n");
	return(TRUE);
	}

int write_refl_pars_blerb(f)
FILE *f;
{
	fprintf(f,"        model_pars_file.txt:\n");
	fprintf(f,"          a model parameter is a group of one or mode nodes on the reflecting\n");
	fprintf(f,"          interface whose position is to be adjusted as a unit\n");
	fprintf(f,"          the file contains any number of groups of\n");
	fprintf(f,"             number_of_nodes in group\n");
	fprintf(f,"             i j k tx xy tz weight\n");
	fprintf(f,"	                                of first node in group\n");
	fprintf(f,"             i j k tx xy tz weight\n");
	fprintf(f,"	                                of first second in group\n");
	fprintf(f,"		etc.\n");
	fprintf(f,"          where i j k are node indices corresponding to the model file\n");
	fprintf(f,"          and where tx xy tz is a unit vector specifying what direction the\n");
	fprintf(f,"              node is to be perturbed (typically, but not necessarily,\n");
	fprintf(f,"              perpendicular to interface \n");
	fprintf(f,"          and where weight scales the overall motion of that node\n");
	fprintf(f,"          relative to others in the group. As the problem is solved\n");
	fprintf(f,"          via least-squares, overlapping groups are perfectly\n");
	fprintf(f,"          acceptable and will impose overall smoothness on the\n");
	fprintf(f,"          node position perturbations. Negative weights are fine\n");
	fprintf(f,"          as well and could be used, for example, to implement\n");
	fprintf(f,"          a best-fitting sloping interface, by defining three model\n");
	fprintf(f,"          parameters for the interface, each containing all\n");
	fprintf(f,"          the nodes on the interface, one model parameter with\n");
	fprintf(f,"          constant weights, one that scales linearly with the\n");
	fprintf(f,"          x-position of the nodes, the other that scales linearly\n");
	fprintf(f,"          with the y-position of the nodes.\n");
	return(TRUE);
	}

int write_traveltime_blerb(f)
FILE *f;
{
	fprintf(f,"      output traveltime_file.tsv\n");
	fprintf(f,"           id is ray number\n");
	fprintf(f,"           x0 y0 z0, source location in km\n");
	fprintf(f,"           x1 y1 z1, receiver location in km\n");
	fprintf(f,"	      T, traveltime in s\n");
	fprintf(f,"	      and when Q is defined in model\n");
	fprintf(f,"	      Q, path-averaged quality factor\n");
	return(TRUE);
	}

int write_predictions_blerb(f)
FILE *f;
{
	fprintf(f,"      output predicted_tt_file.tsv\n");
	fprintf(f,"           x0 y0 z0, source location in km\n");
	fprintf(f,"           x1 y1 z1, receiver location in km\n");
	fprintf(f,"	      T0, origin time in s\n");
	fprintf(f,"	      TT, traveltime in s\n");
	fprintf(f,"	      T, arrival in s\n");
	fprintf(f,"	      dT, residual in s\n");
	fprintf(f,"	      ratio, velocity ratio\n");
	fprintf(f,"	      obs_order, of observed arrival\n");
	fprintf(f,"	      pre_order, of predicted arrival\n");
	fprintf(f,"	      amplitude, geometrical amplitude of predicted arrival\n");
	fprintf(f,"	      and, when Q is defined in the model\n");
	fprintf(f,"	      Q, path-averaged quality factor\n");
	fprintf(f,"	      n_tts, traveltime number, stating with 0\n");

	return(TRUE);
	}

int write_location_blerb(f)
FILE *f;
{
	fprintf(f,"      output location_file.tsv\n");
	fprintf(f,"           xs ys zs, source location in km\n");
	fprintf(f,"	      t0, origin time in s\n");
	fprintf(f,"           sigma-x sigma-y sigma-z, source error in km\n");
	fprintf(f,"           sigma-t, origin time error in s\n");
	fprintf(f,"	      ndata, number of data used in final iteration\n");
	fprintf(f,"	      T, arrival in s\n");
	fprintf(f,"	      dT, residual in s\n");
	fprintf(f,"	      iter, numberof iterations\n");
	fprintf(f,"	      rms, rms traveltime error in s\n");
	return(TRUE);
	}

int write_ray_blerb(f)
FILE *f;
{
	fprintf(f,"      output ray_file.tsv (a filename of - indicates stdout)\n");
	fprintf(f,"           id is ray number\n");
	fprintf(f,"           x y z, location of point on ray, km\n");
	fprintf(f,"           tetra, index number of tetrahedra at that point\n");
	return(TRUE);
	}

int near_tetrahedron( x1, x2, x3, x4, x)
double x1[3], x2[3], x3[3], x4[3], x[3];
{
	double epsilon, y[3];

	epsilon=1.0e-3;

	if( in_tetrahedron(x1, x2, x3, x4, x) ) return(TRUE);

	y[0]=x[0]+epsilon;
	y[1]=x[1];
	y[2]=x[2];
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);

	y[0]=x[0]-epsilon;
	y[1]=x[1];
	y[2]=x[2];
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);

	y[0]=x[0];
	y[1]=x[1]+epsilon;
	y[2]=x[2];
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);

	y[0]=x[0];
	y[1]=x[1]-epsilon;
	y[2]=x[2];
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);

	y[0]=x[0];
	y[1]=x[1];
	y[2]=x[2]+epsilon;
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);

	y[0]=x[0];
	y[1]=x[1];
	y[2]=x[2]-epsilon;
	if( in_tetrahedron(x1, x2, x3, x4, y) ) return(TRUE);
		
	return(FALSE);
	}

int load_star_file(filename)
char *filename;
{
	int i, j, k;
	FILE *f;
	char s0[128], s1[128], s2[128];
	double th, az, x, y, z, T;

	if( (f=fopen(filename,"r")) == NULL ) {
		fprintf(stderr,"error: cant open star file <%s> for read\n", filename);
		return(FALSE);
		}

	if( fscanf(f,"%s%s%s%s%s%s%s%s", s0,s1,s2,s2,s2,s2,s2,s2 ) != 8 ) {
		fprintf(stderr,"error: cant read star header <%s>\n", filename);
		return(FALSE);
		}

	if( sscanf(s0,"th_%d", &m_rays) != 1 ) {
		fprintf(stderr,"error: cant decode header in star file <%s> (1)\n", filename);
		return(FALSE);
		}

	if( sscanf(s1,"az_%d", &n_rays) != 1 ) {
		fprintf(stderr,"error: cant decode header in star file <%s> (2)\n", filename);
		return(FALSE);
		}

	if((ray=(struct ray_rec*)malloc((n_rays*m_rays+10)*sizeof(struct ray_rec)))==NULL){
		fprintf(stderr,"error: out of memory\n");
		return(FALSE);
		}

	for( i=0; i<m_rays; i++ ) for( j=0; j<n_rays; j++ ) ray[i+j*m_rays].ok=FALSE;

	while( fscanf(f,"%d%d%le%le%le%le%le%le",&i,&j,&th,&az,&x,&y,&z,&T)==8 ) {
		if( i<0 || i>m_rays || j<0 || j>n_rays ) {
			fprintf(stderr,"error: bad i,j in star file <%s>\n", filename);
			return(FALSE);
			}
		k=i+j*m_rays;
		ray[k].ok=TRUE;
		ray[k].az=az;
		ray[k].th=th;
		ray[k].x=x;
		ray[k].y=y;
		ray[k].z=z;
		ray[k].T=T;

		}
		
	fclose(f);
	return(TRUE);
	}

int load_turf_file(filename)
char *filename;
{
	int i, j, k, N;
	FILE *f;
	char s0[128], s1[128], s2[128];
	double x0, y0, x, y, z, T;

	if( (f=fopen(filename,"r")) == NULL ) {
		fprintf(stderr,"error: cant open turf file <%s> for read\n", filename);
		return(FALSE);
		}

	if( fscanf(f,"%s%s%s%s%s%s%s%s", s0,s1,s2,s2,s2,s2,s2,s2 ) != 8 ) {
		fprintf(stderr,"error: cant read turf header <%s>\n", filename);
		return(FALSE);
		}

	if( sscanf(s0,"x_%d", &m_turf) != 1 ) {
		fprintf(stderr,"error: cant decode header in turf file <%s> (1)\n", filename);
		return(FALSE);
		}

	if( sscanf(s1,"y_%d", &n_turf) != 1 ) {
		fprintf(stderr,"error: cant decode header in turf file <%s> (2)\n", filename);
		return(FALSE);
		}

	N=n_turf*m_turf;

	if((turf=(struct turf_rec*)malloc((N+10)*sizeof(struct turf_rec)))==NULL){
		fprintf(stderr,"error: out of memory\n");
		return(FALSE);
		}

	for( i=0; i<m_turf; i++ ) for( j=0; j<n_turf; j++ ) {
		k=i+j*m_turf;

		if( k>=N ) {
			fprintf(stderr,"error: bad index (%d %d) in turf file <%s>\n",
				i, j, filename);
			return(FALSE);
			}
		turf[k].ok=FALSE;
		}

	while( fscanf(f,"%d%d%le%le%le%le%le%le",&i,&j,&x0,&y0,&x,&y,&z,&T)==8 ) {
		if( i<0 || i>m_turf || j<0 || j>n_turf ) {
			fprintf(stderr,"error: bad i,j in turf file <%s>\n", filename);
			return(FALSE);
			}
		k=i+j*m_turf;
		if( k>=N ) {
			fprintf(stderr,"error: bad index (%d %d) in turf file <%s>\n",
				i, j, filename);
			return(FALSE);
			}
		turf[k].ok=TRUE;
		turf[k].x0=x0;
		turf[k].y0=y0;
		turf[k].x=x;
		turf[k].y=y;
		turf[k].z=z;
		turf[k].T=T;
		}
		
	fclose(f);
	return(TRUE);
	}

int instarbox(i, j, l, xsta, ysta, th_est, az_est, tri, x00, interface, gs)
int i, j, l;
double xsta, ysta;
double *th_est, *az_est;
struct triangle_rec *tri;
double x00[3];
int interface;
double *gs;
{
	int ii, k[3], sa, hitflag, n_arcs, found, ec;
	double x[3], y[3], z[3], a[3], t[3], T[3], temp;
	double aa, bb, L[3];
	double vertex[4][3], *az[4], *th[4], coef[4], x0[3];
	double tmid, amid, x1[3], tan0[3], tan1[3], xmid, ymid, zmid, Tmid;
	double solangle, triarea, trinormal[3], triraycos, raytan[3], qinvint;

	/* view the box as two triangles, do each separately */
	if( l ) {
		k[0]=i+j*m_rays;
		k[1]=(i+1)+j*m_rays;
		k[2]=i+(j+1)*m_rays;
		}
	else {
		k[0]=(i+1)+(j+1)*m_rays;
		k[1]=(i+1)+j*m_rays;
		k[2]=i+(j+1)*m_rays;
		}

	for( ii=0; ii<3; ii++ ) {
		if( !ray[k[ii]].ok ) return(FALSE);
		x[ii]=ray[k[ii]].x;
		y[ii]=ray[k[ii]].y;
		z[ii]=ray[k[ii]].z;
		a[ii]=ray[k[ii]].az;
		t[ii]=ray[k[ii]].th;
		T[ii]=ray[k[ii]].T;
		}

	if( !in_triangle( xsta, ysta, x[0], y[0], x[1], y[1], x[2], y[2] ) )
		return(FALSE);

	/* attempt to refine by repeatedly dicing triangle into 3 pieces*/
	for( ii=0; ii<10; ii++ ) {
		amid=(a[0]+a[1]+a[2])/3.0;
		tmid=(t[0]+t[1]+t[2])/3.0;
		/* bad things may happen if NaN's appear */
		if( (!finite(amid)) || (!finite(tmid)) ) {
			return(FALSE);
			}
		ray_tangent(tmid,amid,tan0);
		sa=shoot_to_interface( x00, tan0, x1, tan1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &Tmid, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) break;

		/* geometrical speading calculation */
		raytan[0]=tan1[0]; raytan[1]=tan1[1]; raytan[2]=tan1[2];
		if( ii<3 ) if( gs!=NULL ) {
			solangle=solid_angle( t[0], a[0], t[1], a[1], t[2], a[2] );
			triarea = area_of_triagle( x, y, z, trinormal );
			triraycos = fabs(dot(trinormal,raytan));
			(*gs) = (triarea * triraycos) / solangle;
			}
		xmid=x1[0]; ymid=x1[1]; zmid=x1[2];
		found=FALSE;
		if( in_triangle(xsta,ysta, xmid,ymid, x[1],y[1], x[2],y[2] ) ) {
			found=TRUE;
			x[0]=xmid; y[0]=ymid; z[0]=zmid;
			a[0]=amid; t[0]=tmid, T[0]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], xmid,ymid, x[2],y[2] ) ) {
			found=TRUE;
			x[1]=xmid; y[1]=ymid; z[1]=zmid;
			a[1]=amid; t[1]=tmid, T[1]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], x[1],y[1], xmid,ymid ) ) {
			found=TRUE;
			x[2]=xmid; y[2]=ymid; z[2]=zmid;
			a[2]=amid; t[2]=tmid, T[2]=Tmid;
			}
		if( !found ) break;
		}

	/* further refine by linear interpolation within triangle */
	/* augment triangle to tetrahededron to use existing interpolation routine*/
		
	tri->x0=t[0];
	tri->y0=a[0];
	tri->x1=t[1];
	tri->y1=a[1];
	tri->x2=t[2];
	tri->y2=a[2];

	for( ii=0; ii<3; ii++ ) {
		vertex[ii][0]=x[ii];
		vertex[ii][1]=y[ii];
		vertex[ii][2]=0.0;
		az[ii]=(&a[ii]);
		th[ii]=(&t[ii]);
		}

	vertex[3][0]=x[0];
	vertex[3][1]=y[0];
	vertex[3][2]=1.0;
	az[3]=(&a[0]);
	th[3]=(&t[0]);

	x0[0]=xsta;
	x0[1]=ysta;
	x0[2]=0.0;

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], az, coef );
	(*az_est)=eval_velocity2( x0, coef );

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], th, coef );
	(*th_est)=eval_velocity2( x0, coef );
	
	return(TRUE);
	}

int instarbox_refl(i, j, l, xsta, ysta, th_est, az_est, tri, x00, interface, rinterface, gs)
int i, j, l;
double xsta, ysta;
double *th_est, *az_est;
struct triangle_rec *tri;
double x00[3];
int interface, rinterface;
double *gs;
{
	int ii, k[3], sa, hitflag, n_arcs, found, ec;
	double x[3], y[3], z[3], a[3], t[3], T[3], temp;
	double aa, bb, L[3];
	double vertex[4][3], *az[4], *th[4], coef[4], x0[3];
	double tmid, amid, x1[3], tan0[3], tan1[3], xmid, ymid, zmid, Tmid;
	double solangle, triarea, trinormal[3], triraycos, raytan[3], qinvint;

	/* view the box as two triangles, do each separately */
	if( l ) {
		k[0]=i+j*m_rays;
		k[1]=(i+1)+j*m_rays;
		k[2]=i+(j+1)*m_rays;
		}
	else {
		k[0]=(i+1)+(j+1)*m_rays;
		k[1]=(i+1)+j*m_rays;
		k[2]=i+(j+1)*m_rays;
		}

	for( ii=0; ii<3; ii++ ) {
		if( !ray[k[ii]].ok ) return(FALSE);
		x[ii]=ray[k[ii]].x;
		y[ii]=ray[k[ii]].y;
		z[ii]=ray[k[ii]].z;
		a[ii]=ray[k[ii]].az;
		t[ii]=ray[k[ii]].th;
		T[ii]=ray[k[ii]].T;
		}

	if( !in_triangle( xsta, ysta, x[0], y[0], x[1], y[1], x[2], y[2] ) )
		return(FALSE);

	/* attempt to refine by repeatedly dicing triangle into 3 pieces*/
	for( ii=0; ii<10; ii++ ) {
		amid=(a[0]+a[1]+a[2])/3.0;
		tmid=(t[0]+t[1]+t[2])/3.0;
		/* bad things may happen if NaN's appear */
		if( (!finite(amid)) || (!finite(tmid)) ) {
			return(FALSE);
			}
		ray_tangent(tmid,amid,tan0);
		sa=shoot_to_interface_refl( x00, tan0, x1, tan1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &Tmid, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) break;

		/* geometrical speading calculation */
		raytan[0]=tan1[0]; raytan[1]=tan1[1]; raytan[2]=tan1[2];
		if( ii<3 ) if( gs!=NULL ) {
			solangle=solid_angle( t[0], a[0], t[1], a[1], t[2], a[2] );
			triarea = area_of_triagle( x, y, z, trinormal );
			triraycos = fabs(dot(trinormal,raytan));
			(*gs) = (triarea * triraycos) / solangle;
			}
		xmid=x1[0]; ymid=x1[1]; zmid=x1[2];
		found=FALSE;
		if( in_triangle(xsta,ysta, xmid,ymid, x[1],y[1], x[2],y[2] ) ) {
			found=TRUE;
			x[0]=xmid; y[0]=ymid; z[0]=zmid;
			a[0]=amid; t[0]=tmid, T[0]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], xmid,ymid, x[2],y[2] ) ) {
			found=TRUE;
			x[1]=xmid; y[1]=ymid; z[1]=zmid;
			a[1]=amid; t[1]=tmid, T[1]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], x[1],y[1], xmid,ymid ) ) {
			found=TRUE;
			x[2]=xmid; y[2]=ymid; z[2]=zmid;
			a[2]=amid; t[2]=tmid, T[2]=Tmid;
			}
		if( !found ) break;
		}

	/* further refine by linear interpolation within triangle */
	/* augment triangle to tetrahededron to use existing interpolation routine*/
		
	tri->x0=t[0];
	tri->y0=a[0];
	tri->x1=t[1];
	tri->y1=a[1];
	tri->x2=t[2];
	tri->y2=a[2];

	for( ii=0; ii<3; ii++ ) {
		vertex[ii][0]=x[ii];
		vertex[ii][1]=y[ii];
		vertex[ii][2]=0.0;
		az[ii]=(&a[ii]);
		th[ii]=(&t[ii]);
		}

	vertex[3][0]=x[0];
	vertex[3][1]=y[0];
	vertex[3][2]=1.0;
	az[3]=(&a[0]);
	th[3]=(&t[0]);

	x0[0]=xsta;
	x0[1]=ysta;
	x0[2]=0.0;

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], az, coef );
	(*az_est)=eval_velocity2( x0, coef );

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], th, coef );
	(*th_est)=eval_velocity2( x0, coef );
	
	return(TRUE);
	}

int inturfbox(i, j, l, xsta, ysta, x_est, y_est, tri, x00, interface, azi,v0,z0,T0,gs)
int i, j, l;
double xsta, ysta;
double *x_est, *y_est;
struct triangle_rec *tri;
double x00[3];
int interface;
double azi;
double v0;
double z0;
double T0;
double *gs;
{
	int ii, k[3], sa, hitflag, n_arcs, found, ec;
	double x[3], y[3], z[3], x0[3], y0[3], T[3], temp;
	double aa, bb, L[3];
	double vertex[4][3], *X0[4], *Y0[4], coef[4];
	double x0mid, y0mid, x1[3], tan0[3], tan1[3], xmid, ymid, zmid, Tmid, vp;
	double px, py, theta, p0[3];
	int t, my_last_tetra, iii;
	double raytan0[3], raytan1[3], triarea0, triarea1, raycos0, raycos1;
	double trinormal0[3], trinormal1[3], zz0[3], qinvint;

	my_last_tetra=(-1);

	/* view the box as two triangles, do each separately */
	if( l ) {
		k[0]=i+j*m_turf;
		k[1]=(i+1)+j*m_turf;
		k[2]=i+(j+1)*m_turf;
		}
	else {
		k[0]=(i+1)+(j+1)*m_turf;
		k[1]=(i+1)+j*m_turf;
		k[2]=i+(j+1)*m_turf;
		}

	for( ii=0; ii<3; ii++ ) {
		if( !turf[k[ii]].ok ) return(FALSE);
		x[ii]=turf[k[ii]].x;
		y[ii]=turf[k[ii]].y;
		z[ii]=turf[k[ii]].z;
		x0[ii]=turf[k[ii]].x0;
		y0[ii]=turf[k[ii]].y0;
		T[ii]=turf[k[ii]].T;
		}

	if( !in_triangle( xsta, ysta, x[0], y[0], x[1], y[1], x[2], y[2] ) )
		return(FALSE);

	/* attempt to refine by repeatedly dicing triangle into 3 pieces*/
	for( ii=0; ii<10; ii++ ) {
		x0mid=(x0[0]+x0[1]+x0[2])/3.0;
		y0mid=(y0[0]+y0[1]+y0[2])/3.0;
		/* bad things may happen if NaN's appear */
		if( (!finite(x0mid)) || (!finite(y0mid)) ) {
			return(FALSE);
			}
		x00[0]=x0mid;
		x00[1]=y0mid;
		x00[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x00,&t) ) {
			vp = eval_velocity(&tetra[t],x00);
			}
		else {
			return(FALSE);
			}
		if( vp>=v0 ) return(FALSE);
		my_last_tetra=last_tetra;
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,tan0);
		px = tan0[0]/vp; py=tan0[1]/vp;
		sa=shoot_to_interface( x00, tan0, x1, tan1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &Tmid, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) break;
		for( iii=0; iii<3; iii++ ) {
			raytan0[iii]=tan0[iii];
			raytan1[iii]=tan1[iii];
			}


		Tmid += T0 + px*x00[0] + py*x00[1];
		xmid=x1[0]; ymid=x1[1]; zmid=x1[2];
		found=FALSE;
		if( in_triangle(xsta,ysta, xmid,ymid, x[1],y[1], x[2],y[2] ) ) {
			found=TRUE;
			x[0]=xmid; y[0]=ymid; z[0]=zmid;
			x0[0]=x0mid; y0[0]=y0mid, T[0]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], xmid,ymid, x[2],y[2] ) ) {
			found=TRUE;
			x[1]=xmid; y[1]=ymid; z[1]=zmid;
			x0[1]=x0mid; y0[1]=y0mid, T[1]=Tmid;
			}
		else if( in_triangle(xsta,ysta, x[0],y[0], x[1],y[1], xmid,ymid ) ) {
			found=TRUE;
			x[2]=xmid; y[2]=ymid; z[2]=zmid;
			x0[2]=x0mid; y0[2]=y0mid, T[2]=Tmid;
			}
		if( !found ) break;
		/* geometrical spreading */
		if( ii<3 ) if( gs!=NULL ) {
			zz0[0]=z0; zz0[1]=z0; zz0[2]=z0;
			triarea0=area_of_triagle( x0, y0, zz0, trinormal0 );
			triarea1=area_of_triagle( x, y, z, trinormal1 );
			raycos0 = fabs(dot(trinormal0,raytan0));
			raycos1 = fabs(dot(trinormal1,raytan1));
			(*gs) = (triarea1 * raycos1) / (triarea0 * raycos0);
			}
		}


	/* further refine by linear interpolation within triangle */
	/* augment triangle to tetrahededron to use existing interpolation routine*/
		
	tri->x0=x0[0];
	tri->y0=y0[0];
	tri->x1=x0[1];
	tri->y1=y0[1];
	tri->x2=x0[2];
	tri->y2=y0[2];

	for( ii=0; ii<3; ii++ ) {
		vertex[ii][0]=x[ii];
		vertex[ii][1]=y[ii];
		vertex[ii][2]=0.0;
		X0[ii]=(&(x0[ii]));
		Y0[ii]=(&(y0[ii]));
		}

	vertex[3][0]=x[0];
	vertex[3][1]=y[0];
	vertex[3][2]=1.0;
	X0[3]=(&(x0[0]));
	Y0[3]=(&(y0[0]));

	p0[0]=xsta;
	p0[1]=ysta;
	p0[2]=0.0;

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], X0, coef );
	(*x_est)=eval_velocity2( p0, coef );

	velocity_eqn2( vertex[0], vertex[1], vertex[2], vertex[3], Y0, coef );
	(*y_est)=eval_velocity2( p0, coef );

	return(TRUE);
	}

int eqn_of_line( x0, y0, x1, y1, L )
double x0, y0, x1, y1, L[3];
{
	double d;

	d=x0*y1-x1*y0;
	if( d==0.0 ) return(FALSE);
	L[0] = (y0-y1)/d;
	L[1] = (x1-x0)/d;
	L[2] = 1.0;

	return(TRUE);
	}

double eval_line( x, y, L )
double x, y, L[3];
{
	return( L[0]*x+L[1]*y+L[2] );
	}

int refine( x0, xsta, ysta, interface, th_est, az_est, arc, n_arcs, traveltime, tri)
double x0[3], xsta, ysta, *th_est, *az_est;
int interface;
struct arc_rec *arc;
int *n_arcs;
double *traveltime;
struct triangle_rec *tri;
{
	int i, ec;
	double th, az, dth, daz, pth, paz, thold, azold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdth, dydth, dxdaz, dydaz, d;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;

	/* bad things may happen if NaN's appear */
	if( (!finite(*th_est)) || (!finite(*az_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	th = *th_est;
	az = *az_est;
	pth = 1.0e-5;
	paz = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;

	refine_star_status=0;
	refine_star_norm=0.0;

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		ray_tangent(th,az,t0);
		sa=shoot_to_interface( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th+pth,az,t0);
		sb=shoot_to_interface( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th,az+paz,t0);
		sc=shoot_to_interface( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		/* perturbation computed by Newton's method */
		dxdth = (x1b[0]-x1a[0])/pth;
		dydth = (x1b[1]-x1a[1])/pth;
		dxdaz = (x1c[0]-x1a[0])/paz;
		dydaz = (x1c[1]-x1a[1])/paz;

		d = dxdth*dydaz-dydth*dxdaz;

		dth = ( dydaz*(xsta-x1a[0]) - dxdaz*(ysta-x1a[1]) )/d;
		daz = (-dydth*(xsta-x1a[0]) + dxdth*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dth*dth+daz*daz); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dth *= f/e;
		daz *= f/e;

		thold=th;
		azold=az;

		/* bad things happen when Nan's appear */
		if( (!finite(dth)) || (!finite(daz)) ) {
			refine_star_status=1;
			return(FALSE);
			}

		th+=dth;
		az+=daz;

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		refine_star_norm=DX+DY;
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) { refine_star_status=2; return(FALSE); }

	*th_est = th;
	*az_est = az;

	return(TRUE);
	}

int refine_refl( x0, xsta, ysta, interface, rinterface, th_est, az_est, arc, n_arcs, traveltime, tri)
double x0[3], xsta, ysta, *th_est, *az_est;
int interface, rinterface;
struct arc_rec *arc;
int *n_arcs;
double *traveltime;
struct triangle_rec *tri;
{
	int i, ec;
	double th, az, dth, daz, pth, paz, thold, azold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdth, dydth, dxdaz, dydaz, d;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;

	/* bad things may happen if NaN's appear */
	if( (!finite(*th_est)) || (!finite(*az_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	th = *th_est;
	az = *az_est;
	pth = 1.0e-5;
	paz = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;

	refine_star_status=0;
	refine_star_norm=0.0;

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		ray_tangent(th,az,t0);
		sa=shoot_to_interface_refl( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th+pth,az,t0);
		sb=shoot_to_interface_refl( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		ray_tangent(th,az+paz,t0);
		sc=shoot_to_interface_refl( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) {refine_star_status=1; return(FALSE);}

		/* perturbation computed by Newton's method */
		dxdth = (x1b[0]-x1a[0])/pth;
		dydth = (x1b[1]-x1a[1])/pth;
		dxdaz = (x1c[0]-x1a[0])/paz;
		dydaz = (x1c[1]-x1a[1])/paz;

		d = dxdth*dydaz-dydth*dxdaz;

		dth = ( dydaz*(xsta-x1a[0]) - dxdaz*(ysta-x1a[1]) )/d;
		daz = (-dydth*(xsta-x1a[0]) + dxdth*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dth*dth+daz*daz); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dth *= f/e;
		daz *= f/e;

		thold=th;
		azold=az;

		/* bad things happen when Nan's appear */
		if( (!finite(dth)) || (!finite(daz)) ) {
			refine_star_status=1;
			return(FALSE);
			}

		th+=dth;
		az+=daz;

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		refine_star_norm=DX+DY;
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) { refine_star_status=2; return(FALSE); }

	*th_est = th;
	*az_est = az;

	return(TRUE);
	}


int refine_turf( x0, xsta, ysta, interface, x_est, y_est, arc, n_arcs, traveltime, tri,
azi, v0, z0, T0)
double x0[3], xsta, ysta, *x_est, *y_est;
int interface;
struct arc_rec *arc;
int *n_arcs;
double *traveltime;
struct triangle_rec *tri;
double azi, v0, z0, T0;
{
	int i, ec;
	double x, y, dx, dy, px, py, xold, yold;
	double t0[3], x1a[3], x1b[3], x1c[3], t1[3];
	double dxdx, dydx, dxdy, dydy, d, theta, ppx, ppy, vp;
	double e, f, maxe, TT, DX, DY, D, qinvint;
	int status, sa, sb, sc, hitflag, maxiterations;
	int t, my_last_tetra;

	/* bad things may happen if NaN's appear */
	if( (!finite(*x_est)) || (!finite(*y_est)) ) {
		refine_star_status=1;
		return(FALSE);
		}

	x = *x_est;
	y = *y_est;
	px = 1.0e-5;
	py = 1.0e-5;
	maxe=0.1; /*was maxe=0.005;*/
	D=1.0e-3;
	maxiterations=50;
	my_last_tetra=(-1);

	status=FALSE;

	for( i=0; i<maxiterations; i++ ) {

		x0[0]=x;
		x0[1]=y;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		my_last_tetra=last_tetra;
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;

		/* partial derivatives of (x,y) with respect to (th,azi)  */
		/* computed by finite differences with step size pth, paz */
		sa=shoot_to_interface( x0, t0, x1a, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, traveltime, &qinvint, &ec );
		if( (!sa) || (!hitflag) ) return(FALSE);
		if(traveltime!=NULL)
			(*traveltime) += T0 + ppx*x0[0] + ppy*x0[1];

		x0[0]=x+px;
		x0[1]=y;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;
		sb=shoot_to_interface( x0, t0, x1b, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sb) || (!hitflag) ) return(FALSE);

		x0[0]=x;
		x0[1]=y+py;
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else return(FALSE);
		if( vp>=v0 ) return(FALSE);
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		ppx = t0[0]/vp; ppy=t0[1]/vp;
		sc=shoot_to_interface( x0, t0, x1c, t1, NULL, n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &TT, &qinvint, &ec );
		if( (!sc) || (!hitflag) ) return(FALSE);

		/* perturbation computed by Newton's method */
		dxdx = (x1b[0]-x1a[0])/px;
		dydx = (x1b[1]-x1a[1])/px;
		dxdy = (x1c[0]-x1a[0])/py;
		dydy = (x1c[1]-x1a[1])/py;

		d = dxdx*dydy-dydx*dxdy;

		dx = ( dydy*(xsta-x1a[0]) - dxdy*(ysta-x1a[1]) )/d;
		dy = (-dydx*(xsta-x1a[0]) + dxdx*(ysta-x1a[1]) )/d;

		/* but don't alow perturbation to be larger than maxe */
		e = sqrt(dx*dx+dy*dy); if(e<1.0e-6) e=1.0e-6;
		f=e; if( f>maxe ) f=maxe;
		dx *= f/e;
		dy *= f/e;

		xold=x;
		yold=y;
		x+=dx;
		y+=dy;

		/* bad things happen when Nan's appear */
		if( (!finite(x)) || (!finite(y)) ) {
			return(FALSE);
			}

		/* done when within (x,y) is within D of desired position */
		DX=fabs(x1a[0]-xsta);
		DY=fabs(x1a[1]-ysta);
		if( (DX<=D) && (DY<=D) ) {
			status=TRUE;
			break;
			}
		}

	if( !status ) return(FALSE);

	*x_est = x;
	*y_est = y;

	return(TRUE);
	}

int in_triangle( x, y, x0, y0, x1, y1, x2, y2 )
double x, y, x0, y0, x1, y1, x2, y2;
{
	double aa, bb, L[3];

	/* a point is in a triangle if, for all sides, it is */
	/* on the same side of the side as the opposing vertex */

	/* side 0 */
	eqn_of_line( x1, y1, x2, y2, L );
	aa = eval_line( x, y, L );
	bb = eval_line( x0, y0, L );
	if( (aa*bb) < 0.0 ) return(FALSE);

	/* side 1 */
	eqn_of_line( x0, y0, x2, y2, L );
	aa = eval_line( x, y, L );
	bb = eval_line( x1, y1, L );
	if( (aa*bb) < 0.0 ) return(FALSE);

	/* side 2 */
	eqn_of_line( x0, y0, x1, y1, L );
	aa = eval_line( x, y, L );
	bb = eval_line( x2, y2, L );
	if( (aa*bb) < 0.0 ) return(FALSE);

	return(TRUE);
	}

int keep_inside_triangle( xb,yb, xe,ye, x0,y0, x1,y1, x2,y2  )
double xb,yb, *xe,*ye, x0,y0, x1,y1, x2,y2;
{
	int i;
	double tx, ty, s1, s2, s3, xm, ym;

	if( in_triangle( *xe,*ye, x0,y0, x1,y1, x2,y2 ) )
		return(TRUE);

	if( !in_triangle( xb,yb, x0,y0, x1,y1, x2,y2 ) ) {
		return(TRUE);
		}

	tx = (*xe-xb);
	ty = (*ye-yb);

	s1=0.0;
	s3=1.0;
	for( i=0; i<10; i++ ) {
		s2=0.5*(s1+s3);
		xm = xb+s2*tx;
		ym = yb+s2*ty;
		if( in_triangle( xm,ym, x0,y0, x1,y1, x2,y2 ) ) {
			s1=s2;
			}
		else {
			s3=s2;
			}
		}

	*xe = xb+s1*tx;
	*ye = yb+s1*ty;

	return(FALSE);
	}

int ray_tangent( th, az, t )
double th, az, t[3];
{
	t[0]=sin(DTOR*th)*cos(DTOR*az);
	t[1]=sin(DTOR*th)*sin(DTOR*az);
	t[2]=cos(DTOR*th);
	unit_vector(t);
	return(TRUE);
	}

int ray_tangentr( th, az, t )
double th, az, t[3];
{
	t[0]=sin(th)*cos(az);
	t[1]=sin(th)*sin(az);
	t[2]=cos(th);
	unit_vector(t);
	return(TRUE);
	}

int inv_ray_tangentr( th, az, t )
double *th, *az, t[3];
{
	*az = atan2( t[1], t[0] );
	*th = atan2( sqrt(t[0]*t[0]+t[1]*t[1]), t[2] );
	return(TRUE);
	}


int shoot_star(stheta, ftheta, n_thetas, sazi, fazi, n_azis, x0, interface)
double stheta, ftheta, sazi, fazi;
int n_azis, n_thetas;
double *x0;
int interface;
{
	int i, j, k;
	double dtheta, dazi, theta, azi, traveltime;
	double t0[3], x1[3], t1[3], qinvint;
	int status, hitflag, n_arcs, ec;

	m_rays=n_thetas;
	n_rays=n_azis;

	dtheta = (ftheta-stheta)/(double)(n_thetas-1);
	dazi = (fazi-sazi)/(double)(n_azis-1);

	if( ray!=NULL ) free(ray);

	if((ray=(struct ray_rec*)malloc((n_rays*m_rays+10)*sizeof(struct ray_rec)))==NULL){
		fprintf(stderr,"error: out of memory\n");
		return(FALSE);
		}

	for( i=0; i<m_rays; i++ ) for( j=0; j<n_rays; j++ ) ray[i+j*m_rays].ok=FALSE;

	for( i=0; i<m_rays; i++ ) for( j=0; j<n_rays; j++ ) {
		theta=stheta+dtheta*(double)i;
		azi=sazi+dazi*(double)j;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			k=i+j*m_rays;
			ray[k].ok=TRUE;
			ray[k].az=azi;
			ray[k].th=theta;
			ray[k].x=x1[0];
			ray[k].y=x1[1];
			ray[k].z=x1[2];
			ray[k].T=traveltime;
			}
		if( !status ) {
			fprintf(stderr,"warning: shoot_star: lost ray (%d %d) error code (%d)\n", i, j, ec );
			}
		}
	return(TRUE);
	}

int shoot_star_refl(stheta, ftheta, n_thetas, sazi, fazi, n_azis, x0, interface, rinterface)
double stheta, ftheta, sazi, fazi;
int n_azis, n_thetas;
double *x0;
int interface, rinterface;
{
	int i, j, k;
	double dtheta, dazi, theta, azi, traveltime;
	double t0[3], x1[3], t1[3], qinvint;
	int status, hitflag, n_arcs, ec;

	m_rays=n_thetas;
	n_rays=n_azis;

	dtheta = (ftheta-stheta)/(double)(n_thetas-1);
	dazi = (fazi-sazi)/(double)(n_azis-1);

	if( ray!=NULL ) free(ray);

	if((ray=(struct ray_rec*)malloc((n_rays*m_rays+10)*sizeof(struct ray_rec)))==NULL){
		fprintf(stderr,"error: out of memory\n");
		return(FALSE);
		}

	for( i=0; i<m_rays; i++ ) for( j=0; j<n_rays; j++ ) ray[i+j*m_rays].ok=FALSE;

	for( i=0; i<m_rays; i++ ) for( j=0; j<n_rays; j++ ) {
		theta=stheta+dtheta*(double)i;
		azi=sazi+dazi*(double)j;
		ray_tangent(theta,azi,t0);
		status=shoot_to_interface_refl( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			k=i+j*m_rays;
			ray[k].ok=TRUE;
			ray[k].az=azi;
			ray[k].th=theta;
			ray[k].x=x1[0];
			ray[k].y=x1[1];
			ray[k].z=x1[2];
			ray[k].T=traveltime;
			}
		if( !status ) {
			fprintf(stderr,"warning: shoot_star: lost ray (%d %d) error code (%d)\n", i, j, ec );
			}
		}
	return(TRUE);
	}


int shoot_turf(xmin, xmax, n_x, ymin, ymax, n_y, azi, v0, z0, T0, interface)
double xmin, xmax, ymin, ymax;
int n_x, n_y;
double azi, v0, z0, T0;
int interface;
{
	int i, j, k, t, my_last_tetra;
	double dx, dy, x, y, px, py, traveltime, vp, theta;
	double x0[3], t0[3], x1[3], t1[3], qinvint;
	int status, hitflag, n_arcs, ec;

	m_turf=n_x;
	n_turf=n_y;
	if( turf!=NULL ) free(turf);
	if((turf=(struct turf_rec*)malloc((n_turf*m_turf+10)*sizeof(struct turf_rec)))==NULL){
		fprintf(stderr,"error: out of memory\n");
		return(FALSE);
		}

	for( i=0; i<m_turf; i++ ) for( j=0; j<n_turf; j++ ) turf[i+j*m_turf].ok=FALSE;

	dx = (xmax-xmin)/(double)(n_x-1);
	dy = (ymax-ymin)/(double)(n_y-1);

	my_last_tetra=(-1);

	for( i=0; i<n_x; i++ ) for( j=0; j<n_y; j++ ) {
		x0[0]=xmin+dx*(double)i;
		x0[1]=ymin+dy*(double)j;  /*Bug, dy was dx, fixed 6/6/3 by Menke on advice of Zor */
		x0[2]=z0;
		last_tetra=my_last_tetra;
		if( find_tetra(x0,&t) ) {
			vp = eval_velocity(&tetra[t],x0);
			}
		else continue;
		if( vp>=v0 ) continue;
		my_last_tetra=last_tetra;
		theta = 180.0-RTOD*asin(vp/v0);
		ray_tangent(theta,azi,t0);
		px = t0[0]/vp; py=t0[1]/vp;
		status=shoot_to_interface( x0, t0, x1, t1, NULL, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec );
		if( status&&hitflag ) {
			traveltime += T0 + px*x0[0] + py*x0[1];
			k=i+j*m_turf;
			turf[k].ok=TRUE;
			turf[k].x0=x0[0];
			turf[k].y0=x0[1];
			turf[k].x=x1[0];
			turf[k].y=x1[1];
			turf[k].z=x1[2];
			turf[k].T=traveltime;
			}
		if( !status ) {
			fprintf(stderr,"warning: lost ray (%d %d) error code (%d)\n", i, j, ec );
			}
		}
	return(TRUE);
	}

int gjinv(a, b, n, nstore, test, ierror )
double *a, *b, test;
int n, nstore, *ierror;
{

/*
c      a    input matrix
c      b    inverse of a
c      n    size of a and b
c      nstore       dimensioned size of a and b
c      test division by numbers smaller than this generate
c           a divide by zero error
c      ierror       error flag, zero on no error
*/

	double c[MAX_TABLE_COLS];
	int i, j, icol, itriag, ierror1, ierror2;

	/*   build inverse columnwise by solving AB=I */
	ierror1=0;
	for( icol=0; icol<n; icol++ ) {  
		for( i=0; i<n; i++ ) c[i]=0.0;
		c[icol]=1.0;
		if( icol == 0 ) {   /* triangularize a on first call */
                	itriag = TRUE;
			}
		else {
			itriag = FALSE;
			}
		gauss( a, c, n, nstore, test, &ierror2, itriag );
		if( ierror2!=0 ) ierror1++;

		for( i=0; i<n; i++ ) *(b+i*nstore+icol)=c[i];
		}

	*ierror=ierror1;
	if( ierror1==0 ) return(TRUE); else return(FALSE);

       }

/* determinant of a 3x3 matrix */
double det3x3( m )
double m[3][3];
{
	double d;
	d = 0.0;
	d+=m[0][0]*m[1][1]*m[2][2];
	d+=m[0][1]*m[1][2]*m[2][0];
	d+=m[0][2]*m[2][1]*m[1][0];
	d-=m[0][2]*m[1][1]*m[2][0];
	d-=m[0][1]*m[1][0]*m[2][2];
	d-=m[0][0]*m[2][1]*m[1][2];
	return(d);
    }

/* see Rektorys p 143 */
double tetra_volume(t)
int t;
{
	int i;
	double m[3][3];
	struct tetra_rec *p;

	p = &tetra[t];
	for( i=0; i<3; i++ ) {
		m[0][i] = p->x2[i] - p->x1[i];
		m[1][i] = p->x3[i] - p->x1[i];
		m[2][i] = p->x4[i] - p->x1[i];
		}

	return( fabs(det3x3(m))/6.0 );
	}

int findnode(id)
{
	int i;
	struct nodeid_element_rec t, *my_elem;

	if( !nodeidlist_exists ) {
		nodeidlist_exists=TRUE;
		nodeidlist=(struct nodeid_element_rec*)
			malloc(n_nodes*sizeof(struct nodeid_element_rec));
		if( nodeidlist==NULL ) {
			fprintf(stderr,"Error: Out of Memory (findnode)\n");
			exit(-1);
			}
		for( i=0; i<n_nodes; i++ ) {
			nodeidlist[i].id=node[i].id;
			nodeidlist[i].index=i;
			}
		qsort( nodeidlist, n_nodes, sizeof(struct nodeid_element_rec), compare_nodeid );
		}

	t.id=id;
	t.index=(-1);
	
	my_elem=(struct nodeid_element_rec*)bsearch(&t, nodeidlist, n_nodes,
		sizeof(struct nodeid_element_rec), compare_nodeid);
	if( my_elem == NULL ) {
		return(-1);
		}
	else {
		return(my_elem->index);
		}	
	}

int node_in_model_par(n,mp)
int n, mp;
{
	int i, nn;
	struct model_par_rec *m;

	m = &model_par[mp];
	nn = m->n;
	for( i=0; i<nn; i++ ) {
		if( m->node[i] == n ) return(TRUE);
		}
	return(FALSE);
	}

/* Frechet derivative of traveltime with respect to node velocity */
double dTdmpnew( mp, arc, n_arcs, traveltime )
int mp;
struct arc_rec *arc;
int n_arcs;
double traveltime;
{
	int i, nn, method;
	double dv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v += m->weight[i] * dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(phi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(phi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(phi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v -= m->weight[i] * dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dv;

	if( finite(D) ) return(D); else return(0.0);
	
	}

/* Frechet derivative of S traveltime with respect to node P velocity */
double dTsdmpp( mp, arc, n_arcs, traveltime )
int mp;
struct arc_rec *arc;
int n_arcs;
double traveltime;
{
	int i, nn, method;
	double dvP, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	struct node_rec *mynode;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dvP=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = (mynode->v_orig + m->weight[i] * dvP)/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(pgi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(pgi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(pgi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dvP;

	if( finite(D) ) return(D); else return(0.0);
	
	}

/* Frechet derivative of S traveltime with respect to node P:S ratio */
double dTsdmpr( mp, arc, n_arcs, traveltime )
int mp;
struct arc_rec *arc;
int n_arcs;
double traveltime;
{
	int i, nn, method;
	double dr, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	struct node_rec *mynode;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];

	dr=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/(mynode->ratio_orig+m->weight[i]*dr);
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the traveltime integral is now                              */
		/* (integral) d-arclength / velocity 	                       */
		/* (integral) R d(pgi) / [v0 + gz*z + gy*y + gx*x]             */
		/* note v0!=0; z=R*cos(psi), y=0, x=R*sin(psi) on circle       */
		/* R (integral) d(pgi) / [v0 + R*gz*cos(phi) + R*gx*sin(phi)]  */
		/* (integral) d(pgi) / [v0/R + gz*cos(phi) + gx*sin(phi)]      */

		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		v0=eval_velocity(t,arc[i].xc);
		method=342;
		T2 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi2);
		T1 = integral342(v0/R,rgrad[0],rgrad[2],DTOR*psi1);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) {
		mynode = &(node[m->node[i]]);
		mynode->v = mynode->v_orig/mynode->ratio_orig;
		}
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dr;

	if( finite(D) ) return(D); else return(0.0);
	
	}

/* Frechet derivative of reflected traveltime with respect to reflector position */
/* (Tricky because the point of reflection moves as the reflector is perturbed) */
double dTdmpr(mp,interface,rinterface,xia,psia,xib,npsib,arc,n_arcs,traveltime,ng)
int mp, interface, rinterface;
double xia[3], psia[3];
double xib[3], npsib[3];
struct arc_rec *arc;
int n_arcs;
double traveltime;
int ng;
{
	int i, j, nn, method, m_arcs;
	double ds, dTdm;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0;
	double rgrad[3];
	struct node_rec *mynode;
	int sa, sb, hfa, hfb, eca, ecb;
	double xa[3], ta[3], TTa, Qa, tha, aza, TTA;
	double xb[3], tb[3], TTb, Qb, thb, azb, TTB;
	double psib[3], n[3];
	double psiath, psiaaz, psibth, psibaz;
	double dth, daz;
	double xa0[3], xa1[3], xa2[3], TTa0, TTa1, TTa2, psia0[3], psia1[3], psia2[3];
	double xb0[3], xb1[3], xb2[3], TTb0, TTb1, TTb2, psib0[3], psib1[3], psib2[3];
	double ta0[3], ta1[3], ta2[3];
	double tb0[3], tb1[3], tb2[3];
	double epsilon, xrp[3], v1[3], v2[3], v3, P[4], Pr[4], v[3], h[3], s[3][3], st[3][3];
	int face, debug;
	struct tetra_rec *ptetra;

	double tar[3], ta0r[3], ta1r[3], ta2r[3];
	double tbr[3], tb0r[3], tb1r[3], tb2r[3];
	double xar[3], xa0r[3], xa1r[3], xa2r[3];
	double xbr[3], xb0r[3], xb1r[3], xb2r[3];

	double dtadpsi[2][2], dtbdpsi[2][2], dxadpsi[2][2], dxbdpsi[2][2];
	double dpsidxa[2][2], dpsidxb[2][2], dtadxa[2][2], dtbdxb[2][2];
	double b[2], A[2][2], xsoln[2], xbp[3], xbpr[3];
	double *xf1, *xf2, *xf3;
	double dTTadpsi[2], dTTbdpsi[2], dTTadxa[2], dTTbdxb[2];
	double xf1r[3], xf2r[3], xf3r[3];
	double tarest[2], tbrest[2];
	double TTa0est, TTb0est;

	debug=0;

	ds=1.0e-2;           /* perturbation in node position */
	dth=1.0e-4;          /* perturbation in angle of incidence of a ray */
	daz=1.0e-4;          /* perturbation in azimuth of a ray */
	m = &model_par[mp];  /* pointer to this model parameter */
	nn = m->n;           /* number of nodes in this model parameter */

	if(debug) fprintf(stderr,"dTdm: (row col) = (%d %d) with %d nodes\n\n", ng, mp, nn );
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\tstart a (%f %f %f) (%f %f %f)\n",
		xia[0], xia[1], xia[2], psia[0], psia[1], psia[2] );
	if(debug) fprintf(stderr,"\tend   b (%f %f %f) (%f %f %f)\n",
		xib[0], xib[1], xib[2], npsib[0], npsib[1], npsib[2] );
	if(debug) fprintf(stderr,"\n");

	/* Theory:  The goal, to calculate the derivative dT/dm, is reached */
	/* by first calculating how the perturbation dm moves the location  */
	/* of the point of reflection, and then how that motion effects the */
	/* traveltime.  Rays are shot from source and receiver down to the  */
	/* reflector, with perturbations made (alternately) to both the     */
	/* model and to the takeoff angles.  This allows derivatives like   */
	/* dT/dtheta and dx/dtheta to be calculated via finite differences. */
	/* The chain rule is then used to calculate derivatives like dT/dX. */

	/* Step 1a. Shoot rays to unperturbed reflector                     */
	/* source at xia with tangent psia                                  */
        /* has reflection point xa, tangent ta, and traveltime TTA          */
	/* source at xib with tangent psib                                  */
        /* has reflection point xb, tangent tb, and traveltime TTb          */

	/* source to reflector */
	inv_ray_tangentr( &psiath, &psiaaz, psia);
	sa=shoot_to_interface( xia, psia, xa, ta, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTA, &Qa, &eca );
	if( !(sa&&hfa) ) return(0.0);

	if(debug) fprintf(stderr,"\txa (%f %f %f) ta (%f %f %f) TTA %f\n",
		xa[0],xa[1],xa[2], ta[0],ta[1],ta[2], TTA);

	/* Snell's law is equivalent to the condition that the in-plane     */
	/* components two tangents be equal.  Thus we need to identify the  */
	/* normal, n, to the plane.                                         */
	epsilon=1.0e-6;
	for( i=0; i<3; i++ ) xrp[i]=xa[i]-epsilon*ta[i];
	if( !find_tetra( xrp, &i ) ) return(0.0);
	ptetra=tetra+i;
	if( ! ray_exits_tetrahedron( ptetra->x1, ptetra->x2, ptetra->x3, ptetra->x4,
		xrp, ta, epsilon, v1, v2, &v3, &face ) ) return(0.0);
	face+=1;
	if( face==1 )      {xf1=ptetra->x2; xf2=ptetra->x3; xf3=ptetra->x4;}
	else if( face==2 ) {xf1=ptetra->x1; xf2=ptetra->x3; xf3=ptetra->x4;}
	else if( face==3 ) {xf1=ptetra->x1; xf2=ptetra->x2; xf3=ptetra->x4;}
	else if( face==4 ) {xf1=ptetra->x1; xf2=ptetra->x2; xf3=ptetra->x3;}
	else { return(0.0); }
	eqn_of_plane( xf1, xf2, xf3, P );
	normal_to_plane( P, n );

	/* receiver to reflector */
	for(i=0;i<3;i++) psib[i]=(-npsib[i]); /* reverse the tangent, since shooting backwards */
	inv_ray_tangentr( &psibth, &psibaz, psib);
	sb=shoot_to_interface( xib, psib, xb, tb, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTB, &Qb, &ecb );
	if( !(sb&&hfb) ) return(0.0);

	if(debug) fprintf(stderr,"\txb (%f %f %f) tb (%f %f %f) TTB %f\n",
		xb[0],xb[1],xb[2], tb[0],tb[1],tb[2], TTB);
	if(debug) fprintf(stderr,"\n");

	if(debug) {
		fprintf(stderr,"\tunperturbed nodes at\n");
		for( i=0; i<nn; i++ ) {
			mynode= &node[m->node[i]];
			fprintf(stderr,"\t\t%d %f %f %f\n",i,mynode->x[0],mynode->x[1],mynode->x[2]);
			}
		}

	/* Step 2a. Perturb the reflector by moving all the model-parameter's    */
	/* nodes by a small distance ds parallel to the direction specified      */
	/* by the model parameter.  The velocity eqn of all effected tetrahedra  */
	/* must be recalculated, since the spatial velocity gradiant will cahnge */

	perturb_refl_nodes(m,ds);

	if(debug) {
		fprintf(stderr,"\tperturbed nodes at\n");
		for( i=0; i<nn; i++ ) {
			mynode= &node[m->node[i]];
			fprintf(stderr,"\t\t%d %f %f %f\n",i,mynode->x[0],mynode->x[1],mynode->x[2]);
			}
		fprintf(stderr,"\n");
		}

	/* Step 2b. Shoot ray pencil from source thru the perturbed medium          */
	/* the pencil has 3 rays ray0=(th,az), ray1=(th+dth,az), ray2=(th,az+daz)   */
	/* at the reflection point, the positions are xa0, xa1, xa2 and the         */
	/* tangents are ta0, ta2, ta3 and                                           */
	/* traveltimes are TTa0, TTa1, TTa2                                         */

	ray_tangentr( psiath, psiaaz, psia0 );
	sa=shoot_to_interface( xia, psia0, xa0, ta0, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa0, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psiath+dth, psiaaz, psia1 );
	sa=shoot_to_interface( xia, psia1, xa1, ta1, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa1, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); };

	ray_tangentr( psiath, psiaaz+daz, psia2 );
	sa=shoot_to_interface( xia, psia2, xa2, ta2, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfa, &TTa2, &Qa, &eca );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	if(debug) fprintf(stderr,"\txa0 (%f %f %f) ta0 (%f %f %f) TTa0 %f\n",
		xa0[0],xa0[1],xa0[2],ta0[0],ta0[1],ta0[2],TTa0);
	if(debug) fprintf(stderr,"\txa1 (%f %f %f) ta1 (%f %f %f) TTa1 %f\n",
		xa1[0],xa1[1],xa1[2],ta1[0],ta1[1],ta1[2],TTa1);
	if(debug) fprintf(stderr,"\txa2 (%f %f %f) ta2 (%f %f %f) TTa2 %f\n",
		xa2[0],xa2[1],xa2[2],ta2[0],ta2[1],ta2[2],TTa2);
	if(debug) fprintf(stderr,"\n");

	/* Step 2b. Shoot ray pencil from receiver thru perturbed medium            */
	/* the pencil has 3 rays ray0=(th,az), ray1=(th+dth,az), ray2=(th,az+daz)   */
	/* at the reflection point, the positions are xb0, xb1, xb2 and the         */
	/* tangents are tb0, tb2, tb3                                               */
	/* traveltimes are TTb0, TTb1, TTb2                                         */

	ray_tangentr( psibth, psibaz, psib0 );
	sb=shoot_to_interface( xib, psib0, xb0, tb0, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb0, &Qb, &ecb );
	if( !(sb&&hfb) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psibth+dth, psibaz, psib1 );
	sb=shoot_to_interface( xib, psib1, xb1, tb1, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb1, &Qb, &ecb );
	if( !(sb&&hfb) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	ray_tangentr( psibth, psibaz+daz, psib2 );
	sb=shoot_to_interface( xib, psib2, xb2, tb2, NULL, &m_arcs,
		MAX_ARCS, 10000.0, rinterface, &hfb, &TTb2, &Qb, &ecb );
	if( !(sa&&hfa) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	if(debug) fprintf(stderr,"\txb0 (%f %f %f) tb0 (%f %f %f) TTb0 %f\n",
		xb0[0],xb0[1],xb0[2],tb0[0],tb0[1],tb0[2],TTb0);
	if(debug) fprintf(stderr,"\txb1 (%f %f %f) tb1 (%f %f %f) TTb1 %f\n",
		xb1[0],xb1[1],xb1[2],tb1[0],tb1[1],tb1[2],TTb1);
	if(debug) fprintf(stderr,"\txb2 (%f %f %f) tb2 (%f %f %f) TTb2 %f\n",
		xb2[0],xb2[1],xb2[2],tb2[0],tb2[1],tb2[2],TTb2);
	if(debug) fprintf(stderr,"\n");

	/* Step 2c. Rotate positions and tangents to coordinate system of interface */

	tvh( n, v, h );
	newbasisrot( v, h, n, s );
	for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) st[i][j]=s[j][i];
	rotate3(xa,s,xar); rotate3(xa0,s,xa0r); rotate3(xa1,s,xa1r); rotate3(xa2,s,xa2r);
	rotate3(xb,s,xbr); rotate3(xb0,s,xb0r); rotate3(xb1,s,xb1r); rotate3(xb2,s,xb2r);
	rotate3(ta,s,tar); rotate3(ta0,s,ta0r); rotate3(ta1,s,ta1r); rotate3(ta2,s,ta2r);
	rotate3(tb,s,tbr); rotate3(tb0,s,tb0r); rotate3(tb1,s,tb1r); rotate3(tb2,s,tb2r);
	rotate3(xf1,s,xf1r); rotate3(xf2,s,xf2r); rotate3(xf3,s,xf3r);
	eqn_of_plane( xf1r, xf2r, xf3r, Pr );

	if(debug) fprintf(stderr,"\txar (%f %f %f) tar (%f %f %f)\n",
		xar[0],xar[1],xar[2],tar[0],tar[1],tar[2]);
	if(debug) fprintf(stderr,"\txbr (%f %f %f) tbr (%f %f %f)\n",
		xbr[0],xbr[1],xbr[2],tbr[0],tbr[1],tbr[2]);
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\txa0r (%f %f %f) ta0r (%f %f %f)\n",
		xa0r[0],xa0r[1],xa0r[2],ta0r[0],ta0r[1],ta0r[2]);
	if(debug) fprintf(stderr,"\txa1r (%f %f %f) ta1r (%f %f %f)\n",
		xa1r[0],xa1r[1],xa1r[2],ta1r[0],ta1r[1],ta1r[2]);
	if(debug) fprintf(stderr,"\txa2r (%f %f %f) ta2r (%f %f %f)\n",
		xa2r[0],xa2r[1],xa2r[2],ta2r[0],ta2r[1],ta2r[2]);
	if(debug) fprintf(stderr,"\n");
	if(debug) fprintf(stderr,"\txb0r (%f %f %f) tb0r (%f %f %f)\n",
		xb0r[0],xb0r[1],xb0r[2],tb0r[0],tb0r[1],tb0r[2]);
	if(debug) fprintf(stderr,"\txb1r (%f %f %f) tb1r (%f %f %f)\n",
		xb1r[0],xb1r[1],xb1r[2],tb1r[0],tb1r[1],tb1r[2]);
	if(debug) fprintf(stderr,"\txb2r (%f %f %f) tb2r (%f %f %f)\n",
		xb2r[0],xb2r[1],xb2r[2],tb2r[0],tb2r[1],tb2r[2]);
	if(debug) fprintf(stderr,"\n");

	/* Step 2d. Compute finite-difference estimatesd of partial derivatives -   */
	/*      reflector position with respect to change in takefoff angles        */
	/*      tangent with respect to change in takefoff angles                   */
	/*      traveltime with respect to change in takefoff angles                */
	/* for both the source-side "a" ray and the receiver-side "b" ray           */

	dxadpsi[0][0] = (xa1r[0]-xa0r[0])/dth; dxadpsi[0][1] = (xa2r[0]-xa0r[0])/daz;
	dxadpsi[1][0] = (xa1r[1]-xa0r[1])/dth; dxadpsi[1][1] = (xa2r[1]-xa0r[1])/daz;

	dxbdpsi[0][0] = (xb1r[0]-xb0r[0])/dth; dxbdpsi[0][1] = (xb2r[0]-xb0r[0])/daz;
	dxbdpsi[1][0] = (xb1r[1]-xb0r[1])/dth; dxbdpsi[1][1] = (xb2r[1]-xb0r[1])/daz;
	
	dtadpsi[0][0] = (ta1r[0]-ta0r[0])/dth; dtadpsi[0][1] = (ta2r[0]-ta0r[0])/daz;
	dtadpsi[1][0] = (ta1r[1]-ta0r[1])/dth; dtadpsi[1][1] = (ta2r[1]-ta0r[1])/daz;

	dtbdpsi[0][0] = (tb1r[0]-tb0r[0])/dth; dtbdpsi[0][1] = (tb2r[0]-tb0r[0])/daz;
	dtbdpsi[1][0] = (tb1r[1]-tb0r[1])/dth; dtbdpsi[1][1] = (tb2r[1]-tb0r[1])/daz;

	dTTadpsi[0] = (TTa1-TTa0)/dth; dTTadpsi[1] = (TTa2-TTa0)/daz;
	dTTbdpsi[0] = (TTb1-TTb0)/dth; dTTbdpsi[1] = (TTb2-TTb0)/daz;

	if(debug) fprintf(stderr,"\tdxa/dpsi %f %f %f %f\n",
		dxadpsi[0][0], dxadpsi[0][1], dxadpsi[1][0], dxadpsi[1][1] );
	if(debug) fprintf(stderr,"\tdxb/dpsi %f %f %f %f\n",
		dxbdpsi[0][0], dxbdpsi[0][1], dxbdpsi[1][0], dxbdpsi[1][1] );
	if(debug) fprintf(stderr,"\n");

	if(debug) fprintf(stderr,"\tdta/dpsi %f %f %f %f\n",
		dtadpsi[0][0], dtadpsi[0][1], dtadpsi[1][0], dtadpsi[1][1] );
	if(debug) fprintf(stderr,"\tdtb/dpsi %f %f %f %f\n",
		dtbdpsi[0][0], dtbdpsi[0][1], dtbdpsi[1][0], dtbdpsi[1][1] );
	if(debug) fprintf(stderr,"\n");

	if(debug) fprintf(stderr,"\tdTTa/dpsi %f %f\n",
		dTTadpsi[0], dTTadpsi[1] );
	if(debug) fprintf(stderr,"\tdTTb/dpsi %f %f\n",
		dTTbdpsi[0], dTTbdpsi[1] );
	if(debug) fprintf(stderr,"\n");

	/* while we have calculated dx/dpsi and dt/dpsi, we really need dt/dx */
	/* in order to impose Snell's law.  So we must apply the chain rule   */

	/* first rely on the fact that a matrix of partials of form dY/dX is the */
	/* matrix inverse of the partials dX/dY                                  */ 
	if( ! invert2x2( dxadpsi, dpsidxa ) ) { perturb_refl_nodes(m,-ds); return(0.0); }
	if( ! invert2x2( dxbdpsi, dpsidxb ) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	/* then use the chain rule dX/dY = dX/dZ dZ/dX                           */
	mult2x2( dtadpsi, dpsidxa, dtadxa );
	mult2x2( dtbdpsi, dpsidxb, dtbdxb );

	if(debug) fprintf(stderr,"\tdta/dxa %f %f %f %f\n",
		dtadxa[0][0], dtadxa[0][1], dtadxa[1][0], dtadxa[1][1] );
	if(debug) fprintf(stderr,"\tdtb/dxb %f %f %f %f\n",
		dtbdxb[0][0], dtbdxb[0][1], dtbdxb[1][0], dtbdxb[1][1] );
	if(debug) fprintf(stderr,"\n");

	/* Step 3: set up linear system for perturbation in point of reflection that */
	/* matches (to first order) the in-plane components of the two tangents      */
	/* Here we are using Taylor's theorem to write to first order                */
	/* ta(x) = ta(xa) + dta/dxa (x-xa) and                                       */
	/* tb(x) = tb(xa) + dtb/dxb (x-xb)                                           */
	/* if we view these equations in a coorinate system that is normal to the    */
	/* reflector, then Snell's law is the condition that the two in-plane        */
	/* components of the tangents are related by ta(x)=-tb(x) (the minus         */
        /* sign arises because we are propagating from point "b" backwards). This    */
	/* yields a 2x2 matrix equation for x, the position of the                   */
	/* point that satisfies Snell's law                                          */
	/* -[ta(xa)+tb(xb)] + [dta/dxa xa - dtb/dxb xb] =  [dta/dxa + dtb/dxb] x     */

	for( i=0; i<2; i++ ) {
		b[i] = -(ta0r[i]+tb0r[i]);
		for( j=0; j<2; j++ ) {
			b[i] += ( dtbdxb[i][j]*xb0r[j] + dtadxa[i][j]*xa0r[j] );
			}
		}
	for( i=0; i<2; i++ ) for( j=0; j<2; j++ ) {
		A[i][j] = dtbdxb[i][j] + dtadxa[i][j];
		}
	if( !solve2x2(A, b, xsoln ) ) { perturb_refl_nodes(m,-ds); return(0.0); }

	/* The estimated tangent is never used */
	if(debug) {
		for( i=0; i<2; i++ ) {
			tarest[i] = ta0r[i];
			tbrest[i] = tb0r[i];
			for( j=0; j<2; j++ ) {
				tarest[i] += dtadxa[i][j]*(xsoln[j]-xa0r[j]);
				tbrest[i] += dtbdxb[i][j]*(xsoln[j]-xb0r[j]);
				}
			}
		}

	if(debug) fprintf(stderr,"\ttarest %f %f\n", tarest[0], tarest[1] );
	if(debug) fprintf(stderr,"\ttbrest %f %f\n", tbrest[0], tbrest[1] );
	if(debug) fprintf(stderr,"\n");

	/* the matrix equation gives only two components of the position of the     */
	/* point of reflection.  I use the equation of the plane (suitably rotated) */
	/* to find the third                                                        */
	xbpr[0]=xsoln[0];
	xbpr[1]=xsoln[1];
	xbpr[2]= -(Pr[0]*xsoln[0] + Pr[1]*xsoln[1] + Pr[3] )/Pr[2];

	/* The unrotated point of reflection is never used */
	if(debug) rotate3(xbpr,st,xbp);

	if(debug) fprintf(stderr,"\tbounce point at %f %f %f \n", xbp[0], xbp[1], xbp[2] );
	if(debug) fprintf(stderr,"\n");

	/* Step 4: Estimate the derivative of traveltime with respect to reflector */
	/* position                                                                */

	/* Use the chain rule: dTdx = dT/dphi dphi/dx. Note dphi/dx appears as a   */
	/* matrix transpose when this formula is case in a linear algebra          */
	for( i=0;i<2;i++ ) {
		dTTadxa[i]=0.0;
		dTTbdxb[i]=0.0;
		for( j=0; j<2; j++ ) {
			dTTadxa[i] += dTTadpsi[j]*dpsidxa[j][i];
			dTTbdxb[i] += dTTbdpsi[j]*dpsidxb[j][i];
			}
		}

	if(debug) fprintf(stderr,"\tdTTa/dxa %f %f\n", dTTadxa[0], dTTadxa[1] );
	if(debug) fprintf(stderr,"\tdTTb/dxb %f %f\n", dTTbdxb[0], dTTbdxb[1] );
	if(debug) fprintf(stderr,"\n");

	/* Setp 5: Derivative calulation. The derivative is the sum of 2 parts */

	/* estimate the traveltime of the perturbed model where snell's law    */
	/* is satisfied                                                        */

	TTa0est=TTa0;
	TTb0est=TTb0;
	for( j=0; j<2; j++ ) {
		TTa0est += dTTadxa[j]*(xsoln[j]-xa0r[j]);
		TTb0est += dTTbdxb[j]*(xsoln[j]-xb0r[j]);
		}

	if(debug) {
		fprintf(stderr,"\tUnperturbed TT %f + %f = %f\n", TTA,  TTB, TTA+TTB );
		fprintf(stderr,"\tPert uncorr TT %f + %f = %f\n", TTa0, TTb0, TTa0+TTb0 );
		fprintf(stderr,"\tPert & corr TT %f + %f = %f\n", TTa0est, TTb0est, TTa0est+TTb0est );
		fprintf(stderr,"\n");
		}

	/* part associated with change in model, but Snell's law is not satisfied */
	dTdm = ( (TTa0est+TTb0est) - (TTA+TTB) ) / ds;

	if(debug) fprintf(stderr,"\tDerivative %f\n\n", dTdm );

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.             */ 
	perturb_refl_nodes(m,-ds);

	if( finite(dTdm) ) return(dTdm); else return(0.0);
	
	}

int perturb_refl_nodes(m,ds)
struct model_par_rec *m;
double ds;
{
	int i, j, nn;
	struct node_rec *mynode;

	nn = m->n;
	for( i=0; i<n_tetras; i++ ) tetra[i].marked=0;
	for( i=0; i<nn; i++ ) {
		mynode= &node[m->node[i]];
		for( j=0; j<3; j++ ) mynode->x[j]+=ds*m->weight[i]*m->t[3*i+j];
		for( j=0; j<mynode->nt; j++ ) mynode->t[j]->marked=1;
		}
	for( i=0; i<n_tetras; i++ ) if( tetra[i].marked ) velocity_eqn( &tetra[i] );
	return(1);
	}

/* Frechet derivative of direct traveltime with respect to reflector position */
double dTdmprd(mp,interface,rinterface,xia,psia,xib,npsib,arc,n_arcs,traveltime,ng)
int mp, interface, rinterface;
double xia[3], psia[3];
double xib[3], npsib[3];
struct arc_rec *arc;
int n_arcs;
double traveltime;
int ng;
{
	int i, status, m_arcs, debug, t;
	double dTdm, ds;
	struct model_par_rec *m;
	double xc[3], tc[3];
	int hf, ec;
	double TT, Q, dT, dx[3], xcp[3], v, epsilon;

	m = &model_par[mp];  /* pointer to this model parameter */
	ds=1.0e-2;           /* perturbation in node position */
	epsilon = 1.0e-8;    /* point moved back along ray by this much */
	debug=0;

	/* Theory: Do this one by brute force: perturb the model and */
	/* shoot a ray. This ray will not in general pass thru the   */
	/* receiver point, so apply a first order moveout to correct */
	/* its traveltime to the receiver point.  Then compute first */
	/* order finite difference with unperturbed traveltime.      */

	perturb_refl_nodes(m,ds);
	status=shoot_to_interface( xia, psia, xc, tc, NULL, &m_arcs,
		MAX_ARCS, 10000.0, interface, &hf, &TT, &Q, &ec );
	if( !(status&&hf) ) {
		if(debug) fprintf(stderr,"warning: shoot_to_interface failed in dTdmprd\n");
		perturb_refl_nodes(m,-ds);
		return(0.0);
		}

	for( i=0; i<3; i++ ) xcp[i]=xc[i]-epsilon*tc[i];
	if( find_tetra(xcp,&t) ) v = eval_velocity(&tetra[t],xcp);
	else {
		if(debug) fprintf(stderr,"warning: find_tetra failed in dTdmprd\n");
		perturb_refl_nodes(m,-ds);
		return(0.0);
		}

	for( i=0; i<3; i++ ) dx[i]=xib[i]-xc[i];
	dT=dot( dx, tc ) / v;
	dTdm = ( (TT + dT) - traveltime )/ds;

	if( debug ) {
		fprintf(stderr,"dTdmprd: %d %d\n", ng, mp );
		fprintf(stderr,"         x2  %f %f %f\n", xib[0], xib[1], xib[2] );
		fprintf(stderr,"         x2p %f %f %f\n", xc[0], xc[1], xc[2] );
		fprintf(stderr,"         TT=%f dt=%f traveltime=%f v=%f dTdm=%f\n",
			TT, dT, traveltime, v, dTdm );
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.             */ 
	perturb_refl_nodes(m,-ds);

	if( finite(dTdm) ) return(dTdm); else return(0.0);

	}


int solve2x2( double A[2][2], double b[2], double x[2] )
{
	int i, j;
	double d, AI[2][2];

	d = A[0][0]*A[1][1]-A[0][1]*A[1][0];
	if( d==0.0 ) return(0);
	AI[0][0] = A[1][1]/d;
	AI[1][1] = A[0][0]/d;
	AI[1][0] = (-A[1][0])/d;
	AI[0][1] = (-A[0][1])/d;

	for( i=0; i<2; i++ ) {
		x[i]=0.0;
		for( j=0; j<2; j++ ) {
			x[i] += AI[i][j]*b[j];
			}
		}

	return(1);
	}

int invert2x2( a, ai )
double a[2][2], ai[2][2];
{
	double d;
	d = a[0][0]*a[1][1] - a[0][1]*a[1][0];
	if( d==0.0 ) return(0);
	d = 1.0/d;
	ai[0][0]=a[1][1]*d;
	ai[1][1]=a[0][0]*d;
	ai[1][0]=(-a[1][0])*d;
	ai[0][1]=(-a[0][1])*d;
	return(1);
	}

int mult2x2( a, b, c )
double a[2][2], b[2][2], c[2][2];
{
	int i, j, k;
	for( i=0; i<2; i++ ) for( j=0; j<2; j++ ) {
		c[i][j]=0.0;
		for( k=0; k<2; k++ ) c[i][j]+=a[i][k]*b[k][j];
		}
	return(1);
	}
	

double dTdmpold( mp, arc, n_arcs, traveltime )
int mp;
struct arc_rec *arc;
int n_arcs;
double traveltime;
{
	int i, nn, method;
	double dv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, T, T1, T2, v0, qint1, qint2;
	double rgrad[3], gs, dpsi;

	dv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v+=m->weight[i]*dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	/* calculate the traveltime, summing up the contributions for all arcs */
	T=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;

		/* the gradient is no longer parallel to z' */
		/* so recompute its orientation in the rotated coord system */
		rotate3(t->g,arc[i].s,rgrad);

		/* the find the amplitude of the gradient in the (x',z') plane*/
		/* and its angle from the z' direction */
		gs=sqrt(rgrad[0]*rgrad[0]+rgrad[2]*rgrad[2]);
		dpsi = RTOD*atan2(rgrad[0],rgrad[2]);
		
		R=arc[i].R;
		/* view the traveltime intergal in a cordinate system */
		/* rotated about y' so that z'' is now parallel to the */
		/* new velocity gradient.  This changes the integration limits */
		psi1=arc[i].psi1-dpsi;
		psi2=arc[i].psi2-dpsi;
		v0=eval_velocity(t,arc[i].xc);
		T2 = ttintegral(v0,gs,R,psi2,&method);
		T1 = ttintegral(v0,gs,R,psi1,&method);
		T+=(T2-T1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the velocity equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].v-=m->weight[i]*dv;
	for( i=0; i<n_arcs; i++ ) velocity_eqn( arc[i].ptetra );

	D = (T-traveltime)/dv;

	if( finite(D) ) return(D); else return(0.0);
	
	}

double dqinvdmp( mp, arc, n_arcs, traveltime, qinvint )
int mp;
struct arc_rec *arc;
int n_arcs;
double traveltime;
double qinvint;
{
	int i, j, nn, method;
	double dqinv, D;
	struct model_par_rec *m;
	struct tetra_rec *t;
	double psi1, psi2, R, qinvint0, qinvint1, qinvint2, qinv0;
	double grad[3], rgrad[3];

	dqinv=1.0e-5;
	m = &model_par[mp];
	nn = m->n;

	/* perturb all nodes assocated with this model parameter */
	/* and recalculate the qinv equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].qinv += m->weight[i] * dqinv;
	for( i=0; i<n_arcs; i++ ) qinv_eqn( arc[i].ptetra );

	/* calculate the qinv, summing up the contributions for all arcs */
	qinvint0=0.0;
	for( i=0; i<n_arcs; i++ ) {
		t=arc[i].ptetra;
		for( j=0; j<3; j++ ) grad[j]=t->qinvc[j];
		rotate3(grad,arc[i].s,rgrad);
		R=arc[i].R;
		psi1=arc[i].psi1;
		psi2=arc[i].psi2;
		qinv0=eval_qinv(t,arc[i].xc);
		qinvint2=qintegral( qinv0, rgrad[0], rgrad[2], R, t->gs, psi2 );
		qinvint1=qintegral( qinv0, rgrad[0], rgrad[2], R, t->gs, psi1 );
		qinvint0+=(qinvint2-qinvint1);
		}

	/* unperturb all nodes assocated with this model parameter */
	/* and recalculate the qinv equation, etc.           */ 
	for( i=0; i<nn; i++ ) node[m->node[i]].qinv -= m->weight[i] * dqinv;
	for( i=0; i<n_arcs; i++ ) qinv_eqn( arc[i].ptetra );

	D = (qinvint0-qinvint)/(traveltime*dqinv);

	if( finite(D) ) return(D); else return(0.0);
	
	}

int write_tele_note(f)
FILE *f;
{
	fprintf(f,"Note on methodology:\n");
	fprintf(f,"    Suppose a single teleseism is observed with\n");
	fprintf(f,"      arrival time ti on each of i=1...N stations\n");
	fprintf(f,"    Suppose that the teleseism arrives at a reference\n");
	fprintf(f,"	 depth, z0, at the bottom of the model at time\n");
	fprintf(f,"      t0+pxi*xi+pyi*yi.  The horizontal slownesses, (pxi,pyi),\n");
	fprintf(f,"      are assumed known (say from global traveltime models)\n");
	fprintf(f,"      but t0 is unknown.\n");
	fprintf(f,"    There are two ways to deal with the problem of t0:\n");  
	fprintf(f,"    The first is to explicitly remove t0 from consideration (at a price!): \n");
	fprintf(f,"      The predicted arrival time is t0+pxi*xi+pyi*yi+Ti(m)\n");
	fprintf(f,"        where Ti(m) is the traveltime through model m from\n");
	fprintf(f,"        the reference planewave to station i.  It linearizes to:\n");
	fprintf(f,"        t0+pxi*xi+pyi*yi+Ti(m0)+gradTi(dot)dm. Here m0 is\n");
	fprintf(f,"        a reference model and dm a perturbation.\n");
	fprintf(f,"      We eliminate t0 from the equation ti-pred = ti-obs\n");
	fprintf(f,"        by sutracting out the mean (over i=1..N). The mean l.h.s. is:\n");
	fprintf(f,"        t0+mean[pxi*xi+pyi*yi+Ti(m0)]+mean[gradTi](dot)dm\n");
	fprintf(f,"        so the final equation is:\n");
	fprintf(f,"        (gradTi - mean[gradTi]) (dot) dm = \n");
	fprintf(f,"        ti-mean[ti]-(pxi*xi+pyi*yi+Ti(m0))+mean[pxi*xi+pyi*yi+Ti(m0)] =\n");
	fprintf(f,"        [ti-(pxi*xi+pyi*yi+Ti(m0)] - mean[ti-(pxi*xi+pyi*yi+Ti(m0)]\n");
	fprintf(f,"      Note that the resulting equations are insensitive to the mean\n");
	fprintf(f,"        traveltime through the model, and thus can determine only\n");
	fprintf(f,"        horizontal gradients in the velocity model.\n");
	fprintf(f,"    The second method is to solve for t0. As this is non-unique, we \n");
	fprintf(f,"        must apply weights to favor the t0 model parameters over \n");
	fprintf(f,"        the velocity model parameters. This code uses dt0 as\n");
	fprintf(f,"        model parameters, where t0 = t00 + dt0, and t00=<ta>-<tt>,\n");
	fprintf(f,"        where ta are arrival times and tt are traveltimes and <>\n");
	fprintf(f,"        are averages over the plane wave.  dt0 is then forced to\n");
	fprintf(f,"        zero by applying a weight that is 10 times larger than\n");
	fprintf(f,"        for the velocity model parameters, that is, the elemet of\n");
	fprintf(f,"        G that multiplies dt0 is set to 0.1 time the largest element\n");
	fprintf(f,"        of the part of G that multiplies dv.\n");
	fprintf(f,"    I recommend that the second method be used. It preserves the sparseness\n");
	fprintf(f,"        of G, so generally speaking allows larger problems to be solved.\n");
	fprintf(f,"        (Currently, the telesesimic invert allows you to choose\n");
	fprintf(f,"        between methods, while the joint local/telesismic inversion\n");
	fprintf(f,"        just uses the second).\n");
	fprintf(f,"\n");
	return(TRUE);
	}

int write_info_blerb(f)
FILE *f;
{
fprintf(f,"\n");
fprintf(f,"Raytrace3d, by William Menke.  Copyright 2000 by Columbia University.\n");
fprintf(f,"This version complied on %s\n", DATE);

fprintf(f,"Warning: This is prototype software intended for research purposes only.\n");
fprintf(f,"Use at your own risk!\n");
fprintf(f,"\n");
fprintf(f,"Raytrace3d implements raytracing in an isotropic medium described by\n");
fprintf(f,"a set of velocity nodes, with the nodes being organized into adjoining\n");
fprintf(f,"tetrahedra. Within any tetrahedron, the velocity varies linearly with\n");
fprintf(f,"position, and the raypaths are therefore arcs of circles.  The traveltime\n");
fprintf(f,"along along the ray is a known, analytic function of arc-length along\n");
fprintf(f,"the ray.  Thus the major effort in raytracing is the bookkeeping necessary\n");
fprintf(f,"to continue rays between adjacent tetrahedra.\n");
fprintf(f,"\n");
fprintf(f,"The basic raytracing methodology used in Raytrace3d works on arbitrary\n");
fprintf(f,"sets of tetrahedra.  However, I have chosen to make the model specification\n");
fprintf(f,"much more restrictive: a warped cartesan grid of LxMxN nodes.  I chose\n");
fprintf(f,"this simplification because it makes the design of a model by a human\n");
fprintf(f,"being easier and more comprehensible.  Furthermore, it allows one to\n");
fprintf(f,"easily name some interesting subhorizontal interface with the model (which\n");
fprintf(f,"can be useful in the raytracing). Internal to Raytrace3d, each rectangular\n");
fprintf(f,"parallelipiped in the warped cartesian grid is subdivided into several\n");
fprintf(f,"tetrahedra.\n");
fprintf(f,"\n");
fprintf(f,"I use a cartesian, right-handed, (x,y,z) coordinate system with z down.\n");
fprintf(f,"Angle of incidence is measured from +z, and azimuth is measured from +x\n");
fprintf(f,"to +y.  You are of course free to align x with whatever geographic direction\n");
fprintf(f,"you choose.  But if you make (x,y,z) = (N,E,Down) then azimuth will be\n");
fprintf(f,"degrees east of north.\n");
fprintf(f,"\n");
fprintf(f,"With one exception, Raytrace3d only permits you to work with a a single\n");
fprintf(f,"velocity model at a time, so there is no mechanism for simultaneously raytracing\n");
fprintf(f,"through distinct compressional and shear wave models.  However, it does provide\n");
fprintf(f,"a method of specifing a compressional-to-shear wave velocity ratio, so that\n");
fprintf(f,"both compressional and shear wave data can be used in, say, earthquake location.\n");
fprintf(f,"\n");
fprintf(f,"The exception is a joint inversion of P and S traveltimes for compressional\n");
fprintf(f,"velocity and compressional-to-shear wave velocity ratio (see below).\n");
fprintf(f,"\n");
fprintf(f,"Point-to-point (and planewave-to-point) raytracing is accomplished in a\n");
fprintf(f,"two-step process.  First, you compute an exhaustive set of rays from\n");
fprintf(f,"a specific point (or planewave) to a specific interface in the model\n");
fprintf(f,"on which the receivers reside.  I call the resulting ray set a star\n");
fprintf(f,"(or, in the case of the planewave, a turf).  A second step uses the\n");
fprintf(f,"star (or turf) to home-in on the ray that hits a specific receiver. The\n");
fprintf(f,"advantage of this approach is that it allows the systematic discovery of\n");
fprintf(f,"the several different rays that might reach a single station.  Raytrace3d\n");
fprintf(f,"thus allows you to work with data from both first and later-arriving\n");
fprintf(f,"waves.  The disadvantage of the two step process is that it is computationally\n");
fprintf(f,"inefficient, especially if only a few rays share a common source point\n");
fprintf(f,"(or planewave).\n");
fprintf(f,"\n");
fprintf(f,"Raytrace3d is a command-line oriented program that uses simple text\n");
fprintf(f,"file for input and output.  It must be combined with a good graphics\n");
fprintf(f,"package (such as the GMT Generic Mapping Tools) to produce comprehensible\n");
fprintf(f,"results. Raytrace3d is implements these functions:\n");
fprintf(f,"\n");
fprintf(f,"Model Examination. You always need to plot the model.\n");
fprintf(f,"     raytrace3d velocity ... evaluates the velocity model along a plane.\n");
fprintf(f,"          The results can be used an input to a program that makes color images.\n");
fprintf(f,"     raytrace3d grid ... samples the boundaries of the tetrahedra along a\n");
fprintf(f,"	  plane.  These data can be overlain onto a color image of the model.\n");
fprintf(f,"     raytrace3d interface_depth ... determines the depth of a named, subhorizontal\n");
fprintf(f,"          interface at a particular horizontal position.\n");
fprintf(f,"     raytrace3d interface_grid ... determines the depth of a named, subhorizontal\n");
fprintf(f,"          interface on a grid of (x,y) points.\n");
fprintf(f,"     raytrace3d point ... evaluates the velocity model a a single point.\n");
fprintf(f,"     raytrace3d point_list ... evaluates the velocity model a list of points.\n");
fprintf(f,"     raytrace3d dump_model ... writes out model tetrahedralization.\n");
fprintf(f,"\n");
fprintf(f,"Point-to-interface raytracing, handy to quickly generate a plot of some rays.\n");
fprintf(f,"     raytrace3d shoot_arc ... shoots rays at a suite of angles of incidence\n");
fprintf(f,"          and a specified azimuth from a specific point to an interface\n");
fprintf(f,"\n");
fprintf(f,"Point-to-point raytracing, from a point source like an earthquake or explosion.\n");
fprintf(f,"     raytrace3d shoot_star ... shoots a suite of rays at a wide range of\n");
fprintf(f,"          angles of incidence and azimuths, from a single source to a single\n");
fprintf(f,"          named interface.  The results, a star file, are the first step in \n");
fprintf(f,"          point-to-point raytracing.\n");
fprintf(f,"     raytrace3d shoot_star_p ... a slighty different version of shoot_star.\n");
fprintf(f,"     raytrace3d source_to_receivers ... Uses the star file to trace rays from\n");
fprintf(f,"          the point source to receivers on the named interface.\n");
fprintf(f,"\n");

fprintf(f,"Point-to-point raytracing with one reflecting interface is implemented with\n");
fprintf(f,"commands analagous to those above.  These commands allow one to work with PmP,\n");
fprintf(f,"PP and pP phases.  Unfortunately, more complicated senarios senarios involving\n");
fprintf(f,"multiple reflectors have not been implemented).\n");
fprintf(f,"     raytrace3d shoot_arc_refl ... shoots an arc of rays\n");
fprintf(f,"     raytrace3d shoot_star_refl ... shoots a star of rays\n");
fprintf(f,"     raytrace3d source_to_receivers_refl ... performs point-to-point raytracing\n");

fprintf(f,"\n");
fprintf(f,"Planewave-to-point raytracing, from a planewave (e.g. a teleseism).\n");
fprintf(f,"     raytrace3d shoot_turf ... shoots a suite of rays at a wide range of starting\n");
fprintf(f,"          locations, from the planewave to a single named interface.  The results,\n");
fprintf(f,"          a turf file, are the first step in planewave-to-point raytracing.\n");
fprintf(f,"     raytrace3d teleseism_to_receivers ... Uses the turf file to trace rays from\n");
fprintf(f,"          the planewave to receivers on the named interface.\n");
fprintf(f,"\n");
fprintf(f,"Inversion using local (point) sources.\n");
fprintf(f,"     raytrace3d invert ... implements one iteration of a linearized inversion.\n");
fprintf(f,"          for perturbations to nodal velocities using turning wave traveltime.\n");
fprintf(f,"          The results are a new model file.\n");
fprintf(f,"          You must run the inversion successive\n");
fprintf(f,"          times to achieve multiple iterations.\n");
fprintf(f,"     raytrace3d Vinvert ... a synonym for invert\n");
fprintf(f,"     raytrace3d Iinvert ... like Vinvert, but for perturbations to reflector position\n");
fprintf(f,"          using traveltimes from reflected waves\n");
fprintf(f,"     raytrace3d Qinvert ... like Vinvert, but for perturbations to Q\n");
fprintf(f,"     raytrace3d VIinvert ... joint nodal velocity and reflector node position inversion\n");
fprintf(f,"          using both truening wave and reflected wave traveltimes\n");
fprintf(f,"     raytrace3d PSinvert ... joint nodal P-velocity and P:S ratio inversion\n");
fprintf(f,"\n");
fprintf(f,"Velocity inversion using teleseismic (planewave) sources.\n");
fprintf(f,"     raytrace3d teleseismic_invert ... implements one iteration of a linearized\n");
fprintf(f,"          inversion, of a type that is insensitive to the origin time of the\n");
fprintf(f,"          teleseism (see usage for more details). The results are a new model\n");
fprintf(f,"          file.  You must run the inversion successive times to achieve multiple\n");
fprintf(f,"          iterations.\n");
fprintf(f,"     raytrace3d T_invert ... synonym for teleseismic_invert\n");
fprintf(f,"     raytrace3d TV_invert ... joint teleseismic and local event inversion\n");
fprintf(f,"\n");
fprintf(f,"Hypocentral location.\n");
fprintf(f,"     raytrace3d locate ... implements Geiger's method.\n");
fprintf(f,"     raytrace3d tt_table ...  creates a station-specific table of traveltimes.\n");
fprintf(f,"     raytrace3d tt_surface ...  extracts a traveltime surface from a tt_table.\n");
fprintf(f,"     raytrace3d index_traveltimes ...  creates a index of traveltimes.\n");
fprintf(f,"     raytrace3d jhd_table ... creates a table of traveltimes and their derivatives\n");
fprintf(f,"     raytrace3d               that can be used for joint hypocentral determinations\n");
fprintf(f,"\n");
fprintf(f,"Utilities.\n");
fprintf(f,"     raytrace3d roughen_model ... roughens a model (e.g. to make it less idealized)\n");
fprintf(f,"     raytrace3d fit_model ... least-squares adjustment of model to velocity data\n");
fprintf(f,"     raytrace3d perturb_model ... adds elliptical perturbations to a model.\n");
fprintf(f,"     raytrace3d make_model ... build a simple model\n");
fprintf(f,"     raytrace3d model_parameters ... build some simple types of model_parameter\n");
fprintf(f,"     	file from a model file.\n");
return(TRUE);
}

/* great circle azimuth on a sphere in degrees */
/* azimuth is the angle of north_pole - point 1 - point 2 */
double great_circle_azimuth( lat1, lon1, lat2, lon2 )  
double lat1, lon1, lat2, lon2;
{
	double A, B, C, a, b, c;
	double cosA, cosB, cosC, cosa, cosb, cosc;
	double sinA, sinB, sinC, sina, sinb, sinc;

	if( (lat1==lat2) && (lon1==lon2) ) {
		return( (double)0.0 );
		}

	a=DTOR*(90.0-lat2);
	b=DTOR*(90.0-lat1);

	C = DTOR*( lon2 - lon1 );

	cosa = cos(a);
	cosb = cos(b);
	sina = sin(a);
	sinb = sin(b);
	cosC = cos(C);
	sinC = sin(C);

	cosc = cosa*cosb + sina*sinb*cosC;
	if (cosc<-1.0) c=PI; else if (cosc>1.0) c=0.0; else c=acos(cosc);
	sinc = sin(c);

	cosA = (cosa - cosb*cosc) / (sinb*sinc);
	sinA = sinC/(sinc*sina);
	if (cosA<-1.0) A=PI; else if (cosA>1.0) A=0.0; else A=acos(cosA);
	if( sinA < 0.0 ) A=(-A);

	return( A/DTOR );
	}

double solid_angle( colata, lona, colatb, lonb, colatc, lonc )
double colata, lona, colatb, lonb, colatc, lonc;
{
	double lata, latb, latc;
	double A1, A2, B1, B2, C1, C2, A, B, C, E, area;

	lata = 90.0-colata;
	latb = 90.0-colatb;
	latc = 90.0-colatc;

	A1 = great_circle_azimuth( lata,lona, latc,lonc );
	A2 =  great_circle_azimuth( lata,lona, latb,lonb );
	A = A1-A2;
	while( A<0.0) A+=360.0; while( A>=360.0) A-=360.0;

	B1 = great_circle_azimuth( latb,lonb, lata,lona );
	B2 = great_circle_azimuth( latb,lonb, latc,lonc );
	B = B1-B2;
	while( B<0.0) B+=360.0; while( B>=360.0) B-=360.0;

	C1 = great_circle_azimuth( latc,lonc, latb,lonb );
	C2 = great_circle_azimuth( latc,lonc, lata,lona );
	C = C1-C2;
	while( C<0.0) C+=360.0;  while( C>=360.0) C-=360.0;

	E = A + B + C - 180;

	area = E*DTOR;

	if( area > 2.0*PI ) area = 4.0*PI - area;

	return(area);
	}

double area_of_triagle( x, y, z, normal )
double x[3], y[3], z[3], normal[3];
{
	int i;
	double a[3], b[3], c[3], area, len;

	a[0]=x[1]-x[0]; a[1]=y[1]-y[0]; a[2]=z[1]-z[0];
	b[0]=x[2]-x[0]; b[1]=y[2]-y[0]; b[2]=z[2]-z[0];
	cross( a, b, c );
	len = sqrt(fabs(dot(c,c)));
	for( i=0; i<3; i++ ) normal[i]=c[i]/len;
	if( normal[2] < 0.0 ) for( i=0; i<3; i++ ) normal[i]*=(-1.0);
	
	area = 0.5*len;
	return(area);
	}

int point_in_tetra( x, t )
double x[3];
int t;
{
	struct tetra_rec *p;
	if( t>= 0 && t<n_tetras ) {
		p = &tetra[t];
		return( in_tetrahedron( p->x1, p->x2, p->x3, p->x4, x) );
		}
	else return(0);
	}

int neighbor( t, f )
int t, f;
{
	int tn;

	struct tetra_rec *p;
	if( (t<0) || (t>=n_tetras) ) return(-1);

	p = &tetra[t];
	if( f == 0 )      tn=p->t1;
	else if( f == 1 ) tn=p->t2;
	else if( f == 2 ) tn=p->t3;
	else if( f == 3 ) tn=p->t4;
	else tn=(-1);
	return(tn);

	}

double integral292( double a, double x )
{
/* (integral ) tan(ax) dx /                          */
/* see CRC Std Math Tables Integral 292              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */
	double ax, value;
	ax=a*x;
	value = log(fabs(cos(ax)))/(-a);
	return(value);
	}

double integral292a( double x )
{
/* (integral ) tan(x) dx /                           */
/* see CRC Std Math Tables Integral 292              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */
	double value;
	value = (-log(fabs(cos(x))));
	return(value);
	}

double inv_integral294a( double x )
{
/* inverts integral 294a                             */
/* Warning! Only checked on interval -PI<x<PI        */

	double PI=3.141592653589793;
	double value;
	value = 2.0*atan(exp(x))-PI/2.0;
	return(value);
	}


double integral294( double a, double x )
{
/* (integral ) dx / cos(ax)                          */
/* see CRC Std Math Tables Integral 294              */
/* integrand is symmetric in x, so always do |x|     */
/* reversing sign for x<0                            */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan(PI/4.0+fabs(a*x/2.0))))/a;
	if( x>=0.0 ) return(value); else return(-value);
	}

double integral294a( double x )
{
/* (integral ) dx / cos(x)                           */
/* see CRC Std Math Tables Integral 294              */
/* integrand is symmetric in x, so always do |x|     */
/* reversing sign for x<0                            */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan((PI/4.0)+fabs(x/2.0))));
	if( x>=0.0) return(value); else return(-value);
	}

double integral295( double a, double x )
{
/* (integral ) dx / sin(ax)                          */
/* see CRC Std Math Tables Integral 294              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */

	double ax, value;
	double PI=3.141592653589793;
	ax = a*x;
	value = log(fabs(tan(ax/2.0)))/a;
	return(value);
	}

double integral295a( double a, double x )
{
/* (integral ) dx / sin(x)                           */
/* see CRC Std Math Tables Integral 294              */
/* integrand is antisymmetric in x, so always do |x| */
/* using same value for x<0                          */

	double value;
	double PI=3.141592653589793;
	value = log(fabs(tan(fabs(x/2.0))));
	return(value);
	}

double integral340( double a, double b, double x )
/* (integral ) dx / (a + b*sin(x) )                   */
/* see CRC Std Math Tables Integral 340               */
/* integrand is antisymmetric in x, so always do |x|  */
/* using same value for x<0                           */
/* I checked this against numerical integration       */
/* for 4*10**4 points on a grid in a=[0.0,2.03],      */
/* b=[0.1,1.01], x1=[-6.27,6.27], x2=[-6.27,6.27]     */
/* and it seemed to work. William Menke, 07-14-2000.  */
{
	double PI=3.141592653589793;
	double tanbranch();
	double a2, b2, root, tanx2, t1, t2, value, x2;

	a2=a*a; b2=b*b; x2=fabs(x/2.0);

	if( a2>b2 ) {
		root = sqrt(a2-b2);
		tanx2 = tan(x2);
		t1 = a*tanx2+b;
		t2 = atan( t1/root ) + PI*tanbranch(x2);
		value = 2.0*t2/root;
		}
	else {
		root = sqrt(b2-a2);
		tanx2 = tan(x2);
		t1 = a*tanx2 + b - root;
		t2 = a*tanx2 + b + root;
		value = log(fabs(t1/t2))/root;
		}
	return(value);
	}

double integral341( double a, double b, double x )
/* (integral ) dx / (a + b*cos(x) )              */
/* see CRC Std Math Tables Integral 341          */
/* integrand is symmetric in x, so always do |x| */
/* reversing sign of result for x<0              */
{
	double PI=3.141592653589793;
	double tanbranch();
	double a2, b2, root, tanx2, t1, t2, value, x2;

	a2=a*a; b2=b*b; x2=fabs(x/2.0);

	if( a2>b2 ) {
		root = sqrt(a2-b2);
		tanx2 = tan(x2);
		t1 = root*tanx2;
		t2 = atan( t1/(a+b) ) + PI*tanbranch(x2);
		value = 2.0*t2/root;
		}
	else {
		root = sqrt(b2-a2);
		tanx2 = tan(x2);
		t1 = root*tanx2 + a + b;
		t2 = root*tanx2 - a - b;
		value = log(fabs(t1/t2))/root;
		}
	if( x>=0.0) return(value); else return(-value);
	}


double integral342( double a, double b, double c, double x )
/* (integral ) dx / (a + b*sin(x) + c*cos(x) )   */
/* see CRC Std Math Tables Integral 342          */
/* I checked this against numerical integration  */
/* for 4*10**5 points on a grid in a=[0.0,2.03], */
/* b=[0.1,1.01], c=[0.1,1.07], x1=[-6.27,6.27],  */
/* x2=[-6.27,6.27] and it seemed to work.        */
/* William Menke, 07-14-2000.                    */
{
	double PI=3.141592653589793;
	double a2, b2, c2, b2pc2;
	double root, tanx2, x2, t1, t2, t3, cx, sx;
	double value;
	double tanbranch();

	a2=a*a; b2=b*b; c2=c*c; b2pc2=b2+c2; x2=x/2.0;

	if( a2 < b2pc2 ) {
		root = sqrt(b2pc2-a2);
		tanx2 = tan(x2);
		t1 = b - root + (a-c)*tanx2;
		t2 = b + root + (a-c)*tanx2;
		value = log(fabs(t1/t2)) / root;
		}
	else if ( a2 > b2pc2 ) {
		root=sqrt(a2-b2pc2);
		tanx2 = tan(x2);
		t1 = b + (a-c)*tanx2;
		t2 = atan( t1 / root ) + PI*tanbranch(x2);
		value = 2.0 * t2 / root;
		}
	else {
		cx=cos(x); sx=sin(x);
		t1 = a - (b+c)*cx - (b-c)*sx;
		t2 = a - (b-c)*cx + (b+c)*sx;
		value = t1/(t2*a);
		}

	return(value);
	}

double tanbranch(double x)
{
	int i;
	double PI=3.141592653589793;
	i = (int) ( fabs(x) / (PI/2.0) );
	return( copysign( (double)((i+1)/2), x ) );
	}

double qintegral( qinv0, qinvx, qinvz, R, vz, phi )
double qinv0, qinvx, qinvz, R, vz, phi;
{
/* (integral) qinv dT =                                                                */
/* (integral) qinv (1/v) R  d(phi) =                                                   */
/* (integral) R [qinv0 + qinvx*x + qinvz*z] / R*vz*cos(phi) d(phi) =                   */
/* (integral) R [qinv0 + qinvx*R*sin(phi) + qinvz*R*cos(phi)] / R*vz*cos(phi) d(phi) = */
/* (qinv0/vz) (integral) 1/cos(phi) d(phi) +                                           */
/* (R*qinvx/vz) (integral) tan(phi) d(phi) +                                           */
/* (R*qinvz/vz) (integral) d(phi)                                                      */

	double value;

	value = (qinv0/vz)   * integral294a(DTOR*phi)
	      + (R*qinvx/vz) * integral292a(DTOR*phi)
	      + (R*qinvz/vz) * DTOR*phi;

	return(value);
	}

int setorder( tt, n_tts, traveltime, ratio, order )
struct tt_rec *tt;
int n_tts;
double traveltime;
double ratio;
int *order;
{
	int i, j;
	double residual, temp;

	if( n_tts < 1 ) return(1);
	else if (n_tts == 1 ) {
		*order=0;
		return(1);
		}
	else {
		j=0;
		residual=fabs( traveltime - ratio*tt[0].T );

		for( i=0; i<n_tts; i++ ) {
			temp = fabs( traveltime - ratio*tt[i].T );
			if( temp<residual ) {
				residual=temp;
				j=i;
				}
			}
		*order = j;
		return(1);
		}
	}

int timestamp(char *s)
{
	long i;
	time(&i);
	fprintf(stderr,"Time %d seconds: %s\n", i-starttime, s );
	return(1);
	}


int count_lines( char *filename )
{
	FILE *f;
	int i, n, c;

	i=0; n=0;
	if( (f=fopen(filename,"r"))==NULL ) {
		fprintf(stderr,"error: count_lines: cant open file <%s> for read\n", filename);
		return(0);
		}
	while( (c=getc(f)) != EOF ) {
		i++;
		if( c=='\n' ) n++;
		}
	fclose(f);

	return(n);
	}

int make_ttime(char *filename)
{
	int n;

	MAX_TTIMES=count_lines(filename);
	if( MAX_TTIMES==0 ) {
		ttime==NULL;
		}
	else {
		MAX_TTIMES+=10;
		ttime=(struct ttime_rec *)malloc(MAX_TTIMES*sizeof(struct ttime_rec));
		}
	if( ttime==NULL ) {
		fprintf(stderr,"error: out of memory (ttime array)\n");
		exit(-1);
		}
	return(1);
	}
	
/* sparse matrix routines, by William Menke, November 2001 */
/* a simple package that implements efficient storage of   */
/* symmetric and asymmetric sparse matrices, and common    */
/* operations, such as multiplication, applied to them     */

/* you declare a pointer to a n by m sparse matrix, and this  */
/* routine initializes it to a matrix of zeros.  Use sym=0    */
/* for a asymmetric matrix, sym=1 for a square symmetric one  */
struct sparse_matrix *new_sparse_matrix(int n, int m, int sym )
{
	int i;
	struct sparse_matrix *matrix;
	struct sparse_elem **row, **col;

	if( (n<1) || (m<1) || ((n!=m)&&sym) ) return(NULL);

	matrix = (struct sparse_matrix *) malloc(sizeof(struct sparse_matrix));
	if( matrix==NULL ) return(NULL);
	matrix->n=n; matrix->m=m; matrix->count=0; matrix->sym=sym;

	row = (struct sparse_elem **) malloc(n*sizeof(struct sparse_elem *));
	if( row==NULL ) {free(matrix); return(NULL);}
	else { matrix->row=row; for( i=0; i<n; i++ ) row[i]=NULL; }

	col = (struct sparse_elem **) malloc(m*sizeof(struct sparse_elem *));
	if( col==NULL ) {free(matrix); free(row); return(NULL);}
	else { matrix->col=col; for( i=0; i<m; i++ ) col[i]=NULL; }

	return(matrix);
	}

/* copy a sparse matrix */
struct sparse_matrix *copy_sparse_matrix(struct sparse_matrix *A)
{
	int i;
	struct sparse_matrix *matrix;
	struct sparse_elem **row, **col, *e;

	if( A==NULL ) return(NULL);

	matrix = (struct sparse_matrix *) malloc(sizeof(struct sparse_matrix));
	if( matrix==NULL ) return(NULL);
	matrix->n=A->n; matrix->m=A->m; matrix->count=A->count; matrix->sym=A->sym;

	row = (struct sparse_elem **) malloc(A->n*sizeof(struct sparse_elem *));
	if( row==NULL ) {free(matrix); return(NULL);}
	else { matrix->row=row; for( i=0; i<A->n; i++ ) row[i]=NULL; }

	col = (struct sparse_elem **) malloc(A->m*sizeof(struct sparse_elem *));
	if( col==NULL ) {free(matrix); free(row); return(NULL);}
	else { matrix->col=col; for( i=0; i<A->m; i++ ) col[i]=NULL; }

	for( i=0; i<A->n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			if( !set_sparse_value(matrix,e->i,e->j,e->x) ) return(NULL);
			e=e->no;
			}
		}

	return(matrix);
	}

/* transpose a sparse matrix in place (ie A is changed to A-transpose) */
int sparse_transpose(struct sparse_matrix *A)
{
	int i, j;
	struct sparse_matrix *matrix;
	struct sparse_elem **row, *e, *next_e, **p;

	if( A==NULL ) return(0);
	if( A->sym ) return(1);

	for( i=0; i<A->n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			next_e=e->no; e->no=e->nd; e->nd=next_e;
			j=e->j; e->j=e->i; e->i=j;
			e=next_e;
			}
		}
	row=A->row;
	A->row=A->col;
	A->col=row;

	j=A->n; A->n=A->m; A->m=j;

	return(1);
	}


/* accesses the dimensions n, m and number of non-zero elements */
int sparse_dim( struct sparse_matrix *A, int *n, int *m, int *count )
{
	if( A==NULL ) return(0);
	else {
		*n = A->n; *m=A->m; *count=A->count;
		return(1);
		}
	}

/* is the sparse matrix A symmetric ? */
int sparse_issym( struct sparse_matrix *A )
{
	if( A==NULL ) return(0);
	else {
		return(A->sym);
		}
	}

/* return count of non-zero elements*/
int sparse_count( struct sparse_matrix *A )
{
	if( A==NULL ) return(0);
	else return(A->count);
	}

/* delete a sparse matrix */
int free_sparse_matrix( struct sparse_matrix *m )
{
	int i;
	struct sparse_elem *e, *ep;
	for( i=0; i<m->n; i++ ) {
		e=m->row[0];
		while( e!=NULL ) {
			ep=e->no;
			free(e);
			e=ep;
			}
		}
	return(1);
	}

/* gets the value of element m[i][j] */
double get_sparse_value( struct sparse_matrix *m, int i, int j)
{
	struct sparse_elem *e;
	int k;

	if( m->sym && (i>j) ) { k=i; i=j; j=k; };

	if( m->n > m->n ) { 
		e=m->row[i];
		while( e!=NULL ) {
			if( e->j == j ) return(e->x); else e=e->no;
			}
		}
	else {
		e=m->col[j];
		while( e!=NULL ) {
			if( e->i == i ) return(e->x); else e=e->nd;
			}
		}
		
	return((double)0.0);
	}

/* sets the value of m[i][j]=x. */
int set_sparse_value( struct sparse_matrix *m, int i, int j, double x)
{
	struct sparse_elem *r, *rl, *c, *cl, *e;
	int inm, k;

	if( (i<0) || (j<0) || (i>=m->n) || (j>=m->m) ) return(0);

	if( m->sym && (i>j) ) { k=i; i=j; j=k; }

	if( x==0.0 ) {
		c=m->row[i];
		cl=NULL;
		while( c!=NULL ) {
			if( c->j == j ) break;
			cl=c;
			c=c->no;
			}
		/* case 1: this element not in matrix, so do nothing */
		if( c == NULL ) {
			return(1);
			}

		/* case 2: this element in matrix, so remove it */
		if( cl != NULL ) cl->no = c->no;
		if( m->row[i] == c ) m->row[i]=c->no;
		r=m->col[j];
		rl=NULL;
		while( r!=NULL ) {
			if( r == c ) break;
			rl=r;
			r=r->nd;
			}
		if( r != c ) {
			fprintf(stderr,"sparse matrix bug: nomatch\n");
			exit(255);
			}

		if( rl != NULL ) rl->nd = r->nd;
		if( m->col[j] == r ) m->col[j]=r->nd;
		free(c);
		m->count--;
		return(1);
		}

	else {
		inm=0;
		c=NULL; cl=NULL;
		if( m->row[i]!=NULL ) {
			c=m->row[i];
			cl=NULL;
			while( c!=NULL ) {
				if( c->j==j ) { inm=1; break; }
				if( c->j>j ) break;
				cl=c;
				c=c->no;
				}
			}

		if( inm ) { /* case 3: element is the the matrix and needs to be reset */
			c->x = x;
			return(1);
			}

		else { /* case 4: element is no in matrix and must be inserted */

			r=m->col[j];
			rl=NULL;
			while( r!=NULL ) {
				if( r->i>i ) break;
				rl=r;
				r=r->nd;
				}

			e = (struct sparse_elem *) malloc( sizeof(struct sparse_elem) );
			if( e==NULL ) return(0);
			m->count++;
			e->i=i; e->j=j; e->x=x; e->nd=NULL; e->no=NULL;
			if( rl!=NULL ) rl->nd=e; e->nd=r;
			if( cl!=NULL ) cl->no=e; e->no=c;
			if( m->row[i] == NULL ) m->row[i]=e;
			else if( m->row[i] == c ) m->row[i]=e;
			if( m->col[j] == NULL ) m->col[j]=e;
			else if( m->col[j] == r ) m->col[j]=e;
			return(1);
			}
		}
	}

/* add a vector vec to column j of sparse matrix A   */
/* zero-values of vec are ignored.  The element      */
/* is not removed if its new value is exactly zero   */
/* WARNING: Will not work on symmetric matrices!     */
int sparse_add_to_col( struct sparse_matrix *A, int j, double *vec )
{
	int i, n, m;
	int *isnew;
	struct sparse_elem *r, *rl, **p, *e, *c, *cl;
	int inm;

	if( A==NULL ) return(0);
	if( A->sym ) return(0);
	n=A->n; m=A->m;

	/* create column pointer and new-flag */
	p=(struct sparse_elem **)malloc(n*sizeof(struct sparse_elem *));
	if( p==NULL ) return(0);
	isnew=(int*)malloc(n*sizeof(int));
	if( isnew==NULL ) {free(p); return(0); }

	/* initial column pointer and new-flag */
	for( i=0; i<n; i++ ) {
		p[i]=NULL;
		isnew[i]=0;
		}

	/* copy existing elements of column into p */
	e=A->col[j];
	while( e!=NULL ) {
		p[e->i]=e;
		e=e->nd;
		}

	/* add vec to elements of column, creating new ones as necessary */
	for( i=0; i<n; i++ ) if( vec[i]!=0.0 ) {
		if( p[i]==NULL ) {
			p[i]=(struct sparse_elem*)malloc(sizeof(struct sparse_elem));
			if(p[i]==NULL) {free(p);free(isnew);return(0);}
			A->count++;
			p[i]->i=i; p[i]->j=j; p[i]->x=vec[i]; p[i]->nd=NULL; p[i]->no=NULL;
			isnew[i]=1;
			}
		else {
			p[i]->x+=vec[i];
			}
		}

	/* relink the next-down pointers and the start-of-column pointer */
	r=NULL; rl=NULL; A->col[j]=NULL;
	for( i=0; i<n; i++ ) if( p[i]!=NULL ) {
		r=p[i];
		r->nd=NULL;
		if( rl==NULL ) A->col[j]=r; else rl->nd=r;
		rl=r;
		}

	/* relink the next-over pointers and the start-of-row pointer */
	/* only need to process rows with new elements                */
	for( i=0; i<n; i++ ) if( isnew[i] ) {
		if( A->row[i]==NULL ) { /* this is the only element of the row */
			A->row[i]=p[i];
			p[i]->no=NULL;
			}
		else { 
			/* locate position: column j between cl and c */
			c=A->row[i]; cl=NULL;
			while( c!=NULL ) {
				if( c->j > j ) break;
				cl=c;
				c=c->no;
				}
			if( cl==NULL ) { /* no last-column, so thus is first element */
				p[i]->no=A->row[i];
				A->row[i]=p[i];
				}
			else {
				p[i]->no = cl->no;
				cl->no=p[i];
				}
			}
		}

	free(p); free(isnew);							
	return(1);
	}

/* b = A * a, with A sparse and a,b not sparse*/
int sm_times_v( struct sparse_matrix *A, double *a, double *b )
{
	int i, n, k, iscol;
	struct sparse_elem *c;
	double sum;

	n=A->n;
	if( A==NULL) {
		return(0);
		}
	else if( ! A->sym ) {
		for( i=0; i<n; i++ ) {
			sum=0.0;
			c=A->row[i];
			while( c!=NULL ) {
				sum += c->x * a[c->j];
				c = c->no;
				}
			b[i]=sum;
			}
		}
	else {
		for( i=0; i<n; i++ ) {
			iscol=1;
			c=A->col[i]; if(c==NULL){ c=A->row[i]; iscol=0; }
			sum=0.0;
			while( c!=NULL ) {
				if( iscol ) k=c->i; else k=c->j;
				sum += c->x * a[k];
				if( iscol ) {
					c=c->nd;
					if( c==NULL ) {
						iscol=0;
						c=A->row[i];
						if( c!=NULL ) if(c->i==c->j) c=c->no;
						}
					}
				else c=c->no;
				}
			b[i]=sum;
			}
		}

	return(1);
	}

/* b = A-transpose * a, with A sparse and a,b not sparse*/
int smt_times_v( struct sparse_matrix *A, double *a, double *b )
{
	int j, m;
	struct sparse_elem *r;
	double sum;

	if( a==NULL) return(0);
	else if( A->sym ) return( sm_times_v(A,a,b) );
	else {

		m=A->m;
		for( j=0; j<m; j++ ) {
			sum=0.0;
			r=A->col[j];
			while( r!=NULL ) {
				sum += r->x * a[r->i];
				r = r->nd;
				}
			b[j]=sum;
			}
		return(1);
		}

	}


/* C = A * B, with A,B sparse*/
struct sparse_matrix *sm_times_sm( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB;
	double sum;
	int iscolA, iscolB;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( mA != nB ) return(NULL);

	if( (!A->sym) && (!B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			eA=A->row[i]; eB=B->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j; kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no; else eB=eB->nd;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else if( (A->sym) && (!B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if( iscolA ) {
					    eA=eA->nd;
					    if( eA==NULL ) {
					        iscolA=0;
					        eA=A->row[i];
					        if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
					        }
					    }
					else eA=eA->no;
					}
				else eB=eB->nd;
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	else if( (!A->sym) && (B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			eA=A->row[i];
			iscolB=1;
			eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j;
				if(iscolB) kB=eB->i; else kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no;
				else {
					if( iscolB ) {
					    eB=eB->nd;
					    if( eB==NULL ) {
						iscolB=0;
						eB=B->row[j];
						if(eB!=NULL) if(eB->i==eB->j) eB=eB->no;
						}
					    }
					else eB=eB->no;
					}
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	else if( (A->sym) && (B->sym) ) {

		if( A!=B ) {
			C=new_sparse_matrix(nA,mB,0);
			if( C==NULL ) return(NULL);
			for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
				sum=0.0;
				iscolA=1; iscolB=1;
				eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
				eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
				while( (eA!=NULL) && (eB!=NULL) ) {
					if(iscolA) kA=eA->i; else kA=eA->j;
					if(iscolB) kB=eB->i; else kB=eB->j;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) {
					     if(iscolA){
						eA=eA->nd;
						if( eA==NULL ) {
							iscolA=0;
							eA=A->row[i];
							if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
							}
						    }
						else eA=eA->no;
						}
					else {
					    if(iscolB){
						eB=eB->nd;
						if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB=eB->no;
							}
						    }
						else eB=eB->no;
						}
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		else {
			C=new_sparse_matrix(nA,mB,1);
			if( C==NULL ) return(NULL);

			for( i=0; i<nA; i++ ) for( j=i; j<mB; j++ ) {
				sum=0.0;
				iscolA=1; iscolB=1;
				eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
				eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
				while( (eA!=NULL) && (eB!=NULL) ) {
					if(iscolA) kA=eA->i; else kA=eA->j;
					if(iscolB) kB=eB->i; else kB=eB->j;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) {
					    if(iscolA){
						eA=eA->nd;
						     if( eA==NULL ) {
							iscolA=0;
							eA=A->row[i];
							if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
							}
						    }
						else eA=eA->no;
						}
					else {
						if(iscolB){
						    eB=eB->nd;
						    if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB = eB->no;
							}
						    }
						else eB=eB->no;
						}
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		}

	return(C);
	}

/* C = A-transpose * B-transpose, with A,B sparse*/
struct sparse_matrix *smt_times_smt( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( nA != mB ) return(NULL);

	if( A->sym && B->sym ) return( sm_times_sm(A,B) );
	else if( A->sym && !B->sym ) return( sm_times_smt(A,B) );
	else if( !A->sym && B->sym ) return( smt_times_sm(A,B) );

	C=new_sparse_matrix(mA,nB,0);
	if( C==NULL ) return(NULL);

	for( i=0; i<mA; i++ ) for( j=0; j<nB; j++ ) {
		sum=0.0;
		eA=A->col[i]; eB=B->row[j];
		while( (eA!=NULL) && (eB!=NULL) ) {
			kA=eA->i; kB=eB->j;
			if( kA==kB ) sum += eA->x * eB->x;
			if( kA <= kB ) eA=eA->nd; else eB=eB->no;
			}
		set_sparse_value(C,i,j,sum);
		}

	return(C);
	}

/* returns absolute max element of ATA (assuming it to be on main diagonal) */
double ATAabsmax( struct sparse_matrix *A)
{
	struct sparse_elem *eA;
	int i, j, nA, mA, iscolA;
	double sum, t, absmax;

	if( A==NULL ) return(0.0);
	nA=A->n; mA=A->m;

	if( A->sym ) {
		for( i=0; i<mA; i++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			while( eA!=NULL ) {
				sum += eA->x * eA->x;
				if( iscolA ) {
					eA=eA->nd;
					if( eA==NULL ) {
						iscolA=0;
					        eA=A->row[i];
					        if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
					        }
					else {
						eA=eA->no;
						}
					}
				}
			t = fabs(sum);
			if( i==0 ) {
				absmax = t;
				}
			else if ( t > absmax ) {
				absmax = t;
				}
			}
		}
	else {
		for( i=0; i<mA; i++ ) {
			sum=0.0; j=0;
			eA=A->col[i];
			while( eA!=NULL ) {
				sum += eA->x * eA->x;
				j++;
				eA=eA->nd;
				}
			t = fabs(sum);
			if( i==0 ) {
				absmax = t;
				}
			else if ( t > absmax ) {
				absmax = t;
				}
			}
		}

	return(absmax);
	}

/* C = A * B-transpose, with A,B sparse*/
struct sparse_matrix *sm_times_smt( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB, iscolA;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( mA != mB ) return(NULL);

	if( B->sym ) return( sm_times_sm(A,B) );
	else if( !A->sym ) {
		C=new_sparse_matrix(nA,nB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<nB; j++ ) {
			sum=0.0;
			eA=A->row[i]; eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j; kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no; else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else if (A==B) { 
		C=new_sparse_matrix(nA,nB,1);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=i; j<nB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if(iscolA){
					    eA=eA->nd;
					    if( eA==NULL ) {
						iscolA=0;
						eA=A->row[i];
						if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
						}
					    }
					else eA=eA->no;
					}
				else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else { 
		C=new_sparse_matrix(nA,nB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<nB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if(iscolA){
					    eA=eA->nd;
					    if( eA==NULL ) {
						iscolA=0;
						eA=A->row[i];
						if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
						}
					    }
					else eA=eA->no;
					}
				else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	return(C);
	}

/* C = A-transpose * B, with A,B sparse*/
struct sparse_matrix *smt_times_sm( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB, iscolB;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( nA != nB ) return(NULL);

	if( A->sym ) return( sm_times_sm(A,B) );
	else if (!B->sym) {
		if( A==B ) {
			C=new_sparse_matrix(mA,mB,1);
			if( C==NULL ) return(NULL);
			for( i=0; i<mA; i++ ) for( j=i; j<mB; j++ ) {
				sum=0.0;
				eA=A->col[i]; eB=B->col[j];
				while( (eA!=NULL) && (eB!=NULL) ) {
					kA=eA->i; kB=eB->i;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		else {
			C=new_sparse_matrix(mA,mB,0);
			if( C==NULL ) return(NULL);
			for( i=0; i<mA; i++ ) for( j=0; j<mB; j++ ) {
				sum=0.0;
				eA=A->col[i]; eB=B->col[j];
				while( (eA!=NULL) && (eB!=NULL) ) {
					kA=eA->i; kB=eB->i;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		}

	else {
		C=new_sparse_matrix(mA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<mA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			iscolB=1;
			eA=A->col[i];
			eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->i;
				if(iscolB) kB=eB->i; else kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->nd;
				else {
					if(iscolB){
						eB=eB->nd;
						if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB = eB->no;
							}
						}
					else eB=eB->no;
					}
				}
			set_sparse_value(C,i,j,sum);
			}
		}


	return(C);
	}

/* largest absolute value of elements */
double sparse_absmax(struct sparse_matrix *A)
{
	double x, v;
	int i, n;
	struct sparse_elem *e;

	v=0.0;
	n=A->n;
	for( i=0; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			x=fabs(e->x);
			if( x>v ) v=x;
			e=e->no;
			}
		}

	return(v);
	}

/* A = A + xI where A is sparse and x is a constant */
int sparse_addtodiag(struct sparse_matrix *A, double x)
{
	struct sparse_elem *e;
	int i, n;

	if( A==NULL) return(0);

	n=A->n;
	for( i=0; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			if( e->i == e->j ) e->x += x;
			e=e->no;
			}
		}

	return(1);
	}

/* truncate rows off bottom of sparse matrix, A*/
/* Note: A must not be symmetric */
int sparse_truncaterows( struct sparse_matrix *A, int newn ) 
{
	int i, j, n, m;
	struct sparse_elem *e, *ec;

	if( A==NULL ) return(0);
	if( A->sym ) return(0);
	n=A->n; m=A->m;
	if(newn>n) return(0); else if(newn==n) return(1);

	/* adjust all column pointers to cut off last rows */
	for( j=0; j<m; j++ ) {
		e=A->col[j];
		while( e!=NULL ) {
			ec=e->nd;
			if( ec != NULL ) if( ec->i >= newn ) { e->nd=NULL; break; }
			e=ec;
			}
		}

	/* delete rows */
	for( i=newn; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			ec = e;
			e = e->no;
			free(ec);
			}
		}

	A->n=newn;

	return(1);
	}
		
#define EPS (1.0e-14)

/*solves n by n system Ax=b for x by bicongugate gradient method
   A: sparse matrix (must be square)
   b: l.h.s. vector
   x: solution; initialize to guess (e.g. 0.0) before call
   itol: 1, 2, 3, 4 depending on choice of convergence test
   tol: tolerance
   itmax: maximum iterations
   iter: actual iterations
   err: error estimate
   */
int sparse_asolve( struct sparse_matrix *A, double *b, double *x, int transpose );
int sparse_atimes( struct sparse_matrix *A, double *x, double *r, int transpose );
double snorm( int n, double *sx, int itol );

int sparse_biconj( struct sparse_matrix *A, double *b, double *x, int itol, double tol,
	int itmax, int *iter, double *err, int *biec)
{
	int j, n;
	double ak, akden, bk, bkden, bknum, bnorm, dxnorm, xnorm, zm1norm, znorm, t;
	double *p=NULL, *pp=NULL, *r=NULL, *rr=NULL, *z=NULL, *zz=NULL;

	if( A==NULL ) {
		fprintf(stderr,"error: biconj: NULL Matrix\n");
		*biec=1;
		return(0);
		}

	if( A->n != A->m ) {
		fprintf(stderr,"error: biconj: non-square Matrix\n");
		*biec=2;
		return(0);
		}

	if( (itol<1) || (itol>4) ) {
		fprintf(stderr,"error: biconj: bad itol\n");
		*biec=3;
		return(0);
		}

	n=A->n;

	p=(double*)malloc((n+10)*sizeof(double));
	pp=(double*)malloc((n+10)*sizeof(double));
	r=(double*)malloc((n+10)*sizeof(double));
	rr=(double*)malloc((n+10)*sizeof(double));
	z=(double*)malloc((n+10)*sizeof(double));
	zz=(double*)malloc((n+10)*sizeof(double));
	if( (p==NULL) || (pp==NULL) || (r==NULL) || (rr==NULL)
		                    || (z==NULL) || (zz==NULL) ) {
		if( p!=NULL ) free(p);
		if( pp!=NULL ) free(pp);
		if( r!=NULL ) free(r);
		if( rr!=NULL ) free(rr);
		if( z!=NULL ) free(z);
		if( zz!=NULL ) free(zz);
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}

	*iter=0;
	sparse_atimes(A,x,r,0);
	for( j=0; j<n; j++ ) {
		r[j]=b[j]-r[j];
		rr[j]=r[j];
		}
	znorm=1.0;
	if( itol==1) {
		bnorm=snorm(n,b,itol);
		}
	else if( itol==2 ) {
		sparse_asolve(A,b,z,0);
		bnorm=snorm(n,z,itol);
		}
	else {
		sparse_asolve(A,b,z,0);
		bnorm=snorm(n,z,itol);
		sparse_asolve(A,r,z,0);
		znorm=snorm(n,z,itol);
		}

	sparse_asolve(A,r,z,0);

	while( (*iter) <= itmax ) {
		(*iter)++;
		zm1norm=znorm;
		sparse_asolve(A,rr,zz,1);
		bknum=0.0;
		for( j=0; j<n; j++ ) bknum+=z[j]*rr[j];
		if( (*iter)==1 ) {
			for( j=0; j<n; j++ ) {
				p[j]=z[j];
				pp[j]=zz[j];
				}
			}
		else {
			bk=bknum/bkden;
			for( j=0; j<n; j++ ) {
				p[j]=bk*p[j]+z[j];
				pp[j]=bk*pp[j]+zz[j];
				}
			}
		bkden=bknum;
		sparse_atimes(A,p,z,0);
		akden=0.0;
		for( j=0; j<n; j++ ) akden+=z[j]*pp[j];
		ak=bknum/akden;
		sparse_atimes(A,pp,zz,1);
		for( j=0; j<n; j++ ) {
			x[j]+=ak*p[j];
			r[j]-=ak*z[j];
			rr[j]-=ak*zz[j];
			}
		sparse_asolve(A,r,z,0);
		if( (itol==1) || (itol==2) ) {
			znorm=1.0;
			(*err)=snorm(n,r,itol)/bnorm;
			}
		else {
			znorm=snorm(n,z,itol);
			if( (t=fabs(zm1norm-znorm)) > (EPS*znorm) ) {
				dxnorm=fabs(ak)*snorm(n,p,itol);
				(*err)=znorm/t*dxnorm;
				}
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			xnorm=snorm(n,x,itol);
			if( (*err)<=(0.5*xnorm) ) (*err)/=xnorm;
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			}
		if( (*err)<=tol) break;
		}

	free(p); free(pp); free(r); free(rr); free(z); free(zz);
	if( (*err)<=tol) {*biec=0; return(1);}
	else {*biec=(-1); return(0);}
	}

/* internal routine, not intended as user callable */
double snorm( int n, double *sx, int itol )
{
	int i;
	double ans, t;
	if( itol<=3) {
		ans=0.0;
		for(i=0; i<n; i++ ) ans+=sx[i]*sx[i];
		ans=sqrt(ans);
		}
	else {
		ans=fabs(sx[0]);
		for(i=0; i<n; i++ ) if( (t=fabs(sx[i]))>ans) {
			ans=t;
			}
		}
	return(ans);
	}

/* internal routine, not intended as user callable */
int sparse_atimes( struct sparse_matrix *A, double *x, double *r, int transpose )
{
	int i, j;
	if( transpose ) smt_times_v( A, x, r ); else sm_times_v( A, x, r );
	return(1);
	}

/* internal routine, not intended as user callable */
int sparse_asolve( struct sparse_matrix *A, double *b, double *x, int transpose )
{
	int i, n;
	double t;
	n = A->n;
	for( i=0; i<n; i++ ) {
		t=get_sparse_value(A,i,i);
		if( t!=0.0 ) x[i]=b[i]/t;
		else x[i]=0.0;
		}
	return(1);
	}
/* test routine, prints matrix */
int dump_sparse_matrix(struct sparse_matrix *A, FILE *f)
{
	struct sparse_elem *e;
	int i, j, n, m;

	n=A->n; m=A->m;

	if(A->sym)
	fprintf(f,"Dump of symm %d by %d sparse martix with %d non-zero elements\n", n, m, A->count );
	else
	fprintf(f,"Dump of asym %d by %d sparse martix with %d non-zero elements\n", n, m, A->count );


	j=0;
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		while( e != NULL ) {
			j++;
			e=e->no;
			}
		}
	fprintf(f,"elements accessibe from row pointers: %d\n", j );

	i=0;
	for( j=0; j<m; j++ ) {
		e= A->col[j];
		while( e != NULL ) {
			i++;
			e=e->nd;
			}
		}
	fprintf(f,"elements accessibe from col pointers: %d\n", i );

	fprintf(f,"row pointer array:\n");
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		if(e == NULL ) fprintf(f,"%d NULL\n", i);
		else fprintf(f,"%d (%d, %d)\n", i, e->i, e->j);
		}
	fprintf(f,"col pointer array:\n");
	for( j=0; j<m; j++ ) {
		e= A->col[j];
		if(e == NULL ) fprintf(f,"%d NULL\n", i);
		else fprintf(f,"%d (%d, %d)\n", j, e->i, e->j);
		}
	fprintf(f,"elements\n");
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		while( e!=NULL ) {
			fprintf(f,"(%d, %d) value %f ", e->i, e->j, e->x);
			if( e->no == NULL ) fprintf(f,"no NULL ");
			else fprintf(f,"no (%d, %d) ", e->no->i, e->no->j );
			if( e->nd == NULL ) fprintf(f,"nd NULL\n");
			else fprintf(f,"nd (%d, %d)\n", e->nd->i, e->nd->j );
			e=e->no;
			}
		}
	return(0);
	}

int alloc_model_pars(char *filename)
{
	int l, n, count;
	FILE *f;
	char line[1024];

	MAX_MODEL_PARS = 0;

	if( (f=fopen(filename,"r")) == NULL ) {
		fprintf(stderr,"error: cant open model file %s\n", filename);
		exit(-1);
		}

	while( fgets(line,1024,f) != NULL ) {
		count=sscanf(line,"%d", &n);
		if( count<1 ) {
			fprintf(stderr,"error: cant read node count for model par\n", n_model_pars );
			exit(-1);
			}
		for( l=0; l<n; l++ ) {
			if( fgets(line,1024,f) == NULL ) {
				fprintf(stderr,"error: %s: cant read node %d model par %d\n",
					l, n_model_pars );
				exit(-1);
				}
			}
		MAX_MODEL_PARS++;
		}
	MAX_MODEL_PARS+=10;

	fclose(f);

	if( (model_par=(struct model_par_rec *)malloc(MAX_MODEL_PARS*sizeof(struct model_par_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (model pars)\n");
		exit(-1);
		}

	return(1);
	}

int alloc_model_pars_V(char *filename1, char *filename2)
{
	int l, n, count;
	FILE *f;
	char line[1024];

	MAX_MODEL_PARS = 0;

	if( (f=fopen(filename1,"r")) == NULL ) {
		fprintf(stderr,"error: cant open model file %s\n", filename1);
		exit(-1);
		}

	while( fgets(line,1024,f) != NULL ) {
		count=sscanf(line,"%d", &n);
		if( count<1 ) {
			fprintf(stderr,"error: cant read node count for model par\n", n_model_pars );
			exit(-1);
			}
		for( l=0; l<n; l++ ) {
			if( fgets(line,1024,f) == NULL ) {
				fprintf(stderr,"error: %s: cant read node %d model par %d\n",
					l, n_model_pars );
				exit(-1);
				}
			}
		MAX_MODEL_PARS++;
		}
	fclose(f);

	if( (f=fopen(filename2,"r")) == NULL ) {
		fprintf(stderr,"error: cant open model file %s\n", filename2);
		exit(-1);
		}

	while( fgets(line,1024,f) != NULL ) {
		count=sscanf(line,"%d", &n);
		if( count<1 ) {
			fprintf(stderr,"error: cant read node count for model par\n", n_model_pars );
			exit(-1);
			}
		for( l=0; l<n; l++ ) {
			if( fgets(line,1024,f) == NULL ) {
				fprintf(stderr,"error: %s: cant read node %d model par %d\n",
					l, n_model_pars );
				exit(-1);
				}
			}
		MAX_MODEL_PARS++;
		}
	fclose(f);

	MAX_MODEL_PARS+=10;

	if( (model_par=(struct model_par_rec *)malloc(MAX_MODEL_PARS*sizeof(struct model_par_rec))) == NULL ) {
		fprintf(stderr,"error: out of memory (model pars)\n");
		exit(-1);
		}

	return(1);
	}

int load_model_pars_V(mpfile)
FILE *mpfile;
{
	int count, l, n, i, j, k, nodeid;
	char line[1024];
	double weight;

	while( fgets(line,1024,mpfile) != NULL ) {
		count=sscanf(line,"%d", &n);
		if( count<1 ) {
			fprintf(stderr,"error: cant read node count for model par\n", n_model_pars );
			exit(-1);
			}
		if( n_model_pars > (MAX_MODEL_PARS-1) ) {
			fprintf(stderr,"error: maximum model parameters (%d) exceeds limit of %d\n", n_model_pars, MAX_MODEL_PARS-1 );
			exit(-1);
			}

		model_par[n_model_pars].n=n;
		model_par[n_model_pars].node = (int *)
			malloc((model_par[n_model_pars].n+2)*sizeof(int));
		model_par[n_model_pars].weight = (double *)
			malloc((model_par[n_model_pars].n+2)*sizeof(double));
		if(model_par[n_model_pars].node==NULL) {
			fprintf(stderr,"error: out of memory (model_par node)!\n");
			exit(-1);
			}
		if(model_par[n_model_pars].weight==NULL) {
			fprintf(stderr,"error: out of memory (model_par weight)!\n");
			exit(-1);
			}

		for( l=0; l<n; l++ ) {
			if( fgets(line,1024,mpfile) == NULL ) {
				fprintf(stderr,"error: %s: cant read node %d model par %d\n",
					l, n_model_pars );
				exit(-1);
				}
			count=sscanf(line,"%d%d%d%le", &i, &j, &k, &weight);

			if( count < 3 ) {
				fprintf(stderr,"error: cant parse model par %d\n", n_model_pars );
				exit(-1);
				}
			if( i<0 || i>=model_L || j<0 || j>=model_M || k<0 || k>=model_N ) {
				fprintf(stderr,"error: bad node (%d %d %d) in model par %d\n",
					i, j, k, n_model_pars );
				exit(-1);
				}
			nodeid=k*model_L*model_M+j*model_L+i;
			if( (model_par[n_model_pars].node[l]=findnode(nodeid))==(-1) ) {
				fprintf(stderr,"error: couldnt find node (%d %d %d) in model par %d\n",
					i, j, k, n_model_pars );
				exit(-1);
				}

			if( count<4 ) weight=1.0;
			if( weight==0.0 ) {
				fprintf(stderr,"error: weight of 0.0 in model par %d\n", n_model_pars );
				exit(-1);
				}
			model_par[n_model_pars].weight[l]=weight;

			}
		n_model_pars++;
		}

	return(1);
	}

main_VIinvert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est;
	struct triangle_rec tri;
	struct tetra_rec *pt;

	double V_thmin, V_thmax, V_azimin, V_azimax, vs;
	double I_thmin, I_thmax, I_azimin, I_azimax;

	int V_nth, V_nazi;
	int I_nth, I_nazi;

	int ng, itt, ierror, match;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping;
	int tnode[4];
	int bracket_status;
	int biec;
	double new_damping, scx;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, max_dT, qinvint;
	int iter, nx, ny, nz;

	double weight;

	char hitsfile[256];

	int Vmp, Vttimes, rinterface;
	double relative_weight;
	double ntan[3];
	double Gelem, Gmax1, Gmax2, Gmax3, Gmax4;

	Gmax1=0.0;
	Gmax2=0.0;
	Gmax3=0.0;
	Gmax4=0.0;

	if( argc!=23 ) {
		fprintf(stderr,"purpose: invert local direct and reflected traveltime data\n");
		fprintf(stderr,"for improvements to node velocities and reflector node positions\n");
		fprintf(stderr,"usage: %s starting_model_file V_model_pars_file.txt I_model_pars_file.txt\n", argv[0]);
		fprintf(stderr,"       V_traveltimes.tsv I_traveltimes.tsv max_dT damping relative_weight rinterface\n");
		fprintf(stderr,"       V_thmin V_thmax V_nth V_azimin V_azimax V_nazi\n");
		fprintf(stderr,"       I_thmin I_thmax I_nth I_azimin I_azimax I_nazi\n");
		fprintf(stderr,"       new_model_file\n");
		write_model_blerb(stderr);
		fprintf(stderr,"       input V_model_pars_file defines node velocity pertubations\n");
		write_model_pars_blerb(stderr);
		fprintf(stderr,"       input I_model_pars_file defines reflector node position pertubations\n");
		write_refl_pars_blerb(stderr);

		fprintf(stderr,"      input V_traveltimes.tsv defines direct wave traveltimes:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              T0,  origin time in s\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: (-1) indicates closest arrival\n");
		fprintf(stderr,"      input I_traveltimes.tsv defines reflected wave traveltimes\n");
		fprintf(stderr,"              (same format as V_traveltimes.tsv\n");
		fprintf(stderr,"      max_dT: if the absolute value of the traveltime residual, |dT|=|Tobs-Tpred(initial_model)|\n");
		fprintf(stderr,"              is greater than max_dt, that data is ignored\n");
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      relative_weight: relative weight of velocity and position perturbations\n");
		fprintf(stderr,"              (In other words, how much is a km/s of velocity perturbation worth\n");
		fprintf(stderr,"              relative to a km position perturbation?  The dT/dx\n");
		fprintf(stderr,"              position derivatives in the normal equations G dm = dT are\n");
		fprintf(stderr,"              multipled by relative_weight prior to the damped\n");
		fprintf(stderr,"              least-squares solution.  Thus the damping tends to\n");
		fprintf(stderr,"              drive ||dv||**2+||dx/relative_weight||**2 to zero.\n");
		fprintf(stderr,"              A relative_weigth>1 thus reduces the penalty of dx\n");
		fprintf(stderr,"              relative to dv - so large dx's are favored over\n");
		fprintf(stderr,"              large dv's\n");
		fprintf(stderr,"      rinterface: reflector interface (must match model)\n");
		fprintf(stderr,"      direct arrivals are searched for in a star with\n");
		fprintf(stderr,"         V_thmin, V_thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         V_nth, integer number of thetas in star\n");
		fprintf(stderr,"         V_azimin, V_azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         V_nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      reflected arrivals are searched for in a star with\n");
		fprintf(stderr,"         I_thmin, I_thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         I_nth, integer number of thetas in star\n");
		fprintf(stderr,"         I_azimin, I_azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         I_nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d VIinvert.\n\n");
	timestamp("at start");

	fprintf(stderr,"Reflecting interface: %s\n", argv[9] );
	fprintf(stderr,"Reflecting interface: %s\n", argv[9] );
	if( sscanf(argv[9], "%d", &rinterface) != 1 ) {
		fprintf(stderr,"error: %s: cant read rinterface from command line\n", argv[0] );
		exit(-1);
		}

	fprintf(stderr,"Starting model file: %s\n\n", argv[1] );

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load starting model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars_V(argv[2], argv[3]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open V_model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}


	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);

	Vmp=n_model_pars;

	fprintf(stderr,"%d velocity model parameters:\n", Vmp );
	if( Vmp < 1000 ) {
		for( i=0; i<Vmp; i++ ) {
			fprintf(stderr,"velocity model parameter %d has %d nodes\n", i, model_par[i].n );
			for( j=0; j<model_par[i].n; j++ ) {
				fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
				}
			}
		}

	if( (mpfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open I_model_par_file <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	while( fscanf(mpfile,"%d",&(model_par[n_model_pars].n)) == 1 ) {
		if( n_model_pars > (MAX_MODEL_PARS-1) ) {
			fprintf(stderr,"error: %s: maximum model parameters (%d) exceeds limit of %d\n", argv[0], n_model_pars, MAX_MODEL_PARS-1 );
			exit(-1);
			}
		model_par[n_model_pars].node = (int *)
			malloc((model_par[n_model_pars].n+2)*sizeof(int));
		if(model_par[n_model_pars].node==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par node)!\n");
			exit(-1);
			}
		model_par[n_model_pars].weight = (double *)
			malloc((model_par[n_model_pars].n+2)*sizeof(double));
		if(model_par[n_model_pars].weight==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par weight)!\n");
			exit(-1);
			}
		model_par[n_model_pars].t = (double *)
			malloc(3*(model_par[n_model_pars].n+2)*sizeof(double));
		if(model_par[n_model_pars].t==NULL) {
			fprintf(stderr,"error: %s: out of memory (model_par t)!\n");
			exit(-1);
			}
		for( l=0; l<model_par[n_model_pars].n; l++ ) {
			if(fscanf(mpfile,"%d%d%d%le%le%le%le", &i,&j,&k,ntan,ntan+1,ntan+2,&weight)!=7) {
				fprintf(stderr,"error: %s: cant parse model_par_file <%s>\n", argv[0], argv[2] );
				exit(-1);
				}
			if( i<0 || i>=model_L || j<0 || j>=model_M || k<0 || k>=model_N ) {
				fprintf(stderr,"error: %s: bad node (%d %d %d) in model_par_file <%s>\n", argv[0], i, j, k, argv[2] );
				exit(-1);
				}
			nodeid=k*model_L*model_M+j*model_L+i;
			if( (model_par[n_model_pars].node[l]=findnode(nodeid))==(-1) ) {
				fprintf(stderr,"error: %s: couldnt find node (%d %d %d)\n", argv[0], i, j, k );
				exit(-1);
				}
			if( node[model_par[n_model_pars].node[l]].ifc != rinterface ) {
				fprintf(stderr,"error: %s: node (%d %d %d) not on reflector interface %d\n",
				argv[0], i, j, k, rinterface );
				exit(-1);
				}
			if( weight==0.0 ) {
				fprintf(stderr,"error: %s: model parameter %d has 0 weight\n", argv[0], n_model_pars );
				exit(-1);
				}
			model_par[n_model_pars].weight[l]=weight;
			unit_vector(ntan);
			for(i=0;i<3;i++) model_par[n_model_pars].t[3*l+i]=ntan[i];
			}
		n_model_pars++;
		}

	fclose(mpfile);
	fprintf(stderr,"%d reflector node position model parameters:\n", n_model_pars-Vmp );
	if( (n_model_pars-Vmp) < 1000 ) {
		for( i=Vmp; i<n_model_pars; i++ ) {
			fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
			for( j=0; j<model_par[i].n; j++ ) {
				fprintf(stderr,"%d (%f %f %f) %f\n", model_par[i].node[j],
					model_par[i].t[3*j], model_par[i].t[3*j+1], model_par[i].t[3*j+2],
					model_par[i].weight[j]);
				}
			}
		}

	make_ttime2(argv[4],argv[5]);

	if( (ttfile=fopen(argv[4],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open V_traveltime.tsv <%s> for read\n", argv[0], argv[4] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in V_traveltimess.tsv <%s>\n",
			argv[0], argv[4]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Direct Wave Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);
	Vttimes=n_ttimes;

	if( (ttfile=fopen(argv[5],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open I_traveltime.tsv <%s> for read\n", argv[0], argv[5] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in V_traveltimess.tsv <%s>\n",
			argv[0], argv[5]);
		exit(-1);
		}

	fprintf(stderr,"\nInput Reflected Wave Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");	
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);


	fprintf(stderr,"Total of %d data (%d direct, %d reflected)\n\n", n_ttimes, Vttimes, n_ttimes-Vttimes);

	if( sscanf(argv[6], "%le", &max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read max_dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &relative_weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read relative_weight from command line\n", argv[0] );
		exit(-1);
		}

	fprintf(stderr,"max_dT %f  damping %f relative_weight %f\n", max_dT, damping, relative_weight );

	if( sscanf(argv[10], "%le", &V_thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%le", &V_thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%d", &V_nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[13], "%le", &V_azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[14], "%le", &V_azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[15], "%d", &V_nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read V_nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[16], "%le", &I_thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[17], "%le", &I_thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[18], "%d", &I_nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[19], "%le", &I_azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[20], "%le", &I_azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[21], "%d", &I_nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read I_nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[22],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[22] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[22] );

	G=new_sparse_matrix(n_ttimes,n_model_pars,0);
	d=(double*)malloc((n_ttimes+2)*sizeof(double));
	ts=(double*)malloc((n_model_pars+2)*sizeof(double));
	ts2=(double*)malloc((n_ttimes+2)*sizeof(double));
	GTd=(double*)malloc((n_model_pars+2)*sizeof(double));
	dm=(double*)malloc((n_model_pars+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (GTd==NULL) ||
		(ts==NULL) || (ts2==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting Point to Point Ray tracing of data\n\n");
	timestamp("at start of raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	/* direct wave traveltimes */
	for( itt=0; itt<Vttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, V_thmin, V_thmax, V_nth, V_azimin, V_azimax, V_nazi );
		shoot_star(V_thmin, V_thmax, V_nth, V_azimin, V_azimax, V_nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime*ttime[itt].ratio;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	/* this section handles the partial derivative of direct wave */
	/* traveltime with respect to velocity model parameters       */
	for( j=0; j<Vmp; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			Gelem=dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio;
			set_sparse_value( G, ng, j, Gelem);
			Gelem=fabs(Gelem);if(Gmax1<Gelem)Gmax1=Gelem;
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	/* this section handles the partial derivative of direct wave */
	/* traveltime with respect to interface model parameters       */
	for( j=Vmp; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			Gelem=dTdmprd(j,interface,rinterface,x0,t0,x1,t1,arc,n_arcs,
				traveltime,ng) * ttime[itt].ratio;
			set_sparse_value( G, ng, j, Gelem);
			Gelem=fabs(Gelem);if(Gmax2<Gelem)Gmax2=Gelem;
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next direct traveltime */

	/* direct wave traveltimes */
	for( itt=Vttimes; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, I_thmin, I_thmax, I_nth, I_azimin, I_azimax, I_nazi );
		shoot_star_refl(I_thmin, I_thmax, I_nth, I_azimin, I_azimax, I_nazi, x0, interface, rinterface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox_refl(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, rinterface, NULL) ) {
		bracket_status=TRUE;
		if( refine_refl( x0, xsta, ysta, interface, rinterface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface_refl( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, rinterface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime*ttime[itt].ratio;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	/* this section handles the partial derivative of reflected wave */
	/* traveltime with respect to velocity model parameters          */
	for( j=0; j<Vmp; j++ ) {
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			Gelem=dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio;
			set_sparse_value( G, ng, j, Gelem );
			Gelem=fabs(Gelem);if(Gmax3<Gelem)Gmax3=Gelem;
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	/* this section handles the partial derivative of reflected wave */
	/* traveltime with respect to interface model parameters          */
	for( j=Vmp; j<n_model_pars; j++ ) {
		/* Derivatives with respect to reflector node positions */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			Gelem=dTdmpr(j,interface,rinterface,x0,t0,x1,t1,arc,n_arcs,traveltime,ng)
				* ttime[itt].ratio * relative_weight;
			set_sparse_value( G, ng, j, Gelem );
			Gelem=fabs(Gelem);if(Gmax4<Gelem)Gmax4=Gelem;
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of raytracing");
	
	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"The four submatrices in G have absolute max of %f %f %f %f\n",
		Gmax1, Gmax2, Gmax3, Gmax4);
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	timestamp("after forming G and d");
	
	fprintf(stderr,"Damping: %e\n\n", damping );
	
	smt_times_v( G, d, GTd );

	maxd=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars, maxd );

	if( EXPLICIT_GTG ) {
	
	invert_GTG=smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);


	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_model_pars, n_model_pars,
	100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );


	timestamp("after forming GTG");

	for( i=0; i<n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion (explicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after solving linear system");
	
	if( n_model_pars < 1000 ) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	sm_times_v( invert_GTG, dm, ts );
	for( i=0; i<n_model_pars; i++ ) {
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
                if( n_model_pars < 1000 ) fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);
	
	} /* end of explicit GTG */
	else {
		scx = ATAabsmax( G );
		new_damping = damping*scx;
		fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
			damping, scx, new_damping );
		for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	
	} /* end of implicit solution */

	if( n_model_pars < 1000 ) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}

	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	sm_times_v( G, dm, ts2 );

	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* update velocity of nodes */
	for( i=0; i<Vmp; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += weight*dm[i];
			}
		}

	/* update position of nodes */
	for( i=Vmp; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			for( l=0; l<3; l++ )
				node[k].x[l] +=
					dm[i]*model_par[i].weight[j]*model_par[i].t[3*j+l]*relative_weight;
			}
		}

	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);
	timestamp("final");

		
	exit(0);
	}

int make_ttime2(char *filename1, char *filename2)
{
	int n;

	MAX_TTIMES=count_lines(filename1)+count_lines(filename2);
	if( MAX_TTIMES==0 ) {
		ttime==NULL;
		}
	else {
		MAX_TTIMES+=10;
		ttime=(struct ttime_rec *)malloc(MAX_TTIMES*sizeof(struct ttime_rec));
		}
	if( ttime==NULL ) {
		fprintf(stderr,"error: out of memory (ttime array)\n");
		exit(-1);
		}
	return(1);
	}

main_psinvert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], stheta, ftheta, dtheta, theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime, rayx[8000], rayy[8000], rayz[8000];
	int tindex[8000];
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta, th_est, az_est;
	struct triangle_rec tri;
	struct tetra_rec *pt;
	struct model_par_rec *ratio_model_par, *P_model_par;
	int ratio_n_model_pars, P_n_model_pars;
	struct ttime_rec *P_ttime, *S_ttime;
	int P_n_ttimes, S_n_ttimes;

	double thmin, thmax, azimin, azimax, vs;
	int nth, nazi;
	int ng, itt, ierror, match;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping;
	int tnode[4];
	int bracket_status;
	int total_n_model_pars, total_n_ttimes;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, qinvint;
	double P_max_dT, S_max_dT;
	int iter, nx, ny, nz;

	double weight, relative_weight;
	int biec;

	char hitsfile[256];
	double new_damping, scx;


	if( argc!=17 ) {
		fprintf(stderr,"purpose: invert local P & S traveltime data\n");
		fprintf(stderr,"         for improvements to node P velocities and P:S velocity ratio\n");
		fprintf(stderr,"usage: %s starting_model_file P_model_pars_file.txt ratio_model_pars_file.txt P_traveltimes.tsv S_traveltimes.tsv P_max_dT S_max_dT damping relative_weight thmin thmax nth azimin azimax nazi new_model_file\n",argv[0]);
		fprintf(stderr,"      Note: model file must define both P velocity and P:S velocity ratio\n");
		write_model_blerb(stderr);
		fprintf(stderr,"      Note: Two sets of model parameters, one for P-velocities, one for velocity ratios\n");
		write_model_pars_blerb(stderr);
		fprintf(stderr,"      Note: Two sets of traveltimes.  The ratio field is ignored\n");
		fprintf(stderr,"      input traveltimes.tsv:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              T0,  origin time in s\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"                   (but this field is ignored)\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: (-1) indicates closest arrival\n");
		fprintf(stderr,"                 Note: both first ariving P, S have order 0\n");
		fprintf(stderr,"      P_max_dT, S_max_dt: if the absolute value of the traveltime residual, |dT|=|Tobs-Tpred(initial_model)|\n");
		fprintf(stderr,"              is greater than max_dt, that data is ignored\n");
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      relative_weight: of velocity and ratio purturbations\n");
		fprintf(stderr,"              (In other words, how much is a km/s of velocity perturbation, dVp worth\n");
		fprintf(stderr,"              relative to a P:S ratio perturbation, d-ratio?  The dTs/d-ratio\n");
		fprintf(stderr,"              derivatives in the normal equations G dm = dT are\n");
		fprintf(stderr,"              multipled by relative_weight prior to the damped\n");
		fprintf(stderr,"              least-squares solution.  Thus the damping tends to\n");
		fprintf(stderr,"              drive ||dvP||**2+||d-ratio/relative_weight||**2 to zero.\n");
		fprintf(stderr,"              A relative_weigth>1 thus reduces the penalty of d-ratio\n");
		fprintf(stderr,"              relative to dvP - so large d-ratios's are favored over\n");
		fprintf(stderr,"              large dvP's\n");
		fprintf(stderr,"      arrivals are searched for in a star with\n");
		fprintf(stderr,"         thmin, thmax, min, max angles of incidence, (0-180 deg)\n");
		fprintf(stderr,"         nth, integer number of thetas in star\n");
		fprintf(stderr,"         azimin, azimax, azimuth range (0-360 deg)\n");
		fprintf(stderr,"         nazi, integer number of azimuths in star\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d PSinvert. Starting model file: %s\n\n", argv[1] );

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	model_par=NULL; n_model_pars=0;
	alloc_model_pars(argv[2]);
	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open P_model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	load_model_pars_V(mpfile);
	fclose(mpfile);
	P_model_par=model_par;
	P_n_model_pars=n_model_pars;
	fprintf(stderr,"P model parameters from file %s:\n", argv[2]);
	if( P_n_model_pars < 1000 ) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
			}
		}
	fprintf(stderr,"\n\n");

	model_par=NULL; n_model_pars=0;
	alloc_model_pars(argv[3]);
	if( (mpfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open ratio_model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	load_model_pars_V(mpfile);
	fclose(mpfile);
	ratio_model_par=model_par;
	ratio_n_model_pars=n_model_pars;
	fprintf(stderr,"ratio model parameters from file %s:\n", argv[3]);
	if( ratio_n_model_pars < 1000 ) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
			}
		}
	fprintf(stderr,"\n\n");

	model_par=P_model_par; n_model_pars=P_n_model_pars;

	make_ttime(argv[4]);
	if( (ttfile=fopen(argv[4],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open P_traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput P Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);
	fprintf(stderr,"Total of %d P data\n\n", n_ttimes);
	P_ttime = ttime; P_n_ttimes=n_ttimes;

	make_ttime(argv[5]);
	if( (ttfile=fopen(argv[5],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open S_traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput S Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);
	fprintf(stderr,"Total of %d P data\n\n", n_ttimes);
	S_ttime = ttime; S_n_ttimes=n_ttimes;

	ttime=P_ttime; n_ttimes=P_n_ttimes;

	if( sscanf(argv[6], "%le", &P_max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read P_max_dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &S_max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read S_max_dT from command line\n", argv[0] );
		exit(-1);
		}


	if( sscanf(argv[8], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &relative_weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read relative_weight from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[13], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[14], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[15], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( (nmfile=fopen(argv[16],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[16] );

	total_n_model_pars = P_n_model_pars+ratio_n_model_pars;
	total_n_ttimes = P_n_ttimes+S_n_ttimes;
	G=new_sparse_matrix(total_n_ttimes,total_n_model_pars,0);
	d=(double*)malloc((total_n_ttimes+2)*sizeof(double));
	ts=(double*)malloc((total_n_model_pars+2)*sizeof(double));
	ts2=(double*)malloc((total_n_ttimes+2)*sizeof(double));
	GTd=(double*)malloc((total_n_model_pars+2)*sizeof(double));
	dm=(double*)malloc((total_n_model_pars+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (GTd==NULL) ||
		(ts==NULL) || (ts2==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	fprintf(stderr,"Starting P-wave Point to Point Ray tracing of data\n\n");
	timestamp("at start of P-wave raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	ttime=P_ttime; n_ttimes=P_n_ttimes;
	model_par=P_model_par; n_model_pars=P_n_model_pars;

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, 1.0, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmp(j,arc,n_arcs,traveltime) );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}
	/* note also that the block of G corresponding to ratio model pars is zero */

	if( fabs(d[ng]) > P_max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], P_max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of P raytracing");
	fprintf(stderr,"\n");

	fprintf(stderr,"Starting S-wave Point to Point Ray tracing of data\n\n");
	timestamp("at start of S-wave raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	ttime=S_ttime; n_ttimes=S_n_ttimes;

	/* Change to S model */
	for( i=0; i<n_nodes; i++ ) node[i].v /= node[i].ratio;
	for( i=0; i<n_tetras; i++ ) velocity_eqn( &tetra[i] );

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, 1.0, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	/* first the dTs/dVp Frechet derivatives */
	model_par=P_model_par; n_model_pars=P_n_model_pars;
	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTsdmpp(j,arc,n_arcs,traveltime) );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	/* next the dTs/d-ratio Frechet derivatives */
	model_par=ratio_model_par; n_model_pars=ratio_n_model_pars;
	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j+P_n_model_pars,
				relative_weight*dTsdmpr(j,arc,n_arcs,traveltime) );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > S_max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], S_max_dT );
		for( j=0; j<total_n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of S raytracing");
	
	if( ng<100 && total_n_model_pars<6 ) {
		fprintf(stderr,"\nG, d:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<total_n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, total_n_model_pars, 100.0*(double)k/((double)ng*(double)total_n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	timestamp("after forming G and d");
	fprintf(stderr,"Damping: %e Relative weight: %e\n\n", damping, relative_weight );

	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<total_n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n\n",
	total_n_model_pars, maxd );
	
	if( EXPLICIT_GTG ) {

	invert_GTG=smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	k=sparse_count(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	total_n_model_pars, total_n_model_pars,
	100.0*(double)k/((double)total_n_model_pars*(double)total_n_model_pars), maxG );


	timestamp("after forming GTG ");

	for( i=0; i<total_n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,total_n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion (explicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after solving linear system");
	
	if( total_n_model_pars < 1000 ) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	sm_times_v( invert_GTG, dm, ts );
	for( i=0; i<total_n_model_pars; i++ ) {
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
                if( total_n_model_pars < 1000 ) {
                	fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
                		}
		}
	rms = sqrt(rms/(double)total_n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);

	
	} /* end explicit GTG */
	else {
		scx = ATAabsmax( G );
		new_damping = damping*scx;
		fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );
		for( i=0; i<total_n_model_pars; i++ ) dm[i]=0.0;
		if( !sparse_dls_biconj(G,d,dm,new_damping,1,maxd*B_PREC,total_n_model_pars+10,&iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	} /* end implicit GTG */

	if( total_n_model_pars < 1000 ) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<total_n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}

	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	sm_times_v( G, dm, ts2 );

	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* reset nodes to original values */
	for( i=0; i<n_nodes; i++ ) {
		node[i].v=node[i].v_orig;
		node[i].ratio=node[i].ratio_orig;
		}

	/* update P velocity of nodes */
	model_par=P_model_par; n_model_pars=P_n_model_pars;
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += weight*dm[i];
			}
		}

	/* update P:S ratio of nodes */
	model_par=ratio_model_par; n_model_pars=ratio_n_model_pars;
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].ratio += relative_weight*weight*dm[i+P_n_model_pars];
			}
		}

	/* reset nodal qinv, since that's what's written out */
	for( i=0; i<n_nodes; i++ ) {
		node[i].qinv=1.0/node[i].ratio;
		}

	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);
	timestamp("final");

		
	exit(0);
	}


#define MAX_WAVEFRONTS (10000)
main_tt_table(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, n_arcs, n, status, n_azis, n_thetas, ec;
	int n1, n2, n3, n4, n5, n6, n7, n8;
	double x0[3], t0[3], x1[3], t1[3], v[4];
	double stheta, ftheta, dtheta, theta, qinvint;
	double sazi, fazi, dazi, azi;
	double traveltime, max_traveltime, dT, te, tb;
	struct arc_rec arc[MAX_ARCS];
	float X[MAX_WAVEFRONTS], Y[MAX_WAVEFRONTS], Z[MAX_WAVEFRONTS];
	double T[MAX_WAVEFRONTS];
	int TE[MAX_WAVEFRONTS], iarc, iwave, l, method;
	double r[3], r2[3], psi1, psi2, psi, Tprev;

	int do_P;

	if( (argc!=13) && (argc!=14) ) {
		fprintf(stderr,"purpose: constructs station-specific traveltime table file\n");
		fprintf(stderr,"	 (The table is organized by rays radiating out from)\n");
		fprintf(stderr,"	 the station at constant azimuths and angles of incidence)\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z theta_min theta_max n_thetas azi_min azi_max n_azis dT max_traveltime > tt_table_file\n",argv[0]);
		fprintf(stderr,"or\n");
		fprintf(stderr,"usage: %s model_file source_x source_y source_z theta_min theta_max n_thetas azi_min azi_max n_azis dT max_traveltime P_S_flag > tt_table_file\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      source_x, source_y, source_z in km\n");
		fprintf(stderr,"      theta_min, theta_max, 0-180 degrees from +z\n");
		fprintf(stderr,"      azi_min, azi_max, degrees from +x to +y\n");
		fprintf(stderr,"          azi_min<azi_max, but values > 360 allowed\n");
		fprintf(stderr,"      n_thetas, n_azis, integers, control the number of rays\n");
		fprintf(stderr,"      dT between successive wavefronts in s\n");
		fprintf(stderr,"      max_traveltime in s\n");
		fprintf(stderr,"      P_S_flag: P (default) or S traveltimes\n");
		fprintf(stderr,"      the tt_table_file is a binary file\n");
		fprintf(stderr,"         (other keywords use it, no provision is made for\n");
		fprintf(stderr,"         external use)\n");
		exit(-1);
		}

	if( argc==13 ) {
		do_P=1;
		}
	else if( (argv[13][0]=='P') || (argv[13][0]=='p') ) {
		do_P=1;
		}
	else if( (argv[13][0]=='S') || (argv[13][0]=='s') ) {
		do_P=0;
		}
	else {
		fprintf(stderr,"error: %s: bad P_S_flag (must be P or S)\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	/* Change to S model */
	if( !do_P ) {
		if( !QDEFINED ) {
			fprintf(stderr,"error: %s: velocity model must define ratio\n", argv[0], argv[1] );
			exit(-1);
			}
		for( i=0; i<n_nodes; i++ ) node[i].v =  node[i].v_orig/node[i].ratio_orig;
		for( i=0; i<n_tetras; i++ ) velocity_eqn( &tetra[i] );
		}

	if( sscanf(argv[2], "%le", &x0[0]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_x from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[3], "%le", &x0[1]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_y from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[4], "%le", &x0[2]) != 1 ) {
		fprintf(stderr,"error: %s: cant read source_z from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[5], "%le", &stheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &ftheta) != 1 ) {
		fprintf(stderr,"error: %s: cant read theta_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%d", &n_thetas) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_thetas from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &sazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_min from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &fazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read azi_max from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &n_azis) != 1 ) {
		fprintf(stderr,"error: %s: cant read n_azis from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%le", &dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%le", &max_traveltime) != 1 ) {
		fprintf(stderr,"error: %s: cant read max_traveltime from command line\n", argv[0] );
		exit(-1);
		}

	dtheta = (ftheta-stheta)/(double)(n_thetas-1);
	dazi = (fazi-sazi)/(double)(n_azis-1);

	fwrite( x0, sizeof(double), 3, stdout );

	fwrite( &stheta, sizeof(double), 1, stdout );
	fwrite( &ftheta, sizeof(double), 1, stdout );
	fwrite( &dtheta, sizeof(double), 1, stdout );
	fwrite( &n_thetas, sizeof(int), 1, stdout );

	fwrite( &sazi, sizeof(double), 1, stdout );
	fwrite( &fazi, sizeof(double), 1, stdout );
	fwrite( &dazi, sizeof(double), 1, stdout );
	fwrite( &n_azis, sizeof(int), 1, stdout );

	fwrite( &dT, sizeof(double), 1, stdout );

	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		azi=sazi+dazi*(double)j;
		theta=stheta+dtheta*(double)k;
		ray_tangent(theta,azi,t0);
		n_arcs=0.0;
		status=shoot_with_max_traveltime( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, max_traveltime, &traveltime, &qinvint, &ec );
		for( i=1; i<n_arcs; i++ ) arc[i].T+=arc[i-1].T;
		if( status ) {
			X[0]=x0[0]; Y[0]=x0[1]; Z[0]=x0[2]; T[0]=0.0; iarc=0, iwave=1;
			do {
				T[iwave]=T[iwave-1]+dT;
				if( T[iwave] > arc[n_arcs-1].T ) {
					break;
					}
				while( iarc<n_arcs ) if( T[iwave]>arc[iarc].T ) iarc++; else break;
				if( iarc > 0 ) Tprev=arc[iarc-1].T; else Tprev=0.0;
				te = (T[iwave]-Tprev)
				   + ttintegral(0.0, arc[iarc].ptetra->gs, arc[iarc].R,
					 arc[iarc].psi1, &method);

				psi = inv_integral294a( arc[iarc].ptetra->gs*te );

                       		r[0] = arc[iarc].R*sin(psi);
                       		r[1] = 0.0;
                        	r[2] = arc[iarc].R*cos(psi);

				/* untransform */
				for( l=0; l<3; l++ ) r[l] += arc[iarc].shift[l];
				rotate3(r,arc[iarc].st,r2);

				X[iwave] = r2[0];
				Y[iwave] = r2[1];
				Z[iwave] = r2[2];
				TE[iwave] = arc[iarc].ptetra->index;

				if( (iwave+1) > MAX_WAVEFRONTS ) {
					fprintf(stderr,"Error: %s: too many wavefronts\n", argv[0] );
					exit(-1);
					}
				iwave++;
				} while( T[iwave]<=max_traveltime );
			fwrite( &j, sizeof(int), 1, stdout );
			fwrite( &k, sizeof(int), 1, stdout );
			fwrite( &iwave, sizeof(int), 1, stdout );
			fwrite( X, sizeof(float), iwave, stdout );
			fwrite( Y, sizeof(float), iwave, stdout );
			fwrite( Z, sizeof(float), iwave, stdout );
			for( i=0; i<n_tetras; i++ ) tetra[i].marked=0;
			for( i=0; i<n_arcs; i++ ) arc[i].ptetra->marked=1;
			l=0; for( i=0; i<n_tetras; i++ ) if( tetra[i].marked ) l++;
			fwrite( &l, sizeof(int), 1, stdout );
			for( i=0; i<n_tetras; i++ ) if( tetra[i].marked ) {
				fwrite( &i, sizeof(int), 1, stdout );
				}
			}
		else {
			iwave=0;
			fwrite( &j, sizeof(int), 1, stdout );
			fwrite( &k, sizeof(int), 1, stdout );
			fwrite( &iwave, sizeof(int), 1, stdout );
			fprintf(stderr,"warning: lost ray (%d %d) error code (%d)\n", j, k, ec );
			}
		}

	exit(0);
	}

int cmpint(a, b)
int *a, *b;
{
  if( (*a) <  (*b) ) return(-1);
  else if( (*a) == (*b) ) return(0);
  else return(1);
  }

main_tt_surface(argc, argv)
int argc;
char *argv[];
{

	FILE *f;
	int i, j, k, count, iwave, nTT;
	double TT;
	double x0[3], stheta, ftheta, dtheta, sazi, fazi, dazi, dT;
	int n_thetas, n_azis;


	struct jhd_table_rec {
		double azi, theta;
		float *X, *Y, *Z;
		int iwave, *tlist, nt, hit;
		};
	struct jhd_table_rec *jhd;

	if( argc!=3 ) {
		fprintf(stderr,"purpose: compute travetime surface \n");
		fprintf(stderr,"method: interrogates tt_table file\n");
		fprintf(stderr,"usage: tt_table_file traveltime > tt_surface.txt\n", argv[0]);
		fprintf(stderr,"        traveltime: in seconds\n");
		fprintf(stderr,"        note: rounds off to whatever accuracy is in tt_table\n");
		fprintf(stderr,"        tt_surrface.txt file: header line followed by\n");
		fprintf(stderr,"           one (x, y, z, theta, azi) per line\n");
		exit(-1);
		}

	if( (f=fopen(argv[1],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open tt_table file <%s> for read\n",
			argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2],"%lf",&TT) !=1 ) {
		fprintf(stderr,"error: %s: cant read traveltime from cmd line\n",
			argv[0] );
		exit(-1);
		}

	count=0;
	count+=fread( x0, sizeof(double), 3, f );

	count+=fread( &stheta, sizeof(double), 1, f );
	count+=fread( &ftheta, sizeof(double), 1, f );
	count+=fread( &dtheta, sizeof(double), 1, f );
	count+=fread( &n_thetas, sizeof(int), 1, f );

	count+=fread( &sazi, sizeof(double), 1, f );
	count+=fread( &fazi, sizeof(double), 1, f );
	count+=fread( &dazi, sizeof(double), 1, f );
	count+=fread( &n_azis, sizeof(int), 1, f );

	count+=fread( &dT, sizeof(double), 1, f );

	if( count!=12 ) {
		fprintf(stderr,"error: %s: cant read tt_table file (1)\n", argv[0]);
		exit(-1);
		}

	nTT = (int) (0.5+TT/dT);
	if( nTT<0 ) {
		fprintf(stderr,"error: %s: negative traveltime not allowed\n", argv[0]);
		exit(-1);
		}

	jhd = (struct jhd_table_rec *) malloc( n_thetas * n_azis * sizeof(struct jhd_table_rec) );
	if( jhd==NULL ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		jhd[j+k*n_azis].iwave=0; jhd[j+k*n_azis].nt=0; jhd[j+k*n_azis].tlist=NULL;
		jhd[j+k*n_azis].X=NULL; jhd[j+k*n_azis].Y=NULL; jhd[j+k*n_azis].Z=NULL;
		}

	while(1) {
		count=0;
		count+=fread( &j, sizeof(int), 1, f );
		count+=fread( &k, sizeof(int), 1, f );
		count+=fread( &iwave, sizeof(int), 1, f );
		if( count == 0 ) break;
		if( iwave == 0 ) continue;
		if( count != 3 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (2)\n", argv[0] );
			exit(-1);
			}
		jhd[j+k*n_azis].azi=sazi+dazi*(double)j;
		jhd[j+k*n_azis].theta=stheta+dtheta*(double)k;
		jhd[j+k*n_azis].iwave=iwave;

		jhd[j+k*n_azis].X = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Y = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Z = (float *) malloc( sizeof(float)*iwave );

		if((jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)){
			fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( jhd[j+k*n_azis].X, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Y, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Z, sizeof(float), iwave, f );
		if( count != (3*iwave) ) {
			fprintf(stderr,"error: %s: cant read tt_table file (3)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( &(jhd[j+k*n_azis].nt), sizeof(int), 1, f );
		if( count != 1 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (4)\n", argv[0] );
			exit(-1);
			}

		jhd[j+k*n_azis].tlist = (int *) malloc( sizeof(int)*jhd[j+k*n_azis].nt );
		if( jhd[j+k*n_azis].tlist == NULL ) {
			fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
			exit(-1);
			}
		count=0;
		count+=fread( jhd[j+k*n_azis].tlist, sizeof(int), jhd[j+k*n_azis].nt, f );
		if( count != jhd[j+k*n_azis].nt ) {
			fprintf(stderr,"error: %s: cant read tt_table file (5)\n", argv[0] );
			exit(-1);
			}
		}
	fclose(f);

	printf("x\ty\tz\ttheta\tazi\n");
	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		iwave = jhd[j+k*n_azis].iwave;
		if( iwave>nTT ) printf("%f\t%f\t%f\t%f\t%f\n",
			jhd[j+k*n_azis].X[nTT],
			jhd[j+k*n_azis].Y[nTT],
			jhd[j+k*n_azis].Z[nTT],
			stheta+dtheta*(double)k, 
			sazi+dazi*(double)j );
		}

	exit(0);
	}

main_jhd_table(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, m, count, iwave, t, ic, quick;
	struct jhd_table_rec {
		double azi, theta;
		float *X, *Y, *Z;
		int iwave, *tlist, nt, hit;
		};
	struct jhd_table_rec *jhd;

	double x0[3], stheta, ftheta, dtheta, sazi, fazi, dazi, dT;
	int n_thetas, n_azis, iwavemin, iwavemax;

	FILE *f;

	double x1[3], v;
	int nev;

	int r1, r2, r3;
	double b1[3], b2[3], b3[3], t1[3], t2[3], t3[3], T, dTdx[3], Tb, Tt, Tmin, Tmax;
	double rayazi[3], raytheta[3], azi, theta;

	int *cindex, **jindex, **kindex, **lindex, **iindex, buf[4];
	int *itmp, *jtmp, *ktmp, *ltmp, doazth;

	double newazi, newtheta, newT, newdTdx[3];

	int do_P;

	if( !((argc==4) || (argc==5) || (argc==6) ) ) {
		fprintf(stderr,"purpose: output information for joint hypocenter relocation\n");
		fprintf(stderr,"method: Uses a station-specific traveltime table file to construct an\n");
		fprintf(stderr,"        ensemble of ray tubes extending outward from that station.\n");
		fprintf(stderr,"        Finds all the ray tubes that enclose a given station,\n");
		fprintf(stderr,"        and estimates the traveltime and its derivative by\n");
		fprintf(stderr,"        linear interpolation within the ray tube.\n");
		fprintf(stderr,"usage: %s model_file tt_table_file accuracy_flag < events.txt > jhd_table.txt\n", argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"       %s model_file tt_table_file accuracy_flag index_file < events.txt > jhd_table.txt\n", argv[0]);
		fprintf(stderr,"       or\n");
		fprintf(stderr,"       %s model_file tt_table_file accuracy_flag index_file P_S_flag < events.txt > jhd_table.txt\n", argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"       tt_table_file, for a specific station\n");
		fprintf(stderr,"       accuracy_flag, e for exhaustice search (all ray tubes\n");
		fprintf(stderr,"                        in the tt_file are examined to see if\n");
		fprintf(stderr,"                        they contain the event)\n");
		fprintf(stderr,"                      i for indexed search, where the index_file\n");
		fprintf(stderr,"                        has been computed separately via the\n");
		fprintf(stderr,"                        index_traveltime keyword\n");
		fprintf(stderr,"                        (i prefered over e only when there are a hunge number\n");
		fprintf(stderr,"                        of events, since computation of the index file\n");
		fprintf(stderr,"                        is time-consuming)\n");
		fprintf(stderr,"                      q for quick search that may miss arrivals\n");
		fprintf(stderr,"                        (The quick search uses the tetrahedra of\n");
		fprintf(stderr,"                        the model as a hash index, and will fail\n");
		fprintf(stderr,"                        if none of the three rays that constitute\n");
		fprintf(stderr,"                        the ray tube touch the tetrahedron containing\n");
		fprintf(stderr,"                        the event.)\n");
		fprintf(stderr,"       index_file: file produced by index_traveltime keyword\n");
		fprintf(stderr,"       		(include only for accuracy_flag = i)\n");
		fprintf(stderr,"       P_S_flag: P (default) for P traveltimes, S for S\n");
		fprintf(stderr,"                (model must derine ratio for S wave times\n");
		fprintf(stderr,"       events.txt file: one event per line\n");
		fprintf(stderr,"		event_no: arbitrary integer event number\n");
		fprintf(stderr,"		event_x: event x position, km\n");
		fprintf(stderr,"		event_y: event y position, km\n");
		fprintf(stderr,"		event_z: event z position, km\n");
		fprintf(stderr,"		Tmin: minimum traveltime bound, s\n");
		fprintf(stderr,"		Tmax: maximum traveltime bound, s\n");
		fprintf(stderr,"       jdh_table.txt file: one traveltime per line\n");
		fprintf(stderr,"		event_no: arbitrary integer event number\n");
		fprintf(stderr,"		arrival_no: 0=first arrival\n");
		fprintf(stderr,"		event_x: event x position, km\n");
		fprintf(stderr,"		event_y: event y position, km\n");
		fprintf(stderr,"		event_z: event z position, km\n");
		fprintf(stderr,"		T: traveltime in s\n");
		fprintf(stderr,"		dT/dx: derivative in s/km\n");
		fprintf(stderr,"		dT/dy: derivative in s/km\n");
		fprintf(stderr,"		dT/dz: derivative in s/km\n");
		fprintf(stderr,"		v: velocity at (x,y,z) in m/s\n");
		fprintf(stderr,"		tetra: tetrahedron index\n");
		fprintf(stderr,"		theta: takeoff angle of incidence in deg\n");
		fprintf(stderr,"		azimuth: takeoff azimuth in deg\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( (f=fopen(argv[2],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open tt_table file <%s> for read\n",
			argv[0], argv[1] );
		exit(-1);
		}

	if( (argv[3][0]=='q') || (argv[3][0]=='Q') )  quick=1;
	else if( (argv[3][0]=='e') || (argv[3][0]=='E') )  quick=0;
	else if( (argv[3][0]=='i') || (argv[3][0]=='I') )  quick=2;
	else {
		fprintf(stderr,"error: %s: bad accuracy_flag\n", argv[0] );
		}

	if( quick!=2 ) {
		if( argc<5 ) do_P=1;
		else if( (argv[4][0]=='P') || (argv[4][0]=='p') ) do_P=1;
		else if( (argv[4][0]=='S') || (argv[4][0]=='s') ) do_P=0;
		else {
			fprintf(stderr,"error: %s: bad P_S_flag (must be P or S)\n", argv[0] );
			exit(0);
			}
		}
	else {
		if( argc<6 ) do_P=1;
		else if( (argv[5][0]=='P') || (argv[5][0]=='p') ) do_P=1;
		else if( (argv[5][0]=='S') || (argv[5][0]=='s') ) do_P=0;
		else {
			fprintf(stderr,"error: %s: bad P_S_flag (must be P or S)\n", argv[0] );
			exit(0);
			}
		}

	if( !do_P ) {
		for( i=0; i<n_nodes; i++ ) node[i].v = node[i].v_orig/node[i].ratio_orig;
		for( i=0; i<n_tetras; i++ ) velocity_eqn( &tetra[i] );
		}

	count=0;
	count+=fread( x0, sizeof(double), 3, f );

	count+=fread( &stheta, sizeof(double), 1, f );
	count+=fread( &ftheta, sizeof(double), 1, f );
	count+=fread( &dtheta, sizeof(double), 1, f );
	count+=fread( &n_thetas, sizeof(int), 1, f );

	count+=fread( &sazi, sizeof(double), 1, f );
	count+=fread( &fazi, sizeof(double), 1, f );
	count+=fread( &dazi, sizeof(double), 1, f );
	count+=fread( &n_azis, sizeof(int), 1, f );

	count+=fread( &dT, sizeof(double), 1, f );

	if( count!=12 ) {
		fprintf(stderr,"error: %s: cant read tt_table file (1)\n", argv[0]);
		exit(-1);
		}

	jhd = (struct jhd_table_rec *) malloc( n_thetas * n_azis * sizeof(struct jhd_table_rec) );
	if( jhd==NULL ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
		jhd[j+k*n_azis].iwave=0; jhd[j+k*n_azis].nt=0; jhd[j+k*n_azis].tlist=NULL;
		jhd[j+k*n_azis].X=NULL; jhd[j+k*n_azis].Y=NULL; jhd[j+k*n_azis].Z=NULL;
		}

	while(1) {
		count=0;
		count+=fread( &j, sizeof(int), 1, f );
		count+=fread( &k, sizeof(int), 1, f );
		count+=fread( &iwave, sizeof(int), 1, f );
		if( count == 0 ) break;
		if( iwave == 0 ) continue;
		if( count != 3 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (2)\n", argv[0] );
			exit(-1);
			}
		jhd[j+k*n_azis].azi=sazi+dazi*(double)j;
		jhd[j+k*n_azis].theta=stheta+dtheta*(double)k;
		jhd[j+k*n_azis].iwave=iwave;

		jhd[j+k*n_azis].X = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Y = (float *) malloc( sizeof(float)*iwave );
		jhd[j+k*n_azis].Z = (float *) malloc( sizeof(float)*iwave );

		if((jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)||(jhd[j+k*n_azis].X==NULL)){
			fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( jhd[j+k*n_azis].X, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Y, sizeof(float), iwave, f );
		count+=fread( jhd[j+k*n_azis].Z, sizeof(float), iwave, f );
		if( count != (3*iwave) ) {
			fprintf(stderr,"error: %s: cant read tt_table file (3)\n", argv[0] );
			exit(-1);
			}

		count=0;
		count+=fread( &(jhd[j+k*n_azis].nt), sizeof(int), 1, f );
		if( count != 1 ) {
			fprintf(stderr,"error: %s: cant read tt_table file (4)\n", argv[0] );
			exit(-1);
			}

		jhd[j+k*n_azis].tlist = (int *) malloc( sizeof(int)*jhd[j+k*n_azis].nt );
		if( jhd[j+k*n_azis].tlist == NULL ) {
			fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
			exit(-1);
			}
		count=0;
		count+=fread( jhd[j+k*n_azis].tlist, sizeof(int), jhd[j+k*n_azis].nt, f );
		if( count != jhd[j+k*n_azis].nt ) {
			fprintf(stderr,"error: %s: cant read tt_table file (5)\n", argv[0] );
			exit(-1);
			}
		}
	fclose(f);


	if( quick == 2 ) { /* read in index */
		if( (argc<5) || (f=fopen(argv[4],"r"))==NULL ) {
			fprintf(stderr,"error: %s: cant open tt_index file <%s> for read\n",
				argv[0], argv[4] );
			exit(-1);
			}
		
		cindex = (int *) malloc( n_tetras * sizeof(int) );
		jindex = (int **) malloc( n_tetras * sizeof(int*) );
		kindex = (int **) malloc( n_tetras * sizeof(int*) );
		lindex = (int **) malloc( n_tetras * sizeof(int*) );
		iindex = (int **) malloc( n_tetras * sizeof(int*) );
		if( (cindex==NULL) || (jindex==NULL) || (kindex==NULL) ||
		                      (lindex==NULL) || (iindex==NULL) ) {
			fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
			exit(-1);
			}

		for( i=0; i<n_tetras; i++ ) {
			cindex[i]=0;
			jindex[i]=NULL; kindex[i]=NULL; lindex[i]=NULL; iindex[i]=NULL;
			}
		while( (count=fread( buf, sizeof(int), 2, f )) > 0 ) {
			if( count!=2 ) {
				fprintf(stderr,"error: %s: failed to read index (1)\n", argv[0] );
				exit(-1);
				}
			j=buf[0]; k=buf[1];
			cindex[j]=k;
			jtmp = (int *)malloc( k * sizeof(int) );
			ktmp = (int *)malloc( k * sizeof(int) );
			ltmp = (int *)malloc( k * sizeof(int) );
			itmp = (int *)malloc( k * sizeof(int) );
			if( (jtmp==NULL) || (ktmp==NULL) || (ltmp==NULL) || (itmp==NULL) ) {
				fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
				exit(-1);
				}
			jindex[j]=jtmp; kindex[j]=ktmp; lindex[j]=ltmp; iindex[j]=itmp;
			for( i=0; i<k; i++ ) {
				count=fread( buf, sizeof(int), 4, f );
				if( count!=4 ) {
					fprintf(stderr,"error: %s: failed to read index (2)\n", argv[0] );
					exit(-1);
					}
				jtmp[i]=buf[0]; ktmp[i]=buf[1]; ltmp[i]=buf[2]; itmp[i]=buf[3]; 
				}
			}

		fclose(f);
		}

	if( (quick==0) || (quick==1) ) {
	while( scanf("%d%le%le%le%le%le", &nev, &(x1[0]), &(x1[1]), &(x1[2]), &Tmin, &Tmax )==6 ) {
		iwavemin = (int)(Tmin/dT)-1; if( iwavemin<1 ) iwavemin=1;
		iwavemax = (int)(Tmax/dT)+1;
		n_tts=0;
		if( !find_tetra(x1,&t) ) {
			fprintf(stderr,"warning: %s: event %d not in model; skipping\n", argv[0], nev );
			continue;
			}
		v = eval_velocity(&tetra[t],x1);
		if( quick ) {
			for( j=0; j<n_azis; j++ ) for( k=0; k<n_thetas; k++ ) {
				i=j+k*n_azis;
				jhd[i].hit = 0;
				if( jhd[i].tlist != NULL ) {
					jhd[i].hit = (int) bsearch(&t, jhd[i].tlist,
						 jhd[i].nt, sizeof(int), cmpint);
					}
				}
			}
		for( j=0; j<(n_azis-1); j++ ) {
		for( k=0; k<(n_thetas-1); k++ ) {
		for( l=0; l<2; l++ ) {
			if( l==0 ) {
				r1 = j+k*n_azis;
				r2 = (j+1)+k*n_azis;
				r3 = j+(k+1)*n_azis;
				}
			else {
				r1 = (j+1)+k*n_azis;
				r2 = (j+1)+(k+1)*n_azis;
				r3 = j+(k+1)*n_azis;
				}

			if( (jhd[r1].iwave==0) || (jhd[r2].iwave==0) || (jhd[r3].iwave==0) ) {
				continue;
				}

			if( quick ) {
				if( !( jhd[r1].hit || jhd[r2].hit || jhd[r3].hit ) ) {
					continue;
					}
				}
	
			iwave = jhd[r1].iwave;
			if( jhd[r2].iwave < iwave ) iwave=jhd[r2].iwave;
			if( jhd[r3].iwave < iwave ) iwave=jhd[r3].iwave;
			if( iwave>iwavemax ) iwave=iwavemax;
			if( iwavemin>=(iwave-1) ) continue;
			for( i=iwavemin; i<(iwave-1); i++ ) {

				b1[0]=jhd[r1].X[i]; b1[1]=jhd[r1].Y[i]; b1[2]=jhd[r1].Z[i];
				b2[0]=jhd[r2].X[i]; b2[1]=jhd[r2].Y[i]; b2[2]=jhd[r2].Z[i];
				b3[0]=jhd[r3].X[i]; b3[1]=jhd[r3].Y[i]; b3[2]=jhd[r3].Z[i];
				Tb = dT*(double)i;

				t1[0]=jhd[r1].X[i+1]; t1[1]=jhd[r1].Y[i+1]; t1[2]=jhd[r1].Z[i+1];
				t2[0]=jhd[r2].X[i+1]; t2[1]=jhd[r2].Y[i+1]; t2[2]=jhd[r2].Z[i+1];
				t3[0]=jhd[r3].X[i+1]; t3[1]=jhd[r3].Y[i+1]; t3[2]=jhd[r3].Z[i+1];
				Tt = dT*(double)(i+1);

				raytheta[0]=jhd[r1].theta; rayazi[0]=jhd[r1].azi;
				raytheta[1]=jhd[r2].theta; rayazi[1]=jhd[r2].azi;
				raytheta[2]=jhd[r3].theta; rayazi[2]=jhd[r3].azi;

				doazth=1;
				if( in_tri_prism( x1, b1,b2,b3,Tb, t1,t2,t3,Tt, &T, dTdx, 
					doazth, rayazi, raytheta, &azi, &theta ) ) {
					tt[n_tts].T = T;
					tt[n_tts].dTdx[0]=dTdx[0];
					tt[n_tts].dTdx[1]=dTdx[1];
					tt[n_tts].dTdx[2]=dTdx[2];
					tt[n_tts].th = theta;
					tt[n_tts].az = azi;

					if( rayrefine( x0, x1, azi, theta, T, &newazi, &newtheta, &newT, newdTdx) ) {
						tt[n_tts].T = newT;
						tt[n_tts].dTdx[0] = newdTdx[0];
						tt[n_tts].dTdx[1] = newdTdx[1];
						tt[n_tts].dTdx[2] = newdTdx[2];
						tt[n_tts].th = theta;
						tt[n_tts].az = azi;
						}

					if( (n_tts+1) >= MAX_TTS ) {
						fprintf(stderr,"error: %s: too many arrivals, event=%d\n", argv[0],nev);
						exit(-1);
						}
					n_tts++;
					}

				}

			}}}

			if( n_tts > 0 ) {
				qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
				for( i=0; i<n_tts; i++ ) {
					printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\n",
						nev, i, x1[0],x1[1],x1[2], tt[i].T,
						tt[i].dTdx[0], tt[i].dTdx[1], tt[i].dTdx[2],
						v, t, tt[i].th, tt[i].az );
					}
				}
			else {
				fprintf(stderr, "warning: no arrivals for event %d at %f %f %f\n",
					nev, x1[0], x1[1], x1[2] );
				}
		}
		} /* end if quick 0 1 */
	else {
	while( scanf("%d%le%le%le%le%le", &nev, &(x1[0]), &(x1[1]), &(x1[2]), &Tmin, &Tmax )==6 ) {
		iwavemin = (int)(Tmin/dT)-1; if( iwavemin<1 ) iwavemin=1;
		iwavemax = (int)(Tmax/dT)+1;
		n_tts=0;
		if( !find_tetra(x1,&t) ) {
			fprintf(stderr,"warning: %s: event %d not in model; skipping\n", argv[0], nev );
			continue;
			}
		v = eval_velocity(&tetra[t],x1);
		for( ic=0; ic<cindex[t]; ic++ ) {
			j = jindex[t][ic];
			k = kindex[t][ic];
			l = lindex[t][ic];
			i = iindex[t][ic];
			if( l==0 ) {
				r1 = j+k*n_azis;
				r2 = (j+1)+k*n_azis;
				r3 = j+(k+1)*n_azis;
				}
			else {
				r1 = (j+1)+k*n_azis;
				r2 = (j+1)+(k+1)*n_azis;
				r3 = j+(k+1)*n_azis;
				}

			if( (jhd[r1].iwave==0) || (jhd[r2].iwave==0) || (jhd[r3].iwave==0) ) {
				continue;
				}

			b1[0]=jhd[r1].X[i]; b1[1]=jhd[r1].Y[i]; b1[2]=jhd[r1].Z[i];
			b2[0]=jhd[r2].X[i]; b2[1]=jhd[r2].Y[i]; b2[2]=jhd[r2].Z[i];
			b3[0]=jhd[r3].X[i]; b3[1]=jhd[r3].Y[i]; b3[2]=jhd[r3].Z[i];
			Tb = dT*(double)i;

			t1[0]=jhd[r1].X[i+1]; t1[1]=jhd[r1].Y[i+1]; t1[2]=jhd[r1].Z[i+1];
			t2[0]=jhd[r2].X[i+1]; t2[1]=jhd[r2].Y[i+1]; t2[2]=jhd[r2].Z[i+1];
			t3[0]=jhd[r3].X[i+1]; t3[1]=jhd[r3].Y[i+1]; t3[2]=jhd[r3].Z[i+1];
			Tt = dT*(double)(i+1);

			raytheta[0]=jhd[r1].theta; rayazi[0]=jhd[r1].azi;
			raytheta[1]=jhd[r2].theta; rayazi[1]=jhd[r2].azi;
			raytheta[2]=jhd[r3].theta; rayazi[2]=jhd[r3].azi;

			doazth=1;
			if( in_tri_prism( x1, b1,b2,b3,Tb, t1,t2,t3,Tt, &T, dTdx,
				doazth, rayazi, raytheta, &azi, &theta ) ) {
				tt[n_tts].T = T;
				tt[n_tts].dTdx[0]=dTdx[0];
				tt[n_tts].dTdx[1]=dTdx[1];
				tt[n_tts].dTdx[2]=dTdx[2];
				tt[n_tts].th = theta;
				tt[n_tts].az = azi;

				if( rayrefine( x0, x1, azi, theta, T, &newazi, &newtheta, &newT, newdTdx) ) {
					tt[n_tts].T = newT;
					tt[n_tts].dTdx[0] = newdTdx[0];
					tt[n_tts].dTdx[1] = newdTdx[1];
					tt[n_tts].dTdx[2] = newdTdx[2];
					tt[n_tts].th = theta;
					tt[n_tts].az = azi;
					}

				if( (n_tts+1) >= MAX_TTS ) {
					fprintf(stderr,"error: %s: too many arrivals, event=%d\n", argv[0],nev);
					exit(-1);
					}
				if( (T<Tmin) || (T>Tmax) ) continue;
				n_tts++;
				}

			} /* end for ic */

		if( n_tts > 0 ) {
			qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
			for( i=0; i<n_tts; i++ ) {
				printf("%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\n",
					nev, i, x1[0],x1[1],x1[2], tt[i].T,
					tt[i].dTdx[0], tt[i].dTdx[1], tt[i].dTdx[2],
					v, t, tt[i].th, tt[i].az );
				}
			}
		else {
			fprintf(stderr, "warning: no arrivals for event %d at %f %f %f\n",
				nev, x1[0], x1[1], x1[2] );
			}

		} /*end while */

		} /* end quick 2 */
	
	exit(0);
	}

/* Newton method refine of an already pretty good (azi, theta, T) estimate */
int rayrefine( double *x0, double *x1, double azi, double theta, double T, 
                                       double *newazi, double *newtheta, double *newT, double *newdTdx) 
{
	int i, j, status, tetra1, maxiterations, iter, ierror;
	double paz, pth, pT;
	double v, G[3][3], d[3], maxG, absG, D, Dtol;
	double x1ref[3], x1paz[3], x1pth[3];
	double t1ref[3], t1paz[3], t1pth[3];
	double test1, test2, test3;
	int n_arcs;
	struct arc_rec arc[MAX_ARCS];
	
	if( (!finite(theta)) || (!finite(azi)) || (!finite(T)) ) return(0);

	maxiterations=5; Dtol=0.0005;
	*newazi=azi; *newtheta=theta; *newT=T;

	for( iter=0; iter<maxiterations; iter++ ) {

		if( !shoot_to_traveltime( x0, *newazi, *newtheta, *newT, x1ref, t1ref, arc, &n_arcs )) return(0);
		if( !find_tetra(x1ref,&tetra1) ) return(0);
		v = eval_velocity(&tetra[tetra1],x1ref);

		pth = 1.0e-5;
		paz = 1.0e-5;
		if( !shoot_to_traveltime( x0, *newazi+paz, *newtheta, *newT, x1paz, t1paz, arc, &n_arcs )) return(0);
		if( !shoot_to_traveltime( x0, *newazi, *newtheta+pth, *newT, x1pth, t1pth, arc, &n_arcs )) return(0);
		
		D=0.0;
		for( i=0; i<3; i++ ) {
			G[i][0] = (x1paz[i]-x1ref[i])/paz;
			G[i][1] = (x1pth[i]-x1ref[i])/pth;
			G[i][2] = t1ref[i]*v;
			d[i] = x1[i] - x1ref[i];
			newdTdx[i] = t1ref[i]/v;
			D+=fabs(d[i]);
			}
			
		maxG=0.0;
		for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) if( (absG=fabs(G[i][j]))>maxG ) maxG=absG;
		if( maxG <= 0.0 ) return(0);
		
		gauss( G, d, 3, 3, 1.0e-6*maxG, &ierror, TRUE );
		if( ierror != 0 ) return(0);
		
		if( (!finite(d[0])) || (!finite(d[1])) || (!finite(d[2])) ) return(0);

		*newazi += d[0]; *newtheta += d[1]; *newT += d[2];
		
		if( D < Dtol ) break;
			
		}

	/* plausibility check */
	test1 = fabs( (*newT) - T );
	test2 = fmod( fabs( (*newtheta) - theta ) , 360.0 );
	test3 = fmod( fabs( (*newazi) - azi ) , 360.0);
	if( ((*newT)<=0.0) || (test1>=0.01) || (test2>=1.0) || (test3>=1.0) ) return(0);

	return(1);
	}

int shoot_to_traveltime( double *x0, double azi, double theta, double traveltime, double *x1, double *t1,
	 struct arc_rec *arc, int *n_arcs)
{
	int i, l, iarc, status, method, ec;
	double t0[3], xa[3], ta[3], Ta, qinvinta;
	double psi, Tprev, te, r[3];

	ray_tangent(theta,azi,t0);
	*n_arcs=0;
	status=shoot_with_max_traveltime( x0, t0, xa, ta, arc, n_arcs,
			MAX_ARCS, 10000.0, traveltime, &Ta, &qinvinta, &ec );
	if( ! status ) return(0);

	iarc=(*n_arcs)-1;
	Tprev=0.0; for( i=0; i<iarc; i++ ) Tprev+=arc[i].T;

	te = (traveltime-Tprev)
		+ ttintegral(0.0, arc[iarc].ptetra->gs, arc[iarc].R, arc[iarc].psi1, &method);

	psi = inv_integral294a( arc[iarc].ptetra->gs*te );

	/* ray end point */
        r[0] = arc[iarc].R*sin(psi);
        r[1] = 0.0;
        r[2] = arc[iarc].R*cos(psi);
        for( l=0; l<3; l++ ) r[l] += arc[iarc].shift[l];
	rotate3(r,arc[iarc].st,x1);
		
	/* ray end tangent */
	r[0] = cos(psi);
	r[1] = 0.0;
	r[2] = (-sin(psi));
	rotate3(r,arc[iarc].st,t1);

	/* reset last arc to reflect change in endpoint */
	for( l=0; l<3; l++ ) arc[iarc].x1[l]=x1[l];
	for( l=0; l<3; l++ ) arc[iarc].t1[l]=t1[l];
	arc[iarc].psi2 = RTOD*psi;
	arc[iarc].T=traveltime-Tprev;
	arc[iarc].S=DTOR*(arc[iarc].psi2-arc[iarc].psi1)*arc[iarc].R;

	return(1);
	}

int in_tri_prism( x, b1,b2,b3,Tb, t1,t2,t3,Tt, T, dTdx, doazth, rayazi, raytheta, azi, theta )
double x[3];
double b1[3], b2[3], b3[3], Tb;
double t1[3], t2[3], t3[3], Tt;
double *T, dTdx[3], *rayazi, *raytheta, *azi, *theta;
{
	double *v1, *v2, *v3, *v4;
	double T1, T2, T3, T4;
	double A1, A2, A3, A4;
	double TH1, TH2, TH3, TH4;
	double G[4][4], d[4], test, Gabs;
	int i, j, it, ierror;

	for( it=0; it<3; it++ ) {
		if( it==0 ) {
			v1=b1; v2=b2; v3=b3; v4=t1; T1=Tb; T2=Tb; T3=Tb; T4=Tt;
			if( doazth ) {
				A1=rayazi[0]; A2=rayazi[1]; A3=rayazi[2]; A4=rayazi[0];
				TH1=raytheta[0]; TH2=raytheta[1]; TH3=raytheta[2]; TH4=raytheta[0];
				}
			}
		else if( it==1) {
			v1=b2; v2=b3; v3=t1; v4=t2; T1=Tb; T2=Tb; T3=Tt; T4=Tt;
			if( doazth ) {
				A1=rayazi[1]; A2=rayazi[2]; A3=rayazi[0]; A4=rayazi[1];
				TH1=raytheta[1]; TH2=raytheta[2]; TH3=raytheta[0]; TH4=raytheta[1];
				}
			}
		else {
			v1=b3; v2=t1; v3=t2; v4=t3; T1=Tb; T2=Tt; T3=Tt; T4=Tt;
			if( doazth ) {
				A1=rayazi[2]; A2=rayazi[0]; A3=rayazi[1]; A4=rayazi[2];
				TH1=raytheta[2]; TH2=raytheta[0]; TH3=raytheta[1]; TH4=raytheta[2];
				}
			}
		if( in_tetrahedron( v1, v2, v3, v4, x ) ) {
			for( i=0; i<3; i++ ) {
				G[0][i]=v1[i];
				G[1][i]=v2[i];
				G[2][i]=v3[i];
				G[3][i]=v4[i];
				}
			for( i=0; i<4; i++ ) G[i][3]=1.0;
			d[0]=T1; d[1]=T2; d[2]=T3; d[3]=T4;
			test=fabs(G[0][0]);
			for( i=0; i<4; i++ ) for( j=0; j<4; j++ ) {
				Gabs=fabs(G[i][j]);
				if( test<Gabs ) test=Gabs;
				}
			gauss( G, d, 4, 4, 1.0e-6*test, &ierror, TRUE );
			if( ierror ) return(0);
			*T = d[3]+dot(x,d);
			if( !finite(*T) ) return(0);
			for( i=0; i<3; i++ ) {
				if( !finite(d[i]) ) return(0);
				dTdx[i]=d[i];
				}
			if( (*T) < 0.0 ) return(0);

			if( doazth ) {
				d[0]=A1; d[1]=A2; d[2]=A3; d[3]=A4;
				gauss( G, d, 4, 4, 1.0e-6*test, &ierror, FALSE );
				*azi = d[3]+dot(x,d);

				d[0]=TH1; d[1]=TH2; d[2]=TH3; d[3]=TH4;
				gauss( G, d, 4, 4, 1.0e-6*test, &ierror, FALSE );
				*theta = d[3]+dot(x,d);
				}

			return(1);
			}
		}

	return(0);
	}

struct ev_rec {
	int nev, index, count;
	double x, y, z, t0, rms, mean;
	};

struct jhd_rec {
	int nev, order;
	double T, dTdx, dTdy, dTdz, azi, theta;
	};

int cmpev(a, b)
struct ev_rec *a, *b;
{
  if( a->nev <  b->nev ) return(-1);
  else if( a->nev == b->nev ) return(0);
  else return(1);
  }

int cmpjhd(a, b)
struct jhd_rec *a, *b;
{
  if( a->nev <  b->nev ) return(-1);
  else if( a->nev == b->nev ) {
  	if( a->order <  b->order ) return(-1);
  	else if( a->order == b->order ) return(0);
 	else return(1);
	}
  else return(1);
  }

main_jhd_locate(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, m, n, ii, ngood, status, iter, biec, irms, line;
	char str[1024];

	FILE *fsta;
	int *stanum;
	char **jhdfilename;
	int n_sta;
	double x,y,z,t0,c;
	double T, dTdx, dTdy, dTdz;

	FILE *fev;
	struct ev_rec *ev, evkey, *myev;
	int n_ev;

	FILE *fph;
	struct ph_rec {
		double obs, weight, maxres;
		int n, good, row, line;
		int *sta, *ev, *order, *hit;
		double *ratio, *coeff;
		double *T, *dTdx, *dTdy, *dTdz;
		};
	struct ph_rec *ph;
	int n_ph;

	FILE *fjhd;
	struct jhd_rec *jhd;
	int n_jhd;
	struct jhd_rec jhdkey, *myjhd;

	int verbose=0;

	struct sparse_matrix *G, *GTG;
	double *d, *dp, *GTd, *dm, tscale=TSCALEVALUE, maxG, maxd, damping, err, res, rms1, rms2;
	double *starms;
	double new_damping, scx;
	int *stacount;
		
	if( argc!=5 ) {
		fprintf(stderr,"purpose: joint hypocenter relocation\n");
		fprintf(stderr,"method: The user provides a set of observations, where\n");
		fprintf(stderr,"        each observation is assumed to be a linear combination\n");
		fprintf(stderr,"        of traveltimes:\n");
		fprintf(stderr,"           observation = (sum i=1 to n) coefficient[i] * traveltime[i]\n");
		fprintf(stderr,"        A phase file tabulates the observations and the\n");
		fprintf(stderr,"        n coefficients and the station, event and order\n");
		fprintf(stderr,"        of each of the traveltimes.  The observations are\n");
		fprintf(stderr,"        combined with traveltime and traveltime derivatives\n");
		fprintf(stderr,"        to produce an updated estimated of the event locations.\n");
		fprintf(stderr,"usage: %s station_file.txt event_file.txt phase_file.txt damping > updated_event_file.txt\n", argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"       station_file.txt: one station per line, with fields\n");
		fprintf(stderr,"               station_number: arbitrary integer\n");
		fprintf(stderr,"               jhd_table: path of station-specific jhd-table\n");
		fprintf(stderr,"       event_file.txt: one event per line, with fields\n");
		fprintf(stderr,"               event_number: arbitrary integer\n");
		fprintf(stderr,"               x, y, z: preliminary location, km\n");
		fprintf(stderr,"               t0: preliminary origin time, s\n");
		fprintf(stderr,"       phase_file.txt: one group of lines per observation\n");
		fprintf(stderr,"               first line, with fields\n");
		fprintf(stderr,"                     n_phases: number of phases in observation\n");
		fprintf(stderr,"                     observation: a differetial traveltime, s\n");
		fprintf(stderr,"                     weight: weight of observation\n");
		fprintf(stderr,"                     maxres: observation thrown out if weighted residual > maxres, s\n");
		fprintf(stderr,"               subsequent lines in group, with fields\n");
		fprintf(stderr,"                     station_number, corresponding to station_file.txt\n");
		fprintf(stderr,"                     event_number, corresponding to event_file.txt\n");
		fprintf(stderr,"                     order, interger arrival number of phase\n");
		fprintf(stderr,"                     ratio, computed traveltimes multiplied by this\n");
		fprintf(stderr,"                            (i.e. use 1.o for P 1.76 for S)\n");
		fprintf(stderr,"                     coefficient, of phase in observation\n");
		fprintf(stderr,"       damping: damping factor, fraction of max of GTG\n");
		fprintf(stderr,"Note: In the inversion, origin time perturbations are scaled\n");
		fprintf(stderr,"to units of distance using a nominal velocity of %f km/s\n",
				1.0/TSCALEVALUE);
		exit(-1);
		}

	if( sscanf(argv[4],"%le",&damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	/* station file */
	n_sta=count_lines(argv[1]);
	stanum=(int *)malloc(n_sta*sizeof(int));
	stacount=(int *)malloc(n_sta*sizeof(int));
	starms=(double *)malloc(n_sta*sizeof(double));
	jhdfilename = (char**)malloc(n_sta*sizeof(char*));

	if( (stanum==NULL) || (jhdfilename==NULL) || (starms==NULL) || (stacount==NULL)) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	if( (fsta=fopen(argv[1],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open station file <%s> for read\n",argv[0],argv[1]);
		exit(-1);
		}
	n_sta=0;
	while( fscanf(fsta,"%d%s",&i,str)==2 ) {
		stanum[n_sta]=i;
		jhdfilename[n_sta]=(char*)malloc( (strlen(str)+1)*sizeof(char) );
		if( jhdfilename[n_sta]==NULL ) {
			fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
			exit(-1);
			}
		strcpy(jhdfilename[n_sta],str);
		starms[n_sta]=0.0; stacount[n_sta]=0;
		n_sta++;
		}
	fclose(fsta);
	fprintf(stderr,"Stations: %d\n", n_sta);
	if( verbose ) {
		for( i=0; i<n_sta; i++ ) {
			fprintf(stderr,"%d %s\n", stanum[i], jhdfilename[i] );
			}
		fprintf(stderr,"\n");
		}

	/* event file */
	n_ev=count_lines(argv[2]);
	ev=(struct ev_rec *)malloc(n_ev*sizeof(struct ev_rec));
	if( ev==NULL ) {
		fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
		exit(-1);
		}
	if( (fev=fopen(argv[2],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open event file <%s> for read\n",argv[0],argv[2]);
		exit(-1);
		}
	n_ev=0;
	while( fscanf(fev,"%d%le%le%le%le",&i,&x,&y,&z,&t0)==5 ) {
		ev[n_ev].nev = i;
		ev[n_ev].x=x; ev[n_ev].y=y; ev[n_ev].z=z; ev[n_ev].t0=t0;
		ev[n_ev].rms=0.0; ev[n_ev].count=0;
		n_ev++;
		}
	fclose(fev);
	qsort( ev, n_ev, sizeof(struct ev_rec), cmpev );
	for( i=0; i<n_ev; i++ ) ev[i].index=i;
	fprintf(stderr,"Events: %d\n", n_ev);
	if( verbose ) {
		for( i=0; i<n_ev; i++ ) {
			fprintf(stderr,"%d %f %f %f %f\n", ev[i].nev, ev[i].x, ev[i].y, ev[i].z, ev[i].t0 );
			}
		fprintf(stderr,"\n");
		}

	/* phase file */
	if( (fph=fopen(argv[3],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open phase file <%s> for read\n",argv[0],argv[3]);
		exit(-1);
		}
	n_ph=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&k,&k,&x,&x)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			}
		n_ph++;
		}
	rewind(fph);
	ph=(struct ph_rec *)malloc(n_ph*sizeof(struct ph_rec));
	if( ph==NULL ) {
		fprintf(stderr,"error: %s: out of memory (4)\n", argv[0] );
		exit(-1);
		}
	n_ph=0; line=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		line++; ph[n_ph].line=line;
		ph[n_ph].n=i; ph[n_ph].obs=x; ph[n_ph].weight=y; ph[n_ph].maxres=z;
		ph[n_ph].good=0; ph[n_ph].row=(-1);
		ph[n_ph].sta=(int*)malloc(i*sizeof(int));
		ph[n_ph].ev=(int*)malloc(i*sizeof(int));
		ph[n_ph].order=(int*)malloc(i*sizeof(int));
		ph[n_ph].hit=(int*)malloc(i*sizeof(int));
		ph[n_ph].ratio=(double*)malloc(i*sizeof(double));
		ph[n_ph].coeff=(double*)malloc(i*sizeof(double));
		ph[n_ph].T=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdx=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdy=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdz=(double*)malloc(i*sizeof(double));
		if( (ph[n_ph].sta==NULL) || (ph[n_ph].ev==NULL) || (ph[n_ph].order==NULL) || (ph[n_ph].hit==NULL) ||
		    (ph[n_ph].ratio==NULL) || (ph[n_ph].coeff==NULL) || (ph[n_ph].T==NULL) ||
		    (ph[n_ph].dTdx==NULL) || (ph[n_ph].dTdy==NULL) || (ph[n_ph].dTdz==NULL) ) {
			fprintf(stderr,"error: %s: out of memory (5)\n", argv[0] );
			exit(-1);
			}
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&l,&m,&x,&y)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			line++;
			ph[n_ph].sta[j]=k; ph[n_ph].ev[j]=l; ph[n_ph].order[j]=m; ph[n_ph].hit[j]=0;
			ph[n_ph].ratio[j]=x; ph[n_ph].coeff[j]=y; ph[n_ph].T[j]=0.0;
			ph[n_ph].dTdx[j]=0.0; ph[n_ph].dTdy[j]=0.0;ph[n_ph].dTdz[j]=0.0;
			}
		n_ph++;
		}
	fclose(fph);
	fprintf(stderr,"Observations: %d\n", n_ph);

	for( i=0; i<n_sta; i++ ) {
		n_jhd=count_lines(jhdfilename[i]);
		jhd=(struct jhd_rec *)malloc(n_jhd*sizeof(struct jhd_rec));
		if( jhd==NULL ) {
			fprintf(stderr,"error: %s: out of memory (6)\n", argv[0] );
			exit(-1);
			}
		if( (fjhd=fopen(jhdfilename[i],"r")) == NULL ) {
			fprintf(stderr,"error: %s: cant open jhd_table <%s> for read\n",
				argv[0], jhdfilename[i] );
			exit(-1);
			}
		n_jhd=0;
		while( fscanf(fjhd,"%d%d%le%le%le%le%le%le%le%le%d%le%le",&j,&k,&x,&x,&x,
			&T, &dTdx, &dTdy, &dTdz, &x, &l, &x, &x ) == 13 ) {
			jhd[n_jhd].nev=j;
			jhd[n_jhd].order=k;
			jhd[n_jhd].T = T;
			jhd[n_jhd].dTdx=dTdx; jhd[n_jhd].dTdy=dTdy; jhd[n_jhd].dTdz=dTdz;
			n_jhd++;
			}
		fclose(fjhd);
		fprintf(stderr,"jhd_table %s has %d records\n", jhdfilename[i], n_jhd );
		if( verbose ) {
			for( j=0; j<n_jhd; j++ ) {
				fprintf(stderr,"%d %d %f %f %f %f\n", jhd[j].nev, jhd[j].order,
					jhd[j].T, jhd[j].dTdx, jhd[j].dTdy, jhd[j].dTdz );
				}
			fprintf(stderr,"\n");
			}
		qsort( jhd, n_jhd, sizeof(struct jhd_rec), cmpjhd );
		for( j=0; j<n_ph; j++ ) for( k=0; k<ph[j].n; k++ ) {
			if( stanum[i] != ph[j].sta[k] ) continue;
			jhdkey.nev = ph[j].ev[k];
			jhdkey.order = ph[j].order[k];
			myjhd=(struct jhd_rec *)
				bsearch(&jhdkey, jhd, n_jhd, sizeof(struct jhd_rec), cmpjhd );
			if( myjhd == NULL ) continue;
			ph[j].hit[k]=1;
			ph[j].T[k] = myjhd->T;
			ph[j].dTdx[k] = myjhd->dTdx;
			ph[j].dTdy[k] = myjhd->dTdy;
			ph[j].dTdz[k] = myjhd->dTdz;
			}
		free(jhd);
		}

	ngood=0;
	for( j=0; j<n_ph; j++ ) {
		l=0;
		for( k=0; k<ph[j].n; k++ ) l+=ph[j].hit[k];
		if( l==ph[j].n ) {
			ph[j].good=1;
			ngood++;
			}
		}
	fprintf(stderr,"Usable Observations: %d\n", ngood);
	if( ngood==0 ) {
		fprintf(stderr,"warning: %s: no good observations\n");
		for( i=0; i<n_ev; i++ ) {
			x=ev[i].x; y=ev[i].y; z=ev[i].z; t0=ev[i].t0;
			printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
			}
		exit(0);
		}

	if( verbose ) {
		for( j=0; j<n_ph; j++ ) {
			fprintf(stderr,"%d %d %f %f %f %d\n",
				j, ph[j].n, ph[j].obs, ph[j].weight, ph[j].maxres, ph[j].good);
			l=0;
			for( k=0; k<ph[j].n; k++ ) {
				fprintf(stderr, "\t%d %d %d %f %f %f %f %f %f %d\n",
					ph[j].sta[k], ph[j].ev[k], ph[j].order[k], ph[j].ratio[k],
					ph[j].coeff[k], ph[j].T[k], ph[j].dTdx[k], ph[j].dTdy[k],
					ph[j].dTdz[k], ph[j].hit[k] );
				}
			}
		}

	n = ngood;
	m = 4*n_ev;
	G=new_sparse_matrix(n,m,0);
	d=(double*)malloc(n*sizeof(double));
	dp=(double*)malloc(n*sizeof(double));
	dm=(double*)malloc(m*sizeof(double));
	GTd=(double*)malloc(m*sizeof(double));
	if( (G==NULL) || (d==NULL) || (dp==NULL) || (dm==NULL) || (GTd==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (7)\n", argv[1] );
		exit(-1);
		}

	/* Theory                                                                              */
	/* let ta = T0 + T(x,y,z)                                                              */
        /*        = t0 + dt + T(x0,y0,z0) + gradT dot dx                                       */
        /* then (sum i) ci tai = obs =                                                         */
        /*      (sum i) ci t0i + (sum i) ci dt + (sum i) ci T + (sum i) ci gradT dot dx        */
	/* and                                                                                 */
	/*     (sum i) ci gradT dot dx +  (sum i) ci dt = obs - (sum i) ci (t0i+T)             */
	/*                                                                                     */
	/* then note that every row of the equation is multiplied by the weight, x             */
	/*           and every instance of T and gradT is mutliplied by the ratio, y           */
	/*                                                                                     */
	/*     (sum i) x ci y gradT dot dx +  (sum i) x ci dt = obs - (sum i) x ci (t0i+y T)   */
	/*                                                                                     */
	/* finally, we introduce a time scaling, ts, such that dt = ts dtp.  This scaling      */
	/* is chosen to make the units and typical size of dx and dt the same.  A              */
	/* reasonbable choice is ts = 1/mean_velocity.                                         */
	/*                                                                                     */
	/* (sum i) x ci y gradT dot dx +  (sum i) x ci ts dt = obs - (sum i) x ci (t0i+y T)    */
	/*                                                                                     */

	i=0; irms=0; rms1=0.0;
	for( j=0; j<n_ph; j++ ) {
		if( !ph[j].good ) continue;
		x = ph[j].weight;
		d[i]=0.0;
		for( k=0; k<ph[j].n; k++ ) {
			y = ph[j].ratio[k];
			c = ph[j].coeff[k];
			evkey.nev = ph[j].ev[k];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) {
				fprintf(stderr,"error: %s: cant find event %d in event file\n", argv[0], evkey.nev);
				exit(-1);
				}
			l=myev->index;

			d[i] -= x * c * ( y * ph[j].T[k] + myev->t0 );

			status=1;
			z=get_sparse_value(G,i,4*l);
			z += x * c * y * ph[j].dTdx[k];
			status *= set_sparse_value(G,i,4*l,z);

			z=get_sparse_value(G,i,4*l+1);
			z += x * c * y * ph[j].dTdy[k];
			status *= set_sparse_value(G,i,4*l+1,z);

			z=get_sparse_value(G,i,4*l+2);
			z += x * c * y * ph[j].dTdz[k];
			status *= set_sparse_value(G,i,4*l+2,z);

			z=get_sparse_value(G,i,4*l+3);
			z += x * c * tscale;
			status *= set_sparse_value(G,i,4*l+3,z);
			if( !status ) {
				fprintf(stderr,"error: %s: out of memory (8)\n", argv[1] );
				exit(-1);
				}
			}
		if( verbose ) {
			fprintf(stderr,"weighted observation %d (row %d) obs=%f pred=%f res=%f\n",
			j, i, -x*ph[j].obs, d[i], x*ph[j].obs+d[i] );
			}

		res = fabs(x*ph[j].obs+d[i]);
		if(  res > ph[j].maxres ) {
			fprintf(stderr,"observation %d (row %d) thrown out, residual %f > %f\n", j, i,
				res, ph[j].maxres );
			for( l=0; l<m; l++ ) set_sparse_value(G,i,l,0.0);
			d[i]=0.0;
			}
		else {
			rms1 += res*res; irms++;
			d[i]+=x*ph[j].obs;
			ph[j].row=i;
			}

		i++;
		}

	rms1 = sqrt( rms1 / (double) irms );

	maxG=sparse_absmax(G);
	k=sparse_count(G);
	fprintf(stderr,
		"G is %d by %d, is %f percent full, and has absolute maximum of %f\n",
		n, m, 100.0*(double)k/((double)n*(double)m), maxG );

	maxd=0.0;
	for( i=0; i<n; i++ ) {
		x=fabs(d[i]);
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr, "d is %d by 1, and has absolute maximum of %f\n", n, maxd );

	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<m; i++ ) {
		x=fabs(GTd[i]);
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n", m, maxd );

	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );

	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, m+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	
	for( i=0; i<n_ev; i++ ) {
		x=ev[i].x+dm[4*i];
		y=ev[i].y+dm[4*i+1];
		z=ev[i].z+dm[4*i+2];
		t0=ev[i].t0+tscale*dm[4*i+3];
		printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
		}

	sm_times_v( G, dm, dp );

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res =  d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			for( k=0; k<n_sta; k++ ) if( stanum[k]==ph[i].sta[j] ) {
				stacount[k]++;
				starms[k]+= res*res;
				break;
				}
			}
		}
	fprintf(stderr,"station statistics:\n");
	fprintf(stderr,"sta\tcount\trms\n");
	for( i=0; i<n_sta; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\n", stanum[i], stacount[i], sqrt(starms[i]/(double)stacount[i]) );
		}
	fprintf(stderr,"\n");

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res = d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			evkey.nev = ph[i].ev[j];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) continue;
			myev->count++;
			myev->rms += res*res;
			}
		}
	fprintf(stderr,"event statistics:\n");
	fprintf(stderr,"event\tcount\trms\n");
	for( i=0; i<n_ev; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\n",
			ev[i].nev, ev[i].count, sqrt(ev[i].rms/(double)ev[i].count) );
		}
	fprintf(stderr,"\n");

	fprintf(stderr,"phase statistics:\n");
	fprintf(stderr,"phase\tline\terror\n");
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		fprintf(stderr,"%d\t%d\t%f\n", i, ph[i].line, res );
		}
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)irms );
	fprintf(stderr,"rms weighted residual original: %f final %f for %f percent reduction\n",
		rms1, rms2, 100.0-100.0*rms2/rms1 );


	exit(0);
	}

main_TVinvert(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, l, nodeid, interface, hitflag, n_arcs, n, status, order, ec;
	double x0[3], t0[3], x1[3], t1[3], v[4], theta, azi;
	struct arc_rec arc[MAX_ARCS];
	double traveltime;
	FILE *mpfile, *ttfile, *nmfile;
	char ct[128];
	double xsrc, ysrc, zsrc, xsta, ysta, zsta, ratio, T0, Ta;
	struct triangle_rec tri;
	struct tetra_rec *pt;
	double *blocksum, *blockones;

	double xmin, xmax, ymin, ymax, vapp, z0, vs, x_est, y_est, vp, px, py, max_dT;
	int nx, ny, t, my_last_tetra;
	int ng, itt, ierror, match, index;
	double *ts, *ts2, tolerance, rms, rms2, dT, dX;
	double damping, qinvint;
	int tnode[4];
	int biec;

	double thmin, thmax, azimin, azimax;
	int nth, nazi;

	struct sparse_matrix *G, *invert_GTG;

	double *d, *GTd, *dm, maxG, maxd, Gv, dv, err, mean;
	int iter, nz, *dindex, *dptr, bracket_status;

	double weight, relative_weight, t0weight, th_est, az_est;
	struct ttime_rec *T_ttime, *V_ttime;
	int n_T_ttimes, n_V_ttimes, n_total_ttimes;

	char hitsfile[256];
	double new_damping, scx;

	
	my_last_tetra=(-1);

	if( argc!=22 ) {
		fprintf(stderr,"purpose: invert telseismic and local traveltime data to improve model\n");
		fprintf(stderr,"usage: %s starting_model_file model_pars_file.txt teleseismic_traveltimes.tsv local_traveltimes.tsv max_dT relative_weight damping xmin xmax nx ymin ymax ny z0 thmin thmax nth azimin azimax nazi new_model_file\n",argv[0]);
		write_tele_note(stderr);
		write_model_blerb(stderr);
		write_model_pars_blerb(stderr);
		fprintf(stderr,"      input teleseismic_traveltimes.tsv:\n");
		fprintf(stderr,"              index: an integer that is the same for\n");
		fprintf(stderr,"                  all observations of a single teleseismic plane wave\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common index!\n");
		fprintf(stderr,"              azimuth, in deg, of plane wave\n");
		fprintf(stderr,"              apparent-velocity, in km/s, of plane wave\n");
		fprintf(stderr,"                  (but be warned;  azimuth and apparent-velocity ought to be the same for common index!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");
		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common index by interface!)\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"                  Note: The ratio allows S wave traveltimes\n");
		fprintf(stderr,"                  to be used in conjunction with a P wave\n");
		fprintf(stderr,"                  model inversion.  S wave apparent velocities\n");
		fprintf(stderr,"                  are specified in the traveltime file, along\n");
		fprintf(stderr,"                  with an appropriate P:S velocity ratio.  The\n");
		fprintf(stderr,"                  S wave apparent velocity is scaled to a P wave\n");
		fprintf(stderr,"                  apparent velocity prior to raytracing, and then\n");
		fprintf(stderr,"                  the final traveltime is scaled back to an S wave\n");
		fprintf(stderr,"                  traveltime after raytracing. All traveltimes\n");
		fprintf(stderr,"                  in a planewave group must have the same ratio.\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: Closest arrival option (-1) cannot\n");
		fprintf(stderr,"                 be used since the arrival time at the bottom\n");
		fprintf(stderr,"                 of the model is presumed unknown.  It will\n");
		fprintf(stderr,"                 be reset to 0.\n");
		fprintf(stderr,"      input local_traveltimes.tsv:\n");
		fprintf(stderr,"              xs, ys, zs, source location in km\n");
		fprintf(stderr,"                  (but be warned; sort data into groups with common source!\n");
		fprintf(stderr,"              xr, yr, zr, receiver location in km\n");

		fprintf(stderr,"                  (but be warned; z not used!)\n");
		fprintf(stderr,"              receiver interface, integer matching model\n");
		fprintf(stderr,"                  (but be warned; sort data with common source by interface!)\n");
		fprintf(stderr,"              T0,  origin time in s\n");
		fprintf(stderr,"              Ta,  arrival time in s\n");
		fprintf(stderr,"              ratio, 1 for P 1.76 (or whatever) for S\n");
		fprintf(stderr,"              order, 0 for 1-st arrival, 1 for second, etc\n");
		fprintf(stderr,"                 Note: (-1) indicates closest arrival\n");
		fprintf(stderr,"      max_dT: if the absolute value of the traveltime residual, |dT|=|Tobs-Tpred(initial_model)|\n");
		fprintf(stderr,"              is greater than max_dt, that data is ignored\n");
		fprintf(stderr,"      relative_weight: How much are teleseismic errors worth\n");
		fprintf(stderr,"              relative to local errors (i.e. telseismic rows of\n");
		fprintf(stderr,"              G multiplied by relative weight)\n");
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      arrivals are searched for in a turf with\n");
		fprintf(stderr,"         xmin, xmax, min, max x values, km\n");
		fprintf(stderr,"         nx, integer number of x values in turf\n");
		fprintf(stderr,"         ymin, ymax, min, max y values, km\n");
		fprintf(stderr,"         ny, integer number of y values in turf\n");
		fprintf(stderr,"      z0, in km, reference depth for plane wave\n");
		fprintf(stderr,"         (rays shot from this depth, so it should\n");
		fprintf(stderr,"         be within the model but near its bottom)\n");
		fprintf(stderr,"      output new_model_file: same format as starting model file\n");
		fprintf(stderr,"         also output is new_model_file.hits: same format as starting model file, but with velocity replaced with number of rays\n");
		fprintf(stderr,"         interacting with each node\n");
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d TV_invert. Starting model file: %s\n\n", argv[1] );
	timestamp("at start of program");

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	alloc_model_pars(argv[2]);

	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);
	
	fprintf(stderr,"Total of %d velocity node model parameters\n", n_model_pars);

	if( n_model_pars < 1000 ) for( i=0; i<n_model_pars; i++ ) {
		fprintf(stderr,"Model parameter %d has %d nodes\n", i, model_par[i].n );
		for( j=0; j<model_par[i].n; j++ ) {
			fprintf(stderr,"%d %f\n",model_par[i].node[j],model_par[i].weight[j]);
			}
		}

	make_ttime(argv[3]);
	if( (ttfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open telseismic_traveltime.tsv <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 10 ) {
		fprintf(stderr,"error: %s: cant skip header in teleseismic_traveltimess.tsv <%s>\n",
			argv[0], argv[3]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Teleseismic Data\n\n");
	fprintf(stderr,"#\tindex\tazi\tappvel\txr\tyr\tzr\tinterface\tTa\tratio\torder\tpwg\n");
 	while( fscanf(ttfile,"%d%le%le%le%le%le%d%le%le%d", &index,&azi,&vapp,
		&xsta,&ysta,&zsta,&interface,&Ta,&ratio,&order)==10){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		if( order==(-1) ) order=0;
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].index=index;
		ttime[n_ttimes].az=azi;
		ttime[n_ttimes].vapp=vapp;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		/* assign planewave group index */
                if( n_ttimes==0 ) {
                        n_pwg=0;
                        }
                else if( ttime[n_ttimes].index!=ttime[n_ttimes-1].index ) {
                        n_pwg++;
                        }
                ttime[n_ttimes].pwg=n_pwg;

		fprintf(stderr,"%d\t%d\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%d\t%d\n",
			n_ttimes,
			ttime[n_ttimes].index,ttime[n_ttimes].az,ttime[n_ttimes].vapp,
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder, ttime[n_ttimes].pwg );
		n_ttimes++;
		}
	n_pwg++;
	fclose(ttfile);
	T_ttime=ttime; n_T_ttimes=n_ttimes;
	fprintf(stderr,"Total of %d teleseismic data in %d planewave groups\n\n", n_ttimes, n_pwg);

	make_ttime(argv[4]);
	if( (ttfile=fopen(argv[4],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open local_traveltime.tsv <%s> for read\n", argv[0], argv[4] );
		exit(-1);
		}
	if( fscanf(ttfile,"%s%s%s%s%s%s%s%s%s%s%s",
			ct,ct,ct,ct,ct,ct,ct,ct,ct,ct,ct ) != 11 ) {
		fprintf(stderr,"error: %s: cant skip header in local_traveltimess.tsv <%s>\n",
			argv[0], argv[4]);
		exit(-1);
		}

	n_ttimes=0;
	fprintf(stderr,"\nInput Local Data\n\n");
	fprintf(stderr,"#\txs\tys\tzs\txr\tyr\tzr\tinterface\tT0\tTa\tratio\torder\n");
 	while( fscanf(ttfile,"%le%le%le%le%le%le%d%le%le%le%d", &xsrc,&ysrc,&zsrc,
		&xsta,&ysta,&zsta,&interface,&T0,&Ta,&ratio,&order)==11){
		if( (n_ttimes+2) > MAX_TTIMES ) {
			fprintf(stderr,"error: %s: too many traveltimes\n", argv[0] );
			exit(-1);
			}
		ttime[n_ttimes].ok=FALSE;
		ttime[n_ttimes].x0[0]=xsrc;
		ttime[n_ttimes].x0[1]=ysrc;
		ttime[n_ttimes].x0[2]=zsrc;
		ttime[n_ttimes].x1[0]=xsta;
		ttime[n_ttimes].x1[1]=ysta;
		ttime[n_ttimes].x1[2]=zsta;
		ttime[n_ttimes].interface=interface;
		ttime[n_ttimes].T0=T0;
		ttime[n_ttimes].obsTT=Ta-T0;
		ttime[n_ttimes].obsTa=Ta;
		ttime[n_ttimes].preTa=0.0;
		ttime[n_ttimes].preTT=0.0;
		ttime[n_ttimes].ratio=ratio;
		ttime[n_ttimes].obsorder=order;
		ttime[n_ttimes].preorder=order;
		fprintf(stderr,"%d\t%f\t%f\t%f\t%f\t%f\t%f\t%d\t%f\t%f\t%f\t%d\n",
			n_ttimes,
			ttime[n_ttimes].x0[0],ttime[n_ttimes].x0[1],ttime[n_ttimes].x0[2],
			ttime[n_ttimes].x1[0],ttime[n_ttimes].x1[1],ttime[n_ttimes].x1[2],
			ttime[n_ttimes].interface,
			ttime[n_ttimes].T0, ttime[n_ttimes].obsTa, 
			ttime[n_ttimes].ratio, ttime[n_ttimes].obsorder );
		n_ttimes++;
		}
	fclose(ttfile);
	V_ttime = ttime; n_V_ttimes=n_ttimes;
	fprintf(stderr,"Total of %d local data\n\n", n_ttimes);

	if( sscanf(argv[5], "%le", &max_dT) != 1 ) {
		fprintf(stderr,"error: %s: cant read max_dT from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[6], "%le", &relative_weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read relative_weight from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[7], "%le", &damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8], "%le", &xmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9], "%le", &xmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10], "%d", &nx) != 1 ) {
		fprintf(stderr,"error: %s: cant read nx from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[11], "%le", &ymin) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[12], "%le", &ymax) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[13], "%d", &ny) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[14], "%le", &z0) != 1 ) {
		fprintf(stderr,"error: %s: cant read z0 from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[15], "%le", &thmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[16], "%le", &thmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read thmax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[17], "%d", &nth) != 1 ) {
		fprintf(stderr,"error: %s: cant read nth from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[18], "%le", &azimin) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimin from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[19], "%le", &azimax) != 1 ) {
		fprintf(stderr,"error: %s: cant read azimax from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[20], "%d", &nazi) != 1 ) {
		fprintf(stderr,"error: %s: cant read nazi from command line\n", argv[0] );
		exit(-1);
		}


	if( (nmfile=fopen(argv[21],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	sprintf( hitsfile, "%s.hits", argv[21] );

	n_total_ttimes = n_T_ttimes + n_V_ttimes;
	G=new_sparse_matrix(n_total_ttimes,n_model_pars+n_pwg,0);
	ts=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));
	ts2=(double*)malloc((n_total_ttimes+2)*sizeof(double));
	d=(double*)malloc((n_total_ttimes+2)*sizeof(double));
	dindex=(int*)malloc((n_total_ttimes+2)*sizeof(int));
	dptr=(int*)malloc((n_total_ttimes+2)*sizeof(int));
	GTd=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));
	dm=(double*)malloc((n_model_pars+n_pwg+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (dindex==NULL) || (ts==NULL) ||
		(dptr==NULL) || (ts2==NULL) || (GTd==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}

	blocksum=(double*)malloc((n_model_pars+2)*sizeof(double));;
	blockones=(double*)malloc((n_total_ttimes+2)*sizeof(double));
	if( (blocksum==NULL) || (blockones==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (blocksum ...)\n", argv[0] );
		exit(-1);
		}
	ng=0;

	ttime=T_ttime; n_ttimes=n_T_ttimes;
	timestamp("at start of plane-wave-to-point raytracing");

	fprintf(stderr,"Starting Plane-Wave to Point Ray tracing of data\n\n");
	fprintf(stderr,"#\tobsTa\tT-predicted\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].index==index && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		index=ttime[itt].index;
		azi=ttime[itt].az;
		vapp=ttime[itt].vapp;
		interface=ttime[itt].interface;
		ratio=ttime[itt].ratio;
		fprintf(stderr,"Shooting turf pattern from reference depth %f at azimuth %f and app-velocity %f and ratio %f to interface %d\n",
		z0, azi, vapp, ratio, interface);
		fprintf(stderr,"Turf ranges: x (%f %f %d) y (%f %f %d)\n",
		xmin, xmax, nx, ymin, ymax, ny );
		shoot_turf(xmin, xmax, nx, ymin, ymax, ny, azi, vapp*ratio, z0, 0.0, interface);
		fprintf(stderr,"done with turf\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	for( i=0; i<(m_turf-1); i++ )
	for( j=0; j<(n_turf-1); j++ )
	for( l=0; l<2; l++ ) {
			if( inturfbox(i, j, l, xsta, ysta, &x_est,
				&y_est, &tri, x0, interface, azi,vapp*ratio,z0,0.0,NULL) )
			if( refine_turf( x0, xsta, ysta, interface, &x_est, &y_est,
				NULL, &n_arcs, &traveltime, &tri, azi,vapp*ratio,z0,0.0) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=x_est;
				tt[n_tts].th=y_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			}
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
			status=TRUE;
			x0[0]=tt[j].az;
			x0[1]=tt[j].th;
			x0[2]=z0;
			last_tetra=my_last_tetra;
			if( find_tetra(x0,&t) ) {
				vp = eval_velocity(&tetra[t],x0);
				}
			else status=FALSE;
			if( vp>=(vapp*ratio) ) status=FALSE;
			my_last_tetra=last_tetra;
			theta = 180.0-RTOD*asin(vp/(vapp*ratio));
			ray_tangent(theta,azi,t0);
			px = t0[0]/vp; py=t0[1]/vp;
		if( status) if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( status) if( !hitflag ) status=FALSE;
		}
	else status=FALSE;
	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].x0[0]=tt[j].az;
		ttime[itt].x0[1]=tt[j].th;
		ttime[itt].x0[2]=z0;
		ttime[itt].preTa=ratio*(
			traveltime + px*ttime[itt].x0[0] + py*ttime[itt].x0[1] );
		fprintf(stderr,"%d\t%f\t%f\n", itt, ttime[itt].obsTa, ttime[itt].preTa );
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		continue;
		}

	d[ng]=relative_weight*(ttime[itt].obsTa-ttime[itt].preTa);
	dindex[ng]=ttime[itt].index;
	dptr[ng]=itt;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio*relative_weight );
			}
		else {
			/* G[ng][j] = 0.0; */
			;
			}
		}

	ng++;

	} /* next traveltime */

	timestamp("at end of plane-wave-to-point raytracing");

	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d before demean:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	maxG=sparse_absmax(G);
	t0weight=maxG/10.0;
	k=sparse_count(G);
	fprintf(stderr,"\nBefore adjustment for t0 of planewaves\n");
	fprintf(stderr,"Top part of G is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );

        /* add elements of G associate with dt0 model parameters */
        for( i=0; i<ng; i++ ) {
        	j = ttime[dptr[i]].pwg;
        	set_sparse_value( G, i, n_model_pars+j, t0weight );
        	}
        /* subtract <ta> - <tt> from d */
       	i=0;
        while( i<ng ) {
        	j=i;
        	while( (j<ng) && (dindex[j]==dindex[i]) ) j++;
        	mean=0.0;
        	for( k=i; k<j; k++ ) mean+=d[k];
        	mean /= (double)(j-i);
        	for( k=i; k<j; k++ ) d[k]-=mean;
        	i=j;
        	}
        	
        maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

        fprintf(stderr,"\nAfter adjustment for t0 of planewaves\n");
	fprintf(stderr,"Top part of G is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars+n_pwg,
		100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );

	timestamp("after adjustment for t0 of planewaves");

	if( ng<100 && n_model_pars<6 ) {
		fprintf(stderr,"\nG, d after adjustment:\n");
		for(i=0; i<ng; i++ ) {
			fprintf(stderr,"%d\t", i);
			for( j=0; j<n_model_pars; j++ ) {
				fprintf(stderr,"%f\t", get_sparse_value(G,i,j) );
				}
			fprintf(stderr,"%f\n", d[i] );
			}
		fprintf(stderr,"\n");
		}

	ttime=V_ttime; n_ttimes=n_V_ttimes;

	fprintf(stderr,"Starting Point to Point Ray tracing of data\n\n");
	timestamp("at start of point-to-point raytracing");
	fprintf(stderr,"#\tobsTT\tpreTT\tres\n");

	for( itt=0; itt<n_ttimes; itt++ ) {

	if( itt==0 ) {
		match=FALSE;
		}
	else {
		if( ttime[itt].x0[0]==x0[0] && ttime[itt].x0[1]==x0[1] &&
			ttime[itt].x0[2]==x0[2] && ttime[itt].interface==interface ) 
			match=TRUE; else match=FALSE;
		}

	if( !match ) {
		for( i=0; i<3; i++ ) x0[i]=ttime[itt].x0[i];
		interface=ttime[itt].interface;
		if( !find_tetra(x0,&i) ) {
			fprintf(stderr,"error: %s: source location (%f %f %f) not in model\n", argv[0], x0[0], x0[1], x0[2] );
			exit(-1);
			}
		vs=eval_velocity(&tetra[i],x0);

		fprintf(stderr,"Shooting star pattern from source (%f %f %f) at velocity %f to interface %d, theta (%f, %f, %d) azi (%f %f %d)\n",
		x0[0], x0[1], x0[2], vs,
		interface, thmin, thmax, nth, azimin, azimax, nazi );
		shoot_star(thmin, thmax, nth, azimin, azimax, nazi, x0, interface);
		fprintf(stderr,"done with star\n\n");
		}
	xsta=ttime[itt].x1[0];
	ysta=ttime[itt].x1[1];
	n_tts=0;
	bracket_status=FALSE;
	for( i=0; i<(m_rays-1); i++ )
	for( j=0; j<(n_rays-1); j++ )
	for( l=0; l<2; l++ ) {
		if( instarbox(i, j, l, xsta, ysta, &th_est,
			&az_est, &tri, x0, interface, NULL) ) {
		bracket_status=TRUE;
		if( refine( x0, xsta, ysta, interface, &th_est, &az_est,
			NULL, &n_arcs, &traveltime, &tri) ) {
			if( (n_tts+2)<MAX_TTS ) {
				tt[n_tts].az=az_est;
				tt[n_tts].th=th_est;
				tt[n_tts].T=traveltime;
				n_tts++;
				}
			else {
				fprintf(stderr,"error: %s: too many arrivals\n", argv[0]);
				exit(-1);
				}
			} }
		}
	qsort( tt, n_tts, sizeof(struct tt_rec), compare_tt );
	if( order==(-1) ) setorder( tt, n_tts, ttime[itt].obsTa-ttime[itt].T0, ttime[itt].ratio, &ttime[itt].obsorder );

	if( (j=ttime[itt].obsorder) < n_tts ) {
		status=TRUE;
		ray_tangent(tt[j].th,tt[j].az,t0);
		if( !shoot_to_interface( x0, t0, x1, t1, arc, &n_arcs,
			MAX_ARCS, 10000.0, interface, &hitflag, &traveltime, &qinvint, &ec ) ) {
			status=FALSE;
			}
		if( !hitflag ) status=FALSE;
		}
	else status=FALSE;

	if( status ) {
		ttime[itt].ok=TRUE;
		ttime[itt].preorder=j;
		ttime[itt].preTT=traveltime*ttime[itt].ratio;
		ttime[itt].th=tt[j].th;
		ttime[itt].az=tt[j].az;
		fprintf(stderr,"%d\t%f\t%f\t%f\n", itt, 
			ttime[itt].obsTT, ttime[itt].preTT,
			ttime[itt].obsTT-ttime[itt].preTT);
		}
	else {
		ttime[itt].ok=FALSE;
		fprintf(stderr,"warning: lost observation %d\n", itt);
		if( !bracket_status ) {
			fprintf(stderr,"         ray was not bracketed\n" );
			}
		else {
			if( refine_star_status==1 ) {
				fprintf(stderr,"         derivative failed in refine\n" );
				}
			if( refine_star_status==2 ) {
				fprintf(stderr,"         ray came this close: %e\n", refine_star_norm );
				}
			}
		continue;
		}

	d[ng]=ttime[itt].obsTT-traveltime*ttime[itt].ratio;

	for( i=0; i<n_nodes; i++ ) node[i].touched=0;
	for( i=0; i<n_arcs; i++ ) {
		node[ arc[i].ptetra->n1 ].touched=1;
		node[ arc[i].ptetra->n2 ].touched=1;
		node[ arc[i].ptetra->n3 ].touched=1;
		node[ arc[i].ptetra->n4 ].touched=1;
		}
	for( i=0; i<n_nodes; i++ ) {
		if( node[i].touched ) node[i].hits++;
		node[i].touched=0;
		}

	for( j=0; j<n_model_pars; j++ ) {
		/* will varying this model parameter change the traveltime ? */
		match=FALSE;
		for( i=0; i<n_arcs; i++ ) {
			pt=arc[i].ptetra;
			tnode[0]=pt->n1; tnode[1]=pt->n2; tnode[2]=pt->n3; tnode[3]=pt->n4;
			for( k=0; k<4; k++ ) {
				if( node_in_model_par(tnode[k],j) ) {
					match=TRUE;
					break;
					}
				}
			if(match) break;
			}
		if( match ) {
			set_sparse_value( G, ng, j,
				dTdmp(j,arc,n_arcs,traveltime)*ttime[itt].ratio );
			}
		else {
			;/* G[ng][j] is zero */
			}
		}

	if( fabs(d[ng]) > max_dT ) {
		fprintf(stderr,"|%f|>%f, so ignoring this data \n", d[ng], max_dT );
		for( j=0; j<n_model_pars; j++ ) set_sparse_value( G, ng, j, 0.0 );
		}
	else {
		ng++;
		}

	} /* next traveltime */

	timestamp("at end of point_to_point raytracing");

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nAfter point-to-point raytracing\n");
	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n", ng, n_model_pars, 100.0*(double)k/((double)ng*(double)n_model_pars), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	fprintf(stderr,"Damping: %e\n\n", damping );
	
	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}
	fprintf(stderr,"GTd is %d by 1, and has absolute maximum of %f\n\n", n_model_pars, maxd );
	
	if( EXPLICIT_GTG ) {

	timestamp("before forming GTG");
	invert_GTG = smt_times_sm(G,G);
	if( invert_GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (invert_GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(invert_GTG);
	sparse_addtodiag(invert_GTG, damping*maxG);
	k=sparse_count(invert_GTG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_model_pars, n_model_pars,
	100.0*(double)k/((double)n_model_pars*(double)n_model_pars), maxG );

	timestamp("after forming GTG");

	for( i=0; i<n_model_pars; i++ ) {
		dm[i]=0.0;
		}
	if( !sparse_biconj( invert_GTG,GTd,dm,1,maxd*B_PREC,n_model_pars+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion (explicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after solving equations");
	
	sm_times_v( invert_GTG, dm, ts );

	if( n_model_pars < 1000 ) {
		fprintf(stderr,"\nGTG*dm=GTd\n");
		fprintf(stderr,"#\tobs\tpred\tdifference\n");
		}
	rms=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		if( n_model_pars < 1000 ) fprintf(stderr,"%d\t%f\t%f\t%f\n", i, GTd[i], ts[i], GTd[i]-ts[i] );
		rms+=(GTd[i]-ts[i])*(GTd[i]-ts[i]);
		}
	rms = sqrt(rms/(double)n_model_pars);
	fprintf(stderr,"\nrms solution error is %f\n\n", rms);

	} /* end explicit GTG */
	else {

	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );
	
	for( i=0; i<n_model_pars+n_pwg; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, n_model_pars+n_pwg+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion: %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	
	} /* end implicit GTG */

	if( n_model_pars < 1000 ) {
		fprintf(stderr, "Model parameter perturbations\n");
		fprintf(stderr, "#\tdm\n");
		for( i=0; i<n_model_pars; i++ ) {
			fprintf(stderr,"%i\t%f\n", i, dm[i] );
			}
		fprintf(stderr,"\n");
		}
		
	mean=fabs(dm[n_model_pars]);
	for( i=n_model_pars; i<n_model_pars+n_pwg; i++ ) {
		maxd=fabs(dm[i]);
		if( maxd>mean ) mean=maxd;
		}
	fprintf(stderr,"The largest dt0 is %e\n", mean*t0weight );
	
	sm_times_v( G, dm, ts2 );
	fprintf(stderr,"\nG*dm=dt\n");
	fprintf(stderr,"#\tobs_dt\tpred_st\tdifference\n");
	rms=0.0;
	rms2=0.0;
	for( i=0; i<ng; i++ ) {
		fprintf(stderr,"%d\t%f\t%f\t%f\n", i, d[i], ts2[i], d[i]-ts2[i] );
		rms+=(d[i]-ts2[i])*(d[i]-ts2[i]);
		rms2+=d[i]*d[i];
		}
	rms = sqrt(rms/(double)ng);
	rms2 = sqrt(rms2/(double)ng);
	fprintf(stderr,"\nlinearized rms traveltime error is %f compared to original %f\n", rms, rms2);
	fprintf(stderr,"%f percent reduction\n", 100.0*(1-rms/rms2));
	fprintf(stderr,"(but be warned; actual reduction, after re-raytracing, might be different\n");

	/* update velocity of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += weight*dm[i];
			}
		}
	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}

	fclose(nmfile);

	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);

	timestamp("at end");
		
	exit(0);
	}

main_fitmodel(argc, argv)
int argc;
char *argv[];
{
	int i, j, k, nv, nvmax, ng, t, nd[4];
	int nx, ny, nz;
	double x, y, z, vs, v1, v2, v3, v4, dv, damping, dvdn;
	struct v_rec {
		double x[3];
		double v;
		};
	struct v_rec *v;
	char ct[128];
	struct sparse_matrix *G, *GTG;
	double *d, *GTd, *dm, maxG, maxd, err;
	int biec, iter;
	double new_damping, scx;

	if( argc!=3 ) {
		fprintf(stderr,"purpose: adjust nodal velocities to fit v(x,y,z) values\n");
		fprintf(stderr,"usage: %s model_file damping < velocity_data.tsv > new_model_file\n",argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"      damping: damped least squares parameter  (fraction of largest element of GTG)\n");
		fprintf(stderr,"      velocity_data.tsv:\n");
		fprintf(stderr,"          first line: field names (skipped)\n");
		fprintf(stderr,"          subsequent lines: velocity data, one per line, with fields\n");
		fprintf(stderr,"          	x, in km\n");
		fprintf(stderr,"          	y, in km\n");
		fprintf(stderr,"          	z, in km\n");
		fprintf(stderr,"          	v, in km/s\n");
		fprintf(stderr,"      new_model_file: same format as model_file\n");
		exit(-1);
		}

	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	if( sscanf(argv[2],"%le",&damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	nvmax=1000;
	if( (v=(struct v_rec *)malloc((nvmax+2)*sizeof(struct v_rec)))==NULL ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}

	if( scanf("%s%s%s%s", ct,ct,ct,ct ) != 4 ) {
		fprintf(stderr,"error: %s: cant skip header in input\n",argv[0]);
		exit(-1);
		}

	nv=0;
 	while( scanf("%le%le%le%le", &x, &y, &z, &vs) == 4) {
		v[nv].x[0]=x; v[nv].x[1]=y; v[nv].x[2]=z; v[nv].v=vs;
		if( (nv+2) > nvmax ) {
			nvmax+=1000;
			if( (v=(struct v_rec *)realloc(v,(nvmax+2)*sizeof(struct v_rec)))==NULL ) {
				fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
				exit(-1);
				}
			}
			nv++;
		}
	fprintf(stderr,"%d v(x,y,z) data input\n", nv);

	G=new_sparse_matrix(nv,n_nodes,0);
	d=(double*)malloc((nv+2)*sizeof(double));
	GTd=(double*)malloc((n_nodes+2)*sizeof(double));
	dm=(double*)malloc((n_nodes+2)*sizeof(double));

	if( (G==NULL) || (d==NULL) || (GTd==NULL) || (dm==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (G, d ...)\n", argv[0] );
		exit(-1);
		}

	ng=0; dv=1.0e-4;
	for( i=0; i<nv; i++ ) if( find_tetra(v[i].x,&t) ) {
		vs=eval_velocity(&tetra[t],v[i].x);
		d[ng]=v[i].v-vs;
		nd[0]=tetra[t].n1; nd[1]=tetra[t].n2; nd[2]=tetra[t].n3; nd[3]=tetra[t].n4;
		for( j=0; j<4; j++ ) {
			node[nd[j]].v += dv;
			velocity_eqn( &(tetra[t]) );
			dvdn=(eval_velocity(&tetra[t],v[i].x)-vs)/dv;
			node[nd[j]].v = node[nd[j]].v_orig;
			if( !set_sparse_value(G,ng,nd[j],dvdn) ) {
				fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
				exit(-1);
				}
			}
		for( j=0; j<4; j++ ) node[nd[j]].v = node[nd[j]].v_orig;
		velocity_eqn( &(tetra[t]) );
		ng++;
		}

	if( ng!=nv ) fprintf(stderr,"warning: %s: %d data not within the model\n", argv[0], nv-ng );

	maxG=sparse_absmax(G);
	k=sparse_count(G);

	maxd=0.0;
	for( i=0; i<ng; i++ ) {
		dv=fabs(d[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,"\nG is %d by %d, is %f percent full, and has abs max of %f\n",
	ng, n_nodes, 100.0*(double)k/((double)ng*(double)n_nodes), maxG );
	fprintf(stderr,"\nd is %d by 1 and  has abs max of %f\n\n", ng, maxd );
	
	fprintf(stderr,"Damping: %e\n\n", damping );

	smt_times_v( G, d, GTd );

	maxd=0.0;
	for( i=0; i<n_nodes; i++ ) {
		dv=fabs(GTd[i]);
		if( dv>maxd ) maxd=dv;
		}

	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n\n",
	n_nodes, maxd );
	
	if( EXPLICIT_GTG ) {

	GTG = smt_times_sm(G,G);
	if( GTG==NULL ) {
		fprintf(stderr,"error: %s: out of memory (GTG)\n", argv[0] );
		exit(-1);
		}
	maxG=sparse_absmax(GTG);
	sparse_addtodiag(GTG, damping*maxG);
	k=sparse_count(GTG);

	fprintf(stderr,
	"GTG is %d by %d, is %f percent full, and has absolute maximum of %f\n\n",
	n_nodes, n_nodes,
	100.0*(double)k/((double)n_nodes*(double)n_nodes), maxG );

	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_biconj(GTG,GTd,dm,1,maxd*B_PREC,n_nodes+10,&iter,&err,&biec) ) {
		fprintf(stderr,"error: %s: biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
	fprintf(stderr,"Matrix inversion (explicit): %d iterations; rms %f\n\n", iter, err );
	
	}
	else {
	
	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );

	for( i=0; i<n_model_pars; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj(G,d,dm,new_damping,1,maxd*B_PREC,n_nodes+10,&iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	} /* after implicit solution */

	/* write out updated model */
	printf("%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		printf("%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v+dm[i], 1.0/node[i].qinv, node[i].ifc );
		else
		printf("%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v+dm[i], node[i].ifc );
		}


	exit(0);
	}

main_modelpars(argc, argv)
int argc;
char *argv[];
{
	int status;

	if( argc<2 ) {
		fprintf(stderr,"purpose: generate model parameter file from model file,\n");
		fprintf(stderr,"         for certain commonly used types of model parameters\n");
		fprintf(stderr,"usage: %s keyword parameter_1 ... < model_file > model_parameter_file\n", argv[0] );
		fprintf(stderr,"       with the following keywords and parameters:\n");
		fprintf(stderr,"       vnodes interface weight\n");
		fprintf(stderr,"           one velocity model parameter for each node on\n");
		fprintf(stderr,"           the specified interface. All model parameters on\n");
		fprintf(stderr,"           that interface have the same weight. Suggest weight=1.\n");
		fprintf(stderr,"       vnodesavg interface neighbors scale weight\n");
		fprintf(stderr,"           one velocity model parameter for each node on\n");
		fprintf(stderr,"           the specified interface, consisting of a localized\n");
		fprintf(stderr,"           average of nearest neighbor nodes. Setting neighbors=1\n");
		fprintf(stderr,"           creates a model parameters with 9 nodes, neighbors=2\n");
		fprintf(stderr,"           25 nodes, etc. The averaging is gaussian in horizontal\n");
		fprintf(stderr,"           distance, R, from the central node, that is,\n");
		fprintf(stderr,"           exp(-R^2/scale^2).  An overall weight is also\n");
		fprintf(stderr,"           supplied. It is the same for all nodes on the\n");
		fprintf(stderr,"           interface. Suggest weight=1. Warning: Does not\n");
		fprintf(stderr,"           correctly handle case where interface lies on multiple\n");
		fprintf(stderr,"           layers in the model.\n");
		fprintf(stderr,"       vnodesbase interface order weight\n");
		fprintf(stderr,"           implements model parameters in which all nodes\n");
		fprintf(stderr,"           on an interface vary according to a low order\n");
		fprintf(stderr,"           (0<=order<=3) polynomial in horizontal distance:\n");
		fprintf(stderr,"         v(i) =  m0 + m1*x' + m2*y'\n");
		fprintf(stderr,"         	+  m3*(x'**2-4/3)\n");
		fprintf(stderr,"         	+  m4*(y'**2-4/3)\n");
		fprintf(stderr,"         	+  m5*y'*x'\n");
		fprintf(stderr,"         	+  m6*x'**3'\n");
		fprintf(stderr,"         	+  m7*y'*x'**2\n");
		fprintf(stderr,"         	+  m8*y'**2'*x'\n");
		fprintf(stderr,"         	+  m9*y'**3\n");
		fprintf(stderr,"         where the (x',y') coordinates are in the\n");
		fprintf(stderr,"         horizontal plane, centered on the center of\n");
		fprintf(stderr,"         of the model and reach +/-1 at its edges.\n");
		fprintf(stderr,"         The factor of 4/3 is added so that all higher\n");
		fprintf(stderr,"         order terms have zero mean on [-1,+1].\n");
		fprintf(stderr,"       inodes interface Nx Ny Nz weight\n");
		fprintf(stderr,"           one interface position model parameter for each node on\n");
		fprintf(stderr,"           the specified interface. All model parameters on\n");
		fprintf(stderr,"           that interface have the same unit vector and the\n");
		fprintf(stderr,"           same weight. Suggest Nx=0, Ny=0, Nz=1, weight=1.\n");
		fprintf(stderr,"Note: you can build up a model parameter file by concatenating\n");
		fprintf(stderr,"      the output of several calls to this software (e.g. one\n");
		fprintf(stderr,"      call per interface)\n");
		exit(-1);
		}

	if( !strcmp(argv[1],"vnodes") ) {
		status=mp_vnodes(argc-1,argv+1);
		}
	else if( !strcmp(argv[1],"vnodesavg") ) {
		status=mp_vnodesavg(argc-1,argv+1);
		}
	else if( !strcmp(argv[1],"vnodesbase") ) {
		status=mp_vnodesbase(argc-1,argv+1);
		}
	else if( !strcmp(argv[1],"inodes") ) {
		status=mp_inodes(argc-1,argv+1);
		}
	else {
		fprintf(stderr,"Error: %s: no such keyword %s\n", argv[0], argv[1] );
		exit(-1);
		}
	if(status) exit(0); else exit(-1);
	}

int mp_vnodes(int argc, char *argv[])
{
	double weight;
	int i, N, NX, NY, NZ, interface, count;
	char line[2048], str1[128], str2[128];
	int IX, IY, IZ, INTERFACE;
	double X, Y, Z, V;

	if( argc != 3 ) {
		fprintf(stderr,"error: bad arguments, vnodes keyword\n");
		return(0);
		}

	if( sscanf(argv[1], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line, argv[0]\n" );
		return(0);
		}

	if( sscanf(argv[2], "%le", &weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read weight from command line, argv[0]\n" );
		return(0);
		}

	line[0]='\0';
	fgets(line,2040,stdin);
	if( sscanf(line,"%d%d%d", &NX, &NY, &NZ ) != 3 ) {
		fprintf(stderr,"error: %s: cant read model size, argv[0]\n" );
		exit(-1);
		}

	N = NX*NY*NZ;

	for( i=0; i<N; i++ ) {
		line[0]='\0';
		fgets(line,2040,stdin);
		if( (count=sscanf(line,"%d%d%d%le%le%le%le%s%s\n", &IX, &IY, &IZ,
			&X, &Y, &Z, &V, str1, str2 )) < 8 ) {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			return(0);
			}
		if( count==8 ) {
			if( sscanf(str1,"%d", &INTERFACE ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		else if( count==9 ) {
			if( sscanf(str2,"%d", &INTERFACE ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		else {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			exit(0);
			}
		if( interface==INTERFACE ) {
			printf("1\n");
			printf("%d %d %d %f\n", IX, IY, IZ, weight );
			}
		}

	return(1);
	}

int mp_inodes(int argc, char *argv[])
{
	double weight;
	int i, N, NX, NY, NZ, interface, count;
	char line[2048], str1[128], str2[128];
	int IX, IY, IZ, INTERFACE;
	double X, Y, Z, V, tx, ty, tz;

	if( argc != 6 ) {
		fprintf(stderr,"error: bad arguments, inodes keyword\n");
		return(0);
		}

	if( sscanf(argv[1], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[2], "%le", &tx) != 1 ) {
		fprintf(stderr,"error: %s: cant read Nx from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[3], "%le", &ty) != 1 ) {
		fprintf(stderr,"error: %s: cant read Ny from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[4], "%le", &tz) != 1 ) {
		fprintf(stderr,"error: %s: cant read Nz from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[5], "%le", &weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read weight from command line, argv[0]\n" );
		return(0);
		}

	line[0]='\0';
	fgets(line,2040,stdin);
	if( sscanf(line,"%d%d%d", &NX, &NY, &NZ ) != 3 ) {
		fprintf(stderr,"error: %s: cant read model size, argv[0]\n" );
		exit(-1);
		}

	N = NX*NY*NZ;

	for( i=0; i<N; i++ ) {
		line[0]='\0';
		fgets(line,2040,stdin);
		if( (count=sscanf(line,"%d%d%d%le%le%le%le%s%s\n", &IX, &IY, &IZ,
			&X, &Y, &Z, &V, str1, str2 )) < 8 ) {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			return(0);
			}
		if( count==8 ) {
			if( sscanf(str1,"%d", &INTERFACE ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		else if( count==9 ) {
			if( sscanf(str2,"%d", &INTERFACE ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		else {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			exit(0);
			}
		if( interface==INTERFACE ) {
			printf("1\n");
			printf("%d %d %d %f %f %f %f\n", IX, IY, IZ, tx, ty, tz, weight );
			}
		}

	return(1);
	}

int mp_vnodesavg(int argc, char *argv[])
{
	double weight;
	int i, j, N, NX, NY, NZ, interface, neighbors, count;
	int ix, iy;
	char line[2048], str1[128], str2[128];
	int *IX, *IY, *IZ, *INTERFACE;
	double *X, *Y, *Z, *V;
	int IXt, IYt, IZt, INTERFACEt;
	double Xt, Yt, Zt, Vt;
	double sum, R2, scale;

	if( argc != 5 ) {
		fprintf(stderr,"error: bad arguments, vnodesavg keyword\n");
		return(0);
		}

	if( sscanf(argv[1], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[2], "%d", &neighbors) != 1 ) {
		fprintf(stderr,"error: %s: cant read neighbors from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[3], "%le", &scale) != 1 ) {
		fprintf(stderr,"error: %s: cant read scale from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[4], "%le", &weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read weight from command line, argv[0]\n" );
		return(0);
		}

	line[0]='\0';
	fgets(line,2040,stdin);
	if( sscanf(line,"%d%d%d", &NX, &NY, &NZ ) != 3 ) {
		fprintf(stderr,"error: %s: cant read model size, argv[0]\n" );
		exit(-1);
		}

	N = NX*NY*NZ;

	IX = (int *) malloc( N*sizeof(int) );
	if( IX == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	IY = (int *) malloc( N*sizeof(int) );
	if( IY == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	IZ = (int *) malloc( N*sizeof(int) );
	if( IZ == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	INTERFACE = (int *) malloc( N*sizeof(int) );
	if( INTERFACE == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	X = (double*) malloc( N*sizeof(double) );
	if( X == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	Y = (double*) malloc( N*sizeof(double) );
	if( Y == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	Z = (double*) malloc( N*sizeof(double) );
	if( Z == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	V = (double*) malloc( N*sizeof(double) );
	if( V == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};


	for( i=0; i<N; i++ ) {
		line[0]='\0';
		fgets(line,2040,stdin);
		if( (count=sscanf(line,"%d%d%d%le%le%le%le%s%s\n", &IXt, &IYt, &IZt,
			&Xt, &Yt, &Zt, &Vt, str1, str2 )) < 8 ) {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			return(0);
			}
		if( count==8 ) {
			if( sscanf(str1,"%d", &INTERFACEt ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		else if( count==9 ) {
			if( sscanf(str2,"%d", &INTERFACEt ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				return(0);
				}
			}
		j=IXt + IYt*NX + IZt*NY*NY;
		IX[j]=IXt; IY[j]=IYt; IZ[j]=IZt; X[j]=Xt; Y[j]=Yt; Z[j]=Zt; V[j]=Vt;
		INTERFACE[j]=INTERFACEt;
		}

	for( i=0; i<N; i++ ) {
		if( INTERFACE[i]!=interface ) continue;
		count=0;
		sum=0.0;
		for( ix=IX[i]-neighbors; ix<=IX[i]+neighbors; ix++ )
		for( iy=IY[i]-neighbors; iy<=IY[i]+neighbors; iy++ ) {
			if( (ix<0) || (ix>=NX) || (iy<0) || (iy>=NY) ) continue;
			j=ix + iy*NX + IZ[i]*NY*NY;
			if( INTERFACE[j]!=interface ) continue;
			R2 = (X[j]-X[i])*(X[j]-X[i]) + (Y[j]-Y[i])*(Y[j]-Y[i]);
			sum+=exp(-R2/(scale*scale));
			count++;
			}
		printf("%d\n",count);
		for( ix=IX[i]-neighbors; ix<=IX[i]+neighbors; ix++ )
		for( iy=IY[i]-neighbors; iy<=IY[i]+neighbors; iy++ ) {
			if( (ix<0) || (ix>=NX) || (iy<0) || (iy>=NY) ) continue;
			j=ix + iy*NX + IZ[i]*NY*NY;
			if( INTERFACE[j]!=interface ) continue;
			R2 = (X[j]-X[i])*(X[j]-X[i]) + (Y[j]-Y[i])*(Y[j]-Y[i]);
			printf("%d %d %d %e\n", IX[j], IY[j], IZ[j],
				exp(-R2/(scale*scale))*weight/sum );
			}
		}

	return(1);
	}

int mp_vnodesbase(int argc, char *argv[])
{
	double o_weight;
	int i, j, N, NX, NY, NZ, interface, neighbors, count, order, found;
	int ix, iy;
	char line[2048], str1[128], str2[128];
	int *IX, *IY, *IZ, *INTERFACE;
	double *X, *Y, *Z, *V;
	int IXt, IYt, IZt, INTERFACEt;
	double Xt, Yt, Zt, Vt;
	double xmin, xmax, ymin, ymax;
	double xmean, ymean, xscale, yscale;
	double xp, yp, weight, weight_mean;

	if( argc != 4 ) {
		fprintf(stderr,"error: bad arguments, vnodesbase keyword\n");
		return(0);
		}

	if( sscanf(argv[1], "%d", &interface) != 1 ) {
		fprintf(stderr,"error: %s: cant read interface from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[2], "%d", &order) != 1 ) {
		fprintf(stderr,"error: %s: cant read order from command line, argv[0]\n" );
		return(0);
		}
	if( (order<0) || (order>3) ) {
		fprintf(stderr,"error: %s: order must be between 0 and 3\n", argv[0] );
		return(0);
		}
	if( sscanf(argv[3], "%le", &o_weight) != 1 ) {
		fprintf(stderr,"error: %s: cant read weight from command line, argv[0]\n" );
		return(0);
		}

	line[0]='\0';
	fgets(line,2040,stdin);
	if( sscanf(line,"%d%d%d", &NX, &NY, &NZ ) != 3 ) {
		fprintf(stderr,"error: %s: cant read model size, argv[0]\n" );
		exit(-1);
		}

	N = NX*NY*NZ;

	IX = (int *) malloc( N*sizeof(int) );
	if( IX == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	IY = (int *) malloc( N*sizeof(int) );
	if( IY == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	IZ = (int *) malloc( N*sizeof(int) );
	if( IZ == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	INTERFACE = (int *) malloc( N*sizeof(int) );
	if( INTERFACE == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	X = (double*) malloc( N*sizeof(double) );
	if( X == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	Y = (double*) malloc( N*sizeof(double) );
	if( Y == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	Z = (double*) malloc( N*sizeof(double) );
	if( Z == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};

	V = (double*) malloc( N*sizeof(double) );
	if( V == NULL ) {
		fprintf(stderr,"error: %s: out of memory!\n", argv[0] );
		exit(0);
		};


	for( i=0; i<N; i++ ) {
		line[0]='\0';
		fgets(line,2040,stdin);
		if( (count=sscanf(line,"%d%d%d%le%le%le%le%s%s\n", &IX[i], &IY[i], &IZ[i],
			&X[i], &Y[i], &Z[i], &V[i], str1, str2 )) < 8 ) {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			exit(0);
			}
		if( count==8 ) {
			if( sscanf(str1,"%d", &INTERFACE[i] ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				exit(0);
				}
			}
		else if( count==9 ) {
			if( sscanf(str2,"%d", &INTERFACE[i] ) != 1 ) {
				fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
				exit(0);
				}
			}
		else {
			fprintf(stderr,"error: %s: error reading model!\n", argv[0] );
			exit(0);
			}

		if( i==0 ) {
			xmin=X[0]; xmax=X[0];
			ymin=Y[0]; ymax=Y[0];
			}
		else {
			if( X[i]<xmin ) xmin=X[i];
			if( X[i]>xmax ) xmax=X[i];
			if( Y[i]<ymin ) ymin=Y[i];
			if( Y[i]>ymax ) ymax=Y[i];
			}
		}
	xmean = 0.5*(xmax+xmin); ymean = 0.5*(ymax+ymin);
	xscale = 0.5*(xmax-xmin); yscale = 0.5*(ymax-ymin);

	count=0;
	for( j=0; j<N; j++ ) if( INTERFACE[j] == interface ) count++;
	if( count == 0 ) {
		fprintf(stderr,"error: %s: interface not in model\n", argv[0] );
		return(0);
		}

	/* one constant term */
	if( order>=0 ) {
		/* constant term */
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			weight=1.0; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		}

	/* two linear terms */
	if( order>=1 ) {
		/* x term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=xp-weight_mean;  if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		/* y term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=yp-weight_mean;  if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		}

	/* three quadratic terms */
	if( order>=2 ) {
		/* x*x term */
		weight_mean=4.0/3.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(xp*xp)-weight_mean; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		/* y*y term */
		weight_mean=4.0/3.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(yp*yp)-weight_mean;  if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		printf("%d\n", count );
		/* x*y term */
		weight_mean=0.0;
		for( j=0; j<N; j++ ) {
			if( INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(xp*yp)-weight_mean; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		}

	/* four cubic terms */
	if( order>=3 ) {
		/* x*x*x term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if(  INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(xp*xp*xp)-weight_mean; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		/* y*x*x term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if(  INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(yp*xp*xp)-weight_mean; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		/* y*y*x term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if(  INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(yp*yp*xp)-weight_mean; if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		/* y*y*y term */
		weight_mean=0.0;
		printf("%d\n", count );
		for( j=0; j<N; j++ ) {
			if(  INTERFACE[j] != interface ) continue;
			xp=(X[j]-xmean)/xscale; yp=(Y[j]-ymean)/yscale;
			weight=(yp*yp*yp)-weight_mean;  if( weight==0.0 ) weight=1.0e-6;
			printf("%d %d %d %f\n", IX[j], IY[j], IZ[j], weight*o_weight );
			}
		}


	return(1);
	}

main_make_model(argc, argv)
int argc;
char *argv[];
{
	int i, j, k;
	int nx, ny, nz;
	double xmin, xmax, ymin, ymax, zmin, zmax, A, B, C, D;
	double v, dx, dy, dz, x, y, z;

	if( argc<14 ) {
		fprintf(stderr,"purpose: generate a simple, vertically stratified earth model\n");
		fprintf(stderr,"         with equally-spaced nodes parallel to the x,y,z axes.\n");
		fprintf(stderr,"         (This software makes only the simplist of models. To\n");
		fprintf(stderr,"         make more complicated ones, use some scripting software\n");
		fprintf(stderr,"         like awk, perl, MATLAB, etc.)\n");

		fprintf(stderr,"usage: %s xmin xmax nx ymin ymax ny zmin zmax nz A B C D > model_file\n",
				argv[0] );
		fprintf(stderr,"       xmin, xmax, in km, minimum and maximum value of x\n");
		fprintf(stderr,"       nx, number of nodes in x direction\n");
		fprintf(stderr,"       ymin, ymax, in km, minimum and maximum value of y\n");
		fprintf(stderr,"       ny, number of nodes in x direction\n");
		fprintf(stderr,"       zmin, zmax, in km, minimum and maximum value of z\n");
		fprintf(stderr,"       nz, number of nodes in x direction\n");
		fprintf(stderr,"       A, B, C, D coefficients to the velocity equation,\n");
		fprintf(stderr,"       v=A+Bz+C*sqrt(z/D), where v is velocity in km/s\n");
		exit(-1);
		}

	if( sscanf(argv[1], "%le", &xmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmin from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[2], "%le", &xmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read xmax from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[3], "%de", &nx) != 1 ) {
		fprintf(stderr,"error: %s: cant read nx from command line, argv[0]\n" );
		return(0);
		}

	if( sscanf(argv[4], "%le", &ymin) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymin from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[5], "%le", &ymax) != 1 ) {
		fprintf(stderr,"error: %s: cant read ymax from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[6], "%de", &ny) != 1 ) {
		fprintf(stderr,"error: %s: cant read ny from command line, argv[0]\n" );
		return(0);
		}

	if( sscanf(argv[7], "%le", &zmin) != 1 ) {
		fprintf(stderr,"error: %s: cant read zmin from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[8], "%le", &zmax) != 1 ) {
		fprintf(stderr,"error: %s: cant read zmax from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[9], "%de", &nz) != 1 ) {
		fprintf(stderr,"error: %s: cant read nz from command line, argv[0]\n" );
		return(0);
		}

	if( sscanf(argv[10], "%le", &A) != 1 ) {
		fprintf(stderr,"error: %s: cant read A from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[11], "%le", &B) != 1 ) {
		fprintf(stderr,"error: %s: cant read B from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[12], "%le", &C) != 1 ) {
		fprintf(stderr,"error: %s: cant read C from command line, argv[0]\n" );
		return(0);
		}
	if( sscanf(argv[13], "%le", &D) != 1 ) {
		fprintf(stderr,"error: %s: cant read D from command line, argv[0]\n" );
		return(0);
		}

	printf("%d %d %d\n", nx, ny, nz );
	dx = (xmax-xmin)/(double)(nx-1);
	dy = (ymax-ymin)/(double)(ny-1);
	dz = (zmax-zmin)/(double)(nz-1);
	for( i=0; i<nx; i++ ) { x=xmin+dx*(double)i;
	for( j=0; j<ny; j++ ) { y=ymin+dy*(double)j;
	for( k=0; k<nz; k++ ) { z=zmin+dz*(double)k;
		v=A+B*z+C*sqrt(z/D);
		printf("%d %d %d %f %f %f %f %d\n", i, j, k, x, y, z, v, k );
		}}}

	exit(0);
	}

/*solves G x = d by damped-least squares using bicongugate gradients, that is, by solving
  (G-transpose G + e2 I) x = G-transpose b for x , but not ever computing (G-transpose G + e2 I)
  directly.
   G: sparse matrix (must have rows >= columns)
   d: l.h.s. vector
   e2: damping parameter, must be non-negative
   x: solution; initialize to guess (e.g. 0.0) before call
   itol: 1, 2, 3, 4 depending on choice of convergence test
   tol: tolerance
   itmax: maximum iterations
   iter: actual iterations
   err: error estimate
   */
int sparse_dls_biconj( struct sparse_matrix *G, double *d, double *x, double e2, int itol, double tol,
	int itmax, int *iter, double *err, int *biec)
{
	int i, j, m, n, n_diagonal;
	double ak, akden, bk, bkden, bknum, bnorm, dxnorm, xnorm, zm1norm, znorm, t;
	double *p=NULL, *pp=NULL, *r=NULL, *rr=NULL, *z=NULL, *zz=NULL, *b=NULL, *mtemp=NULL, *diagonal=NULL;
	

	if( G==NULL ) {
		fprintf(stderr,"error: biconj: NULL Matrix\n");
		*biec=1;
		return(0);
		}
	
	/* the reversal of the sense of m and n is intentional */
	m=G->n; /*rows*/
	n=G->m; /*columns*/

	if( (itol<1) || (itol>4) ) {
		fprintf(stderr,"error: biconj: bad itol\n");
		*biec=3;
		return(0);
		}
		
	diagonal=AT_times_A_pe2( G, e2, &n_diagonal);
	if( diagonal==NULL ) {
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}
	if( n_diagonal!=n ) {
		fprintf(stderr,"error: bicong: internal error: size mismatch\n");
		*biec=4;
		return(0);
		}
		
	b=(double*)malloc((n+10)*sizeof(double));
	mtemp=(double*)malloc((m+10)*sizeof(double));
	p=(double*)malloc((n+10)*sizeof(double));
	pp=(double*)malloc((n+10)*sizeof(double));
	r=(double*)malloc((n+10)*sizeof(double));
	rr=(double*)malloc((n+10)*sizeof(double));
	z=(double*)malloc((n+10)*sizeof(double));
	zz=(double*)malloc((n+10)*sizeof(double));
	if( (p==NULL) || (pp==NULL) || (r==NULL) || (rr==NULL)
         || (b==NULL) || (z==NULL) || (zz==NULL) || (mtemp==NULL) ) {
         	if( b!=NULL ) free(b);
		if( p!=NULL ) free(p);
         	if( mtemp!=NULL ) free(mtemp);
		if( pp!=NULL ) free(pp);
		if( r!=NULL ) free(r);
		if( rr!=NULL ) free(rr);
		if( z!=NULL ) free(z);
		if( zz!=NULL ) free(zz);
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}
		
	smt_times_v( G, d, b );

	*iter=0;
	sparse_GTGpe2times(G,x,r,e2,mtemp);
	for( j=0; j<n; j++ ) {
		r[j]=b[j]-r[j];
		rr[j]=r[j];
		}
	znorm=1.0;
	if( itol==1) {
		bnorm=snorm(n,b,itol);
		}
	else if( itol==2 ) {
		sparse_GTGpe2solve(diagonal,b,z,n);
		bnorm=snorm(n,z,itol);
		}
	else {
		sparse_GTGpe2solve(diagonal,b,z,n);
		bnorm=snorm(n,z,itol);
		sparse_GTGpe2solve(diagonal,r,z,n);
		znorm=snorm(n,z,itol);
		}

	sparse_GTGpe2solve(diagonal,r,z,n);

	while( (*iter) <= itmax ) {
		(*iter)++;
		zm1norm=znorm;
		sparse_GTGpe2solve(diagonal,rr,zz,n);
		bknum=0.0;
		for( j=0; j<n; j++ ) bknum+=z[j]*rr[j];
		if( (*iter)==1 ) {
			for( j=0; j<n; j++ ) {
				p[j]=z[j];
				pp[j]=zz[j];
				}
			}
		else {
			bk=bknum/bkden;
			for( j=0; j<n; j++ ) {
				p[j]=bk*p[j]+z[j];
				pp[j]=bk*pp[j]+zz[j];
				}
			}
		bkden=bknum;
		sparse_GTGpe2times(G,p,z,e2,mtemp);
		akden=0.0;
		for( j=0; j<n; j++ ) akden+=z[j]*pp[j];
		ak=bknum/akden;
		sparse_GTGpe2times(G,pp,zz,e2,mtemp);
		for( j=0; j<n; j++ ) {
			x[j]+=ak*p[j];
			r[j]-=ak*z[j];
			rr[j]-=ak*zz[j];
			}
		sparse_GTGpe2solve(diagonal,r,z,n);
		if( (itol==1) || (itol==2) ) {
			znorm=1.0;
			(*err)=snorm(n,r,itol)/bnorm;
			}
		else {
			znorm=snorm(n,z,itol);
			if( (t=fabs(zm1norm-znorm)) > (EPS*znorm) ) {
				dxnorm=fabs(ak)*snorm(n,p,itol);
				(*err)=znorm/t*dxnorm;
				}
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			xnorm=snorm(n,x,itol);
			if( (*err)<=(0.5*xnorm) ) (*err)/=xnorm;
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			}
		if( (*err)<=tol) break;
		}

	free(p); free(pp); free(r); free(rr); free(z); free(zz); free(b); free(mtemp); free(diagonal);
	if( (*err)<=tol) {*biec=0; return(1);}
	else {*biec=(-1); return(0);}
	}
	
/* internal routine, not intended as user callable */
/* performs r = ( G-transpose G + e2 I ) x */
int sparse_GTGpe2times( struct sparse_matrix *G, double *x, double *r, double e2, double *mtemp )
{
	int i, m;
	
	m = G->m;
	sm_times_v( G, x, mtemp );
	smt_times_v( G, mtemp, r );
	for( i=0; i<m; i++ ) r[i] += e2*x[i];
	return(1);
	}
	
/* internal routine, not intended as user callable */
int sparse_GTGpe2solve( double *diagonal, double *b, double *x, int n )
/* solves ( G-transpose G + e2 I ) x = b by ignoring all but main diagonal */
{
	int i;
	double t;
	for( i=0; i<n; i++ ) {
		t=diagonal[i];
		if( t!=0.0 ) x[i]=b[i]/t;
		else x[i]=0.0;
		}
	return(1);
	}
	
	
/* diagonal of A-transpose * A + e2 I*/
double  *AT_times_A_pe2( struct sparse_matrix *A, double e2, int *M)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB, iscolB, nC;
	double sum, *diagonal;

	if( A==NULL ) return(NULL);
	nA=A->n; mA=A->m;
	nB=A->n; mB=A->m;
	nC=mA;
	
	if( (diagonal=(double *)malloc(nC*sizeof(double))) == NULL ) {
		*M=0;
		return(NULL);
		}
	else  *M=nC;

	if( A->sym  ) {
		fprintf(stderr,"error: AT_times_A: sorry: case of symmetric A not implemented\n");
		exit(-1);
		}
	else {
		for( i=0; i<nC; i++ ) { 
			j=i;
			sum=0.0;
			eA=A->col[i]; eB=A->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->i; kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
				}
			diagonal[i]=sum+e2;
			}
		}


	return(diagonal);
	}
	

		
main_locate_and_invert(int argc, char **argv)
{
	int i, j, k, l, m, n, ngood, status, iter, biec, irms, line, match;
	int ii, jj, kk;
	char str[1024], hitsfile[1024];
	double x0[3], x1[3], t1[3];
        int tnode[4];
	int nx, ny, nz;

	struct arc_rec arc[MAX_ARCS];
	int n_arcs;

        struct tetra_rec *pt;

	FILE *fsta, *nmfile;
	int *stanum;
	double *stax, *stay, *staz;
	char **jhdfilename;
	int n_sta;
	double x,y,z,t0,c,azi,theta,weight;
	double T, dTdx, dTdy, dTdz;

	FILE *fev;
	struct ev_rec *ev, evkey, *myev;
	int n_ev;

	FILE *fph;
	struct ph_rec {
		double obs, weight, maxres;
		int n, good, row, line;
		int *sta, *ev, *order, *hit;
		double *ratio, *coeff;
		double *T, *dTdx, *dTdy, *dTdz;
		double *azi, *theta;
		double *stax, *stay, *staz;
		};
	struct ph_rec *ph;
	int n_ph;

	FILE *fjhd, *mpfile;
	struct jhd_rec *jhd;
	int n_jhd;
	struct jhd_rec jhdkey, *myjhd;

	int verbose=0;

	struct sparse_matrix *G, *GTG;
	double *d, *dp, *GTd, *dm, tscale=TSCALEVALUE, maxG, maxd, damping, rw, err, res, rms1, rms2;
	double *starms, *stamean;
	int *stacount;
	double *Grow;

	double new_damping, scx;

	if( argc!=9 ) {
		fprintf(stderr,"purpose: joint hypocenter relocation and velocity tomography\n");
		fprintf(stderr,"method: The user provides a set of observations, where\n");
		fprintf(stderr,"        each observation is assumed to be a linear combination\n");
		fprintf(stderr,"        of traveltimes:\n");
		fprintf(stderr,"           observation = (sum i=1 to n) coefficient[i] * traveltime[i]\n");
		fprintf(stderr,"        A phase file tabulates the observations and the\n");
		fprintf(stderr,"        n coefficients and the station, event and order\n");
		fprintf(stderr,"        of each of the traveltimes.  The observations are\n");
		fprintf(stderr,"        used to produce an updated estimate of the event\n");
		fprintf(stderr,"        locations and the velocity model.\n");
		fprintf(stderr,"usage: %s model_file.txt model_parameter_file.txt station_file.txt event_file.txt phase_file.txt new_model_file damping relative_weight > updated_event_file.txt\n", argv[0]);
		write_model_blerb(stderr);
		fprintf(stderr,"       model_file.txt: standard velocity model file\n");
		fprintf(stderr,"       model_parameter_file.txt: standard velocity node model parameter file\n");
		fprintf(stderr,"       station_file.txt: one station per line, with fields\n");
		fprintf(stderr,"               station_number: arbitrary integer\n");
		fprintf(stderr,"               x, y, z: station location, km\n");
		fprintf(stderr,"               jhd_table: path of station-specific jhd-table\n");
		fprintf(stderr,"       event_file.txt: one event per line, with fields\n");
		fprintf(stderr,"               event_number: arbitrary integer\n");
		fprintf(stderr,"               x, y, z: preliminary location, km\n");
		fprintf(stderr,"               t0: preliminary origin time, s\n");
		fprintf(stderr,"       phase_file.txt: one group of lines per observation\n");
		fprintf(stderr,"               first line, with fields\n");
		fprintf(stderr,"                     n_phases: number of phases in observation\n");
		fprintf(stderr,"                     observation: a differetial traveltime, s\n");
		fprintf(stderr,"                     weight: weight of observation\n");
		fprintf(stderr,"                     maxres: observation thrown out if weighted residual > maxres, s\n");
		fprintf(stderr,"               subsequent lines in group, with fields\n");
		fprintf(stderr,"                     station_number, corresponding to station_file.txt\n");
		fprintf(stderr,"                     event_number, corresponding to event_file.txt\n");
		fprintf(stderr,"                     order, interger arrival number of phase\n");
		fprintf(stderr,"                     ratio, computed traveltimes multiplied by this\n");
		fprintf(stderr,"                            (i.e. use 1.o for P 1.76 for S)\n");
		fprintf(stderr,"                     coefficient, of phase in observation\n");
		fprintf(stderr,"       new_model_file.txt: updated model file. A corresponding\n");
		fprintf(stderr,"       		hits file, with suffix .hits, is also created\n");
		fprintf(stderr,"       damping: damping factor, fraction of max of GTG\n");
		fprintf(stderr,"       relative_weight: relative weight of event and velocity model parameters\n");
		fprintf(stderr,"               relative_weigth>1 favors large of velocity model perturbations\n");
		fprintf(stderr,"               relative_weigth<1 favors large of event location perturbations\n");
		fprintf(stderr,"Note: In the inversion, origin time perturbations are scaled\n");
		fprintf(stderr,"to units of distance using a nominal velocity of %f km/s\n",
			1.0/TSCALEVALUE );
		exit(-1);
		}

	if( sscanf(argv[7],"%le",&damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[8],"%le",&rw) != 1 ) {
		fprintf(stderr,"error: %s: cant read relative weight from command line\n", argv[0] );
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d locate_and_invert\n");
	fprintf(stderr,"\tdamping %e\n", damping);
	fprintf(stderr,"\trelative weight %e\n", rw);
	fprintf(stderr,"\tstarting model %s\n", argv[1] );

	/* model file */
	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}

	/* model parameter file */
	alloc_model_pars(argv[2]);
	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);
	fprintf(stderr,"\tmodel parameters %s with %d parameters\n", argv[2], n_model_pars );

	if( (nmfile=fopen(argv[6],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[6] );
		exit(-1);
		}

	/* station file */
	n_sta=count_lines(argv[3]);
	stanum=(int *)malloc(n_sta*sizeof(int));
	stacount=(int *)malloc(n_sta*sizeof(int));
	starms=(double *)malloc(n_sta*sizeof(double));
	stamean=(double *)malloc(n_sta*sizeof(double));
	stax=(double *)malloc(n_sta*sizeof(double));
	stay=(double *)malloc(n_sta*sizeof(double));
	staz=(double *)malloc(n_sta*sizeof(double));
	jhdfilename = (char**)malloc(n_sta*sizeof(char*));
	if( (stanum==NULL) || (jhdfilename==NULL) || (starms==NULL) ||
	    (stacount==NULL) || (stamean==NULL) || (stax==NULL) ||
	    (stay==NULL) || (staz==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	if( (fsta=fopen(argv[3],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open station file <%s> for read\n",argv[0],argv[3]);
		exit(-1);
		}
	n_sta=0;
	while( fscanf(fsta,"%d%le%le%le%s",&i,&x,&y,&z,str)==5 ) {
		stanum[n_sta]=i;
		stax[n_sta]=x; stay[n_sta]=y; staz[n_sta]=z;
		jhdfilename[n_sta]=(char*)malloc( (strlen(str)+1)*sizeof(char) );
		if( jhdfilename[n_sta]==NULL ) {
			fprintf(stderr,"error: %s: out of memory (2)\n", argv[0] );
			exit(-1);
			}
		strcpy(jhdfilename[n_sta],str);
		starms[n_sta]=0.0; stacount[n_sta]=0; stamean[n_sta]=0.0;
		n_sta++;
		}
	fclose(fsta);
	fprintf(stderr,"\tstations: %d\n", n_sta);
	if( verbose ) {
		for( i=0; i<n_sta; i++ ) {
			fprintf(stderr,"%d %s\n", stanum[i], jhdfilename[i] );
			}
		fprintf(stderr,"\n");
		}

	/* event file */
	n_ev=count_lines(argv[4]);
	ev=(struct ev_rec *)malloc(n_ev*sizeof(struct ev_rec));
	if( ev==NULL ) {
		fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
		exit(-1);
		}
	if( (fev=fopen(argv[4],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open event file <%s> for read\n",argv[0],argv[4]);
		exit(-1);
		}
	n_ev=0;
	while( fscanf(fev,"%d%le%le%le%le",&i,&x,&y,&z,&t0)==5 ) {
		ev[n_ev].nev = i;
		ev[n_ev].x=x; ev[n_ev].y=y; ev[n_ev].z=z; ev[n_ev].t0=t0;
		ev[n_ev].rms=0.0; ev[n_ev].count=0; ev[n_ev].mean=0.0; 
		n_ev++;
		}
	fclose(fev);
	qsort( ev, n_ev, sizeof(struct ev_rec), cmpev );
	for( i=0; i<n_ev; i++ ) ev[i].index=i;
	fprintf(stderr,"\tevents: %d\n", n_ev);
	if( verbose ) {
		for( i=0; i<n_ev; i++ ) {
			fprintf(stderr,"%d %f %f %f %f\n", ev[i].nev, ev[i].x, ev[i].y, ev[i].z, ev[i].t0 );
			}
		fprintf(stderr,"\n");
		}

	/* phase file */
	if( (fph=fopen(argv[5],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open phase file <%s> for read\n",argv[0],argv[5]);
		exit(-1);
		}
	n_ph=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&k,&k,&x,&x)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			}
		n_ph++;
		}
	rewind(fph);
	ph=(struct ph_rec *)malloc(n_ph*sizeof(struct ph_rec));
	if( ph==NULL ) {
		fprintf(stderr,"error: %s: out of memory (4)\n", argv[0] );
		exit(-1);
		}
	n_ph=0; line=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		line++; ph[n_ph].line=line;
		ph[n_ph].n=i; ph[n_ph].obs=x; ph[n_ph].weight=y; ph[n_ph].maxres=z;
		ph[n_ph].good=0; ph[n_ph].row=(-1);
		ph[n_ph].sta=(int*)malloc(i*sizeof(int));
		ph[n_ph].ev=(int*)malloc(i*sizeof(int));
		ph[n_ph].order=(int*)malloc(i*sizeof(int));
		ph[n_ph].hit=(int*)malloc(i*sizeof(int));
		ph[n_ph].ratio=(double*)malloc(i*sizeof(double));
		ph[n_ph].coeff=(double*)malloc(i*sizeof(double));
		ph[n_ph].T=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdx=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdy=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdz=(double*)malloc(i*sizeof(double));
		ph[n_ph].azi=(double*)malloc(i*sizeof(double));
		ph[n_ph].theta=(double*)malloc(i*sizeof(double));
		ph[n_ph].stax=(double*)malloc(i*sizeof(double));
		ph[n_ph].stay=(double*)malloc(i*sizeof(double));
		ph[n_ph].staz=(double*)malloc(i*sizeof(double));
		if( (ph[n_ph].sta==NULL) || (ph[n_ph].ev==NULL) ||
			(ph[n_ph].order==NULL) || (ph[n_ph].hit==NULL) ||
		    	(ph[n_ph].ratio==NULL) || (ph[n_ph].coeff==NULL) ||
			(ph[n_ph].T==NULL) || (ph[n_ph].dTdx==NULL) ||
			(ph[n_ph].dTdy==NULL) || (ph[n_ph].dTdz==NULL) ||
		    	(ph[n_ph].azi==NULL) || (ph[n_ph].theta==NULL) ||
			(ph[n_ph].stax==NULL) || (ph[n_ph].stay==NULL) || (ph[n_ph].staz==NULL) ) {
			fprintf(stderr,"error: %s: out of memory (5)\n", argv[0] );
			exit(-1);
			}
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&l,&m,&x,&y)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			line++;
			ph[n_ph].sta[j]=k; ph[n_ph].ev[j]=l; ph[n_ph].order[j]=m; ph[n_ph].hit[j]=0;
			ph[n_ph].ratio[j]=x; ph[n_ph].coeff[j]=y; ph[n_ph].T[j]=0.0;
			ph[n_ph].dTdx[j]=0.0; ph[n_ph].dTdy[j]=0.0;ph[n_ph].dTdz[j]=0.0;
			ph[n_ph].azi[j]=0.0; ph[n_ph].theta[j]=0.0;
			ph[n_ph].stax[j]=0.0; ph[n_ph].stay[j]=0.0; ph[n_ph].staz[j]=0.0;
			}
		n_ph++;
		}
	fclose(fph);
	fprintf(stderr,"\tobservations: %d\n", n_ph);

	for( i=0; i<n_sta; i++ ) {
		n_jhd=count_lines(jhdfilename[i]);
		jhd=(struct jhd_rec *)malloc(n_jhd*sizeof(struct jhd_rec));
		if( jhd==NULL ) {
			fprintf(stderr,"error: %s: out of memory (6)\n", argv[0] );
			exit(-1);
			}
		if( (fjhd=fopen(jhdfilename[i],"r")) == NULL ) {
			fprintf(stderr,"error: %s: cant open jhd_table <%s> for read\n",
				argv[0], jhdfilename[i] );
			exit(-1);
			}
		n_jhd=0;
		while( fscanf(fjhd,"%d%d%le%le%le%le%le%le%le%le%d%le%le",&j,&k,&x,&x,&x,
			&T, &dTdx, &dTdy, &dTdz, &x, &l, &theta, &azi ) == 13 ) {
			jhd[n_jhd].nev=j;
			jhd[n_jhd].order=k;
			jhd[n_jhd].T = T;
			jhd[n_jhd].dTdx=dTdx; jhd[n_jhd].dTdy=dTdy; jhd[n_jhd].dTdz=dTdz;
			jhd[n_jhd].azi=azi; jhd[n_jhd].theta=theta;
			n_jhd++;
			}
		fclose(fjhd);
		fprintf(stderr,"\tjhd_table %s has %d records\n", jhdfilename[i], n_jhd );
		if( verbose ) {
			for( j=0; j<n_jhd; j++ ) {
				fprintf(stderr,"%d %d %f %f %f %f %f %f\n", jhd[j].nev, jhd[j].order,
					jhd[j].T, jhd[j].dTdx, jhd[j].dTdy, jhd[j].dTdz,
					jhd[j].theta, jhd[j].azi);
				}
			fprintf(stderr,"\n");
			}
		qsort( jhd, n_jhd, sizeof(struct jhd_rec), cmpjhd );
		for( j=0; j<n_ph; j++ ) for( k=0; k<ph[j].n; k++ ) {
			if( stanum[i] != ph[j].sta[k] ) continue;
			jhdkey.nev = ph[j].ev[k];
			jhdkey.order = ph[j].order[k];
			myjhd=(struct jhd_rec *)
				bsearch(&jhdkey, jhd, n_jhd, sizeof(struct jhd_rec), cmpjhd );
			if( myjhd == NULL ) continue;
			ph[j].hit[k]=1;
			ph[j].T[k] = myjhd->T;
			ph[j].dTdx[k] = myjhd->dTdx;
			ph[j].dTdy[k] = myjhd->dTdy;
			ph[j].dTdz[k] = myjhd->dTdz;
			ph[j].azi[k]  = myjhd->azi;
			ph[j].theta[k]= myjhd->theta;
			ph[j].stax[k]=stax[i];
			ph[j].stay[k]=stay[i];
			ph[j].staz[k]=staz[i];
			}
		free(jhd);
		} /* next sta */

	ngood=0;
	for( j=0; j<n_ph; j++ ) {
		l=0;
		for( k=0; k<ph[j].n; k++ ) l+=ph[j].hit[k];
		if( l==ph[j].n ) {
			ph[j].good=1;
			ngood++;
			}
		}
	fprintf(stderr,"\tusable observations: %d\n", ngood);
	if( ngood==0 ) {
		fprintf(stderr,"warning: %s: no good observations\n");
		for( i=0; i<n_ev; i++ ) {
			x=ev[i].x; y=ev[i].y; z=ev[i].z; t0=ev[i].t0;
			printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
			}
		exit(0);
		}

	if( verbose ) {
		for( j=0; j<n_ph; j++ ) {
			fprintf(stderr,"%d %d %f %f %f %d\n",
				j, ph[j].n, ph[j].obs, ph[j].weight, ph[j].maxres, ph[j].good);
			l=0;
			for( k=0; k<ph[j].n; k++ ) {
				fprintf(stderr, "\t%d %d %d %f %f %f %f %f %f %f %f %d\n",
					ph[j].sta[k], ph[j].ev[k], ph[j].order[k], ph[j].ratio[k],
					ph[j].coeff[k], ph[j].T[k], ph[j].dTdx[k], ph[j].dTdy[k],
					ph[j].dTdz[k], ph[j].theta, ph[j].azi, ph[j].hit[k] );
				}
			}
		}

	n = ngood;
	m = 4*n_ev+n_model_pars;
	Grow = (double*)malloc(m*sizeof(double));
	G=new_sparse_matrix(n,m,0);
	d=(double*)malloc(n*sizeof(double));
	dp=(double*)malloc(n*sizeof(double));
	dm=(double*)malloc(m*sizeof(double));
	GTd=(double*)malloc(m*sizeof(double));
	if( (G==NULL) || (Grow==NULL ) || (d==NULL) || (dp==NULL) || (dm==NULL) || (GTd==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (7)\n", argv[1] );
		exit(-1);
		}

	/* build matrix equation */
	i=0; irms=0; rms1=0.0;
	for( j=0; j<n_ph; j++ ) { /* loop over observations */
		if( !ph[j].good ) continue;
		for( k=0; k<m; k++ ) Grow[k]=0.0;
		x = ph[j].weight;
		d[i]=0.0;
		for( k=0; k<ph[j].n; k++ ) { /* loop over rays in an observation */
			y = ph[j].ratio[k];
			c = ph[j].coeff[k];

			/* match event number */
			evkey.nev = ph[j].ev[k];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) {
				fprintf(stderr,"error: %s: cant find event %d in event file\n", argv[0], evkey.nev);
				exit(-1);
				}
			l=myev->index;

			/* right hand side of equation */
			d[i] -= x * c * ( y * ph[j].T[k] + myev->t0 );

			/* part of matrix associated with event locations */
			Grow[4*l]   += x * c * y * ph[j].dTdx[k];
			Grow[4*l+1] += x * c * y * ph[j].dTdy[k];
			Grow[4*l+2] += x * c * y * ph[j].dTdz[k];
			Grow[4*l+3] += x * c * tscale;

			/* part of matrix associated with velocity model parameters */
			x0[0]=ph[j].stax[k]; x0[1]=ph[j].stay[k]; x0[2]=ph[j].staz[k];
			if( !shoot_to_traveltime( x0, ph[j].azi[k], ph[j].theta[k], ph[j].T[k],
				x1, t1, arc, &n_arcs ) ) {
				ph[j].good=0;
				break;
				}

			/* count nodes affected by this ray */
			for( ii=0; ii<n_nodes; ii++ ) node[ii].touched=0;
			for( ii=0; ii<n_arcs; ii++ ) {
				node[ arc[ii].ptetra->n1 ].touched=1;
				node[ arc[ii].ptetra->n2 ].touched=1;
				node[ arc[ii].ptetra->n3 ].touched=1;
				node[ arc[ii].ptetra->n4 ].touched=1;
				}
			for( ii=0; ii<n_nodes; ii++ ) {
				if( node[ii].touched ) node[ii].hits++;
				node[i].touched=0;
				}

			/* will varying this velocity model parameter change the traveltime ? */
			for( jj=0; jj<n_model_pars; jj++ ) {
				match=FALSE;
				for( ii=0; ii<n_arcs; ii++ ) {
					pt=arc[ii].ptetra;
					tnode[0]=pt->n1; tnode[1]=pt->n2;
					tnode[2]=pt->n3; tnode[3]=pt->n4;
					for( kk=0; kk<4; kk++ ) {
						if( node_in_model_par(tnode[kk],jj) ) {
							match=TRUE;
							break;
							}
						}
					if(match) break;
					}
				if( match ) {
					rms2 = dTdmp(jj,arc,n_arcs,ph[j].T[k]);
					Grow[4*n_ev+jj] += rw*x*y*c*rms2;
					}
				else {
					;/* Grow is zero */
					}
				} /* next velocity model parameter */
			} /* end loop over rays */

		if( !ph[j].good ) {
			fprintf(stderr,"observation %d (row %d) lost due to raytrace failure\n", j, i );
			continue;
			}

		if( verbose ) {
			fprintf(stderr,"weighted observation %d (row %d) obs=%f pred=%f res=%f\n",
			j, i, -x*ph[j].obs, d[i], x*ph[j].obs+d[i] );
			}

		/* reject observation if residual traveltime is too large */
		res = fabs(x*ph[j].obs+d[i]);
		if(  res > ph[j].maxres ) {
			fprintf(stderr,"observation %d (row %d) thrown out, residual %f > %f\n", j, i,
				res, ph[j].maxres );
			d[i]=0.0;
			}
		else {
			rms1 += res*res; irms++;
			d[i]+=x*ph[j].obs;
			ph[j].row=i;
			for( k=0; k<m; k++ ) if( Grow[k]!=0.0 ) {
				status = set_sparse_value(G,i,k,Grow[k]);
				if( !status ) {
					fprintf(stderr,"error: %s: out of memory (8)\n", argv[1] );
					exit(-1);
					}
				}
			i++;
			}
		} /* next observation */

	rms1 = sqrt( rms1 / (double) irms );

	maxG=sparse_absmax(G);
	k=sparse_count(G);
	fprintf(stderr,
		"G is %d by %d, is %f percent full, and has absolute maximum of %f\n",
		n, m, 100.0*(double)k/((double)n*(double)m), maxG );

	maxd=0.0;
	rms2=0.0;
	for( i=0; i<n; i++ ) {
		x=fabs(d[i]); rms2+=d[i];
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr, "d is %d by 1, and has absolute maximum of %f and a mean of %f\n", n, maxd, rms2/(double)n );
	fprintf(stderr,"Damping: %e\n\n", damping );

	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<m; i++ ) {
		x=fabs(GTd[i]);
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n", m, maxd );

	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );
		
	for( i=0; i<m; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, m+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	timestamp("after implicit solution");
	
	for( i=0; i<n_ev; i++ ) {
		x=ev[i].x+dm[4*i];
		y=ev[i].y+dm[4*i+1];
		z=ev[i].z+dm[4*i+2];
		t0=ev[i].t0+tscale*dm[4*i+3];
		printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
		}

	sm_times_v( G, dm, dp );

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res =  d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			for( k=0; k<n_sta; k++ ) if( stanum[k]==ph[i].sta[j] ) {
				stacount[k]++;
				starms[k]+= res*res;
				stamean[k]+= res;
				break;
				}
			}
		}

	fprintf(stderr,"station statistics:\n");
	fprintf(stderr,"sta\tcount\trms\tmean\n");
	for( i=0; i<n_sta; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\t%f\n", stanum[i], stacount[i],
			sqrt(starms[i]/(double)stacount[i]),
			stamean[i]/(double)stacount[i] );
		}
	fprintf(stderr,"\n");

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res = d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			evkey.nev = ph[i].ev[j];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) continue;
			myev->count++;
			myev->rms += res*res;
			myev->mean += res;
			}
		}

	fprintf(stderr,"event statistics:\n");
	fprintf(stderr,"event\tcount\trms\tmean\n");
	for( i=0; i<n_ev; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\t%f\n",
			ev[i].nev, ev[i].count, sqrt(ev[i].rms/(double)ev[i].count),
			ev[i].mean/(double)ev[i].count);
		}
	fprintf(stderr,"\n");

	fprintf(stderr,"phase statistics:\n");
	fprintf(stderr,"phase\tline\terror\n");
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		fprintf(stderr,"%d\t%d\t%f\n", i, ph[i].line, res );
		}
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ev; i++ ) {
		res = dm[4*i]; rms2 += res*res;
		res = dm[4*i+1]; rms2 += res*res;
		res = dm[4*i+2]; rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)(3*n_ev) );
	fprintf(stderr, "event location statistics: rms change in position %f km\n", rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ev; i++ ) {
		res = tscale*dm[4*i+3]; rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_ev );
	fprintf(stderr, "event origin time statistics: rms change in origin time %f s\n", rms2);
	fprintf(stderr,"\n");

	/* update velocity of nodes */
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v += rw*weight*dm[4*n_ev+i];
			}
		}

	rms2=0.0;
	for( i=0; i<n_model_pars; i++ ) {
		res = dm[4*n_ev+i];
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_model_pars );
	fprintf(stderr, "velocity model parameter statistics: rms size %f\n", rw*rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_nodes; i++ ) {
		res = node[i].v - node[i].v_orig;
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_nodes );
	fprintf(stderr, "velocity nodes statistics: rms change %f km/s\n", rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)irms );
	fprintf(stderr,"rms weighted residual original: %f final %f for %f percent reduction\n",
		rms1, rms2, 100.0-100.0*rms2/rms1 );

	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/* must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, 1.0/node[i].qinv, node[i].ifc );
		else
		fprintf(nmfile,"%d %d %d %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ifc );
		}
	fclose(nmfile);

	sprintf( hitsfile, "%s.hits", argv[6] );
	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);

	exit(0);
	}

struct PS_node {
	double Pv, Sv;
	};
struct PS_tetra {
	double Pvc[4], Pg[3], Pgs, Pgn[3];
	double Svc[4], Sg[3], Sgs, Sgn[3];
	};

main_locate_and_PS_invert(int argc, char **argv)
{
	int i, j, k, l, m, n, ngood, status, iter, biec, irms, line, match;
	int ii, jj, kk;
	char str1[1024], str2[1024], hitsfile[1024];
	double x0[3], x1[3], t1[3];
        int tnode[4];
	int nx, ny, nz;

	struct arc_rec arc[MAX_ARCS];
	int n_arcs;

        struct tetra_rec *pt;

	FILE *fsta, *nmfile;
	int *stanum;
	double *stax, *stay, *staz;
	char **P_jhdfilename, **S_jhdfilename;
	int n_sta;
	double x,y,z,t0,c,azi,theta,weight;
	double T, dTdx, dTdy, dTdz;

	FILE *fev;
	struct ev_rec *ev, evkey, *myev;
	int n_ev;

	FILE *fph;
	struct ph_rec {
		double obs, weight, maxres;
		int n, good, row, line;
		int *sta, *ev, *order, *hit;
		double *ratio, *coeff;
		double *T, *dTdx, *dTdy, *dTdz;
		double *azi, *theta;
		double *stax, *stay, *staz;
		};
	struct ph_rec *ph;
	int n_ph;

	FILE *fjhd, *mpfile;
	struct jhd_rec *P_jhd, *S_jhd;
	int n_P_jhd, n_S_jhd;
	struct jhd_rec jhdkey, *myjhd;

	int verbose=0;

	struct sparse_matrix *G, *GTG;
	double *d, *dp, *GTd, *dm, tscale=TSCALEVALUE, maxG, maxd, damping, rw, rw2, err, res, rms1, rms2;
	double *starms, *stamean;
	int *stacount;
	double *Grow;

	struct model_par_rec *ratio_model_par, *P_model_par;
	int ratio_n_model_pars, P_n_model_pars;

	/* a quick way of switching from P to S model and vice versa */
	struct PS_node *my_PS_node;
	struct PS_tetra *my_PS_tetra;
	int model_is_P;

	double new_damping, scx;


	if( argc!=11 ) {
		fprintf(stderr,"purpose: joint hypocenter relocation and P and S velocity tomography\n");
		fprintf(stderr,"         for improvements to node P velocities and P:S velocity ratio\n");
		fprintf(stderr,"method: The user provides a set of observations, where\n");
		fprintf(stderr,"        each observation is assumed to be a linear combination\n");
		fprintf(stderr,"        of traveltimes:\n");
		fprintf(stderr,"           observation = (sum i=1 to n) coefficient[i] * traveltime[i]\n");
		fprintf(stderr,"        A phase file tabulates the observations and the\n");
		fprintf(stderr,"        n coefficients and the station, event and order\n");
		fprintf(stderr,"        of each of the traveltimes.  The observations are\n");
		fprintf(stderr,"        used to produce an updated estimate of the event\n");
		fprintf(stderr,"        locations and the velocity models.\n");
		fprintf(stderr,"      Note: model file must define both P velocity and P:S velocity ratio\n");
		fprintf(stderr,"usage: %s model_file.txt P_model_parameter_file.txt ratio_model_parameter_file.txt station_file.txt event_file.txt phase_file.txt new_model_file damping EP_relative_weight PS_relative_weight > updated_event_file.txt\n", argv[0]);
		fprintf(stderr,"       model_file.txt: standard velocity model file with ratio defined\n");
		fprintf(stderr,"       P_model_parameter_file.txt: standard velocity node model parameter file\n");
		fprintf(stderr,"       ratio_model_parameter_file.txt: analogous model parameter file for velocity ratios\n");
		fprintf(stderr,"       station_file.txt: one station per line, with fields\n");
		fprintf(stderr,"               station_number: arbitrary integer\n");
		fprintf(stderr,"               x, y, z: station location, km\n");
		fprintf(stderr,"               P_jhd_table: path of station-specific P jhd-table\n");
		fprintf(stderr,"               S_jhd_table: path of station-specific S jhd-table\n");
		fprintf(stderr,"       event_file.txt: one event per line, with fields\n");
		fprintf(stderr,"               event_number: arbitrary integer\n");
		fprintf(stderr,"               x, y, z: preliminary location, km\n");
		fprintf(stderr,"               t0: preliminary origin time, s\n");
		fprintf(stderr,"       phase_file.txt: one group of lines per observation\n");
		fprintf(stderr,"               first line, with fields\n");
		fprintf(stderr,"                     n_phases: number of phases in observation\n");
		fprintf(stderr,"                     observation: a differetial traveltime, s\n");
		fprintf(stderr,"                     weight: weight of observation\n");
		fprintf(stderr,"                     maxres: observation thrown out if weighted residual > maxres, s\n");
		fprintf(stderr,"               subsequent lines in group, with fields\n");
		fprintf(stderr,"                     station_number, corresponding to station_file.txt\n");
		fprintf(stderr,"                     event_number, corresponding to event_file.txt\n");
		fprintf(stderr,"                     order, interger arrival number of phase\n");
		fprintf(stderr,"                     ratio, 1.0 signifies that the P model\n");
		fprintf(stderr,"                        should be used; anthing else, the S model\n");
		fprintf(stderr,"       new_model_file.txt: updated model file. A corresponding\n");
		fprintf(stderr,"       		hits file, with suffix .hits, is also created\n");
		fprintf(stderr,"       damping: damping factor, fraction of max of GTG\n");
		fprintf(stderr,"       EP_relative_weight: relative weight of event and P velocity model parameters\n");
		fprintf(stderr,"               relative_weigth>1 favors large of velocity model perturbations\n");
		fprintf(stderr,"               relative_weigth<1 favors large of event location perturbations\n");
		fprintf(stderr,"       PS_relative_weight: relative weight of P and ratio velocity model parameters\n");
		fprintf(stderr,"               relative_weigth>1 favors large of ratio model parameter perturbations\n");
		fprintf(stderr,"               relative_weigth<1 favors large of P model parameter perturbations\n");
		fprintf(stderr,"Note: In the inversion, origin time perturbations are scaled\n");
		fprintf(stderr,"to units of distance using a nominal velocity of %f km/s\n",
			1.0/TSCALEVALUE );
		exit(-1);
		}

	if( sscanf(argv[8],"%le",&damping) != 1 ) {
		fprintf(stderr,"error: %s: cant read damping from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[9],"%le",&rw) != 1 ) {
		fprintf(stderr,"error: %s: cant read EP relative weight from command line\n", argv[0] );
		exit(-1);
		}

	if( sscanf(argv[10],"%le",&rw2) != 1 ) {
		fprintf(stderr,"error: %s: cant read PS relative weight from command line\n", argv[0] );
		exit(-1);
		}

	fprintf(stderr,"Raytrace3d locate_and_PS_invert\n");
	fprintf(stderr,"\tstarting model %s\n", argv[1] );
	fprintf(stderr,"\tdamping %e\n", damping);
	fprintf(stderr,"\tevent-P relative weight %e\n", rw);
	fprintf(stderr,"\tP-ratio relative weight %e\n", rw2);

	/* model file */
	if( !load_prismatic_model(argv[1]) ) {
		fprintf(stderr,"error: %s: cant load model <%s>\n", argv[0], argv[1] );
		exit(-1);
		}
	if( !QDEFINED ) {
		fprintf(stderr,"error: %s: model <%s> must have Vp/Vs ratio defined\n", argv[0], argv[1] );
		exit(0);
		}

	/* create structures for qickly switching between P and S models */
	my_PS_node = (struct PS_node *) malloc( n_nodes * sizeof(struct PS_node) );
	my_PS_tetra = (struct PS_tetra *) malloc( n_tetras * sizeof(struct PS_tetra) );
	if( (my_PS_node==NULL) || (my_PS_tetra==NULL) ) {
		fprintf(stderr,"error: %s: out of memory! (1a)\n", argv[0]);
		exit(-1);
		}
	for( i=0; i<n_nodes; i++ ) {
		my_PS_node[i].Pv = node[i].v;	
		my_PS_node[i].Sv = node[i].v/node[i].ratio;	
		}
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) my_PS_tetra[i].Pvc[j]=tetra[i].vc[j];
		for(j=0;j<3;j++) my_PS_tetra[i].Pg[j]=tetra[i].g[j];
		my_PS_tetra[i].Pgs = tetra[i].gs;
		for(j=0;j<3;j++) my_PS_tetra[i].Pgn[j]=tetra[i].gn[j];
		}
	for( i=0; i<n_nodes; i++ ) node[i].v = node[i].v_orig/node[i].ratio_orig;
	for( i=0; i<n_tetras; i++ ) velocity_eqn( &tetra[i] );
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) my_PS_tetra[i].Svc[j]=tetra[i].vc[j];
		for(j=0;j<3;j++) my_PS_tetra[i].Sg[j]=tetra[i].g[j];
		my_PS_tetra[i].Sgs = tetra[i].gs;
		for(j=0;j<3;j++) my_PS_tetra[i].Sgn[j]=tetra[i].gn[j];
		}
	model_is_P=0;

	/* P model parameter file */
	alloc_model_pars(argv[2]);
	if( (mpfile=fopen(argv[2],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open P_model_par_file <%s> for read\n", argv[0], argv[2] );
		exit(-1);
		}
	n_model_pars=0;
	load_model_pars_V(mpfile);
	fclose(mpfile);
	fprintf(stderr,"\tP model parameters %s with %d parameters\n", argv[2], n_model_pars );
	P_model_par=model_par; P_n_model_pars=n_model_pars;

	/* ratio model parameter file */
	model_par=NULL; n_model_pars=0;
	alloc_model_pars(argv[3]);
	if( (mpfile=fopen(argv[3],"r"))==NULL ) {
		fprintf(stderr,"error: %s: cant open ratio_model_par_file <%s> for read\n", argv[0], argv[3] );
		exit(-1);
		}
	load_model_pars_V(mpfile);
	fclose(mpfile);
	fprintf(stderr,"\tRatio model parameters %s with %d parameters\n", argv[3], n_model_pars );
	ratio_model_par=model_par; ratio_n_model_pars=n_model_pars;

	if( (nmfile=fopen(argv[7],"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open new_model_file <%s> for write\n", argv[0], argv[6] );
		exit(-1);
		}

	/* station file */
	n_sta=count_lines(argv[3]);
	stanum=(int *)malloc(n_sta*sizeof(int));
	stacount=(int *)malloc(n_sta*sizeof(int));
	starms=(double *)malloc(n_sta*sizeof(double));
	stamean=(double *)malloc(n_sta*sizeof(double));
	stax=(double *)malloc(n_sta*sizeof(double));
	stay=(double *)malloc(n_sta*sizeof(double));
	staz=(double *)malloc(n_sta*sizeof(double));
	P_jhdfilename = (char**)malloc(n_sta*sizeof(char*));
	S_jhdfilename = (char**)malloc(n_sta*sizeof(char*));

	if( (stanum==NULL) || (P_jhdfilename==NULL) || (S_jhdfilename==NULL) || (starms==NULL) ||
	    (stacount==NULL) || (stamean==NULL) || (stax==NULL) || (stay==NULL) || (staz==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (1)\n", argv[0] );
		exit(-1);
		}
	if( (fsta=fopen(argv[4],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open station file <%s> for read\n",argv[0],argv[4]);
		exit(-1);
		}
	n_sta=0;
	while( fscanf(fsta,"%d%le%le%le%s%s",&i,&x,&y,&z,str1,str2)==6 ) {
		stanum[n_sta]=i;
		stax[n_sta]=x; stay[n_sta]=y; staz[n_sta]=z;
		P_jhdfilename[n_sta]=(char*)malloc( (strlen(str1)+1)*sizeof(char) );
		if( P_jhdfilename[n_sta]==NULL ) {
			fprintf(stderr,"error: %s: out of memory (2a)\n", argv[0] );
			exit(-1);
			}
		strcpy(P_jhdfilename[n_sta],str1);
		S_jhdfilename[n_sta]=(char*)malloc( (strlen(str2)+1)*sizeof(char) );
		if( S_jhdfilename[n_sta]==NULL ) {
			fprintf(stderr,"error: %s: out of memory (2b)\n", argv[0] );
			exit(-1);
			}
		strcpy(S_jhdfilename[n_sta],str2);
		starms[n_sta]=0.0; stacount[n_sta]=0; stamean[n_sta]=0.0;
		n_sta++;
		}
	fclose(fsta);
	fprintf(stderr,"\tstations: %d\n", n_sta);

	/* event file */
	n_ev=count_lines(argv[5]);
	ev=(struct ev_rec *)malloc(n_ev*sizeof(struct ev_rec));
	if( ev==NULL ) {
		fprintf(stderr,"error: %s: out of memory (3)\n", argv[0] );
		exit(-1);
		}
	if( (fev=fopen(argv[5],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open event file <%s> for read\n",argv[0],argv[4]);
		exit(-1);
		}
	n_ev=0;
	while( fscanf(fev,"%d%le%le%le%le",&i,&x,&y,&z,&t0)==5 ) {
		ev[n_ev].nev = i;
		ev[n_ev].x=x; ev[n_ev].y=y; ev[n_ev].z=z; ev[n_ev].t0=t0;
		ev[n_ev].rms=0.0; ev[n_ev].count=0; ev[n_ev].mean=0.0; 
		n_ev++;
		}
	fclose(fev);
	qsort( ev, n_ev, sizeof(struct ev_rec), cmpev );
	for( i=0; i<n_ev; i++ ) ev[i].index=i;
	fprintf(stderr,"\tevents: %d\n", n_ev);
	if( verbose ) {
		for( i=0; i<n_ev; i++ ) {
			fprintf(stderr,"%d %f %f %f %f\n", ev[i].nev, ev[i].x, ev[i].y, ev[i].z, ev[i].t0 );
			}
		fprintf(stderr,"\n");
		}

	/* phase file */
	if( (fph=fopen(argv[6],"r")) == NULL ) {
		fprintf(stderr,"error: %s: cant open phase file <%s> for read\n",argv[0],argv[5]);
		exit(-1);
		}
	n_ph=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&k,&k,&x,&x)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			}
		n_ph++;
		}
	rewind(fph);
	ph=(struct ph_rec *)malloc(n_ph*sizeof(struct ph_rec));
	if( ph==NULL ) {
		fprintf(stderr,"error: %s: out of memory (4)\n", argv[0] );
		exit(-1);
		}
	n_ph=0; line=0;
	while( fscanf(fph,"%d%le%le%le",&i,&x,&y,&z)==4 ) {
		line++; ph[n_ph].line=line;
		ph[n_ph].n=i; ph[n_ph].obs=x; ph[n_ph].weight=y; ph[n_ph].maxres=z;
		ph[n_ph].good=0; ph[n_ph].row=(-1);
		ph[n_ph].sta=(int*)malloc(i*sizeof(int));
		ph[n_ph].ev=(int*)malloc(i*sizeof(int));
		ph[n_ph].order=(int*)malloc(i*sizeof(int));
		ph[n_ph].hit=(int*)malloc(i*sizeof(int));
		ph[n_ph].ratio=(double*)malloc(i*sizeof(double));
		ph[n_ph].coeff=(double*)malloc(i*sizeof(double));
		ph[n_ph].T=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdx=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdy=(double*)malloc(i*sizeof(double));
		ph[n_ph].dTdz=(double*)malloc(i*sizeof(double));
		ph[n_ph].azi=(double*)malloc(i*sizeof(double));
		ph[n_ph].theta=(double*)malloc(i*sizeof(double));
		ph[n_ph].stax=(double*)malloc(i*sizeof(double));
		ph[n_ph].stay=(double*)malloc(i*sizeof(double));
		ph[n_ph].staz=(double*)malloc(i*sizeof(double));
		if( (ph[n_ph].sta==NULL) || (ph[n_ph].ev==NULL) ||
			(ph[n_ph].order==NULL) || (ph[n_ph].hit==NULL) ||
		    	(ph[n_ph].ratio==NULL) || (ph[n_ph].coeff==NULL) ||
			(ph[n_ph].T==NULL) || (ph[n_ph].dTdx==NULL) ||
			(ph[n_ph].dTdy==NULL) || (ph[n_ph].dTdz==NULL) ||
		    	(ph[n_ph].azi==NULL) || (ph[n_ph].theta==NULL) ||
			(ph[n_ph].stax==NULL) || (ph[n_ph].stay==NULL) || (ph[n_ph].staz==NULL) ) {
			fprintf(stderr,"error: %s: out of memory (5)\n", argv[0] );
			exit(-1);
			}
		for( j=0; j<i; j++ ) {
			if( fscanf(fph,"%d%d%d%le%le",&k,&l,&m,&x,&y)!=5 ) {
				fprintf(stderr,"error: %s: reading phase file, record %d\n",
					argv[0], n_ph );
				exit(-1);
				}
			line++;
			ph[n_ph].sta[j]=k; ph[n_ph].ev[j]=l; ph[n_ph].order[j]=m; ph[n_ph].hit[j]=0;
			ph[n_ph].ratio[j]=x; ph[n_ph].coeff[j]=y; ph[n_ph].T[j]=0.0;
			ph[n_ph].dTdx[j]=0.0; ph[n_ph].dTdy[j]=0.0;ph[n_ph].dTdz[j]=0.0;
			ph[n_ph].azi[j]=0.0; ph[n_ph].theta[j]=0.0;
			ph[n_ph].stax[j]=0.0; ph[n_ph].stay[j]=0.0; ph[n_ph].staz[j]=0.0;
			}
		n_ph++;
		}
	fclose(fph);
	fprintf(stderr,"\tobservations: %d\n", n_ph);

	for( i=0; i<n_sta; i++ ) {
		n_P_jhd=count_lines(P_jhdfilename[i]);
		P_jhd=(struct jhd_rec *)malloc(n_P_jhd*sizeof(struct jhd_rec));
		if( P_jhd==NULL ) {
			fprintf(stderr,"error: %s: out of memory (6a)\n", argv[0] );
			exit(-1);
			}
		if( (fjhd=fopen(P_jhdfilename[i],"r")) == NULL ) {
			fprintf(stderr,"error: %s: cant open P jhd_table <%s> for read\n",
				argv[0], P_jhdfilename[i] );
			exit(-1);
			}
		n_P_jhd=0;
		while( fscanf(fjhd,"%d%d%le%le%le%le%le%le%le%le%d%le%le",&j,&k,&x,&x,&x,
			&T, &dTdx, &dTdy, &dTdz, &x, &l, &theta, &azi ) == 13 ) {
			P_jhd[n_P_jhd].nev=j;
			P_jhd[n_P_jhd].order=k;
			P_jhd[n_P_jhd].T = T;
			P_jhd[n_P_jhd].dTdx=dTdx; P_jhd[n_P_jhd].dTdy=dTdy; P_jhd[n_P_jhd].dTdz=dTdz;
			P_jhd[n_P_jhd].azi=azi; P_jhd[n_P_jhd].theta=theta;
			n_P_jhd++;
			}
		fclose(fjhd);
		fprintf(stderr,"\tP jhd_table %s has %d records\n", P_jhdfilename[i], n_P_jhd );
		qsort( P_jhd, n_P_jhd, sizeof(struct jhd_rec), cmpjhd );

		n_S_jhd=count_lines(S_jhdfilename[i]);
		S_jhd=(struct jhd_rec *)malloc(n_S_jhd*sizeof(struct jhd_rec));
		if( S_jhd==NULL ) {
			fprintf(stderr,"error: %s: out of memory (6b)\n", argv[0] );
			exit(-1);
			}
		if( (fjhd=fopen(S_jhdfilename[i],"r")) == NULL ) {
			fprintf(stderr,"error: %s: cant open S jhd_table <%s> for read\n",
				argv[0], S_jhdfilename[i] );
			exit(-1);
			}
		n_S_jhd=0;
		while( fscanf(fjhd,"%d%d%le%le%le%le%le%le%le%le%d%le%le",&j,&k,&x,&x,&x,
			&T, &dTdx, &dTdy, &dTdz, &x, &l, &theta, &azi ) == 13 ) {
			S_jhd[n_S_jhd].nev=j;
			S_jhd[n_S_jhd].order=k;
			S_jhd[n_S_jhd].T = T;
			S_jhd[n_S_jhd].dTdx=dTdx; S_jhd[n_S_jhd].dTdy=dTdy; S_jhd[n_S_jhd].dTdz=dTdz;
			S_jhd[n_S_jhd].azi=azi; S_jhd[n_S_jhd].theta=theta;
			n_S_jhd++;
			}
		fclose(fjhd);
		fprintf(stderr,"\tS jhd_table %s has %d records\n", S_jhdfilename[i], n_P_jhd );
		qsort( S_jhd, n_S_jhd, sizeof(struct jhd_rec), cmpjhd );

		for( j=0; j<n_ph; j++ ) for( k=0; k<ph[j].n; k++ ) {
			if( stanum[i] != ph[j].sta[k] ) continue;
			jhdkey.nev = ph[j].ev[k];
			jhdkey.order = ph[j].order[k];
			if( ph[j].ratio[k] == 1.0 ) {
				myjhd=(struct jhd_rec *)
					bsearch(&jhdkey, P_jhd, n_P_jhd, sizeof(struct jhd_rec), cmpjhd );
				}
			else {
				myjhd=(struct jhd_rec *)
					bsearch(&jhdkey, S_jhd, n_S_jhd, sizeof(struct jhd_rec), cmpjhd );
				}
			if( myjhd == NULL ) continue;
			ph[j].hit[k]=1;
			ph[j].T[k] = myjhd->T;
			ph[j].dTdx[k] = myjhd->dTdx;
			ph[j].dTdy[k] = myjhd->dTdy;
			ph[j].dTdz[k] = myjhd->dTdz;
			ph[j].azi[k]  = myjhd->azi;
			ph[j].theta[k]= myjhd->theta;
			ph[j].stax[k]=stax[i];
			ph[j].stay[k]=stay[i];
			ph[j].staz[k]=staz[i];
			}
		free(P_jhd); free(S_jhd);
		} /* next sta */

	ngood=0;
	for( j=0; j<n_ph; j++ ) {
		l=0;
		for( k=0; k<ph[j].n; k++ ) l+=ph[j].hit[k];
		if( l==ph[j].n ) {
			ph[j].good=1;
			ngood++;
			}
		}
	fprintf(stderr,"\tusable observations: %d\n", ngood);
	if( ngood==0 ) {
		fprintf(stderr,"warning: %s: no good observations\n");
		for( i=0; i<n_ev; i++ ) {
			x=ev[i].x; y=ev[i].y; z=ev[i].z; t0=ev[i].t0;
			printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
			}
		exit(0);
		}

	n = ngood;
	m = 4*n_ev+P_n_model_pars+ratio_n_model_pars;
	Grow = (double*)malloc(m*sizeof(double));
	G=new_sparse_matrix(n,m,0);
	d=(double*)malloc(n*sizeof(double));
	dp=(double*)malloc(n*sizeof(double));
	dm=(double*)malloc(m*sizeof(double));
	GTd=(double*)malloc(m*sizeof(double));
	if( (G==NULL) || (Grow==NULL ) || (d==NULL) || (dp==NULL) || (dm==NULL) || (GTd==NULL) ) {
		fprintf(stderr,"error: %s: out of memory (7)\n", argv[1] );
		exit(-1);
		}

	/* build matrix equation */
	i=0; irms=0; rms1=0.0;
	for( j=0; j<n_ph; j++ ) { /* loop over observations */
		if( !ph[j].good ) continue;
		for( k=0; k<m; k++ ) Grow[k]=0.0;
		x = ph[j].weight;
		d[i]=0.0;
		for( k=0; k<ph[j].n; k++ ) { /* loop over rays in an observation */
			y = ph[j].ratio[k];

			if( y==1.0 ) {
				model_to_P( my_PS_node, my_PS_tetra, &model_is_P );
				}
			else {
				model_to_S( my_PS_node, my_PS_tetra, &model_is_P );
				}

			c = ph[j].coeff[k];

			/* match event number */
			evkey.nev = ph[j].ev[k];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) {
				fprintf(stderr,"error: %s: cant find event %d in event file\n", argv[0], evkey.nev);
				exit(-1);
				}
			l=myev->index;

			/* right hand side of equation */
			d[i] -= x * c * ( ph[j].T[k] + myev->t0 );

			/* part of matrix associated with event locations */
			Grow[4*l]   += x * c * ph[j].dTdx[k];
			Grow[4*l+1] += x * c * ph[j].dTdy[k];
			Grow[4*l+2] += x * c * ph[j].dTdz[k];
			Grow[4*l+3] += x * c * tscale;

			/* part of matrix associated with velocity model parameters */
			x0[0]=ph[j].stax[k]; x0[1]=ph[j].stay[k]; x0[2]=ph[j].staz[k];
			if( !shoot_to_traveltime( x0, ph[j].azi[k], ph[j].theta[k], ph[j].T[k],
				x1, t1, arc, &n_arcs ) ) {
				ph[j].good=0;
				break;
				}

			/* count nodes affected by this ray */
			for( ii=0; ii<n_nodes; ii++ ) node[ii].touched=0;
			for( ii=0; ii<n_arcs; ii++ ) {
				node[ arc[ii].ptetra->n1 ].touched=1;
				node[ arc[ii].ptetra->n2 ].touched=1;
				node[ arc[ii].ptetra->n3 ].touched=1;
				node[ arc[ii].ptetra->n4 ].touched=1;
				}
			for( ii=0; ii<n_nodes; ii++ ) {
				if( node[ii].touched ) node[ii].hits++;
				node[i].touched=0;
				}

			/* will varying the P velocity model parameter change the traveltime ? */
			model_par=P_model_par; n_model_pars=P_n_model_pars;
			for( jj=0; jj<n_model_pars; jj++ ) {
				match=FALSE;
				for( ii=0; ii<n_arcs; ii++ ) {
					pt=arc[ii].ptetra;
					tnode[0]=pt->n1; tnode[1]=pt->n2;
					tnode[2]=pt->n3; tnode[3]=pt->n4;
					for( kk=0; kk<4; kk++ ) {
						if( node_in_model_par(tnode[kk],jj) ) {
							match=TRUE;
							break;
							}
						}
					if(match) break;
					}
				if( match ) {
					if( y == 1.0 ) {   /* this is a P ray */
						rms2 = dTdmp(jj,arc,n_arcs,ph[j].T[k]);
						Grow[4*n_ev+jj] += rw*x*c*rms2;
						}
					else { /* this is an S ray */
						rms2 = dTsdmpp(jj,arc,n_arcs,ph[j].T[k]);
						Grow[4*n_ev+jj] += rw*x*c*rms2;
						}
					}
				else {
					;/* Grow is zero */
					}
				} /* next P velocity model parameter */

			/* will varying the ratio velocity model parameter change the S traveltime ? */
			if( y != 1.0 ) { /* this is an S ray */
			model_par=ratio_model_par; n_model_pars=ratio_n_model_pars;
			for( jj=0; jj<n_model_pars; jj++ ) {
				match=FALSE;
				for( ii=0; ii<n_arcs; ii++ ) {
					pt=arc[ii].ptetra;
					tnode[0]=pt->n1; tnode[1]=pt->n2;
					tnode[2]=pt->n3; tnode[3]=pt->n4;
					for( kk=0; kk<4; kk++ ) {
						if( node_in_model_par(tnode[kk],jj) ) {
							match=TRUE;
							break;
							}
						}
					if(match) break;
					}
				if( match ) {
					rms2 = dTsdmpr(jj,arc,n_arcs,ph[j].T[k]);
					Grow[4*n_ev+P_n_model_pars+jj] += rw2*rw*x*c*rms2;
					}
				else {
					;/* Grow is zero */
					}
				}} /* next S velocity model parameter */

			} /* end loop over rays */

		if( !ph[j].good ) {
			fprintf(stderr,"observation %d (row %d) lost due to raytrace failure\n", j, i );
			continue;
			}

		if( verbose ) {
			fprintf(stderr,"weighted observation %d (row %d) obs=%f pred=%f res=%f\n",
			j, i, -x*ph[j].obs, d[i], x*ph[j].obs+d[i] );
			}

		/* reject observation if residual traveltime is too large */
		res = fabs(x*ph[j].obs+d[i]);
		if(  res > ph[j].maxres ) {
			fprintf(stderr,"observation %d (row %d) thrown out, residual %f > %f\n", j, i,
				res, ph[j].maxres );
			d[i]=0.0;
			}
		else {
			rms1 += res*res; irms++;
			d[i]+=x*ph[j].obs;
			ph[j].row=i;
			for( k=0; k<m; k++ ) if( Grow[k]!=0.0 ) {
				status = set_sparse_value(G,i,k,Grow[k]);
				if( !status ) {
					fprintf(stderr,"error: %s: out of memory (8)\n", argv[1] );
					exit(-1);
					}
				}
			i++;
			}
		} /* next observation */

	rms1 = sqrt( rms1 / (double) irms );

	maxG=sparse_absmax(G);
	k=sparse_count(G);
	fprintf(stderr,
		"G is %d by %d, is %f percent full, and has absolute maximum of %f\n",
		n, m, 100.0*(double)k/((double)n*(double)m), maxG );

	maxd=0.0;
	rms2=0.0;
	for( i=0; i<n; i++ ) {
		x=fabs(d[i]); rms2+=d[i];
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr, "d is %d by 1, and has absolute maximum of %f and a mean of %f\n", n, maxd, rms2/(double)n );
	fprintf(stderr,"Damping: %e\n\n", damping );

	smt_times_v( G, d, GTd );
	maxd=0.0;
	for( i=0; i<m; i++ ) {
		x=fabs(GTd[i]);
		if( x>maxd ) maxd=x;
		}
	fprintf(stderr,
	"GTd is %d by 1, and has absolute maximum of %f\n", m, maxd );

	scx = ATAabsmax( G );
	new_damping = damping*scx;
	fprintf(stderr,"Damping: %e abmaxGTG: %e Scaled_damping: %e\n\n",
		damping, scx, new_damping );
		
	for( i=0; i<m; i++ ) dm[i]=0.0;
	if( !sparse_dls_biconj( G, d, dm, new_damping, 1, maxd*B_PREC, m+10, &iter,&err,&biec ) ) {
		fprintf(stderr,"error: %s: dls_biconj fails\n",argv[0]);
		if(biec>0)exit(-1);
		}
		
	fprintf(stderr,"Matrix inversion (implicit): %d iterations; rms %f\n\n", iter, err );
	
	for( i=0; i<n_ev; i++ ) {
		x=ev[i].x+dm[4*i];
		y=ev[i].y+dm[4*i+1];
		z=ev[i].z+dm[4*i+2];
		t0=ev[i].t0+tscale*dm[4*i+3];
		printf("%d\t%f\t%f\t%f\t%f\n", ev[i].nev, x, y, z, t0 );
		}

	sm_times_v( G, dm, dp );

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res =  d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			for( k=0; k<n_sta; k++ ) if( stanum[k]==ph[i].sta[j] ) {
				stacount[k]++;
				starms[k]+= res*res;
				stamean[k]+= res;
				break;
				}
			}
		}

	fprintf(stderr,"station statistics:\n");
	fprintf(stderr,"sta\tcount\trms\tmean\n");
	for( i=0; i<n_sta; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\t%f\n", stanum[i], stacount[i],
			sqrt(starms[i]/(double)stacount[i]),
			stamean[i]/(double)stacount[i] );
		}
	fprintf(stderr,"\n");

	for( i=0; i<n_ph; i++ ) {
		if( (!ph[i].good) || (ph[i].row==(-1)) ) continue;
		res = d[ph[i].row] - dp[ ph[i].row ];
		for( j=0; j<ph[i].n; j++ ) {
			evkey.nev = ph[i].ev[j];
			myev=(struct ev_rec *)
				bsearch(&evkey, ev, n_ev, sizeof(struct ev_rec), cmpev );
			if( myev==NULL ) continue;
			myev->count++;
			myev->rms += res*res;
			myev->mean += res;
			}
		}

	fprintf(stderr,"event statistics:\n");
	fprintf(stderr,"event\tcount\trms\tmean\n");
	for( i=0; i<n_ev; i++ ) {
		fprintf(stderr,"%d\t%d\t%f\t%f\n",
			ev[i].nev, ev[i].count, sqrt(ev[i].rms/(double)ev[i].count),
			ev[i].mean/(double)ev[i].count);
		}
	fprintf(stderr,"\n");

	fprintf(stderr,"phase statistics:\n");
	fprintf(stderr,"phase\tline\terror\n");
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		fprintf(stderr,"%d\t%d\t%f\n", i, ph[i].line, res );
		}
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ev; i++ ) {
		res = dm[4*i]; rms2 += res*res;
		res = dm[4*i+1]; rms2 += res*res;
		res = dm[4*i+2]; rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)(3*n_ev) );
	fprintf(stderr, "event location statistics: rms change in position %f km\n", rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ev; i++ ) {
		res = tscale*dm[4*i+3]; rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_ev );
	fprintf(stderr, "event origin time statistics: rms change in origin time %f s\n", rms2);
	fprintf(stderr,"\n");

	model_to_P( my_PS_node, my_PS_tetra, &model_is_P );

	/* update P velocity of nodes */
	model_par=P_model_par; n_model_pars=P_n_model_pars;
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].v = node[k].v_orig+rw*weight*dm[4*n_ev+i];
			}
		}

	/* update ratio of nodes */
	model_par=ratio_model_par; n_model_pars=ratio_n_model_pars;
	for( i=0; i<n_model_pars; i++ ) {
		for( j=0; j<model_par[i].n; j++ ) {
			k=model_par[i].node[j];
			weight=model_par[i].weight[j];
			node[k].ratio = node[k].ratio_orig + rw2*rw*weight*dm[4*n_ev+P_n_model_pars+i];
			}
		}

	rms2=0.0;
	for( i=0; i<P_n_model_pars; i++ ) {
		res = dm[4*n_ev+i];
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)P_n_model_pars );
	fprintf(stderr, "P velocity model parameter statistics: rms size %f\n", rw*rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<ratio_n_model_pars; i++ ) {
		res = dm[4*n_ev+P_n_model_pars+i];
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)ratio_n_model_pars );
	fprintf(stderr, "ratio velocity model parameter statistics: rms size %f\n", rw2*rw*rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_nodes; i++ ) {
		res = node[i].v - node[i].v_orig;
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_nodes );
	fprintf(stderr, "P velocity nodes statistics: rms change %f km/s\n", rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_nodes; i++ ) {
		res = node[i].ratio - node[i].ratio_orig;
		rms2 += res*res;
		}
	rms2 = sqrt( rms2 / (double)n_nodes );
	fprintf(stderr, "ratio nodes statistics: rms change %f\n", rms2);
	fprintf(stderr,"\n");

	rms2=0.0;
	for( i=0; i<n_ph; i++ ) {
		j=ph[i].row;
		if( (!ph[i].good) || (j==(-1)) ) continue;
		res = d[j]-dp[j];
		rms2 += res*res;
		}

	rms2 = sqrt( rms2 / (double)irms );
	fprintf(stderr,"rms weighted residual original: %f final %f for %f percent reduction\n",
		rms1, rms2, 100.0-100.0*rms2/rms1 );

	/* write out perturbed model */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/* must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		if( QDEFINED )
		fprintf(nmfile,"%d %d %d %f %f %f %f %f %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].v, node[i].ratio, node[i].ifc );
		}
	fclose(nmfile);

	sprintf( hitsfile, "%s.hits", argv[7] );
	if( (nmfile=fopen(hitsfile,"w"))==NULL ) {
		fprintf(stderr,"error: %s: cant open hitsfile <%s> for write\n", argv[0], argv[11] );
		exit(-1);
		}

	/* write out hits file */
	fprintf(nmfile,"%d %d %d\n", model_L, model_M, model_N );
	for( i=0; i<n_nodes; i++ ) {
		k=node[i].id;
		/*must invert id=nz*L*M+ny*L+nx  */
		j = k%(model_L*model_M);
		nz = (k-j)/(model_L*model_M);
		nx = (k-nz*model_L*model_M)%model_L;
		ny = (k-nz*model_L*model_M-nx)/model_L;
		fprintf(nmfile,"%d %d %d %f %f %f %d %d\n", nx, ny, nz,
		node[i].x[0], node[i].x[1], node[i].x[2],
		node[i].hits, node[i].ifc );
		}
	fclose(nmfile);

	exit(0);
	}

int model_to_P( struct PS_node *my_PS_node, struct PS_tetra *my_PS_tetra, int *model_is_P )
{
	int i, j;
	if( *model_is_P ) return(1);
	
	for( i=0; i<n_nodes; i++ ) {
		node[i].v = my_PS_node[i].Pv;	
		}
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) tetra[i].vc[j] = my_PS_tetra[i].Pvc[j];
		for(j=0;j<3;j++) tetra[i].g[j] = my_PS_tetra[i].Pg[j];
		tetra[i].gs = my_PS_tetra[i].Pgs;
		for(j=0;j<3;j++) tetra[i].gn[j] = my_PS_tetra[i].Pgn[j];
		}
	*model_is_P = 1;
	return(1);
	}

int model_to_S( struct PS_node *my_PS_node, struct PS_tetra *my_PS_tetra, int *model_is_P )
{
	int i, j;
	if( ! (*model_is_P) ) return(1);
	
	for( i=0; i<n_nodes; i++ ) {
		node[i].v = my_PS_node[i].Sv;	
		}
	for( i=0; i<n_tetras; i++ ) {
		for(j=0;j<4;j++) tetra[i].vc[j] = my_PS_tetra[i].Svc[j];
		for(j=0;j<3;j++) tetra[i].g[j] = my_PS_tetra[i].Sg[j];
		tetra[i].gs = my_PS_tetra[i].Sgs;
		for(j=0;j<3;j++) tetra[i].gn[j] = my_PS_tetra[i].Sgn[j];
		}
	*model_is_P = 0;
	return(1);
	}

/* do two tetrahedra intersect ? returns TRUE if they do, FALSE otherwise */
int x_tetrahedron( double *x1, double *x2, double *x3, double *x4,
		   double *x5, double *x6, double *x7, double *x8 )
{
	/* theory:
		If two tetrahedra are non-intersecting, then there
		exists a set of planes such that all the vertices of one
		tetrahedron are on one side of the plane, and all
		the vertices of the other tetrahedron are on the
		other side of the plane. Furthermore, if such a set
		exists, then at least one plane defined by three of
		the eight vertices of the two tetrahedra is a member
		of the set.  So we simply test all possible groups
		of three vertices.  If the test plane is defined by three
		vertices, xa, xb, xc is then n = (xb-xa) cross (xc-xa) is
		a normal to that plane.  Two test vertices, x1 and x2
		are on the same side of that plane if (x1-xa) dot n
		and (x2-xa) dot n have the same sign.  Some special
		consideration must be given to the case where the test
		vertex is exactly on the plane.  That vertex cannot
		be used in the decision-making, but must rather be
		ignored. Due to round off error, this code may yield
		unpredictable results in this co-planar case, or in
		other cases where the two tetrahedra are extremely close
		to being intersecting or non-intersecting. */

	int i, ii, jj, kk, nn, first, separate_sides;

	double *x[8];      	/* vertices as list */
	double v1[3], v2[3]; 	/* vectors lying in plane */
	double n[3]; 		/* normal to plane */
	double tv[3]; 		/* test vector */
	double a, b;		/* test numbers */

	x[0]=x1; x[1]=x2; x[2]=x3; x[3]=x4; x[4]=x5; x[5]=x6; x[6]=x7; x[7]=x8;

	/* loop over all combinations of 3 vertices */
	for( ii=0; ii<=5; ii++ ) { for( jj=(ii+1); jj<=6; jj++ ) { for( kk=(jj+1); kk<=7; kk++ ) {
		for( i=0; i<3; i++ ) { /* three vertices define two vector */
			v1[i] = x[jj][i]-x[ii][i];
			v2[i] = x[kk][i]-x[ii][i];
			}
		cross( v1, v2, n ); /* two vectors define plane with this normal */
		separate_sides=TRUE; first=TRUE; b=0.0;
		for( nn=0; nn<8; nn++ ) { /* loop over all vertices */
			if( (nn==ii) || (nn==jj) || (nn==kk) ) continue;
				/* ignore vertices that define the test plane */
			for( i=0; i<3; i++ ) tv[i] = x[nn][i]-x[ii][i]; /* test vector */
			a=dot(n,tv); /* signed distance of vertex x[nn] from plane */
			if( nn>3 ) a=(-a); /* flip sign for vertices in second tetra */
					   /* now all a's will have the same sign only */
					   /* when the plane separates the two tetrahedra */
			if( a==0.0 ) continue;
				/* x[nn] is exactly on the plane, so test is ambiguous */
			else if( first ) { b=a; first=FALSE; }
				/* first x[nn], so save side of the plane its on */
			else if( ((a<0.0)&&(b>0.0)) || ((a>0.0)&&(b<0.0)) ) {
				/*  this x[nn] on different side of plane than first */
				separate_sides=FALSE;
				break;
				}
			}
		if( separate_sides ) return(FALSE);
		}}}

	return(TRUE);
	}

int init_tetra_stack() {
	n_tetra_stack=0;
	max_tetra_stack=100;
	tetra_stack=(int*)malloc(max_tetra_stack*sizeof(int));
	if( tetra_stack == NULL ) {
		tetra_stack=NULL;
		return(FALSE);
		}
	return(TRUE);
	}

int zero_tetra_stack() {
	n_tetra_stack=0;
	return(TRUE);
	}

int destroy_tetra_stack() {
	free(tetra_stack);
	tetra_stack=NULL;
	n_tetra_stack=0;
	return(TRUE);
	}

int push_tetra( int t ) {
	if( t == (-1) ) return(TRUE);
	if( tetra[t].tested ) return(TRUE);
	if( (n_tetra_stack+1) == max_tetra_stack ) {
		max_tetra_stack += 100;
		tetra_stack=(int*)realloc(tetra_stack,max_tetra_stack*sizeof(int));
		if( tetra_stack == NULL ) {
			tetra_stack=NULL;
			return(FALSE);
			}
		}
	tetra_stack[n_tetra_stack]=t;
	n_tetra_stack++;
	return(TRUE);
	}

int pop_tetra() {
	int t;
	if( n_tetra_stack == 0 ) t=(-1);
	else {
		t = tetra_stack[n_tetra_stack-1];
		n_tetra_stack--;
		}
	return(t);
	}
		
